#!/usr/bin/perl

use strict;
use Getopt::Long;
use Data::Dumper;

$Data::Dumper::Sortkeys = 1;

my $maxMapEntities = 50;
my $MissionMapFile;
my $verbose = 0;
my $optGetter = GetOptions(
  "missionmap|m=s" => \$MissionMapFile,
  "verbose!" => \$verbose,
);

if (! defined($MissionMapFile)) {
  die("ERROR: No MissionMap file option provided");
}

if (! -r $MissionMapFile) {
  die("ERROR: Unable to read from file '$MissionMapFile'");
}

my $maps_data;
my $current_map;
my $in_a_map = 0;
my $map_count = 0;
if (open(MM, "<", $MissionMapFile)) {
  L: while (<MM>) {
    chomp;
    chomp;
    my $line = $_;

    # "// Scout Map",
    # Just a comment, but useful to name map
    if ($line =~ /^\s+"\/\/[^\/](.*)"/) {
      $current_map = $1;
      print "Reading map #${map_count}: '$current_map'\n" if ($verbose);
      next(L);
    }

    # start of a map section
    # " +____1____2____3____4____5____6____7____8+",
    if ($line =~ /^\s+"\s+\+[_0-9]+\+"/) {
      $in_a_map = 1;
      $map_count++;
      if (! defined($current_map)) {
        $current_map = "map #${map_count}";
      }
      $maps_data->{'map_name'}->{$map_count} = $MissionMapFile .": ". $current_map;
      next(L);
    }

    # end of a map section
    # "      1    2    3    4    5    6    7    8",
    if ($line =~ /^\s+"[ 0-9]+",/) {
      $in_a_map = 0;
      undef($current_map);
      next(L);
    }

    # skip this line
    # +----------------------------------------+
    if ($line =~ /^\s+"\s+\+-+\+"/) {
      next(L);
    }

    next(L) if (! $in_a_map);

    # if we got here, then it must be map content!
    # "5|...............X.........Y..............|",
    if ($line =~ /^\s+"(\d+|\s+)\|(.+)\|"/) {
      my $content = $2;
      $maps_data->{'content'}->{$map_count} .= $content;
    }
  }
  close(MM);
} else {
  die("ERROR: Unable to open file '$MissionMapFile'");
}

my $census;
foreach my $map_num (sort(keys(%{$maps_data->{'content'}}))) {
  $census->{$map_num}->{'map_name'} = $maps_data->{'map_name'}->{$map_num};
  my @content = split(//, $maps_data->{'content'}->{$map_num}); 
  foreach my $item (@content) {
    $census->{$map_num}->{'items'}->{$item}++ if ($verbose);
    if ($item !~ /[\.a-zA-Z]/) {
      $census->{$map_num}->{'map_entities'}++;
    }
    if ($item =~ /[A-Z]/) {
      $census->{$map_num}->{'ship_entities'}++;
    }
    if ($item eq ' ') {
      print "FIXME: Space detected: ". $maps_data->{'map_name'}->{$map_num} ."\n";
    }
  } 
}

foreach my $map_num (sort(keys(%{$census})))  {
  if ($census->{$map_num}->{'map_entities'} > $maxMapEntities) {
    print Dumper $census->{$map_num};
  }
}

