/********************************************************************************************************\

	File Name:	QShipTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_11ConvoyRaid.h"
#include "QShipTeam.h"
#include "QShipTeamInitialState.h"
#include "QShipTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

#define		kSmallQShipBPV		53
#define		kSmallMerQShipBPV	55
#define		kLargeMerQShipBPV	88
#define		kLargeQShipBPV		98

tClassAllocator<tQShipTeam> gQShipTeamAllocator;

tQShipTeam::tQShipTeam()
{
}
void tQShipTeam::mSetupVictoryCondition()
{
}
void tQShipTeam::mSetupState()
{
	mRegisterState( new tQShipTeamInitialState() );
	mGotoState( typeid( tQShipTeamInitialState ) );
}
void tQShipTeam::mSetupCommState()
{
}
void tQShipTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tQShipTeam::mCreateShipsForTeam\n");
	gST.mCompleteNarrowedList();

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 numberOfQShips = gWC.mGetNumQShips();
	int32 qShipSizeDice = gWC.mGetFrtrSizeDice();
	int32 qShipBPV = 0;

	if ((qShipSizeDice < 60) && (diffLevel < 2)) {
		qShipBPV = 45;
	} else {
		qShipBPV = 90;
	}
	qShipSizeDice = 100;

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {

		int32 fShiplistCount = 0;
		tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
		eRaceName freighterRace = freighterTeam->mGetRace();

		ShipList QShipTeam_sl = gST.narrowed_sl;
		std::list< eRace >  raceList;
		raceList.push_back(freighterRace);
		fShiplistCount = QShipTeam_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", freighterRace, fShiplistCount);

		QShipTeam_sl = gST.stripOutSPECIAL(QShipTeam_sl);
		QShipTeam_sl = gST.stripOutRestricted(QShipTeam_sl);


		// prepare an Auxiliary if available
		ShipList aux_sl = QShipTeam_sl;
		std::list< eEnhancedClassTypes >  auxClassTypeList;
		auxClassTypeList.push_back(kEClassSmallAuxiliary);
		auxClassTypeList.push_back(kEClassLargeAuxiliary);
		fShiplistCount = aux_sl.Subset(auxClassTypeList, true);
		gST.mDebug_fprintf("subset for auxiliary filter completed. ships: %d\n", fShiplistCount);
		// aux classes: warship, carrier, PFT
		std::list< eEnhancedRoles >  auxRolesList;
		auxRolesList.push_back(kWarShip);
		auxRolesList.push_back(kCarrier);
		auxRolesList.push_back(kPFTender);
		fShiplistCount = aux_sl.Subset(auxRolesList, true);
		gST.mDebug_fprintf("subset for auxiliary eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(aux_sl);


		// prepare a CL for the slight chance of generating one
		ShipList cl_sl = QShipTeam_sl;
		std::list< eEnhancedClassTypes >  lightClassTypeList;
		lightClassTypeList.push_back(kEClassLightCruiser);
		lightClassTypeList.push_back(kEClassMediumCruiser);
		lightClassTypeList.push_back(kEClassWarCruiser);
		lightClassTypeList.push_back(kEClassHeavyWarDestroyer);
		fShiplistCount = cl_sl.Subset(lightClassTypeList, true);
		gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
		// cl classes: warship, carrier, PFT
		std::list< eEnhancedRoles >  clRolesList;
		clRolesList.push_back(kLineShip);
		clRolesList.push_back(kWarShip);
		clRolesList.push_back(kLeader);
		clRolesList.push_back(kDroneBombardmentShip);
		clRolesList.push_back(kEscort);
		clRolesList.push_back(kCarrier);
		clRolesList.push_back(kPFTender);
		clRolesList.push_back(kFastCruiser);
		clRolesList.push_back(kMauler);
		clRolesList.push_back(kPolice);
		fShiplistCount = cl_sl.Subset(clRolesList, true);
		gST.mDebug_fprintf("subset for CL eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(cl_sl);


		// Now for the Qships, themselves.
		// military freighters or APTs only.
		std::list< eEnhancedClassTypes >  enhancedClassTypeList;
		enhancedClassTypeList.push_back(kEClassSmallFreighter);
		enhancedClassTypeList.push_back(kEClassPriorityTransport);
		enhancedClassTypeList.push_back(kEClassCivilianFreighter);
		enhancedClassTypeList.push_back(kEClassLargeFreighter);
		fShiplistCount = QShipTeam_sl.Subset(enhancedClassTypeList, true);
		gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);
		// QShips only
		std::list< eEnhancedRoles >  rolesList;
		rolesList.push_back(kQship);
		fShiplistCount = QShipTeam_sl.Subset(rolesList, true);
		gST.mDebug_fprintf("subset for QShip eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(QShipTeam_sl);


		// if there's an auxiliary available, use it as 2nd qship.
		ShipList QShip2sl = QShipTeam_sl;
		if (aux_sl.size() > 0) {
			QShip2sl = aux_sl;
		} 


		Ship s1;
		Ship s2;
		Ship s3;

		if ( (QShipTeam_sl.size() > 0) && (QShipTeam_sl.SelectRandomShip(s1)) && (QShip2sl.SelectRandomShip(s2)) )
		{
			if (numberOfQShips > 0) {
				gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip(
					typeid( tQShipTeamShip ),
					s1.shipCode.c_str(),
					kStartPosition_P,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}

			if (numberOfQShips > 1) {
				gST.mDebug_fprintf("Ship2 selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
				gST.mTrackOrions( s2 );
				mCreateShip(
					typeid( tQShipTeamShip ),
					s2.shipCode.c_str(),
					kStartPosition_O,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );

				if ((qShipSizeDice > 80) && (cl_sl.size() > 0) && (cl_sl.SelectRandomShip(s3))) {
					gST.mDebug_fprintf("Ship3 (CL) selected! %s:  %d BPV\n", s3.shipCode.c_str(), s3.bpv);
					gST.mTrackOrions( s3 );
					mCreateShip( 
						typeid( tQShipTeamShip ),
						s3.shipCode.c_str(),
						kStartPosition_Z, 
						kNoMetaShipID,
						gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset ) );
				} else {
					// fallback method, just in case.
					mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassLightCruiser, 
						35, 
						kStartPosition_Z, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset ) );
				}
			}
		} else {
			mOldCreateShipsForTeam( qShipBPV );
		}
	} else {
		mOldCreateShipsForTeam( qShipBPV );
	}

	gST.mDebug_fprintf("--- Finished: tQShipTeam::mCreateShipsForTeam\n");
}


void tQShipTeam::mOldCreateShipsForTeam( int32 qShipBPV ) {
	gST.mDebug_fprintf("--- Starting: tQShipTeam::mOldCreateShipsForTeam(%d)\n", qShipBPV);

	int32 numberOfQShips = gWC.mGetNumQShips();
	int32 qShipSizeDice = gWC.mGetFrtrSizeDice();

	if (numberOfQShips > 0) {
		mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassFreighter, 
						35, 
						kStartPosition_P, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}

	if (numberOfQShips > 1) {
		qShipBPV += 10;
		mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassFreighter, 
						35, 
						kStartPosition_O, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
		if (qShipSizeDice > 80) {
			mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassLightCruiser, 
						35, 
						kStartPosition_Z, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset ) );
		}
	}

	gST.mDebug_fprintf("--- Finished: tQShipTeam::mOldCreateShipsForTeam(%d)\n", qShipBPV);
}


void tQShipTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tQShipTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tQShipTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tQShipTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}
