/********************************************************************************************************\

	File Name:	AttackTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_12ConvoyEscort.h"
#include "AttackTeam2.h"
#include "AttackTeam2InitialState.h"
//#include "AttackTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

#include "ShipList_API.h"

tClassAllocator<tAttackTeam2> gAttackTeam2Allocator;

tAttackTeam2::tAttackTeam2()
{
	gWC.mAddAttackTeam();
	fdice = fMissionInfo->mRandomInt32( 0, 2 );
}

void tAttackTeam2::mSetupVictoryCondition()
{
	fVictoryCondition = new tAttackVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam2::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tAttackTeam2::mSetupState()
{
	mRegisterState( new tAttackTeam2InitialState() );
	mGotoState( typeid( tAttackTeam2InitialState ) );
}
void tAttackTeam2::mSetupCommState()
{
}


void tAttackTeam2::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam2::mCreateShipsForTeam\n");
	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		ShipList ff_sl = gST.narrowed_sl;

		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		std::list< eRace >  raceList;
		raceList.push_back(mGetRace());
		fShiplistCount = ff_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

		ff_sl = gST.stripOutSPECIAL( ff_sl );
		ff_sl = gST.stripOutRestricted( ff_sl );

		std::list< eEnhancedClassTypes >  FFTypeList;
		FFTypeList.push_back(kEClassFrigate);
		if (diffLevel == 2) {
			FFTypeList.push_back(kEClassDestroyer);
			FFTypeList.push_back(kEClassWarDestroyer);
		}
		fShiplistCount = ff_sl.Subset(FFTypeList);
		gST.mDebug_fprintf("subset for FF filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  ffRolesList;
		ffRolesList.push_back(kLineShip);
		ffRolesList.push_back(kWarShip);
		ffRolesList.push_back(kLeader);
		ffRolesList.push_back(kDroneBombardmentShip);
		ffRolesList.push_back(kEscort);
		ffRolesList.push_back(kCarrier);
		ffRolesList.push_back(kPFTender);
		ffRolesList.push_back(kFastCruiser);
		ffRolesList.push_back(kMauler);
		ffRolesList.push_back(kPolice);
		fShiplistCount = ff_sl.Subset(ffRolesList);
		gST.mDebug_fprintf("subset for FF eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(ff_sl);

		Ship s1;

		if ( (ff_sl.size() > 0) && (ff_sl.SelectRandomShip(s1)) ) {
			mCreateShip( typeid( tShipInfo ),
				s1.shipCode.c_str(),
				kStartPosition_Q + fdice,
				kNoMetaShipID,
				gWC.fNWCustID++, 
				0, 0, -1, -1, NULL, 
				static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tAttackTeam2::mCreateShipsForTeam\n");
}

void tAttackTeam2::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam2::mOldCreateShipsForTeam\n");

	int32 frigateBPV = 70;
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	if( diffLevel == 2 )
	{
		// Any FRIGATE or DESTROYER or WAR_DESTROYER or HEAVY_WAR_DESTROYER  (any sizeclass 4)
		frigateBPV = 200;
	}

	// NOT UPDATED FOR SHIPLIST API: tAttackTeam2 is not used in Met_ED12ConvoyEscort
	mCreateShip( typeid( tShipInfo ),
						  frigateBPV,
						  kClassFrigate, 
						  10, 
						  kStartPosition_Q + fdice,
						  gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("--- Finished: tAttackTeam2::mOldCreateShipsForTeam\n");
}


void tAttackTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// no actually drafting anything

	gST.mDebug_fprintf("--- Finished: tAttackTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam2::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}