//	<Name>	AttackVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "AttackVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_12ConvoyEscort.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "AttackTeam.h"
//#include "TeamScouted.h"
#include "WorldConditions.h"

tAttackVictoryState::tAttackVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tAttackVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );
	attackTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam2 ) );
	attackTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam3 ) );
			
	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	qShipTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kQShipTeam ) );
	
	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 1;
	fPlayerLeftAlready = false;
	fThisPlayerDead = false;
}

void tAttackVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );

	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}
	if( attackTeam2 )
	{
		attackBPV += attackTeam2->mGetCombatBPV();
	}
	if( attackTeam3 )
	{
		attackBPV += attackTeam3->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( freighterTeam )
	{
		defendBPV += freighterTeam->mGetCombatBPV();
	}
	if( qShipTeam )
	{
		defendBPV += qShipTeam->mGetCombatBPV();
	}
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled())))
		{
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}

void tAttackVictoryState::mEndState()
{
}

eVictoryLevels tAttackVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 destroyedFreighters = gWC.mGetFreightersDestroyed();
	int32 attackTeamsRemaining = gWC.mGetAttackTeams();
	int32 escapedFreighters = gWC.mGetFreightersEscaped();
	int32 originalNumberFreighters = gWC.mGetFreightersOriginal();

	if( attackTeamsRemaining > 0 )
	{
		destroyedFreighters = originalNumberFreighters - escapedFreighters;
	}

	if( ! fThisPlayerDead && ( destroyedFreighters > ( originalNumberFreighters / 2 ) ) )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 50 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
	}
	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if( destroyedFreighters == originalNumberFreighters )
	{ 
		return kAstoundingVictory;
	}
	else if( destroyedFreighters >  ( originalNumberFreighters / 2 ) )
	{
		return kVictory;
	}
	else if( destroyedFreighters == 1 )
	{
		return kDefeat;
	}
	else if( destroyedFreighters == 0 )
	{
		if( attackBPV > defendBPV )
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kDevastatingDefeat;
	}
	else
	{
		return kDraw;
	}
}
int32 tAttackVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tAttackVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();


	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveAttackTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		gWC.mCheckForGameEnd();
	}

}
void tAttackVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tAttackVictoryState::mOnCapture( tShipInfo* ship )
{
	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToBonusPrestige( 90 );
	}
	
	mCheckAllTeams( ship );
}

void tAttackVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveAttackTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
//				fTeamDisengaged = true;
				gWC.mCheckForGameEnd();
			}
		}
	}
}
void tAttackVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tAttackVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tAttackVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveAttackTeam();
		fPlayerLeftAlready = true;
		gWC.mCheckForGameEnd();
	}
}