/********************************************************************************************************\

	File Name:	FreighterTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_12ConvoyEscort.h"
#include "FreighterTeam.h"
#include "FreighterTeamInitialState.h"
#include "FreighterTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

#include "ShipList_API.h"

#define		kSmallFtrBPV		0
#define		kLargeFtrBPV		45
#define		kSmallArmedFtrBPV	35
#define		kMediumMerBPV		40
#define		kLargeArmedFtrBPV	50
#define		kLargeMerBPV		55

tClassAllocator<tFreighterTeam> gFreighterTeamAllocator;

tFreighterTeam::tFreighterTeam()
{
}
void tFreighterTeam::mSetupVictoryCondition()
{
}
void tFreighterTeam::mSetupState()
{
    mRegisterState( new tFreighterTeamInitialState() );
    mGotoState( typeid( tFreighterTeamInitialState ) );
}
void tFreighterTeam::mSetupCommState()
{
}
void tFreighterTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateShipsForTeam\n");
	gST.mCompleteNarrowedList();


	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 numberOfFreighters = 4;
	int32 numberOfQShips = 0;
	int32 dice = fMissionInfo->mRandomInt32( 1, 100 );
	int32 QShipDice = fMissionInfo->mRandomInt32( 1, 100 );
	int32 merchantShipDice = fMissionInfo->mRandomInt32( 1, 100 );


	gWC.mSetFrtrSizeDice( merchantShipDice );

	if( QShipDice > 75 )
	{
		numberOfQShips = 2;
	}
	else if( QShipDice > 50 )
	{
		numberOfQShips = 1;
	}

	gWC.mSetNumQShips( numberOfQShips );

	if( dice > 75 )
	{
		numberOfFreighters = 6;
	}
	else if( dice > 5 )
	{
		numberOfFreighters = 5;
	}
	
	numberOfFreighters -= numberOfQShips;

	for( int i = 0; i < numberOfFreighters; ++i )
	{
		gWC.mAddFreighter();
	}


	if ((gST.narrowed_sl.size() != 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {

		int32 fShiplistCount = 0;
		ShipList CargoTeam_sl = gST.narrowed_sl;

		tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
		eRaceName freighterRace = freighterTeam->mGetRace();

		std::list< eRace >  raceList;
		raceList.push_back(freighterRace);
		fShiplistCount = CargoTeam_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", freighterRace, fShiplistCount);

		// CARGO role
		std::list< eEnhancedRoles >  CargoList;
		CargoList.push_back(kCargo);
		fShiplistCount = CargoTeam_sl.Subset(CargoList, true);
		gST.mDebug_fprintf("subset for Cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		// .. but not QSHIPS role
		std::list< eEnhancedRoles >  qShipList;
		qShipList.push_back(kQship);
		fShiplistCount = CargoTeam_sl.Subset(qShipList, false);
		gST.mDebug_fprintf("filter out qShips via eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		CargoTeam_sl = gST.stripOutSPECIAL(CargoTeam_sl);
		CargoTeam_sl = gST.stripOutRestricted(CargoTeam_sl);
		gST.listShiplistContent(CargoTeam_sl);


		
		// Now we want a list just for the Freighters
		ShipList FreighterTeam_sl = CargoTeam_sl;

		// military freighters or APTs only.
		std::list< eEnhancedClassTypes >  enhancedClassTypeListF;
		enhancedClassTypeListF.push_back(kEClassSmallFreighter);
		enhancedClassTypeListF.push_back(kEClassPriorityTransport);
		enhancedClassTypeListF.push_back(kEClassCivilianFreighter);
		enhancedClassTypeListF.push_back(kEClassLargeFreighter);
		enhancedClassTypeListF.push_back(kEClassHeavyFreighter);
		fShiplistCount = FreighterTeam_sl.Subset(enhancedClassTypeListF, true);
		gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

		int32 MinBPV = FreighterTeam_sl.MinimumBPV();
		int32 MaxBPV = FreighterTeam_sl.MaximumBPV();
		int32 kCargoShip1BPV = MinBPV;
		int32 kCargoShip2BPV = MinBPV + ((MaxBPV - MinBPV) / 4);
		int32 kCargoShip3BPV = MinBPV + ((MaxBPV - MinBPV) / 2);
		int32 kCargoShip4BPV = MinBPV + ((MaxBPV - MinBPV) * (3 / 4));
		int32 kCargoShip5BPV = MaxBPV;

		Ship s1, s2, s3, s4, s5, s6;
		int32 MinNPCBPV, MaxNPCBPV;

		ShipList CargoShip1_sl, CargoShip2_sl, CargoShip3_sl, CargoShip4_sl, CargoShip5_sl, CargoShip6_sl;

		CargoShip1_sl = FreighterTeam_sl;
		MinNPCBPV = kCargoShip1BPV * 0.875;
		MaxNPCBPV = kCargoShip1BPV * 1.125;
		fShiplistCount = CargoShip1_sl.Subset(ShipList::kSubsetBPV, MinNPCBPV, MaxNPCBPV);
		gST.mDebug_fprintf("CargoShip1 subset: %d\n", fShiplistCount);

		// if #2 fails, use #1's list
		CargoShip2_sl = FreighterTeam_sl;
		MinNPCBPV = kCargoShip2BPV * 0.875;
		MaxNPCBPV = kCargoShip2BPV * 1.125;
		fShiplistCount = CargoShip2_sl.Subset(ShipList::kSubsetBPV, MinNPCBPV, MaxNPCBPV);
		gST.mDebug_fprintf("CargoShip2 subset: %d\n", fShiplistCount);
		if (fShiplistCount == 0)
		{
			CargoShip2_sl = CargoShip1_sl;
		}

		if ( numberOfFreighters > 2 ) {
			// if #3 fails, use #1's list
			CargoShip3_sl = FreighterTeam_sl;
			MinNPCBPV = kCargoShip3BPV * 0.875;
			MaxNPCBPV = kCargoShip3BPV * 1.125;
			fShiplistCount = CargoShip3_sl.Subset(ShipList::kSubsetBPV, MinNPCBPV, MaxNPCBPV);
			gST.mDebug_fprintf("CargoShip3 subset: %d\n", fShiplistCount);
			if (fShiplistCount == 0)
			{
				CargoShip3_sl = CargoShip1_sl;
			}
		}

		if ( numberOfFreighters > 3 ) {
			// Yes, do #5 before #4. Sanity!
			CargoShip5_sl = FreighterTeam_sl;
			MinNPCBPV = kCargoShip5BPV * 0.875;
			MaxNPCBPV = kCargoShip5BPV * 1.125;
			fShiplistCount = CargoShip5_sl.Subset(ShipList::kSubsetBPV, MinNPCBPV, MaxNPCBPV);
			gST.mDebug_fprintf("CargoShip5 subset: %d\n", fShiplistCount);

			// if #4 fails, use #5's list
			CargoShip4_sl = FreighterTeam_sl;
			MinNPCBPV = kCargoShip4BPV * 0.875;
			MaxNPCBPV = kCargoShip4BPV * 1.125;
			fShiplistCount = CargoShip4_sl.Subset(ShipList::kSubsetBPV, MinNPCBPV, MaxNPCBPV);
			gST.mDebug_fprintf("CargoShip4 subset: %d\n", fShiplistCount);
			if (fShiplistCount == 0)
			{
				CargoShip4_sl = CargoShip5_sl;
			}
		}

		if (numberOfFreighters == 6) {
			// FF, DD, and DW
			// If this fails, use #5
			CargoShip6_sl = CargoTeam_sl;
			std::list< eEnhancedClassTypes >  enhancedClassTypeListFF;
			enhancedClassTypeListFF.push_back(kEClassFrigate);
			enhancedClassTypeListFF.push_back(kEClassDestroyer);
			enhancedClassTypeListFF.push_back(kEClassWarDestroyer);
			fShiplistCount = CargoShip6_sl.Subset(enhancedClassTypeListFF, true);
			gST.mDebug_fprintf("subset for FF filter completed. ships: %d\n", fShiplistCount);
			if (fShiplistCount == 0)
			{
				CargoShip6_sl = CargoShip5_sl;
			}
		}
		
		if ( 
			(FreighterTeam_sl.size() > 0) &&
			(CargoShip1_sl.SelectRandomShip(s1)) &&
			(CargoShip2_sl.SelectRandomShip(s2)) &&
			(CargoShip3_sl.SelectRandomShip(s3) || ( numberOfFreighters <= 2 )) &&
			(CargoShip4_sl.SelectRandomShip(s4) || ( numberOfFreighters <= 3 )) &&
			(CargoShip5_sl.SelectRandomShip(s5) || ( numberOfFreighters <= 4 )) &&
			(CargoShip6_sl.SelectRandomShip(s6) || ( numberOfFreighters <= 5 ))
		) {
			gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );
			mCreateShip(
				typeid( tFreighterTeamShip ),
				s1.shipCode.c_str(),
				kStartPosition_K,
				kNoMetaShipID,
				gWC.fNWCustID++, 0, 0, -1, -1,
				NULL,
				static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );

			gST.mDebug_fprintf("Ship2 selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
			gST.mTrackOrions( s2 );
			mCreateShip(
				typeid( tFreighterTeamShip ),
				s2.shipCode.c_str(),
				kStartPosition_L,
				kNoMetaShipID,
				gWC.fNWCustID++, 0, 0, -1, -1,
				NULL,
				static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );

			if ( numberOfFreighters > 2 )
			{
				gST.mDebug_fprintf("Ship3 selected! %s:  %d BPV\n", s3.shipCode.c_str(), s3.bpv);
				gST.mTrackOrions( s3 );
				mCreateShip(
					typeid( tFreighterTeamShip ),
					s3.shipCode.c_str(),
					kStartPosition_M,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}

			if ( numberOfFreighters > 3 )
			{
				gST.mDebug_fprintf("Ship4 selected! %s:  %d BPV\n", s4.shipCode.c_str(), s4.bpv);
				gST.mTrackOrions( s4 );
				mCreateShip(
					typeid( tFreighterTeamShip ),
					s4.shipCode.c_str(),
					kStartPosition_N,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}

			if ( numberOfFreighters > 4 )
			{
				gST.mDebug_fprintf("Ship5 selected! %s:  %d BPV\n", s5.shipCode.c_str(), s5.bpv);
				gST.mTrackOrions( s5 );
				mCreateShip(
					typeid( tFreighterTeamShip ),
					s5.shipCode.c_str(),
					kStartPosition_O,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}
			if ( numberOfFreighters == 6 )
			{
				gST.mDebug_fprintf("Ship6 selected! %s:  %d BPV\n", s6.shipCode.c_str(), s6.bpv);
				gST.mTrackOrions( s6 );
				mCreateShip(
					typeid( tFreighterTeamShip ),
					s6.shipCode.c_str(),
					kStartPosition_O,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}
		} else {
			mOldCreateShipsForTeam( numberOfFreighters );
		}
	} else {
		mOldCreateShipsForTeam( numberOfFreighters );
	}

	gST.mDebug_fprintf("--- Finished: tFreighterTeam::mCreateShipsForTeam\n");
}


void tFreighterTeam::mOldCreateShipsForTeam( int32 numberOfFreighters )
{
	gST.mDebug_fprintf("--- Entering: tFreighterTeam::mOldCreateShipsForTeam(%d)\n", numberOfFreighters);

	mCreateShip( typeid( tFreighterTeamShip ),
						  kSmallFtrBPV,
						  kClassFreighter, 
						  10, 
						  kStartPosition_K, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );

	mCreateShip( typeid( tFreighterTeamShip ),
						  kSmallArmedFtrBPV,
						  kClassFreighter, 
						  15, 
						  kStartPosition_L, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );

	if( numberOfFreighters > 2 )
	{
		mCreateShip( typeid( tFreighterTeamShip ),
							  kMediumMerBPV,
							  kClassFreighter, 
							  20, 
							  kStartPosition_M, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ));
	}
	if( numberOfFreighters > 3 )
	{
		mCreateShip( typeid( tFreighterTeamShip ),
							  kLargeFtrBPV,
							  kClassFreighter, 
							  25, 
							  kStartPosition_N, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ));
	}
	if( numberOfFreighters > 4 )
	{
		mCreateShip( typeid( tFreighterTeamShip ),
							  kLargeArmedFtrBPV,
							  kClassFreighter, 
							  30, 
							  kStartPosition_O, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}
	if( numberOfFreighters == 6 )
	{
		mCreateShip( typeid( tFreighterTeamShip ),
							  kLargeMerBPV,
							  kClassFreighter, 
							  35, 
							  kStartPosition_O, gWC.fNWCustID++, 0, 0, -1, -1, NULL, static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}

	gST.mDebug_fprintf("--- Finished: tFreighterTeam::mOldCreateShipsForTeam\n");
}

void tFreighterTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tFreighterTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tFreighterTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}