/********************************************************************************************************\

	Mission:	Convoy Escort
	Script:		"Met_12ConvoyEscort"
	Descript:	<DESCRIPTION>

	Created by:		David F		06/29/00
	Last Updated:	DF
					converted from Q-ship suprise
					DJS			10/08/00
					Reversed the order in which victory condition is checked when a freighter is
					destroyed and other freighters have already disengaged.
					LL			1/2/11
					More sensitive ship selection using ShipList API

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Met_12ConvoyEscort.h"
#include "DefendTeam.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "FreighterTeam.h"
#include "QShipTeam.h"
#include "WorldConditions.h"
#include <time.h>

#include "ShipList_API.h"

DEFINE_SCRIPT_CLASS( tMet_12ConvoyEscort );

tMet_12ConvoyEscort::tMet_12ConvoyEscort()
{
	srand(time(NULL));
}
void tMet_12ConvoyEscort::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kPlayerRaceHex|kAlliedRaceHex, kShippingLaneHex|kAnyBaseHex|kAnyPlanetHex );
	Specs->fWait	= 4;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( kHalfBand | k3Client );
	Specs->fGenericFlags = kNoGenericFlags; ;
	
	/*
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAsteroidHex | kNebulaHex | kBlackHoleHex | kShippingLaneHex | kDustCloudsHex | kEmptySpaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;
	*/

	tTeamSpec team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
																		kMaxChance,
																		kTrue,
																		kPrimaryTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kAnyRace,
																		typeid( tDefendTeam ),
																		kNoTeamTag,
																		Messages[kPlayerDefenderTeamTitle_msg] );

	tTeamSpec team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
																		kMaxChance,
																		kTrue,
																		kPrimaryOpponentTeam,
																		kMaxChance,
																		kSpecPlayableRace|kSpecOrion|kSpecMonster,
																		kAnyEnemyOf| kDefendTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );

	tTeamSpec team3 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
																		kMaxChance,
																		kFalse,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kAnyEnemyOf| kDefendTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );

	tTeamSpec team4 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
																		kMinChance,
																		kFalse,
																		kPlayableTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyEnemyOf | kDefendTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );


	tTeamSpec team5 = mCreateTeamSpec(	"Freighter Team", static_cast< eTeamID>( kFreighterTeam ),
																		kMaxChance,
																		kTrue,
																		kNPCTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kSameAs | kDefendTeam1,
																		typeid( tFreighterTeam ) );

	tTeamSpec team6 = mCreateTeamSpec(	"QShip Team", static_cast< eTeamID>( kQShipTeam ),
																		kMinChance,
																		kTrue,
																		kNPCTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kSameAs | kDefendTeam1,
																		typeid( tQShipTeam ) );
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );

}
int32 tMet_12ConvoyEscort::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_12ConvoyEscort::mCreateExternalShips( int32 Mode )
{
}
int32 tMet_12ConvoyEscort::mInitializeFinished( void )
{
	return kDefendBriefing_msg;
}
void tMet_12ConvoyEscort::mNewInternalShip( tShipInfo* Ship )
{
}
void tMet_12ConvoyEscort::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Convoy escort (12)");
	gWC.mCheckCRC();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
		kFreighterTeam,	//Frtr
		kQShipTeam,		//QShp
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Atk2	Atk3	Frtr	Qshp
		/*Def1*/	"--",	"WD",	"WD",	"WD",	"AI",	"AI",
		/*Atk1*/	"WI",	"--",	"AI",	"AI",	"WD",	"WD",
		/*Atk2*/	"WI",	"AI",	"--",	"AI",	"WD",	"WD",
		/*Atk3*/	"WI",	"AI",	"AI",	"--",	"WD",	"WD",
		/*Frtr*/	"AI",	"WI",	"WI",	"WI",	"--",	"AI",
		/*QShp*/	"AI",	"WI",	"WI",	"WI",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_12ConvoyEscort::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainShippingLane, kPlanetAnyPlanet|kPlanetNone, kBaseAnyBase|kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_12ConvoyEscort::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['L'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['M'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['N'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['B'] = kObjectBlackHole;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_12ConvoyEscort::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassDestroyer, kClassHeavyCruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassDestroyer, kClassHeavyCruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassDestroyer, kClassHeavyCruiser, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassHeavyCruiser, 0.65f, 1.2f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassHeavyCruiser, 0.675f, 1.225f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassHeavyCruiser, 0.7f, 1.25f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassLightCruiser, 0.55f, 1.15f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassLightCruiser, 0.575f, 1.175f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassLightCruiser, 0.6f, 1.2f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassLightCruiser, 0.45f, 1.10f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassLightCruiser, 0.475f, 1.124f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassLightCruiser, 0.5f, 1.15f );
}
void tMet_12ConvoyEscort::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		fBriefingIndex[kDefendTeam1][j] = kDefendBriefing_msg;
		fBriefingIndex[kAttackTeam1][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam2][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam3][j] = kAttackBriefing_msg;
	}
}