/********************************************************************************************************\

	File Name:	QShipTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_12ConvoyEscort.h"
#include "QShipTeam.h"
#include "QShipTeamInitialState.h"
#include "QShipTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

#include "ShipList_API.h"

tClassAllocator<tQShipTeam> gQShipTeamAllocator;

tQShipTeam::tQShipTeam()
{
}
void tQShipTeam::mSetupVictoryCondition()
{
}
void tQShipTeam::mSetupState()
{
    mRegisterState( new tQShipTeamInitialState() );
    mGotoState( typeid( tQShipTeamInitialState ) );
}
void tQShipTeam::mSetupCommState()
{
}
void tQShipTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tQShipTeam::mCreateShipsForTeam\n");
	gST.mCompleteNarrowedList();

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 numberOfQShips = gWC.mGetNumQShips();
	int32 qShipSizeDice = gWC.mGetFrtrSizeDice();
	int32 qShipBPV = 0;

	if ((qShipSizeDice < 70) && (diffLevel < 2)) {
		// Small QShip
		qShipBPV = 45;
	} else {
		// Large QShip
		qShipBPV = 90;
	}


	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {

		int32 fShiplistCount = 0;
		ShipList QShipTeam_sl = gST.narrowed_sl;

		tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
		eRaceName freighterRace = freighterTeam->mGetRace();

		std::list< eRace >  raceList;
		raceList.push_back(freighterRace);
		fShiplistCount = QShipTeam_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", freighterRace, fShiplistCount);

		QShipTeam_sl = gST.stripOutSPECIAL( QShipTeam_sl );


		// prepare an Auxiliary if available
		ShipList aux_sl, saved_state_aux_sl = QShipTeam_sl;
		std::list< eEnhancedClassTypes >  auxClassTypeList;
		if ((qShipSizeDice < 70) && (diffLevel < 2)) {
			auxClassTypeList.push_back(kEClassSmallAuxiliary);
		} else {
			auxClassTypeList.push_back(kEClassLargeAuxiliary);
		}
		fShiplistCount = aux_sl.Subset(auxClassTypeList, true);
		if (fShiplistCount == 0) {
			aux_sl = saved_state_aux_sl;
			std::list< eEnhancedClassTypes >  auxClassTypeList_take2;
			auxClassTypeList_take2.push_back(kEClassSmallAuxiliary);
			auxClassTypeList_take2.push_back(kEClassLargeAuxiliary);
			fShiplistCount = aux_sl.Subset(auxClassTypeList_take2, true);
		}
		gST.mDebug_fprintf("subset for auxiliary filter completed. ships: %d\n", fShiplistCount);

		// aux classes: warship, carrier, PFT
		std::list< eEnhancedRoles >  auxRolesList;
		auxRolesList.push_back(kWarShip);
		auxRolesList.push_back(kCarrier);
		auxRolesList.push_back(kPFTender);
		fShiplistCount = aux_sl.Subset(auxRolesList, true);
		gST.mDebug_fprintf("subset for auxiliary eEnhancedRoles completed. ships: %d\n", fShiplistCount);



		// military freighters or APTs only.
		ShipList saved_state_F_sl = QShipTeam_sl;
		std::list< eEnhancedClassTypes >  enhancedClassTypeList;
		if ((qShipSizeDice < 70) && (diffLevel < 2)) {
			enhancedClassTypeList.push_back(kEClassSmallFreighter);
			enhancedClassTypeList.push_back(kEClassPriorityTransport);
			enhancedClassTypeList.push_back(kEClassCivilianFreighter);
		} else {
			enhancedClassTypeList.push_back(kEClassLargeFreighter);
		}
		fShiplistCount = QShipTeam_sl.Subset(enhancedClassTypeList, true);
		if (fShiplistCount == 0) {
			QShipTeam_sl = saved_state_F_sl;
			std::list< eEnhancedClassTypes >  enhancedClassTypeList_take2;
			enhancedClassTypeList_take2.push_back(kEClassSmallFreighter);
			enhancedClassTypeList_take2.push_back(kEClassPriorityTransport);
			enhancedClassTypeList_take2.push_back(kEClassCivilianFreighter);
			enhancedClassTypeList_take2.push_back(kEClassLargeFreighter);
			fShiplistCount = QShipTeam_sl.Subset(enhancedClassTypeList_take2, true);
		}
		gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

		// QShips only
		std::list< eEnhancedRoles >  rolesList;
		rolesList.push_back(kQship);
		fShiplistCount = QShipTeam_sl.Subset(rolesList, true);
		gST.mDebug_fprintf("subset for QShip eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		QShipTeam_sl = gST.stripOutRestricted( QShipTeam_sl );

		// if there's an auxiliary available, use it as 2nd qship.
		ShipList QShip2sl = QShipTeam_sl;
		if (aux_sl.size() > 0) {
			QShip2sl = aux_sl;
		}

		Ship s1;
		Ship s2;

		if ( (QShipTeam_sl.size() > 0) && (QShipTeam_sl.SelectRandomShip(s1)) && (QShip2sl.SelectRandomShip(s2)) )
		{
			if (numberOfQShips > 0) {
				gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip(
					typeid( tQShipTeamShip ),
					s1.shipCode.c_str(),
					kStartPosition_P,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}

			if (numberOfQShips > 1) {
				gST.mDebug_fprintf("Ship2 selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
				gST.mTrackOrions( s2 );
				mCreateShip(
					typeid( tQShipTeamShip ),
					s2.shipCode.c_str(),
					kStartPosition_O,
					kNoMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					NULL,
					static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
			}
		} else {
			mOldCreateShipsForTeam( qShipBPV );
		}
	} else {
		mOldCreateShipsForTeam( qShipBPV );
	}

	gST.mDebug_fprintf("--- Finished: tQShipTeam::mCreateShipsForTeam\n");
}

void tQShipTeam::mOldCreateShipsForTeam( int32 qShipBPV ) 
{
	gST.mDebug_fprintf("--- Starting: tQShipTeam::mOldCreateShipsForTeam(%d)\n", qShipBPV);
	int32 numberOfQShips = gWC.mGetNumQShips();

	if (numberOfQShips > 0) {
		mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassFreighter, 
						35, 
						kStartPosition_P, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}

	if (numberOfQShips > 1) {
		qShipBPV += 10;
		mCreateShip( typeid( tQShipTeamShip ),
						qShipBPV,
						kClassFreighter, 
						35, 
						kStartPosition_O, gWC.fNWCustID++, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}
	gST.mDebug_fprintf("--- Finished: tQShipTeam::mOldCreateShipsForTeam(%d)\n", qShipBPV);
}

void tQShipTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tQShipTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tQShipTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tQShipTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}