/********************************************************************************************************\

	File Name:	AttackTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_14Enigma.h"
#include "AttackTeam.h"
#include "AttackTeamInitialState.h"
#include "AttackTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tAttackTeam> gAttackTeamAllocator;

tAttackTeam::tAttackTeam()
{
	gWC.mAddAttackTeam();
}

void tAttackTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tAttackVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tAttackTeam::mSetupState()
{
	mRegisterState( new tAttackTeamInitialState() );
	mGotoState( typeid( tAttackTeamInitialState ) );
}
void tAttackTeam::mSetupCommState()
{
}
void tAttackTeam::mCreateShipsForTeam()
{
}
void tAttackTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	mCreateShip( 
		typeid( tAttackTeamShip ), 
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_W,
		metaid,
		gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
		shipScriptDescription.fCustomName.c_str(),
		static_cast< eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}