/********************************************************************************************************\

	File Name:	DefendTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "DefendTeamBaseState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "Met_16ShipDefense.h"
#include "ShipList_API.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tDefendTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	mSetTimedInterval( kTimerTurnsSet, kRaiseShieldsID, 5.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tDefendTeamBaseState::mEndState()
{}

void tDefendTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tDefendTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tDefendTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tDefendTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tDefendTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tDefendTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tDefendTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tDefendTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tDefendTeamBaseState::mOnFlashingButtonPressed()
{}
void tDefendTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tDefendTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tDefendTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tDefendTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tDefendTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tDefendTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tDefendTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tDefendTeamBaseState::mOnHostChanged()
{}
void tDefendTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if ( timerID == kStartingID )
	{
		eRaceName  playerRace = kNoRace;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fChildActor->mGetTeam() );
		if ( playerTeam )
			playerRace = playerTeam->mGetRace();

		// return the index to the Starting message we want to use
		switch( playerRace )
		{
			case kFederation:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kFedStartDefend_msg, kNonCommScriptMessage );
			break;
			case kKlingon:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kKliStartDefend_msg, kNonCommScriptMessage );
			break;
			case kRomulan:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kRomStartDefend_msg, kNonCommScriptMessage );
			break;
			case kLyran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kLyrStartDefend_msg, kNonCommScriptMessage );
			break;
			case kGorn:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kGorStartDefend_msg, kNonCommScriptMessage );
			break;
			case kMirak:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kMirStartDefend_msg, kNonCommScriptMessage );
			break;
			case kHydran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kHydStartDefend_msg, kNonCommScriptMessage );
			break;
			case kISC:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kISCStartDefend_msg, kNonCommScriptMessage );
			break;
			default:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kStartDefend_msg, kNonCommScriptMessage );
			break;
		}
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The shipyards are dropping their shields briefly to allow parts transfer sir", kNonCommScriptMessage );
	}

	if (timerID == kRaiseShieldsID) {
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Shipyards are raising their shields", kNonCommScriptMessage );
	}

	if (timerID == kUpgradeFightersID)
	{
#ifdef FIGHTERS
		
		if (
			(fChildActor->mGetTeam() != kDefendTeam1) &&
			(fChildActor->mGetTeam() != kDefendTeam2) &&
			(fChildActor->mGetTeam() != kDefendTeam3)
			)return;
		tTeamInfo* DefendTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
		if (DefendTeam->mIsTeamHumanControlled()) return;               // only for AI teams
		if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

		gST.mDebug_fprintf("--- Entering tDefendTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		gST.fTeam1DeltaFighterBPV += gST.mUpgradeFightersForTeam(DefendTeam);

		gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam1DeltaFighterBPV);
		gST.mDebug_fprintf("--- Finished: tDefendTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
}
void tDefendTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tDefendTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

