/********************************************************************************************************\

	File Name:	OTDBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "OTDBaseState.h"
#include "Met_16ShipDefense.h"
#include "MissionText.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tOTDBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kRaiseShieldsID, 5.0f );
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	// may have to use gGetMapPicked() for some missions
	int32 map = 1;
	int32 cust = fChildActor->mGetCustomID(); if (cust < 1) cust = 1;
	int32 seed = gWC.fMissionSeed; if (seed < 1) seed = 1;
	int32 bpv = fChildActor->mGetShipBPV(); if (bpv < 1) bpv = 1;
	int32 Team = fChildActor->mGetTeam(); if (Team < 1) Team = 1;
	int32 id = fChildActor->mGetShipID(); if (id < 1) id = 1;
	fShipRandValue = map * cust * seed * bpv * Team * id;
	if (fShipRandValue < 0) fShipRandValue *= -1;
}
void tOTDBaseState::mEndState()
{
}

void tOTDBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}


void tOTDBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			if ( gWC.mAllOTDDestroyed() )
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAllOTDSubdued_msg, kNonCommScriptMessage );
			else
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDDestroyed_msg, kNonCommScriptMessage );
		}
	}
}
void tOTDBaseState::mOnCapture( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			if ( gWC.mAllOTDDestroyed() )
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAllOTDSubdued_msg, kNonCommScriptMessage );
			else
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDCaptured_msg, kNonCommScriptMessage );
		}
	}
}
void tOTDBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			if ( gWC.mAllOTDDestroyed() )
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAllOTDSubdued_msg, kNonCommScriptMessage );
			else
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDDisengaged_msg, kNonCommScriptMessage );
		}
	}
}
void tOTDBaseState::mAtDistance( tShipInfo* ship )
{}
void tOTDBaseState::mAtDestination( tShipInfo* ship )
{}
void tOTDBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tOTDBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tOTDBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tOTDBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tOTDBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tOTDBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tOTDBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tOTDBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDBaseState::mOnFlashingButtonPressed()
{}
void tOTDBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tOTDBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tOTDBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tOTDBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tOTDBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tOTDBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tOTDBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tOTDBaseState::mOnHostChanged()
{}
void tOTDBaseState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kStartingID) {
		fChildActor->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f, kTrue );
	} else if (timerID == kRaiseShieldsID) {
		fChildActor->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 1.0f, kTrue );
	}
}
void tOTDBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tOTDBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

