/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_16ShipDefense.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"
#include "ShipScriptDescription.h"


tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

void tOTDTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
	gST.mCompleteNarrowedList();

	eShipOptions options = static_cast< eShipOptions >( kDefaultShipOptions );
	eRace race = mGetRace();

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;

		ShipList OTDTeam_sl = gST.narrowed_sl;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = OTDTeam_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. planets: %d\n", race, fShiplistCount);

		ShipList planet_sl = OTDTeam_sl;

        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  planetRolesList;
		gST.mDebug_fprintf("Selecting planet armed world role.\n");
		planetRolesList.push_back(kArmedWorld);
		fShiplistCount = planet_sl.Subset(planetRolesList, true);
        gST.mDebug_fprintf("subset for ArmedWorld eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        {
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);


        ShipList FRD_sl = OTDTeam_sl;

        // FRDs
        std::list< eEnhancedClassTypes >  FRDClassTypeList;
        FRDClassTypeList.push_back(kEClassFleetRepairDock);
        fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
        gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(FRD_sl);

        if ((planet_sl.size() > 0) && (FRD_sl.size() > 0)) {
			Ship s1;
			planet_sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("Planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );
            mCreateShip(
                typeid( tOTD ),
                s1.shipCode.c_str(),
                kStartPosition_W,
                kNoMetaShipID,
                kOTD, 
				0, 0, -1, -1, 
				NULL,
                options );

			for ( int32 i = 1; i < kNumFRDs; i++ )
			{
				FRD_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip(
					typeid( tOTD ), 
					s1.shipCode.c_str(), 
					kStartPosition_W + i, 
					kNoMetaShipID,
					kOTD, 
					0, 0, -1, -1, 
					NULL, 
					options );
			}
        } else {
            mOldCreateShipsForTeam();
        }
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string shipyard = "";
	eRace race = mGetRace();
	eShipOptions options = static_cast< eShipOptions >( kDefaultShipOptions );

	switch ( race )
	{
	case kFederation:	shipyard = "F-PLPh0";		break;
	case kKlingon:		shipyard = "K-PLPh0";		break;
	case kRomulan:		shipyard = "R-PLPh0";		break;
	case kLyran:		shipyard = "L-PLPh0";		break;
	case kGorn:			shipyard = "G-PLPh0";		break;
	case kHydran:		shipyard = "H-PLPh0";		break;
	case kISC:			shipyard = "I-PLPh0";		break;
	case kMirak:		shipyard = "Z-PLPh0";		break;
	default:			shipyard = "O-PLPh0";		break;
	}

	mCreateShip(	typeid( tOTD ), shipyard.c_str(), kStartPosition_W, kNoMetaShipID,
					kOTD, 0, 0, -1, -1, NULL, options );

	switch ( race )
	{
	case kFederation:	shipyard = "F-FRD";		break;
	case kKlingon:		shipyard = "K-FRD";		break;
	case kRomulan:		shipyard = "R-FRD";		break;
	case kLyran:		shipyard = "L-FRD";		break;
	case kGorn:			shipyard = "G-FRD";		break;
	case kHydran:		shipyard = "H-FRD";		break;
	case kISC:			shipyard = "I-FRD";		break;
	case kMirak:		shipyard = "Z-FRD";		break;
	default:			shipyard = "O-FRD";		break;
	}

	for ( int32 i = 1; i < kNumFRDs; i++ )
	{
		mCreateShip(	typeid( tOTD ), shipyard.c_str(), kStartPosition_W + i, kNoMetaShipID,
						kOTD, 0, 0, -1, -1, NULL, options );
	}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}

void tOTDTeam::mEmerCreateShipsForTeam()
// used if admins eliminate the stock FRDs from the shiplist,
// substitutes the smallest bases available instead
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mEmerCreateShipsForTeam\n");
	int32 minbpv = 1;

	for (int32 i = 1; i < kNumFRDs; i++) {
		mCreateShip( typeid( tOTD ),
						minbpv,
						kClassListeningPost,
						kClassBaseStation, 
						0, 
						kStartPosition_W + i,
						-1, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mEmerCreateShipsForTeam\n");
}


void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}