/********************************************************************************************************\

	File Name:	AttackTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AttackShip.h"
#include "AttackTeam.h"
#include "AttackTeamBaseState.h"
#include "BaseVictory.h"
#include "Met_16ShipDefense.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tClassAllocator<tAttackTeam> gAttackTeamAllocator;

tAttackTeam::tAttackTeam(): fCustomID( kAttacker1 ), fNumShips ( 0 )
{
	gWC.mIncrementTotalAttackTeams();
}

void tAttackTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinnerAttack_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoserAttack_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tAttackTeam::mSetupState()
{
	mRegisterState( new tAttackTeamBaseState() );
	mGotoState( typeid( tAttackTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tAttackTeam::mSetupCommState()
{}

void tAttackTeam::mCreateShipsForTeam()
{}

void tAttackTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	if ( mIsTeamHumanControlled() )
	{	
		tShipIterator ship = mGetFirstShip();

		Assert ( *ship != NULL );

		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
        metaid = kNoMetaShipID;
    }
#endif


	mCreateShip( 
		typeid( tAttackShip ), 
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_G + shipScriptDescription.fTeamID,
		metaid,
		fCustomID, 0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		kDefaultShipOptions );

	fCustomID++;
	fNumShips++;
	gWC.mIncrementTotalAttackShips();

	gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}