/********************************************************************************************************\

	File Name:	DefendTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "DefendShip.h"
#include "DefendTeam.h"
#include "DefendTeamBaseState.h"
#include "Met_16ShipDefense.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tClassAllocator<tDefendTeam> gDefendTeamAllocator;


tDefendTeam::tDefendTeam():fCustomID( kDefender1 ), fNumShips( 0 )
{
	gWC.mIncrementTotalDefendTeams();
}

void tDefendTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinnerDefend_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoserDefend_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tDefendTeam::mSetupState()
{
	mRegisterState( new tDefendTeamBaseState() );
	mGotoState( typeid( tDefendTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tDefendTeam::mSetupCommState()
{}

void tDefendTeam::mCreateShipsForTeam()
{}

void tDefendTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	if ( mIsTeamHumanControlled() )
	{
		tShipIterator ship = mGetFirstShip();

		Assert ( *ship != NULL );

		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tDefendTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	//
	//	The first ship of each team should go inder the FRD.
	//
	eShipOptions options = kDefaultShipOptions;
	if ( !fNumShips )
		options = static_cast< eShipOptions >( kStartPositionOffsetOrigin | kDefaultShipOptions );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
        metaid = kNoMetaShipID;
    }
#endif


	mCreateShip( 
		typeid( tDefendShip ), 
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_Q + shipScriptDescription.fTeamID,
		metaid,
		fCustomID, 0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		options );

	fCustomID++;
	fNumShips++;

	gST.mDebug_fprintf("--- Finished: tDefendTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tDefendTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}
