/********************************************************************************************************\

	Mission:	Met_18HomeworldAssault
	Script:		"Met_18HomeworldAssault"
	Descript:	Fleet attempts to take over base

	Created by:		Daniel J. Suleski		07/06/00
	Last Updated:	DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta


\********************************************************************************************************/

#include "AttackTeam.h"
#include "DefendTeam.h"
#include "Met_18HomeworldAssault.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "OTDTeam.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_18HomeworldAssault );

tMet_18HomeworldAssault::tMet_18HomeworldAssault()
{
	srand(time(NULL));
}

void tMet_18HomeworldAssault::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kHomeworldHex );
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fWait	= 1;
	Specs->fGenericFlags = kStopMetaMovement | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Attack Team 1",
												static_cast< eTeamID >( kAttackTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team 2",
												static_cast< eTeamID >( kAttackTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Attack Team 3",
												static_cast< eTeamID >( kAttackTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team 1",
												static_cast< eTeamID >( kDefendTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Defend Team 2",
												static_cast< eTeamID>( kDefendTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tOTDTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team7 );
}

int32 tMet_18HomeworldAssault::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tMet_18HomeworldAssault::mCreateExternalShips( int32 Mode )
{}

void tMet_18HomeworldAssault::mNewInternalShip( tShipInfo* Ship )
{}

void tMet_18HomeworldAssault::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Homeworld assault");
	gWC.mCheckCRC();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kAttackTeam1,
		kAttackTeam2,
		kAttackTeam3,
		kDefendTeam1,
		kDefendTeam2,
		kPlanetTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  PTm4  PTm5   StTm
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD",  "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD",  "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD",  "WD",
		/*PTm4*/ "WD", "WD", "WD", "AI", "AI",  "AI",
		/*PTm5*/ "WD", "WD", "WD", "AI", "AI",  "AI",
		/*StTm*/ "WD", "WD", "WD", "AI", "AI",  "AI",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

tMapTerrain tMet_18HomeworldAssault::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetHomeWorld, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula, kPlanetHomeWorld, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainBlackholes,		kPlanetHomeWorld, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainDustclouds,		kPlanetHomeWorld, kBaseAnyBase );
	case 5:		return tMapTerrain( kTerrainShippingLane,		kPlanetHomeWorld, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_18HomeworldAssault::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_18HomeworldAssault::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kAttackTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingAttack_msg;
		}
		for ( i = kDefendTeam1; i <= kPlanetTeam; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingDefend_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_18HomeworldAssault::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.3f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.675f, 1.275f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.65f, 1.25f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.625f, 1.25f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.6f, 1.225f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.575f, 1.2f );

	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassHeavyCruiser, 75l, 125l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassHeavyCruiser, 80l, 130l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassHeavyCruiser, 85l, 135l );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassHeavyCruiser, 65l, 115l );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassHeavyCruiser, 70l, 120l );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassHeavyCruiser, 75l, 125l );

	int32 homeBPV = 500;
	mSetTeamShipStrength( kPlanetTeam, kClassPlanets, kClassPlanets, homeBPV, kMaxBPV );
}


//
//	The following two functions determine the size and race of the planet and the race that should be
//	defending it.
//
iMissSpecRace tMet_18HomeworldAssault::mTranslateHexIntoRace( tMetaLocationSpec hex )
{
	iMissSpecRace returnRace = kSpecPlayableRace;

	if ( hex.fLocationWhoHexFlags & kFederationRaceHex )
		returnRace = kSpecFed;
	else if ( hex.fLocationWhoHexFlags & kKlingonRaceHex )
		returnRace = kSpecKling;
	else if ( hex.fLocationWhoHexFlags & kRomulanRaceHex )
		returnRace = kSpecRomulan;
	else if ( hex.fLocationWhoHexFlags & kGornRaceHex )
		returnRace = kSpecGorn;
	else if ( hex.fLocationWhoHexFlags & kLyranRaceHex )
		returnRace = kSpecLyran;
	else if ( hex.fLocationWhoHexFlags & kHydranRaceHex )
		returnRace = kSpecHydran;
	else if ( hex.fLocationWhoHexFlags & kMirakRaceHex )
		returnRace = kSpecMirak;
	else if ( hex.fLocationWhoHexFlags & kISCRaceHex )
		returnRace = kSpecISC;

	return returnRace;
}
