/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

/*

	HELPER NOTE:	Add or delete from this list.  It is not necessary to have these exact maps in each
					multiplayer mission; however, each map should have valid starting positions for each
					of the 6 teams.

*/
int32 TotalMaps = 6;
const char* MapTypes[] = { "Open Space", "Asteroid Space", "Nebula Space", "Blackhole Space", "Dustclouds Space", "ShippingLane Space" };
const char* MapLines[]=
{
	// Asteroids: a,A,*
	// Black Holes: b,B
	// Nebula: c,C
	// Dust clouds: d,D
	// Start positions: G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z
	// End positions:   g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z
	// Planets: 1-6
	// Suns: 7-9
	//
   "Met_18HomeworldAssault",
   "// Open Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........tT..............|",
   " |.........................|",
   "1|....Z....................|",
   " |.....z...YVv.............|",
   " |..........y..............|",
   " |.sS..uU..................|",
   " |........W................|",
   "2|.....X...w...............|",
   " |......x..................|",
   " |.........................|",
   " |.........................|",
   " |..................h......|",
   "3|..............g..........|",
   " |...............G.........|",
   " |.........................|",
   " |.............i....H......|",
   " |..............I..........|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........*...........*...|",
   " |.........................|",
   "1|....Z.....Vv......*......|",
   " |.....z...Y...*...........|",
   " |........sSy..............|",
   " |..Uu..Tt..............*..|",
   " |......X.W........*.......|",
   "2|...*.....w..*............|",
   " |......x..................|",
   " |..............*..........|",
   " |.........................|",
   " |..*......*........h.*....|",
   "3|.....*........g..........|",
   " |...............G.........|",
   " |.................H.......|",
   " |.........*...i...........|",
   " |....*.........I......*...|",
   "4|.........................|",
   " |...........*.............|",
   " |...*...........*.........|",
   " |...................*.....|",
   " |.........*...............|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Nebula Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |..........vV.............|",
   "1|....Z.............d......|",
   " |.....z...................|",
   " |........Y.yUu............|",
   " |.d.......................|",
   " |..sS..TtW................|",
   "2|.....X...w.....d.........|",
   " |......x......d...........|",
   " |...........d.............|",
   " |.........d...............|",
   " |..................h......|",
   "3|..............g..........|",
   " |...............G..H......|",
   " |.........................|",
   " |.............i...I.......|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |..d.....d................|",
   " |.ddd.....................|",
   " |.........................|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Blackhole Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |...........Vv............|",
   "1|....Z....................|",
   " |.....z...................|",
   " |........YSys.............|",
   " |.......tT................|",
   " |........W................|",
   "2|..Uu.X...w...............|",
   " |......x..................|",
   " |.........................|",
   " |.........................|",
   " |..................h......|",
   "3|..............g..........|",
   " |...............G..H......|",
   " |.........................|",
   " |......B......i...I.......|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Dustclouds Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|....Z.......vV....d......|",
   " |.....z...................|",
   " |........Y.y..............|",
   " |.d.........sS............|",
   " |..tT..uUW................|",
   "2|.....X...w.....d.........|",
   " |......x......d...........|",
   " |...........d.............|",
   " |.........d...............|",
   " |..................h......|",
   "3|..............g..........|",
   " |...............G..H......|",
   " |.........................|",
   " |.............i...I.......|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |..d.....d................|",
   " |.ddd.....................|",
   " |.........................|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// ShippingLane Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |......vV.................|",
   " |.........................|",
   "1|....Z....................|",
   " |.....z...Y...............|",
   " |........uUy..............|",
   " |.........................|",
   " |...Ss.tTW................|",
   "2|.....X...w...............|",
   " |......x..................|",
   " |.........................|",
   " |.........................|",
   " |..................h......|",
   "3|..............g..........|",
   " |...............G.........|",
   " |.........................|",
   " |.............i....H......|",
   " |..............I..........|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "6|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "7|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "///",
   0
};

