/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_18HomeworldAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ShipScriptDescription.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{
	std::string drafted_homeworld = "";
	int32 drafted_homeworld_MetaID = -1;
	std::string drafted_homeworld_name = "";
}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tOTDTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    std::string shiptype, frdtype;
    int32 i;
    iMapPosition hexes[4] = { kStartPosition_T,
                                kStartPosition_U,
                                kStartPosition_V,
                                kStartPosition_S, };

    if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
        int32 fShiplistCount = 0;
        ShipList OTDTeam_sl = gST.narrowed_sl;

        std::list< eRace >  raceList;
        raceList.push_back(mGetRace());
        fShiplistCount = OTDTeam_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

        // Defense platforms
        ShipList DEF_sl = OTDTeam_sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);

        // FRDs
        ShipList FRD_sl = OTDTeam_sl;
        std::list< eEnhancedClassTypes >  FRDClassTypeList;
        FRDClassTypeList.push_back(kEClassFleetRepairDock);
        fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
        gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(FRD_sl);

        if ( (DEF_sl.size() > 0) && (FRD_sl.size() > 0) ) {

            int32 skipchance = 5;
            if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
                tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
                if (drafter) {
                    if (drafter->mGetRace() == kLyran) skipchance = 1;
                    else if (drafter->mGetRace() == kGorn) skipchance = 2;
                    else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
                    else skipchance = 4;
                }
            }

            Ship s1;
			Ship s2;

            for (i = 0; i < 4; i++) {
                if ((rand() % 5) < skipchance) {
                    DEF_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
					gST.mTrackOrions( s1 );
                    mCreateShip( typeid( tShipInfo ),
                        s1.shipCode.c_str(),
                        hexes[i],
                        kNoMetaShipID,
                        -1,
                        0, 0,
                        -1, -1,
                        "",
                        kDefaultShipOptions);
                }
                if ((rand() % 5) < skipchance) {
                    FRD_sl.SelectRandomShip(s2);
					gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
					gST.mTrackOrions( s2 );
                    mCreateShip( typeid( tShipInfo ),
                        s2.shipCode.c_str(),
                        hexes[i],
                        kNoMetaShipID,
                        -1,
                        0, 0,
                        -1, -1,
                        "",
                        kDefaultShipOptions);
                }
            }

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
        int32 fShiplistCount = 0;
        ShipList planet_sl = gST.narrowed_sl;

		eRace race = mGetRace();

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = planet_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", race, fShiplistCount);

        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  planetRolesList;
        gST.mDebug_fprintf("Selecting planet homeworld role.\n");
        planetRolesList.push_back(kHomeWorld);
        fShiplistCount = planet_sl.Subset(planetRolesList, true);
		gST.mDebug_fprintf("subset for planet eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        {
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission, and inapropriate PE planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);

        Ship p1;
        if ((planet_sl.size() > 0) && (planet_sl.SelectRandomShip(p1))) {
            gST.mDebug_fprintf("planet selected! %s:  %d BPV\n", p1.shipCode.c_str(), p1.bpv);
			gST.mTrackOrions( p1 );

            mCreateShip(
                typeid( tOTD ),
                p1.shipCode.c_str(),
                kStartPosition_Z,
                drafted_homeworld_MetaID,
                kOTD, 0, 0, 0, 0, 
				drafted_homeworld_name.c_str(),
                kCanAddToFleetUponCapture );

        } else {
            mOldCreateNewShip();
        }
	} else {
		mOldCreateNewShip();
	}

    gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string shiptype, frdtype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };


	switch (mGetRace()) {
		case kFederation: shiptype = "F-DEF"; frdtype = "F-FRD"; break;
		case kGorn: shiptype = "G-DEF";  frdtype = "G-FRD";break;
		case kHydran: shiptype = "H-DEF";  frdtype = "H-FRD";break;
		case kISC: shiptype = "I-DEF";  frdtype = "I-FRD";break;
		case kKlingon: shiptype = "K-DEF";  frdtype = "K-FRD";break;
		case kLyran: shiptype = "L-DEF";  frdtype = "L-FRD";break;
		case kMirak: shiptype = "Z-DEF";  frdtype = "Z-FRD";break;
		case kRomulan: shiptype = "R-DEF";  frdtype = "R-FRD";break;
		default: // used for both Orion and neutral/monster
			shiptype = "O-DEF";  frdtype = "O-FRD";break;
	}
	int32 skipchance = 5;
	if (gWC.fDraftedHumans < 1) {
		tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
		if (drafter) { 
			if (drafter->mGetRace() == kLyran) skipchance = 1;
			else if (drafter->mGetRace() == kGorn) skipchance = 2;
			else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
			else skipchance = 4;
		}
	}

	for (i = 0; i < 4; i++) {
		if ((rand() % 5) < skipchance) {
			mCreateShip( typeid( tShipInfo ), 
				shiptype.c_str(), 
				hexes[i], 
				kNoMetaShipID, 
				-1, 
				0, 0, 
				-1, -1,	
				"", 
				kDefaultShipOptions);
		}
		if ((rand() % 5) < skipchance) {
			mCreateShip( typeid( tShipInfo ), 
				frdtype.c_str(), 
				hexes[i], 
				kNoMetaShipID, 
				-1, 
				0, 0, 
				-1, -1,	
				"", 
				kDefaultShipOptions);
		}
	}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}

void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	drafted_homeworld = shipScriptDescription.fClassName;
	drafted_homeworld_name = shipScriptDescription.fCustomName;

	drafted_homeworld_MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		drafted_homeworld_MetaID = kNoMetaShipID;
	}
#endif

   	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOTDTeam::mOldCreateNewShip()
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateNewShip: %s\n", drafted_homeworld.c_str());

	mCreateShip(	typeid( tOTD ),
					drafted_homeworld.c_str(), 
					kStartPosition_Z,
					drafted_homeworld_MetaID,
					kOTD, 0, 0, -1, -1,
					drafted_homeworld_name.c_str(),
					kDefaultShipOptions );

   	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateNewShip: %s\n", drafted_homeworld.c_str());
}


void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}