//	<Name>	AttackVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "AttackVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_19Scan.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "AttackTeam.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"

#include "WorldConditions.h"

tAttackVictoryState::tAttackVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tAttackVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisTeamShipsInPlay = 1;
	fTeamDisengaged = false;
	fThisPlayerDead = false;
}

void tAttackVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())))
		{
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}

void tAttackVictoryState::mEndState()
{
}

eVictoryLevels tAttackVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 attackDamagePoints = 0;
	float attackDamageRatio = 0.0;
	int32 defendDamagePoints = 0;
	float defendDamageRatio = 0.0;
	float damageRatio = 0.0;
	int i;

	for( i = 1; i <= gWC.mGetNumAttackShips(); ++i )
	{
		attackDamagePoints +=	gWC.mGetAttackDamageCond( i ); 
	}
	for( i = 1; i <= gWC.mGetNumDefendShips(); ++i )
	{
		defendDamagePoints +=	gWC.mGetDefendDamageCond( i );
	}
	attackDamageRatio = (float)attackDamagePoints / gWC.mGetNumAttackShips();
	defendDamageRatio = (float)defendDamagePoints / gWC.mGetNumDefendShips();

	if( attackDamageRatio )
	{
		damageRatio = defendDamageRatio / attackDamageRatio;
	}
	else if( defendDamageRatio )
	{
		damageRatio = 3;
	}
	else
	{
		damageRatio = 0.5;
	}


	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if ( fTeamDisengaged )
	{
		return kDefeat;
	}
	else if( damageRatio > 2 )
	{
		return	kAstoundingVictory;
	}
	else if( damageRatio > 1 )
	{
		return kVictory;
	}
	else if( damageRatio > 0.5 )
	{
		return kDraw;
	}
	else if( damageRatio > 0.25 )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}
}
int32 tAttackVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tAttackVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			gWC.mRemoveAttackShip();
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveAttackTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		gWC.mCheckForGameEnd();
	}

}
void tAttackVictoryState::mOnDeath( tShipInfo* ship )
{

	if (!ship) return;
	int32 thisShipID;

	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		eTeamID teamID = ship->mGetTeam();
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID))
				mAddToBonusPrestige(125);
		}
	}

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
		//Empty
		break;
	}

	mCheckAllTeams( ship );
}
void tAttackVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;
	if (!ship) return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		eTeamID teamID = ship->mGetTeam();
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID))
				mAddToBonusPrestige(175);
		}
	}

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
			//Break;
		break;
	}

	mCheckAllTeams( ship );
}

void tAttackVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if (!ship) return;
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
			tTeamInfo* myteam = fChildActor->mGetTeam();
			if (myteam) {
				eTeamID myteamID = myteam->mGetTeam();
				if (gWC.mShipIsMyEnemy(teamID, myteamID))
					mAddToBonusPrestige(75);
			}
		}
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveAttackTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				gWC.mCheckForGameEnd();
			}
		}
	}
}
void tAttackVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tAttackVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tAttackVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveAttackTeam();
		fPlayerLeftAlready = true;
		gWC.mCheckForGameEnd();
	}
}