//	<Name>	PlanetTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "PlanetTeam.h"
#include "Planet.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "Met_19Scan.h"
#include "ShipList_API.h"

tClassAllocator<tPlanetTeam> gPlanetTeamAllocator;

tPlanetTeam::tPlanetTeam() : tTeamInfo()
{
}

void tPlanetTeam::mSetupState()
{
}

void tPlanetTeam::mSetupCommState()
{

}

void tPlanetTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tPlanetTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
        int32 fShiplistCount = 0;
		eRace race = mGetRace();

        ShipList planet_sl = gST.narrowed_sl;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = planet_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. planets: %d\n", race, fShiplistCount);

        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  planetRolesList;
        gST.mDebug_fprintf("Selecting planet armed world role.\n");
        planetRolesList.push_back(kArmedWorld);
		planetRolesList.push_back(kHomeWorld);
        fShiplistCount = planet_sl.Subset(planetRolesList, false);
        gST.mDebug_fprintf("Filtered out armed or home worlds. ships: %d\n", fShiplistCount);

        {
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);

		Ship s1;
        if ((planet_sl.size() > 0) && (planet_sl.SelectRandomShip(s1))) {

            gST.mDebug_fprintf("Planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );
            mCreateShip(
                typeid( tPlanet ),
                s1.shipCode.c_str(),
                kStartPosition_Z,
                kNoMetaShipID,
                -1,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

			// overlay the planet with another astronomical body
			char objectPlace = 'Z';
			int32 x1 = 0, y1 = 0;
			switch (gWC.mGetSpecialMissionType()) {
				case kSunMission:
					objectPlace = 'T';
					fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
					fMissionInfo->mCreateMapObject( '(', x1,	y1 );
					break;
				case kBlackHoleMission:
					fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
					fMissionInfo->mCreateMapObject( ':', x1,	y1 );
					break;
				case kAsteroidMission:
					fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
					fMissionInfo->mCreateMapObject( '*', x1,	y1 );
					break;
				default:
					break;
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tPlanetTeam::mCreateShipsForTeam\n");
}

void tPlanetTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlanetTeam::mOldCreateShipsForTeam\n");

	static int32 fNumPlanets = 0;

	if (fNumPlanets > 0) return;
	fNumPlanets++;

	int32 planetDice = fMissionInfo->mRandomInt32( 1, 100 );

	if( planetDice < 6 )
	{
		mCreateShip( typeid( tPlanet ), "PL1", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 12 )
	{
		mCreateShip( typeid( tPlanet ), "PL2", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 18 )
	{
		mCreateShip( typeid( tPlanet ), "PL3", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 24 )
	{
		mCreateShip( typeid( tPlanet ), "PL4", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 30 )
	{
		mCreateShip( typeid( tPlanet ), "PL0", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 36 )
	{
		mCreateShip( typeid( tPlanet ), "PL6", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 42 )
	{
		mCreateShip( typeid( tPlanet ), "PL7", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 48 )
	{
		mCreateShip( typeid( tPlanet ), "PL8", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 54 )
	{
		mCreateShip( typeid( tPlanet ), "PL9", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 60 )
	{
		mCreateShip( typeid( tPlanet ), "PL10", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 66 )
	{
		mCreateShip( typeid( tPlanet ), "PL11", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 72 )
	{
		mCreateShip( typeid( tPlanet ), "PL12", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 78 )
	{
		mCreateShip( typeid( tPlanet ), "PL13", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 84 )
	{
		mCreateShip( typeid( tPlanet ), "PL14", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 90 )
	{
		mCreateShip( typeid( tPlanet ), "PL15", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( planetDice < 95 )
	{
		mCreateShip( typeid( tPlanet ), "PL16", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else
	{
		mCreateShip( typeid( tPlanet ), "PL17", kStartPosition_Z, kNoMetaShipID,
										-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}

	// overlay the planet with another astronomical body
	char objectPlace = 'Z';
	int32 x1 = 0, y1 = 0;
	switch (gWC.mGetSpecialMissionType()) {
		case kSunMission:
			objectPlace = 'T';
			fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
			fMissionInfo->mCreateMapObject( '(', x1,	y1 );
			break;
		case kBlackHoleMission:
			fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
			fMissionInfo->mCreateMapObject( ':', x1,	y1 );
			break;
		case kAsteroidMission:
			fMissionInfo->mGetGridPosition( objectPlace, &x1, &y1 );
			fMissionInfo->mCreateMapObject( '*', x1,	y1 );
			break;
		default:
			break;
	}

	gST.mDebug_fprintf("---- Finished: tPlanetTeam::mOldCreateShipsForTeam\n");
}

void tPlanetTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlanetTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Just taking census

	gST.mDebug_fprintf("--- Finished: tPlanetTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}