//	<Name>	ScanTeamInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Met_19Scan.h"
#include "ShipInfo.h"
#include "DefendTeam.h"
#include "ScanTeamInitialState.h"
#include "WorldConditions.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "DefendTeam2.h"
#include "ShipList_API.h"

tScanTeamInitialState::tScanTeamInitialState( )
{
}

void tScanTeamInitialState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
}

void tScanTeamInitialState::mEndState()
{
}
void tScanTeamInitialState::mOnTimedInterval( int32 timerID )
{


	if( timerID == kStartingID )
	{
		eRaceName  playerRace;
		fChildActor->mGetTeam();
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fChildActor->mGetTeam() );
		playerRace = playerTeam->mGetRace();

		// return the index to the Starting message we want to use
		switch( playerRace )
		{
			case kFederation:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kFedScanStart_msg, kNonCommScriptMessage );
			break;
			case kKlingon:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kKliScanStart_msg, kNonCommScriptMessage );
			break;
			case kRomulan:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kRomScanStart_msg, kNonCommScriptMessage );
			break;
			case kLyran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kLyrScanStart_msg, kNonCommScriptMessage );
			break;
			case kGorn:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kGorScanStart_msg, kNonCommScriptMessage );
			break;
			case kMirak:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kMirScanStart_msg, kNonCommScriptMessage );
			break;
			case kHydran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kHydScanStart_msg, kNonCommScriptMessage );
			break;
			case kISC:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kISCScanStart_msg, kNonCommScriptMessage );
			break;
		}

		// see if there is anything on the surface we need to pickup
		if (gWC.mGetSpecialMissionType() == kPickupMission) {
			std::string pickupstr("It seems there is something on the planet surface we are supposed to retrieve if possible");
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   pickupstr, kScienceScriptMessage );
		} else if (gWC.mGetSpecialMissionType() == kSunMission) {
			std::string abnormalstr("This is a good chance to observe the damage a planet takes when it finally crashes into a small sun");
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   abnormalstr, kScienceScriptMessage );
		} else if (gWC.mGetSpecialMissionType() == kBlackHoleMission) {
			std::string abnormalstr("This is a good chance to observe the damage a planet takes when it finally reaches the gravitational well of a black hole");
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   abnormalstr, kScienceScriptMessage );
		} else if (gWC.mGetSpecialMissionType() == kAsteroidMission) {
			std::string abnormalstr("This is a good chance to observe a planet immediately following a major asteroid collision");
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   abnormalstr, kScienceScriptMessage );
		}

		// check and see if there are any enemy ships that must be dealt with
		tTeamInfo *attacker = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kAttackTeam1) );
		int32 enemies = 0;
		if (attacker) enemies += attacker->mShipsCurrentlyControlled();
		attacker = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kAttackTeam2) );
		if (attacker) enemies += attacker->mShipsCurrentlyControlled();
		if (enemies > 0) {
			std::string warningstr("Captain, we also detect enemy ships in the vicinity - we will need to deal with them too");
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   warningstr, kNonCommScriptMessage );
		}


	}//startingid

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* DefendTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
        if (DefendTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (DefendTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tScanTeamInitialState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        gST.fTeam1DeltaFighterBPV += gST.mUpgradeFightersForTeam(DefendTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam1DeltaFighterBPV);
        gST.mDebug_fprintf("--- Finished: tScanTeamInitialState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tScanTeamInitialState::mOnDeath( tShipInfo* ship )
{
}
void tScanTeamInitialState::mOnDisengaged( tShipInfo* ship )
{
}
void tScanTeamInitialState::mOnCapture( tShipInfo* ship )
{
}
