/********************************************************************************************************\

	File Name:	ScoutTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ScoutTeam.h"
#include "ScoutTeamInitialState.h"
#include "ScoutTeamShip.h"
#include "ShipScriptDescription.h"
#include "ScoutTeamVictoryCondition.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tScoutTeam> gScoutTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/

void tScoutTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tScoutTeamVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
	fNumships = 0;
}

void tScoutTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{

	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kScoutAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kScoutVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kScoutDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kScoutDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kScoutDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tScoutTeam::mSetupState()
{
	   mRegisterState( new tScoutTeamInitialState() );
	   mGotoState( typeid( tScoutTeamInitialState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tScoutTeam::mSetupCommState()
{
}
void tScoutTeam::mCreateShipsForTeam()
{
}
void tScoutTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tScoutTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tScoutTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
	int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
		MetaID = kNoMetaShipID;
	}
#endif

	if (fNumships <= 0) {
		fNumships++;
		iMapPosition mappos = kStartPosition_G;
		if (shipScriptDescription.fMetaDatabaseID % 2) mappos = kStartPosition_H;
		mCreateShip( typeid( tScoutTeamShip ), 
					 shipScriptDescription.fClassName.c_str(), 
					 mappos,
					 MetaID,
					 gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
					 shipScriptDescription.fCustomName.c_str(),
					 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
	}

	gST.mDebug_fprintf("--- Finished: tScoutTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tScoutTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}