/********************************************************************************************************\

	File Name:	TeamScouted.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "TeamScouted.h"
#include "TeamScoutedInitialState.h"
#include "ShipScriptDescription.h"
#include "TeamScoutedVictoryCondition.h"
#include "TeamScoutedShip.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tTeamScouted> gTeamScoutedAllocator;

tTeamScouted::tTeamScouted()
{
	fDice = fMissionInfo->mRandomInt32( 0,2 );
}
void tTeamScouted::mSetupVictoryCondition()
{
	fVictoryCondition = new tTeamScoutedVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tTeamScouted::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kTeamScoutedAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kTeamScoutedVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kTeamScoutedDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kTeamScoutedDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kTeamScoutedDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}
void tTeamScouted::mSetupState()
{
		mRegisterState( new tTeamScoutedInitialState() );
	    mGotoState( typeid( tTeamScoutedInitialState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tTeamScouted::mSetupCommState()
{}

void tTeamScouted::mCreateShipsForTeam()
{}

void tTeamScouted::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}

}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tTeamScouted::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tTeamScouted::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam()); 
		MetaID = kNoMetaShipID;
	}
#endif

	mCreateShip( typeid( tTeamScoutedShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_X + fDice,
				 MetaID,
				 gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("--- Finished: tTeamScouted::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tTeamScouted::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}