//	<Name>	TeamScoutedVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "TeamScoutedVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_1Scout.h"
#include "TeamScoutedVictoryCondition.h"
#include "MissionText.h"
#include "ScoutTeam.h"
#include "TeamScouted.h"
#include "WorldConditions.h"

tTeamScoutedVictoryState::tTeamScoutedVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tTeamScoutedVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	ftotalTeamScoutedShips = 0;
	
	fScoutTeamShipsInPlay = 1;
	fTeamScoutedShipsInPlay = 1;
	fScoutCaptured = false;
	fPlayerDied= false;
	fScoutHasAlreadyWon = kFalse;
}

void tTeamScoutedVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
				mMyInvokeGameStatus( (eTeamID)kScoutTeam, kLeftEarly );
				mMyInvokeGameStatus( (eTeamID)kTeamScouted, kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
	}
}

void tTeamScoutedVictoryState::mEndState()
{
}

eVictoryLevels tTeamScoutedVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
	int32 shipsScanned = 0;

	for( int32 i = 0; i < 20; ++i )
	{
		if( gWC.mGetScannedStatus( i ) == 1)
		{
			++shipsScanned;
		}
	}

	if( fPlayerDied )
	{
		return kDevastatingDefeat;
	}
	else if( (fScoutCaptured == true) && (fScoutTeamShipsInPlay == 0) )
	{
		return kAstoundingVictory;
	}
	else if( fScoutTeamShipsInPlay == 0 )
	{
		return kVictory;
	}
	else if(( shipsScanned > 0 ) || (fScoutHasAlreadyWon))
	{
		return kDefeat;
	}
	else
	{
		return kDraw;
	}

}
int32 tTeamScoutedVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tTeamScoutedVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	tTeamInfo* scoutTeam   = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );
	tTeamInfo* teamScouted = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTeamScouted ) );

	fScoutTeamShipsInPlay = scoutTeam->mShipsCurrentlyControlled();

	fTeamScoutedShipsInPlay = teamScouted->mShipsCurrentlyControlled();

	if( fTeamScoutedShipsInPlay == 0 )
	{
		if (fScoutHasAlreadyWon) {
			mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
			mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
		} else if (fScoutTeamShipsInPlay > 0) {
			// wins if managed to get enough scanning done before the scouted team died or left
			int32 shipsScanned = 0;
			for( int32 i = 0; i < 20; ++i ) {
				if( gWC.mGetScannedStatus( i ) == 1) {
					++shipsScanned;
				}
			}
			int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
			if (totalTeamScoutedShips <= (2 * shipsScanned)) {
				mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
				mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
			} else {
				mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerLoses );
				mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
			}
		} else {
			mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerLoses );
			mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
		}
	}
	else if( fScoutTeamShipsInPlay == 0 )
	{
		fPlayerDied = true;
		if (fScoutHasAlreadyWon) {
			mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
			mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
		} else {
			mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerLoses );
			mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
		}
	}

}
void tTeamScoutedVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tTeamScoutedVictoryState::mOnCapture( tShipInfo* ship )
{
	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}
	if (ship->mGetOriginalTeam() == kScoutTeam) {
		fScoutCaptured = kTrue;
	}
	mCheckAllTeams( ship );
}

void tTeamScoutedVictoryState::mOnDisengaged( tShipInfo* ship )
{
	int32 shipsScanned = 0;

	if( fTeam )
	{
		if (ship->mGetTeam() == kScoutTeam)
		{
			// A scout ship got away, see if they had collected enough
			// data before they left
				for( int32 i = 0; i < 20; ++i )
				{
					if( gWC.mGetScannedStatus( i ) == 1)
					{
						++shipsScanned;
					}
				}
				int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
				if (totalTeamScoutedShips <= (2 * shipsScanned)) {
					fScoutHasAlreadyWon = kTrue;
				}
			// the enemy getting away with a captured ship is an automatic loss
			if (ship->mGetOriginalTeam() != kScoutTeam)
					fScoutHasAlreadyWon = kTrue;
		}
	}
	mCheckAllTeams( ship );
}
void tTeamScoutedVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tTeamScoutedVictoryState::mOnTimedInterval( int32 timerID )
{
		if( timerID == kStartingID )
		{
			int32 scoutTeamShips = 0;


			tTeamInfo* scoutTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );

			if (scoutTeam) scoutTeamShips = scoutTeam->mShipsCurrentlyControlled();

			gWC.mSetTotalScoutShips( scoutTeamShips );	

			if( scoutTeamShips < 1 )
			{
				mMyInvokeGameStatus( (eTeamID)kScoutTeam, kPlayerLoses );
				mMyInvokeGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
			}

		}
}

void tTeamScoutedVictoryState::mMyInvokeGameStatus(eTeamID team, eGameStatus stat)
{

	tMissionInfo* fMissionInfo = gScriptInterface.mMission();
	if (!fMissionInfo) return;
	tTeamInfo *fTeam =fMissionInfo->mGetTeamHandle( team );
	if (!fTeam) return;

	fTeam->fStatus = stat;
	if (fMissionInfo) fMissionInfo->fStatusArray[fTeam->mGetTeam()] = stat;

	if (fTeam->fVictoryCondition) {
		// make sure the status is properly set for every team
		fTeam->fVictoryLevel = fTeam->fVictoryCondition->mCalcVictoryStatus(fTeam->mGetTeam());
		fTeam->mScheduleNextMission(fTeam->fVictoryCondition);
		// calculate prestige and debriefing messages for human teams
		if (!fTeam->fHasEndedHost) {
			if (fTeam->mIsTeamHumanControlled()) {
				int32 result = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				result += fTeam->fVictoryCondition->mGetBonusPrestige();
				gWC.mLogTeamResults(team, fTeam->mGetRace(), result, stat, fTeam->mIsTeamHumanControlled());
				tShipSystemsInfo info;
				info.fTeam = fTeam->mGetTeam();
				info.fAction = kPlayerEvent;
				info.fInt32[0] = kPlayerStatusChange;
				info.fInt32[1] = fTeam->fStatus;
				info.fInt32[2] = fTeam->fVictoryLevel;
				info.fInt32[3] = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				info.fInt32[4] = fTeam->fVictoryCondition->mGetBonusPrestige();
				fMissionInfo->mResetDebriefingCount( fTeam->mGetTeam() );
				fTeam->mSetupDebriefingList( fTeam->fVictoryLevel );
				gScriptInterface.mEntityMgr(info);
				if (gScriptInterface.mIsHost()) {
					fTeam->fHasEndedHost = kTrue;
					fTeam->fHasEndedClient = kTrue;
				}
			}
			// return the mission results and scheduling for the team
			tTeamInfo::tMissionScheduler *nextMission = fTeam->mGetNextMissionInfo();
			tShipSystemsInfo info;
			info.fTeam = fTeam->mGetTeam();
			info.fAction = kOther;
			info.fInt32[0] = kMissionResults;
			info.fInt32[1] = fTeam->fVictoryLevel;
			if (nextMission) info.fInt32[2] = nextMission->fPrestige;
			else info.fInt32[2] = 0;
			info.fInt32[3] = fTeam->fVictoryCondition->mGetBonusPrestige();
			if (nextMission) info.fInt32[4] = nextMission->fNextMissionScore;
			else info.fInt32[4] = 0;
			if (nextMission) info.fFloat[0] = static_cast<float>(nextMission->fMedal);
			else info.fFloat[0] = 0.0f;
			if (nextMission) info.fFloat[1] = static_cast<float>(nextMission->fCampaignEvent);
			else info.fFloat[1] = 0.0f;
			info.fFloat[2] = static_cast<float>(kFalse);
			info.fFloat[3] = static_cast<float>(gScriptInterface.mIsHost());
			if (!nextMission->fNextMissionTitle.empty())
				info.fTextPtr = nextMission->fNextMissionTitle.c_str();
			else info.fTextPtr = NULL;
			gScriptInterface.mEntityMgr(info);
		}
		if (fMissionInfo) fMissionInfo->mCheckForMissionEnd();
	}
	if (gScriptInterface.mIsHost()) fTeam->fHasEndedHost = kTrue;
	fTeam->fHasEndedClient = kTrue;
}
