/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include <time.h>
#include <fstream.h>

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	gTotalTeamScoutedShips = 0;
	gTotalScoutShips = 0;
	for( int32 i = 0; i < 20; ++i )
	{
		gShipsToScan[i] = 0;
	}
	fResultsIn = kFalse;
	fBadCRC = kFalse;
	fStartShipsInMotion = 1;

   // initialize each of the default values in case anything goes wrong
   fLegendaryAICaptain = 0.00f;
   fLegendaryAIWeapon = 0.00f;
   fLegendaryAIEngineer = 0.00f;
   fLegendaryAIScience = 0.00f;
   fLegendaryAIMarine = 0.00f;
   fLegendaryAIDoctor = 0.00f;
   fLegendaryAIComm = 0.00f;
   fLegendaryAIHelm = 0.00f ;
   fAIDroneModifier = 1.0f;
   fAIPartsModifier = 1.0f;
   fAIMarinesModifier = 1.0f;
   fFreighterSpareLimit = 40;
   fFrigateSpareLimit = 40;
   fDestroyerSpareLimit = 40;
   fWarDestroyerSpareLimit = 40;
   fLightCruiserSpareLimit = 40;
   fHeavyCruiserSpareLimit = 40;
   fNewHeavyCruiserSpareLimit = 40;
   fHeavyBattlecruiserSpareLimit = 40;
   fCarrierSpareLimit = 40;
   fDreadnoughtSpareLimit = 40;
   fBattleshipSpareLimit = 40;
   fBaseSpareLimit = 40;
   fBatsSpareLimit = 40;
   fStarbaseSpareLimit = 40;
   fPlanetSpareLimit = 40;
   fSpecialSpareLimit = 40;
   fOtherSpareLimit = 40;
   fAIBPVModifier = 1.0;
   fAllowCallForHelp = 1;
   fBPVPenaltyForCall = 0;
   fBPVBonusForNotCalling = 0;
   fDisableAIDuringPvP = 0;
   fAstoundingVictoryPvPBonus = 500;
   fVictoryPvPBonus = 400;
   fDrawPvPBonus = 300;
   fDefeatPvPBonus = 200;
   fDevastatingDefeatPvPBonus = 100;
   fFederation = 1;
   fKlingon = 2;
   fRomulan = 2;
   fLyran = 2;
   fMirak = 1;
   fGorn = 1;
   fHydran = 1;
   fISC = 3;
   fMonster = 4;
   fNeutral = 5;
   fOrion = 6;
   fOther = 7;
   fOrionOrion = 1;
   fPrime = 8;
   fWyldefire = 9;
   fTigerHeart = 2;
   fBeastRaiders = 1;
   fKorgath = 2;
   fSyndicate = 13;
   fNWCustID = 950;
   fCamboro = 14;
}

iTruth tWorldConditions::mAppendToFile(std::string filename, std::string textline)
{
	ofstream fpout;
	fpout.open(filename.c_str(), ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	fpout << textline.c_str() << endl;
	fpout.close();
	return kTrue;
}

iTruth tWorldConditions::mLogTeamResults(eTeamID team, eRace race, int32 prestige, eGameStatus result, iTruth human)
{
	std::string teamname;
	switch (team) {
			case kTeam1: teamname = "*Scout player"; break;
			case kTeam2: teamname = "Scouted player"; break;
			default: teamname = "AI"; break;
	}

	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "ERROR: unable to open BattleLog.txt during results logging" << endl;
			fpout.close();
		 }
		return kFalse;
	}
	if (!fResultsIn) {
		fResultsIn = kTrue;
		time_t timer1, timer2;
		timer2 = time(&timer1);
		struct tm *datetime1 = localtime(&timer1);
		int32 month = 0;
		int32 day = 0;
		int32 year = 0;
		int32 hour = 0;
		int32 min = 0;
		if (datetime1) {
			hour = datetime1->tm_hour;
			if (hour == 0) hour = 12;
			min = datetime1->tm_min;
		}
		fpout << "mission completed at: " << hour << ":";
		if (min < 10) fpout << "0";
		fpout << min <<  endl;
	}
	fpout << "   " << teamname.c_str() << " (";
	switch (race) {
		case kFederation: fpout << "Fed"; break;
		case kKlingon: fpout << "Kli"; break;
		case kRomulan: fpout << "Rom"; break;
		case kLyran: fpout << "Lyr"; break;
		case kMirak: fpout << "Mir"; break;
		case kHydran: fpout << "Hyd"; break;
		case kGorn: fpout << "Gor"; break;
		case kISC: fpout << "ISC"; break;
		case kNeutralRace: fpout << "Neu"; break;
		case kMonster: fpout << "Mons"; break;
		default: fpout << "Ori"; break;
	}
	if (human) fpout << ", human";
	else fpout << ", ai";
	fpout << ") " << prestige << "pp,";
	if (result < 0x00000100) fpout << " drafting player won\n";
	else if (result < kLeftEarly) fpout << " drafting player lost\n";
	else fpout << " mission was a draw or disconnect\n";
	fpout.close();
	return kTrue;
}

iTruth tWorldConditions::mStartMissionLog(std::string missionname)
{
	time_t timer1, timer2;
	timer2 = time(&timer1);
	struct tm *datetime1 = localtime(&timer1);
	int32 month = 0;
	int32 day = 0;
	int32 year = 0;
	int32 hour = 0;
	int32 min = 0;
	if (datetime1) {
		month = datetime1->tm_mon + 1;
	    day = datetime1->tm_mday;
		year = datetime1->tm_year - 100;
		hour = datetime1->tm_hour;
		if (hour == 0) hour = 12;
		min = datetime1->tm_min;
	}
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	fpout << endl << month << "/" << day << "/0" << year << " at " << hour << ":";
	if (min < 10) fpout << "0";
	fpout << min << "," << missionname.c_str() << ": " ;
	if (gScriptInterface.mIsHost()) fpout << "(Hosting) ";
	else fpout << "(you are client) ";
	fpout.close();
	return kTrue;
}

// mission script routines to read text from a file named NWMissionConfig.scr 
//    located in the specs/scripts directory,

iTruth tWorldConditions::mGetSetting(char *line, char *settingname, int32 &value, int32 len)
{
   int32 linesize, numstart;
   std::string setting;

   if( strncmp(line, settingname, len) == 0 )
   {
      if( line )
      {
         setting = line;
         ::ZeroMemory( line, sizeof(line) );
         linesize = setting.size();
         numstart = setting.find("=") + 1;
         setting.copy( line, linesize - numstart, numstart );
         value = atoi( line );
		 ofstream fpout;
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "int32 " << settingname << " " << value << endl;
			fpout.close();
		 }
         return kTrue;
      }
   }
   return kFalse;
}

iTruth tWorldConditions::mGetSetting(char *line, char *settingname, float &value, int32 len)
{
   int32 linesize, numstart;
   std::string setting;

    if( strncmp(line, settingname, len) == 0 )
    {
       if( line )
       {
         setting = line;
         ::ZeroMemory( line, sizeof(line) );
         linesize = setting.size();
         numstart = setting.find("=") + 1;
         setting.copy( line, linesize - numstart, numstart );
         value = atof( line );
 		 ofstream fpout;
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "float " << settingname << " " << value << endl;
			fpout.close();
		 }
         return kTrue;
       }
    }
    return kFalse;
}

void tWorldConditions::mGetSettingsFromConfig()
{
   char *line     = new char[256];
   int32 bufsize    = 256;

   ifstream infile( ".\\assets\\scripts\\NWMissionConfig.scr", ios::nocreate | ios::in );
   if( infile.fail() )
   {
	   ofstream fpout;
 	    fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		if (!fpout.fail()) {
			fpout << "unable to open config file" << endl;
			fpout.close();
		}
        delete[] line;
        return;
   }


   while( !infile.eof() )
   {
      infile.getline( line, bufsize );
      if (mGetSetting(line, "LegendaryAICaptain", fLegendaryAICaptain, 14)) continue;
      if (mGetSetting(line, "LegendaryAIWeapon", fLegendaryAIWeapon, 14)) continue;
      if (mGetSetting(line, "LegendaryAIEngineer", fLegendaryAIEngineer, 14)) continue;
      if (mGetSetting(line, "LegendaryAIScience", fLegendaryAIScience, 14)) continue;
      if (mGetSetting(line, "LegendaryAIMarine", fLegendaryAIMarine, 14)) continue;
      if (mGetSetting(line, "LegendaryAIDoctor", fLegendaryAIDoctor, 14)) continue;
      if (mGetSetting(line, "LegendaryAIComm", fLegendaryAIComm, 14)) continue;
      if (mGetSetting(line, "LegendaryAIHelm", fLegendaryAIHelm, 14)) continue; 
 
      if (mGetSetting(line, "AIDroneModifier", fAIDroneModifier, 5)) continue;  
      if (mGetSetting(line, "AIPartsModifier", fAIPartsModifier, 5)) continue;  
      if (mGetSetting(line, "AIMarinesModifier", fAIMarinesModifier, 5)) continue;
  
      if (mGetSetting(line, "FreighterSpareLimit", fFreighterSpareLimit, 16)) continue;  
      if (mGetSetting(line, "FrigateSpareLimit", fFrigateSpareLimit, 16)) continue;  
      if (mGetSetting(line, "DestroyerSpareLimit", fDestroyerSpareLimit, 16)) continue;  
      if (mGetSetting(line, "WarDestroyerSpareLimit", fWarDestroyerSpareLimit, 16)) continue;  
      if (mGetSetting(line, "LightCruiserSpareLimit", fLightCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "HeavyCruiserSpareLimit", fHeavyCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "NewHeavyCruiserSpareLimit", fNewHeavyCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "HeavyBattlecruiserSpareLimit", fHeavyBattlecruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "CarrierSpareLimit", fCarrierSpareLimit, 16)) continue;  
      if (mGetSetting(line, "DreadnoughtSpareLimit", fDreadnoughtSpareLimit, 16)) continue;  
      if (mGetSetting(line, "BattleshipSpareLimit", fBattleshipSpareLimit, 16)) continue;  
      if (mGetSetting(line, "BaseSpareLimit", fBaseSpareLimit, 13)) continue;  
      if (mGetSetting(line, "BatsSpareLimit", fBatsSpareLimit, 13)) continue;  
      if (mGetSetting(line, "StarbaseSpareLimit", fStarbaseSpareLimit, 16)) continue;  
      if (mGetSetting(line, "PlanetSpareLimit", fPlanetSpareLimit, 16)) continue;  
      if (mGetSetting(line, "SpecialSpareLimit", fSpecialSpareLimit, 16)) continue;  
      if (mGetSetting(line, "OtherSpareLimit", fOtherSpareLimit, 15)) continue;  
 
      if (mGetSetting(line, "AIBPVModifier", fAIBPVModifier, 12)) continue;  
 
      if (mGetSetting(line, "AllowCallForHelp", fAllowCallForHelp, 16)) continue;  
      if (mGetSetting(line, "BPVPenaltyForCall", fBPVPenaltyForCall, 16)) continue;  
      if (mGetSetting(line, "BPVBonusForNotCalling", fBPVBonusForNotCalling, 16)) continue;  
 
      if (mGetSetting(line, "DisableAIDuringPvP", fDisableAIDuringPvP, 18)) continue;  
 
      if (mGetSetting(line, "AstoundingVictoryPvPBonus", fAstoundingVictoryPvPBonus, 12)) continue;  
      if (mGetSetting(line, "VictoryPvPBonus", fVictoryPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DrawPvPBonus", fDrawPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DefeatPvPBonus", fDefeatPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DevastatingDefeatPvPBonus", fDevastatingDefeatPvPBonus, 12)) continue;  
 
      if (mGetSetting(line, "Federation", fFederation, 5)) continue;  
      if (mGetSetting(line, "Klingon", fKlingon, 5)) continue;  
      if (mGetSetting(line, "Romulan", fRomulan, 5)) continue;  
      if (mGetSetting(line, "Lyran", fLyran, 5)) continue;  
      if (mGetSetting(line, "Mirak", fMirak, 5)) continue;  
      if (mGetSetting(line, "Gorn", fGorn, 4)) continue;  
      if (mGetSetting(line, "Hydran", fHydran, 5)) continue;  
      if (mGetSetting(line, "ISC", fISC, 3)) continue;  
      if (mGetSetting(line, "Monster", fMonster, 5)) continue;  
      if (mGetSetting(line, "Neutral", fNeutral, 5)) continue;  
      if (mGetSetting(line, "Other", fOther, 5)) continue;  
      if (mGetSetting(line, "OrionOrion", fOrionOrion, 9)) continue;  
      if (mGetSetting(line, "Orion", fOrion, 5)) continue;  
      if (mGetSetting(line, "Prime", fPrime, 5)) continue;  
      if (mGetSetting(line, "Wyldefire", fWyldefire, 5)) continue;  
      if (mGetSetting(line, "TigerHeart", fTigerHeart, 5)) continue;  
      if (mGetSetting(line, "BeastRaiders", fBeastRaiders, 5)) continue;  
      if (mGetSetting(line, "Korgath", fKorgath, 5)) continue;  
      if (mGetSetting(line, "Syndicate", fSyndicate, 5)) continue;  
      if (mGetSetting(line, "Camboro", fCamboro, 5)) continue;  

	  int32 testmotion = 1;
      if (mGetSetting(line, "StartShipsInMotion", testmotion, 5)) {
		  if (testmotion == 0) fStartShipsInMotion = kFalse;
		  continue;  
	  }
   }
   infile.close();
   delete[] line;
}



void tWorldConditions::mSetTotalTeamScoutedShips( int32 setValue )
{
	gTotalTeamScoutedShips = setValue;
}
int32 tWorldConditions::mGetTotalTeamScoutedShips( void )
{
	return gTotalTeamScoutedShips;
}
void tWorldConditions::mSetTotalScoutShips( int32 setValue )
{
	gTotalScoutShips = setValue;
}
int32 tWorldConditions::mGetTotalScoutShips( void )
{
	return gTotalScoutShips;
}
void tWorldConditions::mSetShipScanned( int32 index )
{
	gShipsToScan[index] = 1;
}
int32 tWorldConditions::mGetScannedStatus( int32 index )
{
	return gShipsToScan[index];
}

/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.

*/
void tWorldConditions::mCheckForGameEnd( void )
{}
iTruth tWorldConditions::mCheckCRC()
{
	int32 crc = 0;
	int32 backup = 0;

   ifstream infile( ".\\assets\\scripts\\NWMissionConfig.scr", ios::nocreate | ios::in );
   if( infile.fail() )
   {
	   // cannot read config file, treat as bad CRC check
	   ofstream fpout;
 	    fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		if (!fpout.fail()) {
			fpout << "unable to open config file for CRC check" << endl;
			fpout.close();
		}
		fBadCRC = kTrue;
        return kFalse;
   } else {
		// compute the crc for the current config file
		char c;
		while (!infile.eof()) {
			infile.get(c);
			crc += 0l & c;
		}
		infile.close();
		time_t timer1, timer2;
		timer2 = time(&timer1);
		struct tm *datetime1 = localtime(&timer1);
		if (datetime1) {
			crc +=  (datetime1->tm_yday);
			backup = crc + ((datetime1->tm_yday - 1) % 365);
		}
		crc = crc % 15649;
		backup = backup % 15649;

		// read the recorded CRC file from the debug file
	   ifstream fpin;
	   fpin.open("NWMissionDebug.txt");
	   if (fpin.fail()) {
		   // no debug file found, treat as bad crc check
		   fBadCRC = kTrue;
		   return kFalse;
	   } else {
		   int32 crc_chk;
		   fpin >> crc_chk;
		   fpin.close();
		   if ((crc_chk != crc) && (crc_chk != backup)) {
			   fBadCRC = kTrue;
			   return kFalse;
		   }
	   }
   }
   return kTrue;
}
