/********************************************************************************************************\

	File Name:	BaseTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_26AsteroidAssualt.h"
#include "BaseTeam.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "BaseTeamInitialState.h"

tClassAllocator<tBaseTeam> gBaseTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tBaseTeam::mSetupState()
{
	mRegisterState( new tBaseTeamInitialState() );
	mGotoState( typeid( tBaseTeamInitialState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tBaseTeam::mSetupCommState()
{
}

void tBaseTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tBaseTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {

		int32 fShiplistCount = 0;
		tTeamInfo* defendTeam = fMissionInfo->mGetTeamHandle(  static_cast< eTeamID >( kDefendTeam1 ) );
		eRace race = defendTeam->mGetRace();
        ShipList AB_sl = gST.narrowed_sl;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        raceList.push_back(kNeutralRace);
        fShiplistCount = AB_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d and kNeutralRace) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

        std::list< eEnhancedClassTypes >  ABClassTypeList;
        ABClassTypeList.push_back(kEClassLargeAsteroid);
        fShiplistCount = AB_sl.Subset(ABClassTypeList, true);
        gST.mDebug_fprintf("subset for asteroid base class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  ABRolesList;
        ABRolesList.push_back(kBase);
        fShiplistCount = AB_sl.Subset(ABRolesList, true);
        gST.mDebug_fprintf("subset for kBase eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(AB_sl);

		Ship s1;
		if ((AB_sl.size() > 0) && (AB_sl.SelectRandomShip(s1))) {
			gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );
			mCreateShip(	
				typeid( tShipInfo ), 
				s1.shipCode.c_str(), 
				kStartPosition_M,			
				kNoMetaShipID, 
				-1, 0, 0, -1, -1, 
				NULL, 
				static_cast<eShipOptions>(kDeathUponCapture|kStartPositionCanBeOffset) );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tBaseTeam::mCreateShipsForTeam\n");
}

void tBaseTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tBaseTeam::mOldCreateShipsForTeam\n");

	std::string basetype = "N-AB";
	mCreateShip(	
		typeid( tShipInfo ), 
		basetype.c_str(), 
		kStartPosition_M,			
		kNoMetaShipID, 
		-1, 0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>(kDeathUponCapture|kStartPositionCanBeOffset) );

	gST.mDebug_fprintf("---- Finished: tBaseTeam::mOldCreateShipsForTeam\n");
}

void tBaseTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tBaseTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tBaseTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// for the census only. Not actually drafting.

   	gST.mDebug_fprintf("--- Finished: tBaseTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}