/********************************************************************************************************\

	Mission:	Asteriod Assault
	Script:		"Met_26AsteroidAssualt" //Assualt on an Aseteroid base (not an actual meta object)
	Descript:	<DESCRIPTION>

	Created by:		David F		02/07/01
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "AttackTeam3.h"
#include "Met_26AsteroidAssualt.h"
#include "BaseTeam.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_26AsteroidAssualt );

tMet_26AsteroidAssualt::tMet_26AsteroidAssualt()
{
}
void tMet_26AsteroidAssualt::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex , kAsteroidHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 5;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tAttackTeam ),
														kNoTeamTag,
														Messages[kAttackTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryOpponentTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kAttackTeam1,
														typeid( tDefendTeam ),
														kNoTeamTag,
														Messages[kDefenseTeamTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kAttackTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														Messages[kDefenseTeamTitle_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kAttackTeam1,
														typeid( tDefendTeam3 ),
														kNoTeamTag,
														Messages[kDefenseTeamTitle_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyAllyOf | kAttackTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														Messages[kAttackTeamTitle_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyAllyOf | kAttackTeam1,
														typeid( tAttackTeam3 ),
														kNoTeamTag,
														Messages[kAttackTeamTitle_msg] );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Base Team", static_cast< eTeamID>( kBaseTeam ),
														kMaxChance,
														kTrue,
														kNPCTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kAttackTeam1,
														typeid( tBaseTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}
int32 tMet_26AsteroidAssualt::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_26AsteroidAssualt::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_26AsteroidAssualt::mInitializeFinished( void )
{
	return kAttackBriefing_msg;
}

void tMet_26AsteroidAssualt::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_26AsteroidAssualt::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 7;
	int32 tagList[kRelationsCount] =
	{
		kAttackTeam1,	//Atk1
		kDefendTeam1,	//Def1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
		kBaseTeam,		//Orn
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Atk1	Def1	Def2	Def3	Atk2	Atk3	Base
		/*Atk1*/	"--",	"WD",	"WD",	"WD",	"AI",	"AI",	"WD",
		/*Def1*/	"WD",	"--",	"AI",	"AI",	"WD",	"WD",	"AI",
		/*Def2*/	"WD",	"AI",	"--",	"AI",	"WD",	"WD",	"AI",
		/*Def3*/	"WD",	"AI",	"AI",	"--",	"WD",	"WD",	"AI",
		/*Atk2*/	"AI",	"WD",	"WD",	"WD",	"--",	"AI",	"WD",	
		/*Atk3*/	"AI",	"WD",	"WD",	"WD",	"AI",	"--",	"WD",
		/*Base*/	"WD",	"AI",	"AI",	"AI",	"WD",	"WD",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );


}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_26AsteroidAssualt::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainAsteroids1, kPlanetNone, kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids2, kPlanetNone, kBaseNone );
	case 2:		return tMapTerrain( kTerrainAsteroids3, kPlanetNone, kBaseNone );
	case 3:		return tMapTerrain( kTerrainAsteroids4, kPlanetNone, kBaseNone );
	case 4:		return tMapTerrain( kTerrainAsteroids5, kPlanetNone, kBaseNone );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_26AsteroidAssualt::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['Y'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['G'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['M'] = kObjectEnemyBase;
}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_26AsteroidAssualt::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 60l, 110l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 65l, 115l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 70l, 120l );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.65f, 1.2f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.625f, 1.2f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.2f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tMet_26AsteroidAssualt::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		fBriefingIndex[kAttackTeam1][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam2][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam3][j] = kAttackBriefing_msg;
		fBriefingIndex[kDefendTeam1][j] = kDefenseBriefing_msg;
		fBriefingIndex[kDefendTeam2][j] = kDefenseBriefing_msg;
		fBriefingIndex[kDefendTeam3][j] = kDefenseBriefing_msg;

	}
}