/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "sfcAssert.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "Met_26AsteroidAssualt.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	fOrigNumAttackTeams = 0;
	fOrigNumDefendTeams = 0;
	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	fNumAttackShips = 0;	//The total number of attack ships at game start;
	fRemainingAttackShips = 0;	//The current count of Attack Ships
	fNumDefendShips = 0;
	fRemainingDefendShips = 0;
	fAsteroidBaseDestroyed = false;

	fNumTeams = 6;	//Hard coded max number of teams

	for( int32 i = 0; i < 21; ++i )
	{
		fAttackShipsDamageChart[i] = 0;
		fDefendShipsDamageChart[i] = 0;
	}
}
bool tWorldConditions::mGetIsAsteroidBaseDestroyed()
{
	return fAsteroidBaseDestroyed;
}
void tWorldConditions::mSetAsteroidBaseDestroyed()
{
	fAsteroidBaseDestroyed = true;
}
void tWorldConditions::mSetDefendDamageCond( int32 index, int32 condition )
{
	fDefendShipsDamageChart[index] = condition;
}
int32 tWorldConditions::mGetDefendDamageCond( int32 index )
{
	return fDefendShipsDamageChart[index];
}
void tWorldConditions::mSetAttackDamageCond( int32 index, int32 condition )
{
	fAttackShipsDamageChart[index] = condition;
}
int32 tWorldConditions::mGetAttackDamageCond( int32 index )
{
	return fAttackShipsDamageChart[index];
}
void tWorldConditions::mAddAttackTeam()
{
	++fNumAttackTeams;
	++fOrigNumAttackTeams;
}
void tWorldConditions::mRemoveAttackTeam()
{
	--fNumAttackTeams;
}
int32 tWorldConditions::mGetAttackTeams()
{
	return fNumAttackTeams;
}
void tWorldConditions::mAddDefendTeam()
{
	++fNumDefendTeams;
	++fOrigNumDefendTeams;
}
void tWorldConditions::mRemoveDefendTeam()
{
	--fNumDefendTeams;
}
int32 tWorldConditions::mGetDefendTeams()
{
	return fNumDefendTeams;
}
void tWorldConditions::mAddDefendShip()
{
	++fNumDefendShips;
	++fRemainingDefendShips;
}
void tWorldConditions::mRemoveDefendShip()
{
	--fRemainingDefendShips;
}
int32 tWorldConditions::mGetNumDefendShips()
{
	return fNumDefendShips;
}
int32 tWorldConditions::mGetRemainingDefendShips()
{
	return fRemainingDefendShips;
}
void tWorldConditions::mAddAttackShip()
{
	++fNumAttackShips;
	++fRemainingAttackShips;
}
int32 tWorldConditions::mGetNumAttackShips()
{
	return fNumAttackShips;
}
void tWorldConditions::mRemoveAttackShip()
{
	--fRemainingAttackShips;
}
int32 tWorldConditions::mGetRemainingAttackShips()
{
	return fRemainingAttackShips;
}
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo*	thisMission = gScriptInterface.mMission();
	if (!thisMission) return;

	tTeamInfo* currentTeam;
	int32 humans = 0;
	int32 numattackers = 0;
	int32 numdefenders = 0;
	int32 numbases = 0;
	int i;

	for ( i = kAttackTeam1; i <= kBaseTeam; i++ )
	{

		currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		if (currentTeam->mShipsCurrentlyControlled() > 0) {
			if ((i == kAttackTeam1) || (i == kAttackTeam2) || (i == kAttackTeam3)) numattackers++;
			else if (i == kBaseTeam) numbases++;
			else if ((i == kDefendTeam1) || (i == kDefendTeam2) || (i == kDefendTeam3)) numdefenders++;
			if (currentTeam->mIsTeamHumanControlled()) humans++;
		}

	}

	for ( i = kAttackTeam1; i <= kAttackTeam3; i++ )
	{

		currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		
		
		if( numdefenders == 0 && numbases == 0 )
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
		}
		else if( numattackers == 0 )
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
		}
		else if( humans == 0  )
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
		}
	
	}
}
