//	<Name>	AttackVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "AttackVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_27AsteroidDefense.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "AttackTeam.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"

#include "WorldConditions.h"

tAttackVictoryState::tAttackVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tAttackVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );
	attackTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam2 ) );
	attackTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam3 ) );
			
	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	defendTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam2 ) );
	defendTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam3 ) );
	baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );

	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 1;
	fTeamDisengaged = false;
	fThisPlayerDead = false;

}

void tAttackVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );

	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}
	if( attackTeam2 )
	{
		attackBPV += attackTeam2->mGetCombatBPV();
	}
	if( attackTeam3 )
	{
		attackBPV += attackTeam3->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( defendTeam2 )
	{
		defendBPV += defendTeam2->mGetCombatBPV();
	}
	if( defendTeam3 )
	{
		defendBPV += defendTeam3->mGetCombatBPV();
	}
	if( baseTeam )
	{
		defendBPV += baseTeam->mGetCombatBPV();
	}
}

void tAttackVictoryState::mEndState()
{
}

eVictoryLevels tAttackVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 attackDamagePoints = 0;
	float attackDamageRatio = 0.0;
	int32 defendDamagePoints = 0;
	float defendDamageRatio = 0.0;
	float damageRatio = 0.0;
	int i;

	for( i = 1; i <= gWC.mGetNumAttackShips(); ++i )
	{
		attackDamagePoints +=	gWC.mGetAttackDamageCond( i ); 
	}
	for( i = 1; i <= gWC.mGetNumDefendShips(); ++i )
	{
		defendDamagePoints +=	gWC.mGetDefendDamageCond( i );
	}
	attackDamageRatio = (float)attackDamagePoints / gWC.mGetNumAttackShips();
	defendDamageRatio = (float)defendDamagePoints / gWC.mGetNumDefendShips();

	if( attackDamageRatio )
	{
		damageRatio = defendDamageRatio / attackDamageRatio;
	}
	else if( defendDamageRatio )
	{
		damageRatio = 3;
	}
	else
	{
		damageRatio = 0.5;
	}

	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if( damageRatio > 1 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 50 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return	kAstoundingVictory;
	}
	else if( damageRatio > 0.5 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 25 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 62 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kVictory;
	}
	else if( damageRatio > 0.25 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 12 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 31 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kDraw;
	}
	else if( damageRatio > 0.125 )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}

}
int32 tAttackVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsCurrentlyControlled();

	return teamShipsInPlay;
}
void tAttackVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kBaseTeam ) );
	int32	baseShipsInPlay;

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			gWC.mRemoveAttackShip();
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveAttackTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}

	if( ship->mGetOriginalTeam() == baseTeam->mGetTeam() )
	{
		baseShipsInPlay = mShipsInPlay( baseTeam );

		if( baseShipsInPlay == 0 )
		{
			gWC.mSetAsteroidBaseDestroyed();
			mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
		}
	}

	tTeamInfo* attack1 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ) );
	tTeamInfo* attack2 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam2 ) );
	tTeamInfo* attack3 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam3 ) );
	tTeamInfo* defend1 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	tTeamInfo* defend2 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam2 ) );
	tTeamInfo* defend3 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam3 ) );
	int32 numattackers = 0;
	int32 numdefenders = 0;
	if (attack1) numattackers += attack1->mShipsCurrentlyControlled();
	if (attack2) numattackers += attack2->mShipsCurrentlyControlled();
	if (attack3) numattackers += attack3->mShipsCurrentlyControlled();
	if (defend1) numdefenders += defend1->mShipsCurrentlyControlled();
	if (defend2) numdefenders += defend2->mShipsCurrentlyControlled();
	if (defend3) numdefenders += defend3->mShipsCurrentlyControlled();
	if (baseTeam) numdefenders += baseTeam->mShipsCurrentlyControlled();
	if ((numattackers < 1) || (numdefenders < 1)) {
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}


}
void tAttackVictoryState::mOnDeath( tShipInfo* ship )
{

	int32 thisShipID;

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
		case kAttackTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
		case kDefendTeam3:
		//Empty
		break;
	}

	mCheckAllTeams( ship );
}
void tAttackVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
		case kAttackTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
		case kDefendTeam3:
			//Break;
		break;
	}

	mCheckAllTeams( ship );
}

void tAttackVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if (!ship) return;
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveAttackTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
			}
		}
	}

	mCheckAllTeams( ship );

}
void tAttackVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tAttackVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tAttackVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveAttackTeam();
		fPlayerLeftAlready = true;
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}
}