/********************************************************************************************************\

	File Name:	DefendTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_27AsteroidDefense.h"
#include "DefendTeam2.h"
#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "DefendVictoryCondition.h"
#include "DefendTeam2InitialState.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tDefendTeam2> gDefendTeam2Allocator;

tDefendTeam2::tDefendTeam2()
{
	gWC.mAddDefendTeam();
}

void tDefendTeam2::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam2::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tDefendTeam2::mSetupState()
{
	mRegisterState( new tDefendTeam2InitialState() );
	mGotoState( typeid( tDefendTeam2InitialState ) );	
}
void tDefendTeam2::mSetupCommState()
{
}
void tDefendTeam2::mCreateShipsForTeam()
{
}
void tDefendTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tDefendTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	mCreateShip( typeid( tDefendTeamShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_H,
				 metaid,
				 -1, 0, 0, -1, -1, //Default data,so we can set the custom name
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

   	gST.mDebug_fprintf("--- Finished: tDefendTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tDefendTeam2::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}