/********************************************************************************************************\

	File Name:	CommonEnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonEnemyShip.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


tClassAllocator<tCommonEnemyTeam> gCommonEnemyTeamAllocator;

tCommonEnemyTeam::tCommonEnemyTeam() : fNumShips( 0 ), fReachedListeningPost( kFalse )
{}

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tCommonEnemyTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonEnemyTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if (!fMissionInfo) return;

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinnerEnemy_msg );
			fMissionInfo->mAddToDebriefingList( fTeam, fMissionInfo->mRandomInt32( kData1_msg, kData5_msg ) );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoserEnemy_msg );
			fMissionInfo->mAddToDebriefingList( fTeam, fMissionInfo->mRandomInt32( kData1_msg, kData5_msg ) );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCommonEnemyTeam::mSetupState()
{
	mRegisterState( new tCommonEnemyTeamBaseState() );
	mGotoState( typeid( tCommonEnemyTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tCommonEnemyTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tCommonEnemyTeam::mCreateShipsForTeam()
{
	if (gWC.fPvP) return;
	// ship is actually drafted. See fShipClass here? It's from mCreateNewShip
	mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipClass.c_str(), 
					kStartPosition_G + fShipTeamID,
					fmetaid, 
					gWC.fNWCustID++, 0, 0, -1, -1,
					fShipName.c_str(),
					kDefaultShipOptions );

	fNumShips++;
}

void tCommonEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( mIsTeamHumanControlled() )
	{
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	//
	//	Set this team to 'won'.  All teams that are in the game should be set to this value.
	//
	gWC.mSetTeamWon( mGetTeam(), kTrue );
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tCommonEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tCommonEnemyTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	fmetaid = shipScriptDescription.fMetaDatabaseID;
	if (!mIsTeamHumanControlled()) {
		fShipClass = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		fShipTeamID = shipScriptDescription.fTeamID;
#ifdef ERA_BASED_DRONE_SPEEDS
		if (fMissionInfo->mGetDifficultyLevel() >= 1) {
			gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
			fmetaid = kNoMetaShipID; // no meta ID means era-correct drones
		}
#endif
	} else {
		mCreateShip(
			typeid( tCommonEnemyShip ), 
			shipScriptDescription.fClassName.c_str(), 
			kStartPosition_G + shipScriptDescription.fTeamID,
			fmetaid,
			gWC.fNWCustID++, 0, 0, -1, -1,
			shipScriptDescription.fCustomName.c_str(),
			kDefaultShipOptions );

		fNumShips++;
	}

   	gST.mDebug_fprintf("--- Finished: tCommonEnemyTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}





void tCommonEnemyTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonEnemyTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonEnemyTeam::mNumShips()
{
	return fNumShips;
}

iTruth tCommonEnemyTeam::mReachedListeningPost( void )
{
	return fReachedListeningPost;
}

void tCommonEnemyTeam::mSetReachedListeningPost( iTruth x )
{
	fReachedListeningPost = x;
}

void tCommonEnemyTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	if (VictoryCondition) {
		fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
		fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
		fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	} else {
		fMissionScheduler.fVictoryLevel = kDraw;
		fMissionScheduler.fPrestige = 0;
		fMissionScheduler.fBonusPrestige = 0;
	}
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}