/********************************************************************************************************\

	File Name:	CommonEnemyTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_2HoldingAction.h"
#include "ShipInfo.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonEnemyTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tCommonEnemyTeamBaseState::mEndState()
{}

void tCommonEnemyTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonEnemyTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonEnemyTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonEnemyTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonEnemyTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonEnemyTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonEnemyTeamBaseState::mOnFlashingButtonPressed()
{}
void tCommonEnemyTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonEnemyTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonEnemyTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonEnemyTeamBaseState::mOnHostChanged()
{}
void tCommonEnemyTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if ( timerID == kStartingID )
	{
		if (!fChildActor) return;
		if (!fMissionInfo) return;
		if (!fChildActor->mIsTeamHumanControlled()) return;
		eRaceName  playerRace = kNoRace;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fChildActor->mGetTeam() );
		if ( playerTeam )
			playerRace = playerTeam->mGetRace();

		// return the index to the Starting message we want to use
		switch( playerRace )
		{
			case kFederation:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kFedStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kKlingon:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kKliStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kRomulan:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kRomStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kLyran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kLyrStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kGorn:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kGorStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kMirak:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kMirStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kHydran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kHydStartEnemy_msg, kNonCommScriptMessage );
			break;
			case kISC:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kISCStartEnemy_msg, kNonCommScriptMessage );
			break;
			default:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kStartEnemy_msg, kNonCommScriptMessage );
			break;
		}
	}

	if (timerID == kUpgradeFightersID)
	{
#ifdef FIGHTERS
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;
		if (! gScriptInterface.mIsHost()) return;

		int32 fTeamID = fChildActor->mGetTeam();
		tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fTeamID ) );
		if ((fTeamID != kEnemyTeam1) && (fTeamID != kEnemyTeam2) && (fTeamID != kEnemyTeam3)) return;
		if (enemyTeam->mIsTeamHumanControlled()) return;

		gST.mDebug_fprintf("--- Entering tCommonEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		gST.fTeam1DeltaFighterBPV += gST.mUpgradeFightersForTeam(enemyTeam);

		gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam1DeltaFighterBPV);
		gST.mDebug_fprintf("--- Finished: tCommonEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
}
void tCommonEnemyTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonEnemyTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

