/********************************************************************************************************\

	File Name:	CommonPlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_2HoldingAction.h"
#include "ShipInfo.h"
#include "CommonPlayerTeam.h"
#include "CommonPlayerTeamBaseState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "ShipList_API.h"



/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tCommonPlayerTeamBaseState::mEndState()
{}

void tCommonPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tCommonPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonPlayerTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonPlayerTeamBaseState::mOnHostChanged()
{}
void tCommonPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if ( timerID == kStartingID )
	{
		if (!fMissionInfo) return;
		if (!fChildActor) return;
		if (!fChildActor->mIsTeamHumanControlled()) return;
		eRaceName  playerRace = kNoRace;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fChildActor->mGetTeam() );
		if ( playerTeam )
			playerRace = playerTeam->mGetRace();

		// return the index to the Starting message we want to use
		switch( playerRace )
		{
			case kFederation:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kFedStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kKlingon:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kKliStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kRomulan:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kRomStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kLyran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kLyrStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kGorn:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kGorStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kMirak:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kMirStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kHydran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kHydStartPlayer_msg, kNonCommScriptMessage );
			break;
			case kISC:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kISCStartPlayer_msg, kNonCommScriptMessage );
			break;
			default:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kStartPlayer_msg, kNonCommScriptMessage );
			break;
		}
	}

	if (timerID == kUpgradeFightersID)
	{
#ifdef FIGHTERS
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;
		if (! gScriptInterface.mIsHost()) return;

		int32 fTeamID = fChildActor->mGetTeam();
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fTeamID ) );
		if ((fTeamID != kPlayerTeam1) && (fTeamID != kPlayerTeam2) && (fTeamID != kPlayerTeam3)) return;
		if (playerTeam->mIsTeamHumanControlled()) return;

		gST.mDebug_fprintf("--- Entering tCommonPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		gST.fTeam2DeltaFighterBPV += gST.mUpgradeFightersForTeam(playerTeam);

		gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam2DeltaFighterBPV);
		gST.mDebug_fprintf("--- Finished: tCommonPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
}

void tCommonPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

