/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "CommonVictoryState.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "MissionText.h"
#include "CommonDefs.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState(	int32 astoundingVictory,
											int32 victory,
											int32 draw,
											int32 defeat,
											int32 devastatingDefeat ) : tChildActorVictoryState< tCommonVictory >(	astoundingVictory,
																													victory,
																													draw,
																													defeat,
																													devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus(fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled())))
		{
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}
void tCommonVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tCommonVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	//
	//	Now figure out which victory level this team should be.
	//
	if (gWC.fListeningPostDestroyed) return kDefeat;
	if (!fMissionInfo) return kDraw;

	tCommonEnemyTeam* enemyTeam = dynamic_cast< tCommonEnemyTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kEnemyTeam1) ) );
	tCommonEnemyTeam* enemyTeam2 = dynamic_cast< tCommonEnemyTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kEnemyTeam2) ) );
	tCommonEnemyTeam* enemyTeam3 = dynamic_cast< tCommonEnemyTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kEnemyTeam3) ) );
	int32 numenemies = 0;
	if (enemyTeam) numenemies += enemyTeam->mShipsCurrentlyControlled();
	if (enemyTeam2) numenemies += enemyTeam2->mShipsCurrentlyControlled();
	if (enemyTeam3) numenemies += enemyTeam3->mShipsCurrentlyControlled();

	tCommonPlayerTeam* playerTeam = dynamic_cast< tCommonPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam1) ) );
	tCommonPlayerTeam* playerTeam2 = dynamic_cast< tCommonPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam2) ) );
	tCommonPlayerTeam* playerTeam3 = dynamic_cast< tCommonPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam3) ) );
	int32 numplayers = 0;
	if (playerTeam) numplayers += playerTeam->mShipsCurrentlyControlled();
	if (playerTeam2) numplayers += playerTeam2->mShipsCurrentlyControlled();
	if (playerTeam3) numplayers += playerTeam3->mShipsCurrentlyControlled();

	switch(ID) {
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
			if (gWC.fEscapedWithData) {
				if ((numplayers > 0) && (numenemies < 1)) return kDraw;
				else if ((numplayers > 0) && (numenemies > 0)) return kDraw;
				else if ((numplayers < 1) && (numenemies < 1)) return kVictory;
				else return kAstoundingVictory;
			} else if (gWC.fReachedListeningPost) {
				if ((numplayers > 0) && (numenemies < 1)) return kDefeat;
				else if ((numplayers > 0) && (numenemies > 0)) return kDraw;
				else if ((numplayers < 1) && (numenemies < 1)) return kVictory;
				else return kAstoundingVictory;
			} else {
				if ((numplayers > 0) && (numenemies < 1)) return kDevastatingDefeat;
				else if ((numplayers > 0) && (numenemies > 0)) return kDefeat;
				else if ((numplayers < 1) && (numenemies < 1)) return kDraw;
				else return kAstoundingVictory;
			}
			break;
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			if (gWC.fEscapedWithData) {
				if ((numplayers > 0) && (numenemies < 1)) return kDraw;
				else if ((numplayers > 0) && (numenemies > 0)) return kDefeat;
				else if ((numplayers < 1) && (numenemies < 1)) return kDefeat;
				else return kDefeat;
			} else if (gWC.fReachedListeningPost) {
				if ((numplayers > 0) && (numenemies < 1)) return kVictory;
				else if ((numplayers > 0) && (numenemies > 0)) return kDefeat;
				else if ((numplayers < 1) && (numenemies < 1)) return kDraw;
				else return kDevastatingDefeat;
			} else {
				if ((numplayers > 0) && (numenemies < 1)) return kAstoundingVictory;
				else if ((numplayers > 0) && (numenemies > 0)) return kDefeat;
				else if ((numplayers < 1) && (numenemies < 1)) return kVictory;
				else return kDevastatingDefeat;
			}
			break;
		default: return kDraw;
	}
	
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tCommonVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/

iTruth tCommonVictoryState::mTeamsAreEnemies(eTeamID team1, eTeamID team2)
{
	switch (team1) {
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
			if (team2 == kPlayerTeam1) return kTrue;
			if (team2 == kPlayerTeam2) return kTrue;
			if (team2 == kPlayerTeam3) return kTrue;
			if (team2 == kGoalTeam) return kFalse;
			return kFalse;
			break;
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			if (team2 == kPlayerTeam1) return kFalse;
			if (team2 == kPlayerTeam2) return kFalse;
			if (team2 == kPlayerTeam3) return kFalse;
			if (team2 == kGoalTeam) return kFalse;
			return kTrue;
			break;
		default:
			return kFalse;
	}
}

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if (!team) return;

	if (!fTeam) return;
	eTeamID thisteamID = fTeam->mGetTeam();
	if (fTeam->mIsTeamHumanControlled()) {
		if (teamID == thisteamID) mAddToPrestige(-25);
		else if (mTeamsAreEnemies(thisteamID, teamID)) mAddToPrestige(25);
		else mAddToPrestige(-10);
	}


	if ( ship->mGetCustomID() == kListeningPostID )
	{
		gWC.fListeningPostDestroyed = kTrue;
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if (!fTeam) return;
	eTeamID thisteamID = fTeam->mGetTeam();
	if (fTeam->mIsTeamHumanControlled()) {
		if (teamID == thisteamID) mAddToPrestige(-50);
		else if (ship->mGetTeam() == thisteamID) mAddToPrestige(50);
		else if (mTeamsAreEnemies(thisteamID, teamID)) mAddToPrestige(25);
		else mAddToPrestige(-25);
	}


	if ( ship->mGetCustomID() == kListeningPostID )
	{
		gWC.fListeningPostDestroyed = kTrue;
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if (!fTeam) return;
	eTeamID thisteamID = fTeam->mGetTeam();
	if (fTeam->mIsTeamHumanControlled()) {
		if (thisteamID == teamID) mAddToPrestige(-10);
		else if (mTeamsAreEnemies(thisteamID, teamID)) mAddToPrestige(10);
		else mAddToPrestige(-5);
	}

	//
	//	The exception to the above rule is if the ship was on the enemy side and already transferred the data.
	//
	bool gotData = false;
	if ( teamID == kEnemyTeam1 || teamID == kEnemyTeam2 || teamID == kEnemyTeam3 )
	{
		if (gWC.fReachedListeningPost) gWC.fEscapedWithData = kTrue;
		tCommonEnemyTeam* enemyTeam = dynamic_cast< tCommonEnemyTeam* >( team );
		if ( enemyTeam )
		{
			if ( gWC.fReachedListeningPost )
			{
				gotData = true;
			}
		}
	}


	if ( ship->mGetCustomID() == kListeningPostID )
	{
		gWC.fListeningPostDestroyed = kTrue;
	}
	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

