/********************************************************************************************************\

	File Name:	GoalTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "GoalTeam.h"
#include "GoalTeamBaseState.h"
#include "ListeningPost.h"
#include "CommonDefs.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tGoalTeam> gGoalTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tGoalTeam::mSetupState()
{
	mRegisterState( new tGoalTeamBaseState() );
	mGotoState( typeid( tGoalTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tGoalTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tGoalTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tGoalTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();
	int32 fShiplistCount = 0;

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		// Players are holding the enemy.
		eRaceName race = kOrion;  // Mission text says Pirate

		ShipList goal_sl;
		goal_sl = gST.mFilterShipListToSingleRace(gST.narrowed_sl, race);
		goal_sl = gST.stripOutRestricted( goal_sl );

		// goal is a platform. Any platform.
		std::list< eEnhancedClassTypes >  platformClassTypeList;
		platformClassTypeList.push_back(kEClassPlatform);
		fShiplistCount = goal_sl.Subset(platformClassTypeList, true);
		gST.mDebug_fprintf("subset for platform filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(goal_sl);

		Ship s1;
		if ( (goal_sl.size() > 0) && (goal_sl.SelectRandomShip(s1)) ) {
			gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );

			mCreateShip( 
				typeid( tListeningPost ), 
				s1.shipCode.c_str(), 
				kStartPosition_M, 
				kNoMetaShipID, 
				kListeningPostID, 
				0, 
				0, 
				-1, 
				-1, 
				"Base", 
				kDefaultShipOptions );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}


   	gST.mDebug_fprintf("--- Entering: tGoalTeam::mCreateShipsForTeam\n");
}

void tGoalTeam::mOldCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tGoalTeam::mOldCreateShipsForTeam\n");

	mCreateShip( 
		typeid( tListeningPost ), 
		"O-DEF", kStartPosition_M, 
		kNoMetaShipID, kListeningPostID, 
		0, 
		0, 
		-1, 
		-1, 
		"Base", 
		kDefaultShipOptions );

   	gST.mDebug_fprintf("--- Entering: tGoalTeam::mOldCreateShipsForTeam\n");
}

void tGoalTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tGoalTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tGoalTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tGoalTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}
