/********************************************************************************************************\

	File Name:	GoalVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "GoalVictoryState.h"
#include "MissionText.h"
#include "GoalTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"

//
//	Serialization events
//
tGoalVictoryState::tGoalVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tTeam1BaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tGoalVictoryState::mBeginState()
{}
void tGoalVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tGoalVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tGoalVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tGoalVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( ship->mGetTeam() == kPlayerTeam )
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if ( !team->mShipsRemaining() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( ship->mGetTeam() == kPlayerTeam )
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if ( !team->mShipsRemaining() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( ship->mGetTeam() == kPlayerTeam )
	{
		gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDisengaged );
	}
}
void tGoalVictoryState::mAtDistance( tShipInfo* ship )
{}
void tGoalVictoryState::mAtDestination( tShipInfo* ship )
{}
void tGoalVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tGoalVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tGoalVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tGoalVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tGoalVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tGoalVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tGoalVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tGoalVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tGoalVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tGoalVictoryState::mOnFlashingButtonPressed()
{}
void tGoalVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tGoalVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tGoalVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tGoalVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tGoalVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tGoalVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tGoalVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tGoalVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tGoalVictoryState::mOnHostChanged()
{}
void tGoalVictoryState::mOnTimedInterval( int32 timerID )
{}
void tGoalVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tGoalVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
