/********************************************************************************************************\

	Mission:	Met_2HoldingAction
	Script:		"Met_2HoldingAction"
	Descript:	Small force is assigned to delay a larger force.

	Created by:		Daniel J. Suleski		07/05/00
	Last Updated:	DJS						07/06/00
					Added extra traits to mission that were not found in Met_4FleetAction.  This
					included the event catching for an enemy ship making it off of the 'far' edge
					of the map and the event catching for the the holding team destroying half of
					the enemy ships.
					DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta


\********************************************************************************************************/

#include "Met_2HoldingAction.h"
#include "CommonDefs.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "GoalTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_2HoldingAction );

tMet_2HoldingAction::tMet_2HoldingAction()
{
	srand(time(NULL));
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMet_2HoldingAction::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAnyTerrainHex);
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement;

	/*
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAsteroidHex | kNebulaHex | kBlackHoleHex | kShippingLaneHex | kDustCloudsHex | kEmptySpaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;
	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Enemy Team 1",
												static_cast< eTeamID >( kEnemyTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitleEnemy_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast< eTeamID >( kEnemyTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kEnemyTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitleEnemy_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team 3",
												static_cast< eTeamID >( kEnemyTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyAllyOf | kEnemyTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitleEnemy_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Holding Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kEnemyTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitlePlayer_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Holding Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kEnemyTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitlePlayer_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Holding Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kEnemyTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitlePlayer_msg] );

	const tTeamSpec& goalTeam = mCreateTeamSpec(	"Goal Team",
													static_cast< eTeamID >( kGoalTeam ),
													kMaxChance,
													kTrue,
													kNPCTeam,
													kMaxChance,
													kSpecOrion,
													kAnyRace,
													typeid( tGoalTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( goalTeam );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMet_2HoldingAction::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMet_2HoldingAction::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMet_2HoldingAction::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMet_2HoldingAction::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Holding action");
	gWC.mCheckCRC();

	const int kRelationsCount = 7;
	int32 teamList[kRelationsCount] =
	{
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kGoalTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 ETm1  ETm2  ETm3  PTm1  PTm2  PTm3  Goal
		/*ETm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*ETm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "HI",
		/*ETm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "HI",
		/*PTm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "HI",
		/*PTm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "HI",
		/*PTm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "HI",
		/*Goal*/ "WD", "HI", "HI", "HI", "HI", "HI", "AI",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_2HoldingAction::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace,		kPlanetNone, kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids,	kPlanetNone, kBaseNone );
	case 2:		return tMapTerrain( kTerrainNebula,		kPlanetNone, kBaseNone );
	case 3:		return tMapTerrain( kTerrainBlackholes,		kPlanetNone, kBaseNone );
	case 4:		return tMapTerrain( kTerrainDustclouds,		kPlanetNone, kBaseNone );
	case 5:		return tMapTerrain( kTerrainShippingLane,		kPlanetNone, kBaseNone );
	default:	break;
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_2HoldingAction::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['M'] = kObjectShipEnd;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_2HoldingAction::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefingPlayer_msg;
		}
		for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefingEnemy_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_2HoldingAction::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, 0.575f, 1.2f );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, 0.55f, 1.175f );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, 0.525f, 1.15f );
	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.25f );
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, 0.725f, 1.23f );
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, 0.75f, 1.21f );
	mSetTeamShipStrength( kPlayerTeam2, kClassFrigate, kClassBattleship, 0.625f, 1.19f );
	mSetTeamShipStrength( kPlayerTeam2, kClassFrigate, kClassBattleship, 0.65f, 1.17f );
	mSetTeamShipStrength( kPlayerTeam2, kClassFrigate, kClassBattleship, 0.675f, 1.15f );
	mSetTeamShipStrength( kPlayerTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}