/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

/*

	HELPER NOTE:	Add or delete from this list.  It is not necessary to have these exact maps in each
					multiplayer mission; however, each map should have valid starting positions for each
					of the 6 teams.

*/
int32 TotalMaps = 6;
const char* MapTypes[] = { "Open Space", "Asteroid Space", "Nebula Space", "Blackhole Space", "Dustclouds Space", "ShippingLane Space" };
const char* MapLines[]=
{
	// Asteroids: a,A,*
	// Black Holes: b,B
	// Nebula: c,C
	// Dust clouds: d,D
	// Start positions: G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z
	// End positions:   g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z
	// Planets: 1-6
	// Suns: 7-9
	//
   "Met_2HoldingAction",
   "// Open Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "1|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...........L.K.....................|",
   "2|............M......................|",
   " |............m......................|",
   " |............J......................|",
   " |............j......................|",
   " |...................................|",
   "3|........k.......l..................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "4|....g..............................|",
   " |....G.h............................|",
   " |.....H..i..........................|",
   " |.......I...........................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Asteroid Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................*...............|",
   "1|...................................|",
   " |.........*.........................|",
   " |..*.........[......*...............|",
   " |.....*..........*..................|",
   " |........[...*......................|",
   "2|.......[....M..]...................|",
   " |............m.......*..............|",
   " |...*...]..].J..[]..................|",
   " |.....[]....LjK.....................|",
   " |.........].........................|",
   "3|........k.*.].*.l.[................|",
   " |...........*....]..................|",
   " |....*.....*...[..*.................|",
   " |..........[........................|",
   " |........*.i.*.h.*..................|",
   "4|...................................|",
   " |.........I......H..................|",
   " |............g......................|",
   " |............G.........*............|",
   " |...................................|",
   "5|..................*................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Nebula Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |.?.............H...................|",
   " |................G.I................|",
   " |...................................|",
   "1|...................................|",
   " |...................>...............|",
   " |......>.....>......................|",
   " |.........>......<..................|",
   " |...................................|",
   "2|........<...m..>...................|",
   " |.....>......M.....>................|",
   " |............j......................|",
   " |............J......................|",
   " |........k.......l..................|",
   "3|........K...>...L..................|",
   " |...................................|",
   " |.................>.................|",
   " |...................................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |........i...g...h..................|",
   " |...................................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Blackhole Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...............H...................|",
   " |................G.I................|",
   " |...................................|",
   "1|...................................|",
   " |...................>...............|",
   " |......>.....>......................|",
   " |.........>......<....B.............|",
   " |...................................|",
   "2|........<...m..>...................|",
   " |.....>......M.....>................|",
   " |............j......................|",
   " |............J......................|",
   " |........k.......l..................|",
   "3|........K...>...L..................|",
   " |...................................|",
   " |.................>.................|",
   " |...................................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |........i...g...h..................|",
   " |...................................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Dustclouds Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...............H...................|",
   " |................G.I................|",
   " |...................................|",
   "1|...................................|",
   " |...................>...............|",
   " |......>.....>......................|",
   " |.........>......<..................|",
   " |...................................|",
   "2|........<...m..>...................|",
   " |.....>......M.....>................|",
   " |............j......................|",
   " |............J......................|",
   " |........k.......l..................|",
   "3|........K...>...L..................|",
   " |...................................|",
   " |.................>.................|",
   " |...................................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |........i...g...h..................|",
   " |...................................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// ShippingLane Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "1|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...........L.K.....................|",
   "2|............M......................|",
   " |............m......................|",
   " |............J......................|",
   " |............j......................|",
   " |...................................|",
   "3|........k.......l..................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "4|....g..............................|",
   " |....G.h............................|",
   " |.....H..i..........................|",
   " |.......I...........................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "///",
   0
};

