/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "GoalTeam.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include <time.h>
#include <fstream.h>

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions()
{
	fMissionSeed = 91;
	for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ )
	{
		fWasCaptured[i] = kFalse;
		fTeamWon[i] = kFalse;
	}
	fPvP = kFalse;
	fListeningPostDestroyed = kFalse;
	fReachedListeningPost = kFalse;
	fEscapedWithData = kFalse;
	fResultsIn = kFalse;
	fBadCRC = kFalse;
  	fAIBombsModifier = 1.0f;
	fBaseDestructProb = 0.50f;
	fBatsDestructProb = 0.25f;
	fSBDestructProb = 0.10f;

   // initialize each of the default values in case anything goes wrong
   fLegendaryAICaptain = 0.00f;
   fLegendaryAIWeapon = 0.00f;
   fLegendaryAIEngineer = 0.00f;
   fLegendaryAIScience = 0.00f;
   fLegendaryAIMarine = 0.00f;
   fLegendaryAIDoctor = 0.00f;
   fLegendaryAIComm = 0.00f;
   fLegendaryAIHelm = 0.00f ;
   fAIDroneModifier = 1.0f;
   fAIPartsModifier = 1.0f;
   fAIMarinesModifier = 1.0f;
   fFreighterSpareLimit = 40;
   fFrigateSpareLimit = 40;
   fDestroyerSpareLimit = 40;
   fWarDestroyerSpareLimit = 40;
   fLightCruiserSpareLimit = 40;
   fHeavyCruiserSpareLimit = 40;
   fNewHeavyCruiserSpareLimit = 40;
   fHeavyBattlecruiserSpareLimit = 40;
   fCarrierSpareLimit = 40;
   fDreadnoughtSpareLimit = 40;
   fBattleshipSpareLimit = 40;
   fBaseSpareLimit = 40;
   fBatsSpareLimit = 40;
   fStarbaseSpareLimit = 40;
   fPlanetSpareLimit = 40;
   fSpecialSpareLimit = 40;
   fOtherSpareLimit = 40;
   fAIBPVModifier = 1.0;
   fAllowCallForHelp = 1;
   fBPVPenaltyForCall = 0;
   fBPVBonusForNotCalling = 0;
   fDisableAIDuringPvP = 0;
   fAstoundingVictoryPvPBonus = 500;
   fVictoryPvPBonus = 400;
   fDrawPvPBonus = 300;
   fDefeatPvPBonus = 200;
   fDevastatingDefeatPvPBonus = 100;
   fFederation = 1;
   fKlingon = 2;
   fRomulan = 2;
   fLyran = 2;
   fMirak = 1;
   fGorn = 1;
   fHydran = 1;
   fISC = 3;
   fMonster = 4;
   fNeutral = 5;
   fOrion = 6;
   fOther = 7;
   fOrionOrion = 1;
   fPrime = 8;
   fWyldefire = 9;
   fTigerHeart = 2;
   fBeastRaiders = 1;
   fKorgath = 2;
   fSyndicate = 13;
   fCamboro = 14;
   fNWCustID = 950;
}

iTruth tWorldConditions::mAppendToFile(std::string filename, std::string textline)
{
	ofstream fpout;
	fpout.open(filename.c_str(), ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	fpout << textline.c_str() << endl;
	fpout.close();
	return kTrue;
}

iTruth tWorldConditions::mLogTeamResults(eTeamID team, eRace race, int32 prestige, eGameStatus result, iTruth human)
{
	std::string teamname;
	switch (team) {
			case kTeam1: teamname = "*Blockade runner"; break;
			case kTeam2:
			case kTeam3: teamname = "Blockade runner"; break;
			case kTeam4:
			case kTeam5:
			case kTeam6: teamname = "Defending player"; break;
			default: teamname = "AI"; break;
	}

	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	if (!fResultsIn) {
		fResultsIn = kTrue;
		time_t timer1, timer2;
		timer2 = time(&timer1);
		struct tm *datetime1 = localtime(&timer1);
		int32 month = 0;
		int32 day = 0;
		int32 year = 0;
		int32 hour = 0;
		int32 min = 0;
		if (datetime1) {
			hour = datetime1->tm_hour;
			if (hour == 0) hour = 12;
			min = datetime1->tm_min;
		}
		fpout << "mission completed at: " << hour << ":";
		if (min < 10) fpout << "0";
		fpout << min << "," << endl;
	}
	fpout << "   " << teamname.c_str() << " (";
	switch (race) {
		case kFederation: fpout << "Fed"; break;
		case kKlingon: fpout << "Kli"; break;
		case kRomulan: fpout << "Rom"; break;
		case kLyran: fpout << "Lyr"; break;
		case kMirak: fpout << "Mir"; break;
		case kHydran: fpout << "Hyd"; break;
		case kGorn: fpout << "Gor"; break;
		case kISC: fpout << "ISC"; break;
		case kNeutralRace: fpout << "Neu"; break;
		case kMonster: fpout << "Mons"; break;
		default: fpout << "Ori"; break;
	}
	fpout << ") " << prestige << "pp,";
	if (result < 0x00000100) fpout << " drafting player won\n";
	else if (result < kLeftEarly) fpout << " drafting player lost\n";
	else fpout << " mission was a draw\n";
	fpout.close();
	return kTrue;
}

iTruth tWorldConditions::mStartMissionLog(std::string missionname)
{
	time_t timer1, timer2;
	timer2 = time(&timer1);
	struct tm *datetime1 = localtime(&timer1);
	int32 month = 0;
	int32 day = 0;
	int32 year = 0;
	int32 hour = 0;
	int32 min = 0;
	if (datetime1) {
		month = datetime1->tm_mon + 1;
	    day = datetime1->tm_mday;
		year = datetime1->tm_year - 100;
		hour = datetime1->tm_hour;
		if (hour == 0) hour = 12;
		min = datetime1->tm_min;
	}
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	fpout << endl << month << "/" << day << "/0" << year << " at " << hour << ":";
	if (min < 10) fpout << "0";
	fpout << min << "," << missionname.c_str() << ": " ;
	if (gScriptInterface.mIsHost()) fpout << "(Hosting) ";
	else fpout << "(you are client) ";
	fpout.close();
	return kTrue;
}

// mission script routines to read text from a file named NWMissionConfig.scr 
//    located in the specs/scripts directory,

iTruth tWorldConditions::mGetSetting(char *line, char *settingname, int32 &value, int32 len)
{
   int32 linesize, numstart;
   std::string setting;

   if( strncmp(line, settingname, len) == 0 )
   {
      if( line )
      {
         setting = line;
         ::ZeroMemory( line, sizeof(line) );
         linesize = setting.size();
         numstart = setting.find("=") + 1;
         setting.copy( line, linesize - numstart, numstart );
         value = atoi( line );
		 ofstream fpout;
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "int32 " << settingname << " " << value << endl;
			fpout.close();
		 }
         return kTrue;
      }
   }
   return kFalse;
}

iTruth tWorldConditions::mGetSetting(char *line, char *settingname, float &value, int32 len)
{
   int32 linesize, numstart;
   std::string setting;

    if( strncmp(line, settingname, len) == 0 )
    {
       if( line )
       {
         setting = line;
         ::ZeroMemory( line, sizeof(line) );
         linesize = setting.size();
         numstart = setting.find("=") + 1;
         setting.copy( line, linesize - numstart, numstart );
         value = atof( line );
 		 ofstream fpout;
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "float " << settingname << " " << value << endl;
			fpout.close();
		 }
         return kTrue;
       }
    }
    return kFalse;
}

void tWorldConditions::mGetSettingsFromConfig()
{
   char *line     = new char[256];
   int32 bufsize    = 256;

   ifstream infile( ".\\assets\\scripts\\NWMissionConfig.scr", ios::nocreate | ios::in );
   if( infile.fail() )
   {
	   ofstream fpout;
 	    fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		if (!fpout.fail()) {
			fpout << "unable to open config file" << endl;
			fpout.close();
		}
        delete[] line;
        return;
   }


   while( !infile.eof() )
   {
      infile.getline( line, bufsize );
      if (mGetSetting(line, "LegendaryAICaptain", fLegendaryAICaptain, 14)) continue;
      if (mGetSetting(line, "LegendaryAIWeapon", fLegendaryAIWeapon, 14)) continue;
      if (mGetSetting(line, "LegendaryAIEngineer", fLegendaryAIEngineer, 14)) continue;
      if (mGetSetting(line, "LegendaryAIScience", fLegendaryAIScience, 14)) continue;
      if (mGetSetting(line, "LegendaryAIMarine", fLegendaryAIMarine, 14)) continue;
      if (mGetSetting(line, "LegendaryAIDoctor", fLegendaryAIDoctor, 14)) continue;
      if (mGetSetting(line, "LegendaryAIComm", fLegendaryAIComm, 14)) continue;
      if (mGetSetting(line, "LegendaryAIHelm", fLegendaryAIHelm, 14)) continue; 
 
      if (mGetSetting(line, "AIDroneModifier", fAIDroneModifier, 5)) continue;  
      if (mGetSetting(line, "AIPartsModifier", fAIPartsModifier, 5)) continue;  
      if (mGetSetting(line, "AIMarinesModifier", fAIMarinesModifier, 5)) continue;
      if (mGetSetting(line, "AIBombsModifier", fAIBombsModifier, 5)) continue;
 
	  if (mGetSetting(line, "BaseDestructProb", fBaseDestructProb, 15)) continue;  
      if (mGetSetting(line, "BatsDestructProb", fBatsDestructProb, 15)) continue;  
      if (mGetSetting(line, "SBDestructProb", fSBDestructProb, 15)) continue;  
  
      if (mGetSetting(line, "FreighterSpareLimit", fFreighterSpareLimit, 16)) continue;  
      if (mGetSetting(line, "FrigateSpareLimit", fFrigateSpareLimit, 16)) continue;  
      if (mGetSetting(line, "DestroyerSpareLimit", fDestroyerSpareLimit, 16)) continue;  
      if (mGetSetting(line, "WarDestroyerSpareLimit", fWarDestroyerSpareLimit, 16)) continue;  
      if (mGetSetting(line, "LightCruiserSpareLimit", fLightCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "HeavyCruiserSpareLimit", fHeavyCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "NewHeavyCruiserSpareLimit", fNewHeavyCruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "HeavyBattlecruiserSpareLimit", fHeavyBattlecruiserSpareLimit, 16)) continue;  
      if (mGetSetting(line, "CarrierSpareLimit", fCarrierSpareLimit, 16)) continue;  
      if (mGetSetting(line, "DreadnoughtSpareLimit", fDreadnoughtSpareLimit, 16)) continue;  
      if (mGetSetting(line, "BattleshipSpareLimit", fBattleshipSpareLimit, 16)) continue;  
      if (mGetSetting(line, "BaseSpareLimit", fBaseSpareLimit, 13)) continue;  
      if (mGetSetting(line, "BatsSpareLimit", fBatsSpareLimit, 13)) continue;  
      if (mGetSetting(line, "StarbaseSpareLimit", fStarbaseSpareLimit, 16)) continue;  
      if (mGetSetting(line, "PlanetSpareLimit", fPlanetSpareLimit, 16)) continue;  
      if (mGetSetting(line, "SpecialSpareLimit", fSpecialSpareLimit, 16)) continue;  
      if (mGetSetting(line, "OtherSpareLimit", fOtherSpareLimit, 15)) continue;  
 
      if (mGetSetting(line, "AIBPVModifier", fAIBPVModifier, 12)) continue;  
 
      if (mGetSetting(line, "AllowCallForHelp", fAllowCallForHelp, 16)) continue;  
      if (mGetSetting(line, "BPVPenaltyForCall", fBPVPenaltyForCall, 16)) continue;  
      if (mGetSetting(line, "BPVBonusForNotCalling", fBPVBonusForNotCalling, 16)) continue;  
 
      if (mGetSetting(line, "DisableAIDuringPvP", fDisableAIDuringPvP, 18)) continue;  
 
      if (mGetSetting(line, "AstoundingVictoryPvPBonus", fAstoundingVictoryPvPBonus, 12)) continue;  
      if (mGetSetting(line, "VictoryPvPBonus", fVictoryPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DrawPvPBonus", fDrawPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DefeatPvPBonus", fDefeatPvPBonus, 12)) continue;  
      if (mGetSetting(line, "DevastatingDefeatPvPBonus", fDevastatingDefeatPvPBonus, 12)) continue;  
 
      if (mGetSetting(line, "Federation", fFederation, 5)) continue;  
      if (mGetSetting(line, "Klingon", fKlingon, 5)) continue;  
      if (mGetSetting(line, "Romulan", fRomulan, 5)) continue;  
      if (mGetSetting(line, "Lyran", fLyran, 5)) continue;  
      if (mGetSetting(line, "Mirak", fMirak, 5)) continue;  
      if (mGetSetting(line, "Gorn", fGorn, 4)) continue;  
      if (mGetSetting(line, "Hydran", fHydran, 5)) continue;  
      if (mGetSetting(line, "ISC", fISC, 3)) continue;  
      if (mGetSetting(line, "Monster", fMonster, 5)) continue;  
      if (mGetSetting(line, "Neutral", fNeutral, 5)) continue;  
      if (mGetSetting(line, "Other", fOther, 5)) continue;  
      if (mGetSetting(line, "OrionOrion", fOrionOrion, 9)) continue;  
      if (mGetSetting(line, "Orion", fOrion, 5)) continue;  
      if (mGetSetting(line, "Prime", fPrime, 5)) continue;  
      if (mGetSetting(line, "Wyldefire", fWyldefire, 5)) continue;  
      if (mGetSetting(line, "TigerHeart", fTigerHeart, 5)) continue;  
      if (mGetSetting(line, "BeastRaiders", fBeastRaiders, 5)) continue;  
      if (mGetSetting(line, "Korgath", fKorgath, 5)) continue;  
      if (mGetSetting(line, "Syndicate", fSyndicate, 5)) continue;  
      if (mGetSetting(line, "Camboro", fCamboro, 5)) continue;  
   }
   infile.close();
   delete[] line;

   if (fAIBPVModifier < 0.5) fAIBPVModifier = 0.5;
   if (fAIBPVModifier > 2.0) fAIBPVModifier = 2.0;
}




/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	Find out which teams have ships remaining in play.
	//
	int32 attackers = 0;
	int32 defenders = 0;
	int32 humans = 0;
	int32 base = 0;
	tTeamInfo *team;
	iTruth gotdata = kFalse;

	team = thisMission->mGetTeamHandle( static_cast< eTeamID >( kGoalTeam ) );
	if (team) {
		if (team->mShipsCurrentlyControlled() > 0) base++;
	}


	for ( int32 k = kEnemyTeam1; k <= kEnemyTeam3; k++ )
	{
		team = thisMission->mGetTeamHandle( static_cast< eTeamID >( k ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() > 0) {
				attackers++;
				if (team->mIsTeamHumanControlled()) humans++;
			}
		}
	}

	for ( k = kPlayerTeam1; k <= kPlayerTeam3; k++ )
	{
		team = thisMission->mGetTeamHandle( static_cast< eTeamID >( k ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() > 0) {
				defenders++;
				if (team->mIsTeamHumanControlled()) humans++;
			}
		}
	}

	if (gWC.fListeningPostDestroyed) { // base died, no one wins
			for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ ) {
				mMyInvokeGameStatus(static_cast<eTeamID>(i),kLeftEarly);
			}
	} else if ((attackers > 0) && (!defenders)) { // base and attackers survived, defenders gone. automatic win for attackers
			for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ ) {
				mMyInvokeGameStatus(static_cast<eTeamID>(i),kPlayerWins);
			}
	} else if ((!defenders) && ( gWC.fEscapedWithData)) { // defenders gone, base survived, atackers got data
			for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ ) {
				mMyInvokeGameStatus(static_cast<eTeamID>(i),kPlayerWins);
			}
	} else if ((!attackers) || (!humans)) { // base survived, attackers gone, success depends on whether or not they got the data
			for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ ) {
				if (gWC.fEscapedWithData) mMyInvokeGameStatus(static_cast<eTeamID>(i),kLeftEarly);
				else mMyInvokeGameStatus(static_cast<eTeamID>(i),kPlayerLoses);
			}
	} 


	//
	//	If we got this far, then the game is not considered to be completed yet.
	//
}


void tWorldConditions::mSetWasCaptured( int32 x, iTruth y )
{
	fWasCaptured[x] = y;
}
void tWorldConditions::mSetTeamWon( int32 x, iTruth y )
{
	fTeamWon[x] = y;
}

iTruth tWorldConditions::mWasCaptured( int32 x )
{
	return fWasCaptured[x];
}
iTruth tWorldConditions::mTeamWon( int32 x )
{
	return fTeamWon[x];
}

void tWorldConditions::mMyInvokeGameStatus(eTeamID team, eGameStatus stat)
{

	tMissionInfo* fMissionInfo = gScriptInterface.mMission();
	if (!fMissionInfo) return;
	tTeamInfo *fTeam =fMissionInfo->mGetTeamHandle( team );
	if (!fTeam) return;

	fTeam->fStatus = stat;
	if (fMissionInfo) fMissionInfo->fStatusArray[team] = stat;

	if (fTeam->fVictoryCondition) {
		// make sure the status is properly set for every team
		fTeam->fVictoryLevel = fTeam->fVictoryCondition->mCalcVictoryStatus(team);
		fTeam->mScheduleNextMission(fTeam->fVictoryCondition);
		// calculate prestige and debriefing messages for human teams
		if (!fTeam->fHasEndedHost) {
			if (fTeam->mIsTeamHumanControlled()) {
				int32 result = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				result += fTeam->fVictoryCondition->mGetBonusPrestige();
				gWC.mLogTeamResults(team, fTeam->mGetRace(), result, stat, fTeam->mIsTeamHumanControlled());
				tShipSystemsInfo info;
				info.fTeam = fTeam->mGetTeam();
				info.fAction = kPlayerEvent;
				info.fInt32[0] = kPlayerStatusChange;
				info.fInt32[1] = fTeam->fStatus;
				info.fInt32[2] = fTeam->fVictoryLevel;
				info.fInt32[3] = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				info.fInt32[4] = fTeam->fVictoryCondition->mGetBonusPrestige();
				fMissionInfo->mResetDebriefingCount( fTeam->mGetTeam() );
				fTeam->mSetupDebriefingList( fTeam->fVictoryLevel );
				gScriptInterface.mEntityMgr(info);
				if (gScriptInterface.mIsHost()) {
					fTeam->fHasEndedHost = kTrue;
					fTeam->fHasEndedClient = kTrue;
				}
			}
			// return the mission results and scheduling for the team
			tTeamInfo::tMissionScheduler *nextMission = fTeam->mGetNextMissionInfo();
			tShipSystemsInfo info;
			info.fTeam = fTeam->mGetTeam();
			info.fAction = kOther;
			info.fInt32[0] = kMissionResults;
			info.fInt32[1] = fTeam->fVictoryLevel;
			if (nextMission) info.fInt32[2] = nextMission->fPrestige;
			else info.fInt32[2] = 0;
			info.fInt32[3] = fTeam->fVictoryCondition->mGetBonusPrestige();
			if (nextMission) info.fInt32[4] = nextMission->fNextMissionScore;
			else info.fInt32[4] = 0;
			if (nextMission) info.fFloat[0] = static_cast<float>(nextMission->fMedal);
			else info.fFloat[0] = 0.0f;
			if (nextMission) info.fFloat[1] = static_cast<float>(nextMission->fCampaignEvent);
			else info.fFloat[1] = 0.0f;
			info.fFloat[2] = static_cast<float>(kFalse);
			info.fFloat[3] = static_cast<float>(gScriptInterface.mIsHost());
			if (!nextMission->fNextMissionTitle.empty())
				info.fTextPtr = nextMission->fNextMissionTitle.c_str();
			else info.fTextPtr = NULL;
			gScriptInterface.mEntityMgr(info);
		}
		if (fMissionInfo) fMissionInfo->mCheckForMissionEnd();
	}
	if (gScriptInterface.mIsHost()) fTeam->fHasEndedHost = kTrue;
	fTeam->fHasEndedClient = kTrue;
}

iTruth tWorldConditions::mCheckCRC()
{
	int32 crc = 0;
	int32 backup = 0;

   ifstream infile( ".\\assets\\scripts\\NWMissionConfig.scr", ios::nocreate | ios::in );
   if( infile.fail() )
   {
	   // cannot read config file, treat as bad CRC check
	   ofstream fpout;
 	    fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		if (!fpout.fail()) {
			fpout << "unable to open config file for CRC check" << endl;
			fpout.close();
		}
		fBadCRC = kTrue;
        return kFalse;
   } else {
		// compute the crc for the current config file
		char c;
		while (!infile.eof()) {
			infile.get(c);
			crc += 0l & c;
		}
		infile.close();
		time_t timer1, timer2;
		timer2 = time(&timer1);
		struct tm *datetime1 = localtime(&timer1);
		if (datetime1) {
			crc +=  (datetime1->tm_yday);
			backup = crc + ((datetime1->tm_yday - 1) % 365);
		}
		crc = crc % 15649;
		backup = backup % 15649;

		// read the recorded CRC file from the debug file
	   ifstream fpin;
	   fpin.open("NWMissionDebug.txt");
	   if (fpin.fail()) {
		   // no debug file found, treat as bad crc check
		   fBadCRC = kTrue;
		   return kFalse;
	   } else {
		   int32 crc_chk;
		   fpin >> crc_chk;
		   fpin.close();
		   if ((crc_chk != crc) && (crc_chk != backup)) {
			   fBadCRC = kTrue;
			   return kFalse;
		   }
	   }
   }
   return kTrue;
}
