/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackTeam.h"
#include "BaseVictoryState.h"
#include "DefendTeam.h"
#include "MissionText.h"
#include "Met_7BaseAssault.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisPlayerDisengaged = false;	
}
void tBaseVictoryState::mBeginState()
{
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled()))) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( ID );
	if ( team )
	{
		iTruth teamHasShips = kFalse;
		if ( team->mShipsCurrentlyControlled() )
			teamHasShips = kTrue;

		//
		//	Find out which team the argument ID is on.
		//
		if( fThisPlayerDisengaged )
		{
			return kDefeat;
		}

		if ( team->mGetTag() == kAttackTag )
		{
			if( gWC.mOTDDestroyed() && teamHasShips )
			{
				return kVictory;
			}
			else
			{
				return kDefeat;
			}
		}
		else if ( team->mGetTag() == kDefendTag )
		{
			//
			//	Find out which teams have ships remaining in play.
			//
			iTruth	attackTeamHasShips = kFalse;		// what a fix...?!?!?!
			for ( int32 k = kAttackTeam1; k <= kAttackTeam3; k++ )
			{
				tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( k ) );
				if ( team )
				{
					if ( team->mShipsCurrentlyControlled() )
						attackTeamHasShips = kTrue;
				}
			}

			if ( !attackTeamHasShips && teamHasShips )
			{
				return kVictory;
			}
			else
			{
				return kDefeat;
			}
		}
	}

	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	The starbase has been destroyed, so the game should end.
			//
			gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeDestroyOTD );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( ( ( int )fTeam == kAttackTeam1 ) || ( ( int )fTeam == kAttackTeam2 ) || ( ( int )fTeam == kAttackTeam3 ) )
				mAddToBonusPrestige( 100 );

			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( kPrestigeDestroyShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( ( ( int )fTeam == kDefendTeam1 ) || ( ( int )fTeam == kDefendTeam2 ) || ( ( int )fTeam == kDefendTeam3 ) )
				mAddToBonusPrestige( 100 );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeDestroyShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	mSetTimedInterval( kTimerSecondsSet, kEndGameDelay, 2.0f );
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	The starbase has been destroyed, so the game should end.
			//
			gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeCaptureOTD );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( kPrestigeCaptureShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeCaptureShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	mSetTimedInterval( kTimerSecondsSet, kEndGameDelay, 2.0f );
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
	{
		int32 fThisTeamShipsInPlay = team->mShipsRemainingInPlay();

		if( fThisTeamShipsInPlay < 1 )
		{
			fThisPlayerDisengaged = true;
		}
	}

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	There is no way in the world that a starbase is going to disengage.  If it does,
			//	get a good night's rest.
			//			
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	mSetTimedInterval( kTimerSecondsSet, kEndGameDelay, 2.0f );
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kEndGameDelay) {
		gST.mDebug_fprintf("Sending to gWC.mCheckForGameEnd()\n");
		gWC.mCheckForGameEnd();
	}
}


void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
