/********************************************************************************************************\

	Mission:	Met_7BaseAssault
	Script:		"Met_7BaseAssault"
	Descript:	Fleet attempts to take over base

	Created by:		Daniel J. Suleski		07/06/00
	Last Updated:	DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta


\********************************************************************************************************/

#include "AttackTeam.h"
#include "DefendTeam.h"
#include "Met_7BaseAssault.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "OTDTeam.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_7BaseAssault );

tMet_7BaseAssault::tMet_7BaseAssault()
{
	srand(time(NULL));
	fBaseClassType = kClassBattleStation;

}

void tMet_7BaseAssault::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile(); 

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kBattleStationHex );
	Specs->fWait	= 1;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase;

	/*
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAsteroidHex | kNebulaHex | kBlackHoleHex | kShippingLaneHex | kDustCloudsHex | kEmptySpaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;
	*/

	const tTeamSpec& team1 = mCreateTeamSpec(	"Attack Team 1",
												static_cast< eTeamID >( kAttackTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team 2",
												static_cast< eTeamID >( kAttackTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Attack Team 3",
												static_cast< eTeamID >( kAttackTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team 1",
												static_cast< eTeamID >( kDefendTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Defend Team 2",
												static_cast< eTeamID>( kDefendTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Defend Team 3",
												static_cast< eTeamID>( kDefendTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Starbase Team",
												static_cast< eTeamID >( kStarbaseTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace | kOrion,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tOTDTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}

int32 tMet_7BaseAssault::mInitializeStart( tDynaverseScriptInfo& info )
{
	gSetMapPicked(info.fMapPicked);
	return 1;
}

void tMet_7BaseAssault::mCreateExternalShips( int32 Mode )
{}

void tMet_7BaseAssault::mNewInternalShip( tShipInfo* Ship )
{}

void tMet_7BaseAssault::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Bats Assault (7)");
	gWC.mCheckCRC();

	const int kRelationsCount = 7;
	int32 teamList[kRelationsCount] =
	{
		kAttackTeam1,
		kAttackTeam2,
		kAttackTeam3,
		kDefendTeam1,
		kDefendTeam2,
		kDefendTeam3,
		kStarbaseTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  PTm4  PTm5  PTm6  StTm
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*PTm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*PTm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*StTm*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

tMapTerrain tMet_7BaseAssault::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetNone, kBaseBattleStation );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetNone, kBaseBattleStation );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseBattleStation );
	case 3:		return tMapTerrain( kTerrainShippingLane, kPlanetNone, kBaseBattleStation );
	case 4:		return tMapTerrain( kTerrainShippingLane, kPlanetAnyPlanet, kBaseBattleStation );
	case 5:		return tMapTerrain( kTerrainNebula, kPlanetNone, kBaseBattleStation );
	case 6:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseBattleStation );
	case 7:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseBattleStation );
	case 8:		return tMapTerrain( kTerrainDustclouds, kPlanetNone, kBaseBattleStation );
	case 9:		return tMapTerrain( kTerrainShippingLane, kPlanetNone, kBaseBattleStation );
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_7BaseAssault::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_7BaseAssault::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kAttackTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingAttack_msg;
		}
		for ( i = kDefendTeam1; i <= kStarbaseTeam; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingDefend_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_7BaseAssault::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.2f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.575f, 1.18f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.55f, 1.16f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.675f, 1.14f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.65f, 1.12f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.625f, 1.10f );

	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 60l, 110l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 65l, 115l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 70l, 120l );

	mSetTeamShipStrength( kDefendTeam2, kClassDestroyer, kClassHeavyCruiser, 70l, 120l );
	mSetTeamShipStrength( kDefendTeam2, kClassDestroyer, kClassHeavyCruiser, 75l, 125l );
	mSetTeamShipStrength( kDefendTeam2, kClassDestroyer, kClassHeavyCruiser, 80l, 130l );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.6f, 1.1f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.625f, 1.125f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.65f, 1.15f );

	mSetTeamShipStrength( kStarbaseTeam, fBaseClassType, fBaseClassType, kMinBPV, kMaxBPV );
}


//
//	The following two functions determine the size and race of the starbase and the race that should be
//	defending it.
//
iMissSpecRace tMet_7BaseAssault::mTranslateHexIntoRace( tMetaLocationSpec hex )
{
	iMissSpecRace returnRace = kSpecPlayableRace;

	if ( hex.fLocationWhoHexFlags & kFederationRaceHex )
		returnRace = kSpecFed;
	else if ( hex.fLocationWhoHexFlags & kKlingonRaceHex )
		returnRace = kSpecKling;
	else if ( hex.fLocationWhoHexFlags & kRomulanRaceHex )
		returnRace = kSpecRomulan;
	else if ( hex.fLocationWhoHexFlags & kGornRaceHex )
		returnRace = kSpecGorn;
	else if ( hex.fLocationWhoHexFlags & kLyranRaceHex )
		returnRace = kSpecLyran;
	else if ( hex.fLocationWhoHexFlags & kHydranRaceHex )
		returnRace = kSpecHydran;
	else if ( hex.fLocationWhoHexFlags & kMirakRaceHex )
		returnRace = kSpecMirak;
	else if ( hex.fLocationWhoHexFlags & kISCRaceHex )
		returnRace = kSpecISC;

	return returnRace;
}
