/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.  You may want to change this for multiplayer
					missions, though.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team as shown below.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );			break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPC1 ),
					nPCBPV,
					kClassFrigate,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPC1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPC2 ), "PL3" , kStartPosition_J, kNoMetaShipID, kPlayerNPC2 );

*/
void tPlayerTeam::mCreateShipsForTeam()
{
	mCreateShip(	typeid( tPlayerShip ),
					mGetCombatBPV(),
					kClassFrigate,
					kClassFrigate,
					0,
					kStartPosition_G,
					kPlayerShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
}

/*

	HELPER NOTE:	This is where you want to set the player ship as player controlled.  Also, you have
					the ability to setup state callback events here such as:

						for( ; *ship != NULL; ship++ )
						{
							ship->mSetShipCallbackEvents( kNotifyDamage );
						}

*/
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

/*

	HELPER NOTE:	Normally, in campaign missions, you will want to create the exact same ship that the player
					is currently controlling in the Metaverse.  Well, this is where to do this.  The parameter
					that comes through here contains all of the information necessary to create this ship.  You
					will likely decide to create the player ship(s) using this function rather than
					::mCreateShipsForTeam().  Following is an example of how to use this parameter:

						mCreateShip( typeid( tPlayerShip ), 
									 shipScriptDescription.fClassName.c_str(), 
									 kStartPosition_G,
									 kNoMetaShipID,
									 shipNumber, 0, 0, -1, -1,
									 shipScriptDescription.fCustomName.c_str(),
									 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

*/
void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}

/*

	HELPER NOTE:	For each non-kNPCTeam, you must set the battle results.  Below is a brief description
					of the fields in tMissionScheduler:

						fVictoryLevel		- kAstoundingVictory, kDefeat, etc...
						fPrestige			- Amount of normal prestige rewarded based on fVictoryLevel
						fBonusPrestige		- Amount of bonus prestige rewarded regardless of fVictoryLevel
						fNextMissionScore	- 1000 means that the fNextMissionTitle will definitely be the next
											  mission chosen
						fNextMissionTitle	- The title of the suggested next mission -- this mission should
											  be referred to by it's title (assigned in tMissionSpec), not
											  by it's .scr file
						fMedal				- Bitwise constants that suggest a medal upon completion
						fCampaignEvent		- kNone, kRetire, or kLost determines any special campaign screens

*/
void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();

	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
	fMissionScheduler.fMedal = kNoMedals;
	fMissionScheduler.fCampaignEvent = tTeamInfo::tMissionScheduler::kNone;
}
