/********************************************************************************************************\

	File Name:	OTDBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "OTDBaseState.h"
#include "Met_7BaseAssault.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include "ShipList_API.h"
#include "BaseMetaIDShip.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tOTDBaseState::mBeginState()
{	
	fRestoreSpares = 0;
	// may have to use gGetMapPicked() for some missions
	int32 map = gGetMapPicked(); if (map < 1) map = 1;
	int32 cust = fChildActor->mGetCustomID(); if (cust < 1) cust = 1;
	int32 seed = gWC.fMissionSeed; if (seed < 1) seed = 1;
	int32 bpv = fChildActor->mGetShipBPV(); if (bpv < 1) bpv = 1;
	int32 Team = fChildActor->mGetTeam(); if (Team < 1) Team = 1;
	int32 id = fChildActor->mGetShipID(); if (id < 1) id = 1;
	fShipRandValue = map * cust * seed * bpv * Team * id;
	if (fShipRandValue < 0) fShipRandValue *= -1;
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ));
	if (team && (team->mGetRace() == kRomulan) && (!team->mIsTeamHumanControlled())) {
		fChildActor->mCloak( kFalse );
		fChildActor->mChangeAIOfficerState( kCloakOfficer, kFalse );
	}

#ifndef NOMODIFIEDOFFICERS
	tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
	if (drafter && ((drafter->mGetRace() == kLyran) || (drafter->mGetRace() == kGorn)
		|| (drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan))) {
		if ((gWC.fDraftingHumans == 1) && (!team->mIsTeamHumanControlled()) && (gWC.fDraftedHumans == 0)) {
			if ((rand() % 3) > 0) {
				fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse); 
			} else {
				fChildActor->mChangeAIOfficerState( kRepairOfficer, kFalse ); 
				fChildActor->mSetOfficerRank( kEngineeringOfficer, kRookie ); 
				fChildActor->mSetOfficerRank( kWeaponsOfficer, kRookie ); 
				fChildActor->mChangeAIOfficerState( kECCMOfficer, kFalse ); 
				fChildActor->mChangeAIOfficerState( kShuttleOfficer, kFalse); 
			}
		}
	}
#endif

}

void tOTDBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
	// adjust the loadout of the AI ship
	return;

	// quit if it isn't the desired kind of update
	if (Type != kShipCurrentValue) return;

	// quit if there is no ship to update
	if (!Ship) return;

	// quit if this isn't the ship we're looking for
	if (Ship->mGetCustomID() != fChildActor->mGetCustomID()) return;

	// quit if it's on a different team
	if (Ship->mGetTeam() != fChildActor->mGetTeam()) return;

	// quit if the ship is, or was, on a human controlled team
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( Ship->mGetTeam() ));
	if (!team) return;

	// adjust spare parts handling for humans
	if (team->mIsTeamHumanControlled()) {
		// adjust spare parts handling here
		int32 limit = 40;
		eClassTypes HullType = fChildActor->mGetClassType();
		if (HullType == kClassFrigate) HullType = kClassCarrier;
		else if ((HullType > kClassFreighter) && (HullType < kClassDreadnought))
			HullType = static_cast<eClassTypes>(HullType - 1);
		switch (HullType){
			case kClassFreighter: limit = gWC.fFreighterSpareLimit; break;
			case kClassFrigate: limit = gWC.fFrigateSpareLimit; break;
			case kClassDestroyer: limit = gWC.fDestroyerSpareLimit; break;
			case kClassWarDestroyer: limit = gWC.fWarDestroyerSpareLimit; break;
			case kClassLightCruiser: limit = gWC.fLightCruiserSpareLimit; break;
			case kClassHeavyCruiser: limit = gWC.fHeavyCruiserSpareLimit; break;
			case kClassNewHeavyCruiser: limit = gWC.fNewHeavyCruiserSpareLimit; break;
			case kClassHeavyBattlecruiser: limit = gWC.fHeavyBattlecruiserSpareLimit; break;
			case kClassCarrier: limit = gWC.fCarrierSpareLimit; break;
			case kClassDreadnought: limit = gWC.fDreadnoughtSpareLimit; break;
			case kClassBattleship: limit = gWC.fBattleshipSpareLimit; break;
			case kClassBaseStation: limit = gWC.fBaseSpareLimit; break;
			case kClassBattleStation: limit = gWC.fBatsSpareLimit; break;
			case kClassStarBase: limit = gWC.fStarbaseSpareLimit; break;
			case kClassPlanets: limit = gWC.fPlanetSpareLimit; break;
			case kClassSpecial: limit = gWC.fSpecialSpareLimit; break;
			default: limit = gWC.fOtherSpareLimit; break;
		}
		if ((Value > limit) && (limit > 0)) {
			fRestoreSpares = Value - limit;
			Value = limit;
			fChildActor->mSetItemInStores( Item, SubItem, Value );	
		}
#ifdef DIZZYPACK
		// include specialized drones limit for romulans in dizzy's server
		if ((team->mGetRace() == kRomulan) && (Item == kMissilesTypeI) && (subitem != kStandard)) {
			Ship->mSetItemInStores( kMissilesTypeI, kStandard, Value );	
		}
#endif
		return;
	} 
		
	// designate some uniqueness to the AI
	if (team->mGetRace() == kRomulan) {
		Ship->mCloak( kFalse );
		Ship->mSetSystemHealthFromOrig( kCloakHardPoint, 0, kFalse );
		Ship->mChangeAIOfficerState( kCloakOfficer, kFalse );
		Ship->mCloak( kFalse );
	} 
#ifndef NOMODIFIEDOFFICERS
    else if (team->mGetRace() != kLyran) {
		int32 speed = 24;;
		switch (fShipRandValue % 29) {
			case 1: Ship->mChangeAIOfficerState( kECMOfficer, kFalse); break;
			case 2: Ship->mChangeAIOfficerState( kECCMOfficer, kFalse ); break;
			case 3: Ship->mChangeAIOfficerState( kTractorOfficer, kFalse ); break;
			case 4: Ship->mChangeAIOfficerState( kRepairOfficer, kFalse ); break;
			case 5: Ship->mChangeAIOfficerState( kSeekingWeaponDefenseOfficer, kFalse ); break;
			case 6:  
			case 7: Ship->mChangeAIOfficerState( kHitAndRunOfficer, kFalse ); break;
			case 8: 
			case 9: Ship->mChangeAIOfficerState( kTBombOfficer, kFalse ); break;
			case 10: Ship->mChangeAIOfficerState( kHETOfficer, kFalse ); break;
			case 11: break;
			case 12: Ship->mChangeAIOfficerState( kFighterOfficer, kFalse ); break;
			case 13: Ship->mChangeAIOfficerState( kShieldReinforcementOfficer, kFalse ); break;
			case 14: Ship->mChangeAIOfficerState( kShuttleOfficer, kFalse ); break;
			default:
				int32 officerroll = fShipRandValue % 101;
				float officerchance = gWC.fLegendaryAIEngineer;
				if ((officerroll < 1) || (officerroll > 99)) break;
				if (officerroll < (100 * officerchance)) {
					Ship->mSetOfficerRank( kEngineeringOfficer, kLegendary ); break;
				} 
				officerchance += gWC.fLegendaryAIScience;
				if (officerroll < (100 * officerchance)) {
					Ship->mSetOfficerRank( kScienceOfficer, kLegendary ); break;
				}
				officerchance += gWC.fLegendaryAIMarine;
				if (officerroll < (100 * officerchance)) {
					Ship->mSetOfficerRank( kSecurityOfficer, kLegendary ); break;
				}
				officerchance += gWC.fLegendaryAIComm;
				if (officerroll < (100 * officerchance)) {
					Ship->mSetOfficerRank( kCommOfficer, kLegendary ); break;
				}
				officerchance += gWC.fLegendaryAIHelm;
				if (officerroll < (100 * officerchance)) 
					Ship->mSetOfficerRank( kHelmOfficer, kLegendary );
				break;
		}
	}
#endif
	
	// compute an adjusted quantity for the ship stores
	switch (Item) {
		case kMissilesTypeI:
		case kMissilesTypeIV:
			if (SubItem == kSpeed_Fast) {
				if (!(fShipRandValue % 3)) SubItem = kSpeed_Med;
			}
			if (SubItem == kSpeed_Med) {
				if (!(fShipRandValue % 2)) SubItem = kStandard;
			}
			Value *= gWC.fAIDroneModifier;
			break;
		case kMarines:
			Value *= gWC.fAIMarinesModifier;
			break;
		case kSmall_Mines:
			Value *= gWC.fAIBombsModifier;
			break;
		case kSpareParts:
			Value *= gWC.fAIPartsModifier;
			break;
	}

#ifdef DIZZYPACK
		// include specialized drones limit for romulans in dizzy's server
		if ((team->mGetRace() == kRomulan) && (Item == kMissilesTypeI) && (subitem != kStandard)) {
			Ship->mSetItemInStores( kMissilesTypeI, kStandard, Value );	
		}
		return;
#endif

	// establish the adjusted quantity on board
	Ship->mSetItemInStores( Item, SubItem, Value );	

}


void tOTDBaseState::mEndState()
{
	if (fRestoreSpares > 0)
		fChildActor->mAddItemToStores( kSpareParts, kStandard, fRestoreSpares );	
}

void tOTDBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDDestroyed_msg, kNonCommScriptMessage );
		}

#ifdef ERA_BASED_DRONE_SPEEDS
		// This device will carry the metaid back to the D2 instance when base is destroyed.
		// it'll blow up immediately after creation
		gST.mDebug_fprintf("Base era-correct drone hack: Creating a box with the base's metaid (%d)..\n", gWC.fBaseMetaID);
		tTeamInfo *StarbaseTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kStarbaseTeam));
		StarbaseTeam->mCreateShip( 
			typeid( tBaseMetaIDShip ),
			"N-BOX", 
			kStartPosition_Z,
			gWC.fBaseMetaID,
			kBaseMetaIDCustomID, 
			0, 0, -1, -1,
			"",
			kNoShipOptions );
#endif
	}

	int32 enemies = 0;
	tTeamInfo *team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
	if ((fRestoreSpares > 0) && (ship->mGetCustomID() != fChildActor->mGetCustomID())) {
		fChildActor->mAddItemToStores( kSpareParts, kStandard, fRestoreSpares );	
		fRestoreSpares = 0;
	}
}

void tOTDBaseState::mOnCapture( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDCaptured_msg, kNonCommScriptMessage );
		}

#ifdef ERA_BASED_DRONE_SPEEDS
		// This device will carry the metaid back to the D2 instance when base is destroyed.
		// it'll blow up immediately after creation
		gST.mDebug_fprintf("Base era-correct drone hack: Creating a box with the base's metaid (%d)..\n", gWC.fBaseMetaID);
		tTeamInfo *StarbaseTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kStarbaseTeam));
		StarbaseTeam->mCreateShip( 
			typeid( tBaseMetaIDShip ),
			"N-BOX", 
			kStartPosition_Z,
			gWC.fBaseMetaID,
			kBaseMetaIDCustomID, 
			0, 0, -1, -1,
			"",
			kNoShipOptions );
#endif
	}

	int32 enemies = 0;
	tTeamInfo *team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
	if (fRestoreSpares > 0) {
		fChildActor->mAddItemToStores( kSpareParts, kStandard, fRestoreSpares );	
		fRestoreSpares = 0;
	}	
}

void tOTDBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kOTDDisengaged_msg, kNonCommScriptMessage );
		}

#ifdef ERA_BASED_DRONE_SPEEDS
		// This device will carry the metaid back to the D2 instance when base is destroyed.
		// it'll blow up immediately after creation
		gST.mDebug_fprintf("Base era-correct drone hack: Creating a box with the base's metaid (%d)..\n", gWC.fBaseMetaID);
		tTeamInfo *StarbaseTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kStarbaseTeam));
		StarbaseTeam->mCreateShip( 
			typeid( tBaseMetaIDShip ),
			"N-BOX", 
			kStartPosition_Z,
			gWC.fBaseMetaID,
			kBaseMetaIDCustomID, 
			0, 0, -1, -1,
			"",
			kNoShipOptions );
#endif
	}

	if ((ship->mGetCustomID() == fChildActor->mGetCustomID()) && (fRestoreSpares > 0)) {
		fChildActor->mAddItemToStores( kSpareParts, kStandard, fRestoreSpares );
		fRestoreSpares = 0;
	}	
	int32 enemies = 0;
	tTeamInfo *team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
		team = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
	if ((team) && (team->mShipsCurrentlyControlled() != 0)) return;
	if (fRestoreSpares > 0) {
		fChildActor->mAddItemToStores( kSpareParts, kStandard, fRestoreSpares );	
		fRestoreSpares = 0;
	}
}

void tOTDBaseState::mAtDistance( tShipInfo* ship )
{}
void tOTDBaseState::mAtDestination( tShipInfo* ship )
{}
void tOTDBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tOTDBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
}
void tOTDBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tOTDBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tOTDBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tOTDBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tOTDBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tOTDBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDBaseState::mOnFlashingButtonPressed()
{}
void tOTDBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tOTDBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tOTDBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tOTDBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tOTDBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tOTDBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tOTDBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tOTDBaseState::mOnHostChanged()
{}
void tOTDBaseState::mOnTimedInterval( int32 timerID )
{}
void tOTDBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tOTDBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

