/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_7BaseAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{}

void tOTDTeam::mSetupVictoryCondition()
{
	gST.mDebug_fprintf("DEBUG: when do we do mSetupVictoryCondition?\n");

	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );

	gST.mDebug_fprintf("DEBUG: finished mSetupVictoryCondition\n");
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	gST.mDebug_fprintf("DEBUG: when do we do mSetupState?\n");

	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );

	gST.mDebug_fprintf("DEBUG: finished mSetupState\n");
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

void tOTDTeam::mOldCreateShipsForTeam()
{
	return; //debugging!

   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	std::string shiptype, frdtype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };


	switch (mGetRace()) {
		case kFederation: shiptype = "F-DEF"; frdtype = "F-FRD"; break;
		case kGorn: shiptype = "G-DEF";  frdtype = "G-FRD";break;
		case kHydran: shiptype = "H-DEF";  frdtype = "H-FRD";break;
		case kISC: shiptype = "I-DEF";  frdtype = "I-FRD";break;
		case kKlingon: shiptype = "K-DEF";  frdtype = "K-FRD";break;
		case kLyran: shiptype = "L-DEF";  frdtype = "L-FRD";break;
		case kMirak: shiptype = "Z-DEF";  frdtype = "Z-FRD";break;
		case kRomulan: shiptype = "R-DEF";  frdtype = "R-FRD";break;
		default: // used for both Orion and neutral/monster
			shiptype = "O-DEF";  frdtype = "O-FRD";break;
	}
	int32 skipchance = 5;
	if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
		tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
		if (drafter) { 
			if (drafter->mGetRace() == kLyran) skipchance = 1;
			else if (drafter->mGetRace() == kGorn) skipchance = 2;
			else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
			else skipchance = 4;
		}
	}
	for (i = 0; i < 4; i++) {
		if ((rand() % 5) < skipchance)
			mCreateShip( typeid( tShipInfo ), 
				frdtype.c_str(), 
				hexes[i], 
				kNoMetaShipID, 
				-1, 
				0, 0, 
				-1, -1,	
				"", 
				kDefaultShipOptions);
		}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string shiptype, frdtype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		ShipList OTDTeam_sl = gST.narrowed_sl;

		std::list< eRace >  raceList;
		raceList.push_back(mGetRace());
		fShiplistCount = OTDTeam_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

		// FRDs
		ShipList FRD_sl = OTDTeam_sl;
		std::list< eEnhancedClassTypes >  FRDClassTypeList;
		FRDClassTypeList.push_back(kEClassFleetRepairDock);
		fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
		gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(FRD_sl);

		if (FRD_sl.size() > 0) {

			int32 skipchance = 5;
			if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
				tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
				if (drafter) { 
					if (drafter->mGetRace() == kLyran) skipchance = 1;
					else if (drafter->mGetRace() == kGorn) skipchance = 2;
					else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
					else skipchance = 4;
				}
			}

			Ship s1;

			for (i = 0; i < 4; i++) {
				if ((rand() % 5) < skipchance) {
					FRD_sl.SelectRandomShip(s1);
					gST.mTrackOrions(s1);
					mCreateShip( typeid( tShipInfo ), 
						s1.shipCode.c_str(), 
						hexes[i], 
						kNoMetaShipID, 
						-1, 
						0, 0, 
						-1, -1,	
						"", 
						kDefaultShipOptions);
				}
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}

void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
		gWC.fBaseMetaID = shipScriptDescription.fMetaDatabaseID;
	} else {
		gWC.fBaseMetaID = kNoMetaShipID;
	}
	gST.mDebug_fprintf("base metaid is going to be %d\n", gWC.fBaseMetaID);
#endif

	int32 TeamRandValue = 1;
	if (gGetMapPicked() > 1) TeamRandValue *= gGetMapPicked();
	if (mGetTeam() > 1) TeamRandValue *= mGetTeam();
	if (mGetCombatBPV() > 1) TeamRandValue *= mGetCombatBPV();
	if (gWC.fMissionSeed > 1) TeamRandValue *= gWC.fMissionSeed;
	if (metaid > 1) TeamRandValue *= metaid;
	if (mShipsCurrentlyControlled() > 1) TeamRandValue *= mShipsCurrentlyControlled();
	if (TeamRandValue < 0) TeamRandValue *= -1;
	TeamRandValue = TeamRandValue % 100;

	mCreateShip( 
		typeid( tOTD ),
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_Z,
		metaid,
		kOTD, 0, 0, -1, -1,
		"",
		kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

eClassTypes tOTDTeam::mTranslateHexIntoType( tMetaLocationSpec hex )
{
	eClassTypes returnClass = kClassStarBase;

	if ( hex.fLocationObjectFlags & kBaseStationHex )
		returnClass = kClassBaseStation;
	else if ( hex.fLocationObjectFlags & kBattleStationHex )
		returnClass = kClassBattleStation;
	else if ( hex.fLocationObjectFlags & kStarbaseHex )
		returnClass = kClassStarBase;

	return returnClass;
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}