/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackTeam.h"
#include "BaseVictoryState.h"
#include "DefendTeam.h"
#include "MissionText.h"
#include "Met_7BaseAssault.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "WorldConditions.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisPlayerDisengaged = false;	
}
void tBaseVictoryState::mBeginState()
{
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kDefendTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled()))) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( ID );
	if ( team )
	{
		iTruth teamHasShips = kFalse;
		if ( team->mShipsCurrentlyControlled() )
			teamHasShips = kTrue;

		//	Find out which teams have ships remaining in play.
		//
		iTruth	attackTeamHasShips = kFalse;		// what a fix...?!?!?!
		for ( int32 k = kAttackTeam1; k <= kAttackTeam3; k++ )
		{
			tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( k ) );
			if ( team )
			{
				if ( team->mShipsCurrentlyControlled() )
						attackTeamHasShips = kTrue;
			}
		}

		//
		//	Find out which team the argument ID is on.
		//

		if ( ID <= kAttackTeam3 )
		{
			if( gWC.mOTDDestroyed() && attackTeamHasShips )
			{
				return kVictory;
			}
			else if (gWC.mOTDDestroyed()) 
			{
				return kDraw;
			} 
			else
			{
				return kDefeat;
			}
		}
		else if ( ID <= kDefendTeam3 )
		{
			//

			if ( !attackTeamHasShips && !gWC.mOTDDestroyed() && teamHasShips )
			{
				return kVictory;
			}
			else if (!attackTeamHasShips && (!gWC.mOTDDestroyed() || teamHasShips)) 
			{
				return kDraw;
			}
			else
			{
				return kDefeat;
			}
		}
	}

	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	tTeamInfo*	baseteam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kStarbaseTeam ) );
	Assert( team );
	if ( !team )
		return;

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	Check if all freighters have been destroyed.
			//
			if ((baseteam->mShipsCurrentlyControlled() <= 0) && (gWC.fOTDDisengaged == false))
				gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( 50 );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( 50 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( 50 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	tTeamInfo*	baseteam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kStarbaseTeam ) );
	Assert( team );
	if ( !team )
		return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 100 );
	}

	if ( team != fChildActor->mGetTeam() ) return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	Check if all freighters have been destroyed.
			//
			if ((baseteam->mShipsCurrentlyControlled() <= 0) && (gWC.fOTDDisengaged == false))
				gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( 100 );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( 100 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( 100 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	tTeamInfo*	baseteam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kStarbaseTeam ) );
	Assert( team );
	if ( !team )
		return;

	if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
	{
		int32 fThisTeamShipsInPlay = team->mShipsRemainingInPlay();

		if( fThisTeamShipsInPlay < 1 )
		{
			fThisPlayerDisengaged = true;
		}
	}

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			// at least one freighter managed to disengage
			if (!gWC.mOTDDestroyed()) gWC.fOTDDisengaged = true;		
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( 25 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( 25 );

			else if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
