/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_7BaseAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{
	fNumShips = 0;
}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
   }
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tOTDTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		ShipList freighter_sl = gST.narrowed_sl;
		eRaceName race = mGetRace();

		std::list< eRace >  raceList;
		raceList.push_back(race);
		fShiplistCount = freighter_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", race, fShiplistCount);

		// CARGO role
		std::list< eEnhancedRoles >  CargoList;
		CargoList.push_back(kCargo);
		fShiplistCount = freighter_sl.Subset(CargoList);
		gST.mDebug_fprintf("subset for Cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		// Ships only! no bases.
		std::list< eEnhancedClassTypes >  enhancedClassTypeListShip;
		for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
			enhancedClassTypeListShip.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = freighter_sl.Subset(enhancedClassTypeListShip);
		gST.mDebug_fprintf("subset for ships-only filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(freighter_sl);

		// s1 is fShiptype: already chosen through draft.
		Ship s2;
		if ((freighter_sl.size() > 0) && (freighter_sl.SelectRandomShip(s2))) {
			gST.mDebug_fprintf("Ship2 selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
			gST.mTrackOrions( s2 );
			mCreateShip( typeid( tOTD ),
					s2.shipCode.c_str(), 
					kStartPosition_X,
					kNoMetaShipID,
					kOTD, 0, 0, -1, -1,
					"Convoy X",
					kDeathUponCapture );


			// keep only freighters.
			std::list< eEnhancedClassTypes >  enhancedClassTypeListF;
			enhancedClassTypeListF.push_back(kEClassSmallFreighter);
			enhancedClassTypeListF.push_back(kEClassPriorityTransport);
			enhancedClassTypeListF.push_back(kEClassCivilianFreighter);
			enhancedClassTypeListF.push_back(kEClassLargeFreighter);
			enhancedClassTypeListF.push_back(kEClassHeavyFreighter);
			fShiplistCount = freighter_sl.Subset(enhancedClassTypeListF);
			gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

			// no Qship role for this ship
			std::list< eEnhancedRoles >  qshipList;
			qshipList.push_back(kQship);
			fShiplistCount = freighter_sl.Subset(qshipList, false);
			gST.mDebug_fprintf("subset for no qship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

			gST.listShiplistContent(freighter_sl);

			// replacement for duplicated drafted ship
			Ship s1;
			if ((freighter_sl.size() > 0) && (freighter_sl.SelectRandomShip(s1))) {
				gST.mDebug_fprintf("Ship1 (replacement) selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip( 
					typeid( tOTD ),
					s1.shipCode.c_str(), 
					kStartPosition_W,
					kNoMetaShipID,
					kOTD, 0, 0, -1, -1,
					"Convoy W",
					kDeathUponCapture );
			} else {
				gST.mDebug_fprintf("Failed to select s1 (replacement) cargo ship. Not a big problem. Using drafted ship.\n");
				// fShiptype is drafted.
				mCreateShip( typeid( tOTD ),
					fShiptype.c_str(), 
					kStartPosition_W,
					kNoMetaShipID,
					kOTD, 0, 0, -1, -1,
					"Convoy W",
					kDeathUponCapture );
			}

			// if it's a really big cargo ship, ie tug, do not draft another.
			if (s2.bpv < 80) {
				Ship s3;
				if ((freighter_sl.size() > 0) && (freighter_sl.SelectRandomShip(s3))) {
					gST.mDebug_fprintf("Ship3 selected! %s:  %d BPV\n", s3.shipCode.c_str(), s3.bpv);
					gST.mTrackOrions( s3 );
					mCreateShip( typeid( tOTD ),
							s3.shipCode.c_str(), 
							kStartPosition_Y,
							kNoMetaShipID,
							kOTD, 0, 0, -1, -1,
							"Convoy Y",
							kDeathUponCapture );
				} else {
					gST.mDebug_fprintf("Failed to select s3 cargo ship. Not a big problem. Moving on.\n");
				}
			} else {
				gST.mDebug_fprintf("Not selecting a third cargo ship. s2 was big enough.\n");
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	switch (mGetRace()) {
		case kFederation:
			fShiptype2 = "F-FSPh";
			fShiptype3 = "F-FLPh";
			break;
		case kKlingon:
			fShiptype2 = "K-FSPh";
			fShiptype3 = "K-FLPh";
			break;
		case kGorn:
			fShiptype2 = "G-FSPh";
			fShiptype3 = "G-FLPh";
			break;
		case kRomulan:
			fShiptype2 = "R-FSPh";
			fShiptype3 = "R-FLPh";
			break;
		case kLyran:
			fShiptype2 = "L-FSPh";
			fShiptype3 = "L-FLPh";
			break;
		case kMirak:
			fShiptype2 = "Z-FSPh";
			fShiptype3 = "Z-FLPh";
			break;
		case kISC:
			fShiptype2 = "I-FSPh";
			fShiptype3 = "I-FLPh";
			break;
		case kHydran:
			fShiptype2 = "H-FSPh";
			fShiptype3 = "H-FLPh";
			break;
		default:
			fShiptype2 = "F-FSPh";
			fShiptype3 = "K-FLPh";
			break;
	}

	// fShiptype is drafted.
	mCreateShip( typeid( tOTD ),
		fShiptype.c_str(), 
		kStartPosition_W,
		kNoMetaShipID,
		kOTD, 0, 0, -1, -1,
		"Convoy W",
		kDeathUponCapture );

	mCreateShip( typeid( tOTD ),
			fShiptype2.c_str(), 
			kStartPosition_X,
			kNoMetaShipID,
			kOTD, 0, 0, -1, -1,
			"Convoy X",
			kDeathUponCapture );

	mCreateShip( typeid( tOTD ),
			fShiptype3.c_str(), 
			kStartPosition_Y,
			kNoMetaShipID,
			kOTD, 0, 0, -1, -1,
			"Convoy Y",
			kDeathUponCapture );

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}


void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed.\n");
		metaid = kNoMetaShipID;
	}
#endif

	int32 TeamRandValue = 1;
	if (gGetMapPicked() > 1) TeamRandValue *= gGetMapPicked();
	if (mGetTeam() > 1) TeamRandValue *= mGetTeam();
	if (mGetCombatBPV() > 1) TeamRandValue *= mGetCombatBPV();
	if (gWC.fMissionSeed > 1) TeamRandValue *= gWC.fMissionSeed;
	if (metaid > 1) TeamRandValue *= metaid;
	if (mShipsCurrentlyControlled() > 1) TeamRandValue *= mShipsCurrentlyControlled();
	if (TeamRandValue < 0) TeamRandValue *= -1;
	TeamRandValue = TeamRandValue % 100;

	if (fNumShips == 0) {
		fShiptype = shipScriptDescription.fClassName;
		fShiptype2 = fShiptype;
		fShiptype3 = fShiptype;
	} else if (fNumShips == 1) fShiptype2 = shipScriptDescription.fClassName;
	else fShiptype3 = shipScriptDescription.fClassName;
	fNumShips++;

	mCreateShip( typeid( tOTD ),
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_Z,
		metaid,
		kOTD, 0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		kDefaultShipOptions );

   	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

eClassTypes tOTDTeam::mTranslateHexIntoType( tMetaLocationSpec hex )
{
	eClassTypes returnClass = kClassStarBase;

	if ( hex.fLocationObjectFlags & kBaseStationHex )
		returnClass = kClassBaseStation;
	else if ( hex.fLocationObjectFlags & kBattleStationHex )
		returnClass = kClassBattleStation;
	else if ( hex.fLocationObjectFlags & kStarbaseHex )
		returnClass = kClassStarBase;

	return returnClass;
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}