/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_7BaseAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tOTDTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

	std::string shiptype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		ShipList OTDTeam_sl = gST.narrowed_sl;

		std::list< eRace >  raceList;
		raceList.push_back(mGetRace());
		fShiplistCount = OTDTeam_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

		// Defense platforms
		ShipList DEF_sl = OTDTeam_sl;
		std::list< eEnhancedClassTypes >  DEFClassTypeList;
		DEFClassTypeList.push_back(kEClassPlatform);
		fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
		gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);
		std::list< eEnhancedRoles >  DEFRolesList;
		DEFRolesList.push_back(kLocalDefense);
		fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
		gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(DEF_sl);

		if  (DEF_sl.size() > 0) {
			int32 skipchance = 5;
			if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
				tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
				if (drafter) { 
					if (drafter->mGetRace() == kLyran) skipchance = 1;
					else if (drafter->mGetRace() == kGorn) skipchance = 2;
					else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
					else skipchance = 4;
				}
			}

			Ship s1;

			for (i = 0; i < 4; i++) {
				if ((rand() % 5) < skipchance) {
					DEF_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					mCreateShip( typeid( tShipInfo ), 
						s1.shipCode.c_str(), 
						hexes[i], 
						kNoMetaShipID, 
						-1, 
						0, 0, 
						-1, -1,	
						"", 
						kDefaultShipOptions);
				}
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
    gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string shiptype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };


	switch (mGetRace()) {
		case kFederation: shiptype = "F-DEF"; break;
		case kGorn: shiptype = "G-DEF"; break;
		case kHydran: shiptype = "H-DEF"; break;
		case kISC: shiptype = "I-DEF"; break;
		case kKlingon: shiptype = "K-DEF"; break;
		case kLyran: shiptype = "L-DEF"; break;
		case kMirak: shiptype = "Z-DEF"; break;
		case kRomulan: shiptype = "R-DEF"; break;
		default: // used for both Orion and neutral/monster
			shiptype = "O-DEF"; break;
	}
	int32 skipchance = 5;
	if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
		tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
		if (drafter) { 
			if (drafter->mGetRace() == kLyran) skipchance = 1;
			else if (drafter->mGetRace() == kGorn) skipchance = 2;
			else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
			else skipchance = 4;
		}
	}
	for (i = 0; i < 4; i++) {
		if ((rand() % 5) < skipchance) 
			mCreateShip( typeid( tShipInfo ), 
				shiptype.c_str(), 
				hexes[i], 
				kNoMetaShipID, 
				-1, 
				0, 0, 
				-1, -1,	
				"", 
				kDefaultShipOptions);
	}
    gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}


void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// drone speed workaround for drafted ships
	int32 metaid = shipScriptDescription.fMetaDatabaseID;
 #ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
		gWC.fBaseMetaID = shipScriptDescription.fMetaDatabaseID;
	} else {
		gWC.fBaseMetaID = kNoMetaShipID;
	}
 #endif
 
	int32 TeamRandValue = 1;
	if (gGetMapPicked() > 1) TeamRandValue *= gGetMapPicked();
	if (mGetTeam() > 1) TeamRandValue *= mGetTeam();
	if (mGetCombatBPV() > 1) TeamRandValue *= mGetCombatBPV();
	if (gWC.fMissionSeed > 1) TeamRandValue *= gWC.fMissionSeed;
	if (metaid > 1) TeamRandValue *= metaid;
	if (mShipsCurrentlyControlled() > 1) TeamRandValue *= mShipsCurrentlyControlled();
	if (TeamRandValue < 0) TeamRandValue *= -1;
	TeamRandValue = TeamRandValue % 100;

	mCreateShip( 
		typeid( tOTD ),
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_Z,
		metaid,
		kOTD, 0, 0, -1, -1,
		"",
		kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

eClassTypes tOTDTeam::mTranslateHexIntoType( tMetaLocationSpec hex )
{
	eClassTypes returnClass = kClassStarBase;

	if ( hex.fLocationObjectFlags & kBaseStationHex )
		returnClass = kClassBaseStation;
	else if ( hex.fLocationObjectFlags & kBattleStationHex )
		returnClass = kClassBattleStation;
	else if ( hex.fLocationObjectFlags & kStarbaseHex )
		returnClass = kClassStarBase;

	return returnClass;
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}