/********************************************************************************************************\

	Mission:	Met_8ShipAssault
	Script:		"Met_8ShipAssault"
	Descript:	Fleet attempts to take over base

	Created by:		Daniel J. Suleski		07/06/00
	Last Updated:	DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta


\********************************************************************************************************/

#include "AttackTeam.h"
#include "DefendTeam.h"
#include "Met_8ShipAssault.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "OTDTeam.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_8ShipAssault );

tMet_8ShipAssault::tMet_8ShipAssault()
{
	srand(time(NULL));
}

void tMet_8ShipAssault::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAnyBaseHex);
	Specs->fWait	= 2;
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase;


	const tTeamSpec& team1 = mCreateTeamSpec(	"Attack Team 1",
												static_cast< eTeamID >( kAttackTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team 2",
												static_cast< eTeamID >( kAttackTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Attack Team 3",
												static_cast< eTeamID >( kAttackTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kAttackTeam1,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ),
												Messages[kTeamTitleAttack_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team 1",
												static_cast< eTeamID >( kDefendTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Defend Team 2",
												static_cast< eTeamID>( kDefendTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Defend Team 3",
												static_cast< eTeamID >( kDefendTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tDefendTeam ),
												static_cast< eTeamTag >( kDefendTag ),
												Messages[kTeamTitleDefend_msg] );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Shipyard Team",
												static_cast< eTeamID >( kShipyardTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kAttackTeam1,
												typeid( tOTDTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}

int32 tMet_8ShipAssault::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tMet_8ShipAssault::mCreateExternalShips( int32 Mode )
{}

void tMet_8ShipAssault::mNewInternalShip( tShipInfo* Ship )
{}

void tMet_8ShipAssault::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Shipyard assault");
	gWC.mCheckCRC();

	const int kRelationsCount = 7;
	int32 teamList[kRelationsCount] =
	{
		kAttackTeam1,
		kAttackTeam2,
		kAttackTeam3,
		kDefendTeam1,
		kDefendTeam2,
		kDefendTeam3,
		kShipyardTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  PTm4  PTm5  PTm6  StTm
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "WD",
		/*PTm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*PTm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*PTm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
		/*StTm*/ "WD", "WD", "WD", "AI", "AI", "AI", "AI",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

tMapTerrain tMet_8ShipAssault::mAssignMapTerrainTypes( int32 Map )
{

	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetNone, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula, kPlanetNone, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_8ShipAssault::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectShipEnd;
	Map->fTacticalMapIcon['X'] = kObjectShipEnd;
	Map->fTacticalMapIcon['Y'] = kObjectShipEnd;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_8ShipAssault::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kAttackTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingAttack_msg;
		}
		for ( i = kDefendTeam1; i <= kDefendTeam3; i++ )
		{
			fBriefingIndex[i][j] = kTeamBriefingDefend_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_8ShipAssault::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.55f, 1.0f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.525f, 0.95f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.5f, 0.9f );

	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 75l, 120l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 80l, 125l );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassLightCruiser, 85l, 130l );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.6f, 1.05f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.625f, 1.075f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.65f, 1.1f );
}


//
//	The following two functions determine the size and race of the shipyards and the race that should be
//	defending it.
//
iMissSpecRace tMet_8ShipAssault::mTranslateHexIntoRace( tMetaLocationSpec hex )
{
	iMissSpecRace returnRace = kSpecPlayableRace;

	if ( hex.fLocationWhoHexFlags & kFederationRaceHex ) {
		returnRace = kSpecFed;
	} else if ( hex.fLocationWhoHexFlags & kKlingonRaceHex ) {
		returnRace = kSpecKling;
	} else if ( hex.fLocationWhoHexFlags & kRomulanRaceHex ) {
		returnRace = kSpecRomulan;
	} else if ( hex.fLocationWhoHexFlags & kGornRaceHex ) {
		returnRace = kSpecGorn;
	} else if ( hex.fLocationWhoHexFlags & kLyranRaceHex ) {
		returnRace = kSpecLyran;
	} else if ( hex.fLocationWhoHexFlags & kHydranRaceHex ) {
		returnRace = kSpecHydran;
	} else if ( hex.fLocationWhoHexFlags & kMirakRaceHex ) {
		returnRace = kSpecMirak;
	} else if ( hex.fLocationWhoHexFlags & kISCRaceHex ) {
		returnRace = kSpecISC;
	}

	return returnRace;
}
