/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_8ShipAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"
#include "worldconditions.h"
#include "ShipScriptDescription.h"

tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

void tOTDTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		tTeamInfo* defendTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam1 ) );
		eRace race = defendTeam->mGetRace();

		ShipList FRD_sl = gST.narrowed_sl;

		std::list< eRace >  raceList;
		raceList.push_back(race);
		fShiplistCount = FRD_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", race, fShiplistCount);

		std::list< eEnhancedClassTypes >  FRDClassTypeList;
		FRDClassTypeList.push_back(kEClassFleetRepairDock);
		fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
		gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(FRD_sl);

		if (FRD_sl.size() > 0) {
		eShipOptions options = static_cast< eShipOptions >( kStartPositionOffsetOrigin | kStartPositionCanBeOffset );//| kDeathUponCapture );
			for ( int32 i = 0; i < kNumFRDs; i++ )
			{
				Ship s1;
				FRD_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip(	
					typeid( tOTD ), 
					s1.shipCode.c_str(), 
					kStartPosition_W + i, 
					kNoMetaShipID,		
					kOTD, 
					0, 0, -1, -1, NULL, 
					options );
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
   	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
   	gST.mDebug_fprintf("---- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string shipyard = "";
	tTeamInfo* defendTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam1 ) );
	eRace race = defendTeam->mGetRace();

	switch ( race )
	{
	case kFederation:	shipyard = "F-FRD";		break;
	case kKlingon:		shipyard = "K-FRD";		break;
	case kRomulan:		shipyard = "R-FRD";		break;
	case kLyran:		shipyard = "L-FRD";		break;
	case kGorn:			shipyard = "G-FRD";		break;
	case kHydran:		shipyard = "H-FRD";		break;
	case kISC:			shipyard = "I-FRD";		break;
	case kMirak:		shipyard = "Z-FRD";		break;
	default:			shipyard = "O-FRD";		break;
	}

	eShipOptions options = static_cast< eShipOptions >( kStartPositionOffsetOrigin | kStartPositionCanBeOffset );//| kDeathUponCapture );
	for ( int32 i = 0; i < kNumFRDs; i++ )
	{
		mCreateShip(	typeid( tOTD ), shipyard.c_str(), kStartPosition_W + i, kNoMetaShipID,
						kOTD, 0, 0, -1, -1, NULL, options );
	}
   	gST.mDebug_fprintf("---- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}

void tOTDTeam::mEmerCreateShipsForTeam()
// used if admins eliminate the stock FRDs from the shiplist,
// substitutes the smallest bases available instead
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mEmerCreateShipsForTeam\n");
	int32 minbpv = 1;

	for (int32 i = 0; i < kNumFRDs; i++) {
		mCreateShip( typeid( tOTD ),
						minbpv,
						kClassListeningPost,
						kClassBaseStation, 
						0, 
						kStartPosition_W + i,
						-1, 0, 0, -1, -1, 
						NULL, 
						static_cast<eShipOptions>( kStartPositionCanBeOffset | kDeathUponCapture ) );
	}
	gST.mDebug_fprintf("--- Finished: tOTDTeam::mEmerCreateShipsForTeam\n");
}

void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}
