/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_9PlanetaryAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "BaseVictory.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{
	int32 shipMetaID = -1;
	std::string shiptype = "";
	std::string shipname = "";
}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tOTDTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
   }
}

void tOTDTeam::mSetupCommState()
{}

void tOTDTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();


	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
		int32 fShiplistCount = 0;
		ShipList planet_sl = gST.narrowed_sl;
		eRaceName race = mGetRace();

		planet_sl = gST.mFilterShipListToSingleRace( planet_sl, race );

		std::list< eEnhancedClassTypes >  planetClassTypeList;
		planetClassTypeList.push_back(kEClassPlanet);
		fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
		gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  planetRolesList;
		switch (gWC.fMapPicked) {
		case 8: 
			gST.mDebug_fprintf("Map picked was for an armed world.\n");
			planetRolesList.push_back(kArmedWorld);
			fShiplistCount = planet_sl.Subset(planetRolesList);
			break;
		case 9: 
			gST.mDebug_fprintf("Map picked was for a homeworld.\n");
			planetRolesList.push_back(kHomeWorld);
			fShiplistCount = planet_sl.Subset(planetRolesList);
			break;
		default:
			gST.mDebug_fprintf("Map picked was for a colony world.\n");
			planetRolesList.push_back(kArmedWorld);
			planetRolesList.push_back(kHomeWorld);
			fShiplistCount = planet_sl.Subset(planetRolesList, false);
			break;
		}
		gST.mDebug_fprintf("subset for planet eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		{
		SHIP_LIST_ITER(planet_sl)
			std::string badplanet = "";
			switch (race) {
				case kKlingon: badplanet = "K-PE"; break;
				case kRomulan: badplanet = "R-PE"; break;
				case kLyran: badplanet = "L-PE"; break;
				case kGorn: badplanet = "G-PE"; break;
				case kHydran: badplanet = "H-PE"; break;
				case kMirak: badplanet = "Z-PE"; break;
				case kISC: badplanet = "I-PE"; break;
				default: break;
			}
			for (;;) {
				if (gST.mGetModelSize(iter->geometry) > 151) {
					iter = planet_sl.erase(iter);
					if (iter != planet_sl.end()) continue; else break;
				}
				if (badplanet.compare(iter->shipCode) == 0) {
					iter = planet_sl.erase(iter);
					if (iter != planet_sl.end()) continue; else break;
				}
				break;
			}
		SHIP_LIST_ITER_END
		}
		gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
		gST.listShiplistContent(planet_sl);



		ShipList OTDTeam_sl = gST.narrowed_sl;
		OTDTeam_sl = gST.mFilterShipListToSingleRace( OTDTeam_sl, race );

		// Defense platforms
		ShipList DEF_sl = OTDTeam_sl;
		std::list< eEnhancedClassTypes >  DEFClassTypeList;
		DEFClassTypeList.push_back(kEClassPlatform);
		fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
		gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);
		std::list< eEnhancedRoles >  DEFRolesList;
		DEFRolesList.push_back(kLocalDefense);
		fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
		gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(DEF_sl);

		// FRDs
		ShipList FRD_sl = OTDTeam_sl;
		std::list< eEnhancedClassTypes >  FRDClassTypeList;
		FRDClassTypeList.push_back(kEClassFleetRepairDock);
		fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
		gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(FRD_sl);

		if ( (DEF_sl.size() > 0) && (FRD_sl.size() > 0) && (planet_sl.size() > 0) ) {
			// create planet
			Ship p1;
			planet_sl.SelectRandomShip(p1);
			gST.mDebug_fprintf("Planet selected! %s:  %d BPV\n", p1.shipCode.c_str(), p1.bpv);

#ifdef OP  // see below for the planet selection during draft.
			mCreateShip(	
				typeid( tOTD ),
				p1.shipCode.c_str(), 
				kStartPosition_Z,
				shipMetaID,
				kOTD, 0, 0, -1, -1,
				shipname.c_str(),
				static_cast< eShipOptions >( kDefaultShipOptions ) );
#endif

			int32 skipchance = 5;
			if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
				tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
				if (drafter) {
					if (drafter->mGetRace() == kLyran) skipchance = 1;
					else if (drafter->mGetRace() == kGorn) skipchance = 2;
					else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
					else skipchance = 4;
				}
			}

			Ship s1;
			for (i = 0; i < 4; i++) {
				if ((i % 2) == 0) {
				  if ((rand() % 5) < skipchance) {
					DEF_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("DEF Platform selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
					gST.mTrackOrions( s1 );
					mCreateShip( typeid( tShipInfo ),
					  s1.shipCode.c_str(),
					  hexes[i],
					  kNoMetaShipID,
					  -1,
					  0, 0,
					  -1, -1,
					  "",
					  kDefaultShipOptions);
				  }
				} else {
				  if ((rand() % 5) < skipchance) {
					FRD_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
					gST.mTrackOrions( s1 );
					mCreateShip( typeid( tShipInfo ),
					  s1.shipCode.c_str(),
					  hexes[i],
					  kNoMetaShipID,
					  -1,
					  0, 0,
					  -1, -1,
					  "",
					  kDefaultShipOptions);
				  }
				}
			}

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

    gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}


void tOTDTeam::mOldCreateShipsForTeam()
{
   	gST.mDebug_fprintf("---- Entering: tOTDTeam::mOldCreateShipsForTeam\n");
	
#ifdef OP // see below for planet selection during draft.
	mCreateShip(	
		typeid( tOTD ),
		shiptype.c_str(), 
		kStartPosition_Z,
		shipMetaID,
		kOTD, 0, 0, -1, -1,
		shipname.c_str(),
		static_cast< eShipOptions >( kDefaultShipOptions ) );
#endif

	std::string shiptype, frdtype;
	int32 i;
	iMapPosition hexes[4] = { kStartPosition_T,
								kStartPosition_U,
								kStartPosition_V,
								kStartPosition_S, };

	switch (mGetRace()) {
		case kFederation: shiptype = "F-DEF"; frdtype = "F-FRD"; break;
		case kGorn: shiptype = "G-DEF";  frdtype = "G-FRD";break;
		case kHydran: shiptype = "H-DEF";  frdtype = "H-FRD";break;
		case kISC: shiptype = "I-DEF";  frdtype = "I-FRD";break;
		case kKlingon: shiptype = "K-DEF";  frdtype = "K-FRD";break;
		case kLyran: shiptype = "L-DEF";  frdtype = "L-FRD";break;
		case kMirak: shiptype = "Z-DEF";  frdtype = "Z-FRD";break;
		case kRomulan: shiptype = "R-DEF";  frdtype = "R-FRD";break;
		default: // used for both Orion and neutral/monster
			shiptype = "O-DEF";  frdtype = "O-FRD";break;
	}
	int32 skipchance = 5;
	if ((gWC.fDraftingHumans < 2) && (gWC.fDraftedHumans < 1)) {
		tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
		if (drafter) { 
			if (drafter->mGetRace() == kLyran) skipchance = 1;
			else if (drafter->mGetRace() == kGorn) skipchance = 2;
			else if ((drafter->mGetRace() == kISC) || (drafter->mGetRace() == kRomulan)) skipchance = 3;
			else skipchance = 4;
		}
	}
	for (i = 0; i < 4; i++) {
		if ((i % 2) == 0) {
			if ((rand() % 5) < skipchance)
				mCreateShip( typeid( tShipInfo ), 
					shiptype.c_str(), 
					hexes[i], 
					kNoMetaShipID, 
					-1, 
					0, 0, 
					-1, -1,	
					"", 
					kDefaultShipOptions);
		} else {
			if ((rand() % 5) < skipchance)
				mCreateShip( typeid( tShipInfo ), 
					frdtype.c_str(), 
					hexes[i], 
					kNoMetaShipID, 
					-1, 
					0, 0, 
					-1, -1,	
					"", 
					kDefaultShipOptions);
		}
	}

   	gST.mDebug_fprintf("---- Entering: tOTDTeam::mOldCreateShipsForTeam\n");
}


void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

#ifdef EAW
	// This code here exists to make things easier for Lyrans in SFC-EAW
	// Lyrans in SFC-OP are .. well.. more dangerous: maulers. PFs.
	if ((gWC.fDraftedHumans < 1) && (! gST.global_sl.mIsEnhancedShiplist())) {
		tTeamInfo* drafter = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ));
		if (drafter) {
			if (drafter->mGetRace() == kLyran) {
				std::string shipname = "PL0";
				bool upgrade = false;
				if ((rand() % 19) > 13) upgrade = true;
				switch (mGetRace()) {
					case kFederation:
						if (upgrade) shipname = "F-PLPh0";
						else shipname = "F-PL0";
						break;
					case kKlingon:
						if (upgrade) shipname = "K-PLPh0";
						else shipname = "K-PL0";
						break;
					case kRomulan:
						if (upgrade) shipname = "R-PLPh0";
						else shipname = "R-PL0";
						break;
					case kLyran:
						if (upgrade) shipname = "L-PLPh0";
						else shipname = "L-PL0";
						break;
					case kHydran:
						if (upgrade) shipname = "H-PLPh0";
						else shipname = "H-PL0";
						break;
					case kISC:
						if (upgrade) shipname = "I-PLPh0";
						else shipname = "I-PL0";
						break;
					case kGorn:
						if (upgrade) shipname = "G-PLPh0";
						else shipname = "G-PL0";
						break;
					case kMirak:
						if (upgrade) shipname = "Z-PLPh0";
						else shipname = "Z-PL0";
						break;
					default:
						break;
				}
				gST.mDebug_fprintf("Lyran primary attacker detected. Making it easier. Planet: %s\n", shipname.c_str());
				mCreateShip(	typeid( tOTD ),
					shipname.c_str(), 
					kStartPosition_Z,
					shipScriptDescription.fMetaDatabaseID,
					kOTD, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

				gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());

				return;
			}
		}
	}
#endif // EAW

	shipMetaID = shipScriptDescription.fMetaDatabaseID;
	shiptype = shipScriptDescription.fClassName.c_str();
	shipname = "";

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}