/********************************************************************************************************\

	File Name:	AttackTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AttackShip.h"
#include "AttackTeam.h"
#include "AttackTeamBaseState.h"
#include "BaseVictory.h"
#include "Met_9PlanetaryAssault.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tAttackTeam> gAttackTeamAllocator;

tAttackTeam::tAttackTeam(): fCustomID( kAttacker1 ), fNumShips ( 0 )
{
	gWC.mIncrementTotalAttackTeams();
}

void tAttackTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinnerAttack_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoserAttack_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tAttackTeam::mSetupState()
{
	mRegisterState( new tAttackTeamBaseState() );
	mGotoState( typeid( tAttackTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tAttackTeam::mSetupCommState()
{}


void tAttackTeam::mCreateShipsForTeam()
{
#ifdef EAW
    gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if (mGetRace() == kLyran) {
        if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
            int32 fShiplistCount = 0;
            ShipList basebuster_sl = gST.narrowed_sl;

            std::list< eRace >  raceList;
            raceList.push_back(kOrion);
            fShiplistCount = basebuster_sl.Subset(raceList);
            gST.mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", mGetRace(), fShiplistCount);

            std::list< eEnhancedClassTypes >  CAClassTypeList;
            CAClassTypeList.push_back(kEClassMediumCruiser);
            CAClassTypeList.push_back(kEClassHeavyCruiser);
            CAClassTypeList.push_back(kEClassNewHeavyCruiser);
            fShiplistCount = basebuster_sl.Subset(CAClassTypeList, true);
            gST.mDebug_fprintf("subset for CA class filter completed. ships: %d\n", fShiplistCount);

            std::list< eEnhancedRoles >  CARolesList;
            CARolesList.push_back(kLineShip);
            CARolesList.push_back(kWarShip);
            CARolesList.push_back(kLeader);
            CARolesList.push_back(kDroneBombardmentShip);
            CARolesList.push_back(kEscort);
            CARolesList.push_back(kCarrier);
            CARolesList.push_back(kPFTender);
            CARolesList.push_back(kFastCruiser);
            CARolesList.push_back(kMauler);
            CARolesList.push_back(kPolice);
            fShiplistCount = basebuster_sl.Subset(CARolesList, true);
            gST.mDebug_fprintf("subset for CA eEnhancedRoles completed. ships: %d\n", fShiplistCount);
            gST.listShiplistContent(basebuster_sl);

            Ship s1;
            if ( (basebuster_sl.size() > 0) && (basebuster_sl.SelectRandomShip(s1)) ) {
				gST.mDebug_fprintf("Hired Gun selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
                mCreateShip( typeid( tShipInfo ),
                    s1.shipCode.c_str(),
                    kStartPosition_G,
                    kNoMetaShipID,
                    -1,
                    0, 0,
                    -1, -1,
                    "Hired Gun",
                    static_cast< eShipOptions >( kDefaultShipOptions ) );
            } else {
                mOldCreateShipsForTeam();
            }
        } else {
            mOldCreateShipsForTeam();
        }
    }
    gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateShipsForTeam\n");
#endif
}



void tAttackTeam::mOldCreateShipsForTeam()
{
#ifdef EAW
	gST.mDebug_fprintf("--- Entering: tAttackTeam::mOldCreateShipsForTeam\n");

	if (mGetTeam() == kAttackTeam1) {
		if (mGetRace() == kLyran) {
			mCreateShip( typeid( tShipInfo ), 
					"O-CA2", 
					kStartPosition_G, 
					kNoMetaShipID, 
					-1, 
					0, 0,
					-1, -1,
					"Hired Gun", 
					static_cast< eShipOptions >( kDefaultShipOptions ) ); 
		}
	}
	gST.mDebug_fprintf("--- Finished: tAttackTeam::mOldCreateShipsForTeam\n");
#endif
}

void tAttackTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	if ( mIsTeamHumanControlled() )
	{	
		tShipIterator ship = mGetFirstShip();

		Assert ( *ship != NULL );

		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	int32 metaid = shipScriptDescription.fMetaDatabaseID;
	eShipOptions option = kDefaultShipOptions;
	if (mIsTeamHumanControlled()) {
		option = kDeathUponCapture;
		gWC.fAttackingHumans++;
#ifdef ERA_BASED_DRONE_SPEEDS
	} else {
		if (fMissionInfo->mGetDifficultyLevel() >= 1) {
			gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
			metaid = kNoMetaShipID;
		}
#endif
	}

	mCreateShip( 
		typeid( tAttackShip ), 
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_G + shipScriptDescription.fTeamID,
		metaid,
		fCustomID, 0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		option );

	fCustomID++;
	fNumShips++;
	gWC.mIncrementTotalAttackShips();
	if (gWC.fAttackingHumans > 1) gWC.fAdj = kFalse;
	if ((mGetRace() != kLyran) && (mGetRace() != kGorn)) gWC.fAdj = kFalse;

   	gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}