/********************************************************************************************************\

	File Name:	AttackTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackTeamBaseState.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include "Met_9PlanetaryAssault.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAttackTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );
	mSetTimedInterval( kTimerTurnsSet, kTooLongID, 25.0f );
	mSetTimedInterval( kTimerTurnsSet, kWarningID, 23.0f );
	mSetTimedInterval( kTimerCountdownSet, kCountDownID, 25.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tAttackTeamBaseState::mEndState()
{}

void tAttackTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tAttackTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tAttackTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tAttackTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	static int lastmsg = 0;
	if ( damagedShip->mGetCustomID() != kOTD )  return;
		if (damagedShip->mGetDamagePercent() < 20.0f) {
			if (lastmsg < 1) {
				lastmsg = 1;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Minor damage has been inflicted on the planet Sir" );
			}
		}
		else if (damagedShip->mGetDamagePercent() < 35.0f) {
			if (lastmsg < 2) {
				lastmsg = 2;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Modest damage has been inflicted on the planet Captain" );
			}
		}
		else if (damagedShip->mGetDamagePercent() < 60.0f) {
			if (lastmsg < 3) {
				lastmsg = 3;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The planet has been subdued, Sir!" );
				gWC.fPlanetSubdued = kTrue;
				gWC.mCheckForGameEnd();
			}
		}
		else if (damagedShip->mGetDamagePercent() < 75.0f) {
			if (lastmsg < 4) {
				lastmsg = 4;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Major damage has been inflicted on the planet Captain - " );
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "they are offering anything to end the attack!" );
			}
		}
		else if (damagedShip->mGetDamagePercent() > 90.0f) {
			if (lastmsg < 5) {
				lastmsg = 5;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Crippling damage has been inflicted on the planet Sir - it" );
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "will be long before they can rebuild their installations." );
			}
		}
	// AI teams should no longer fire on a subdued planet
	if ((gWC.fPlanetSubdued) && (!fChildActor->mIsTeamHumanControlled())) {
		fChildActor->mSetRelations(kTeam7, kTeamNeutral, kGoalIgnore, 0.0f );
	}

}
void tAttackTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tAttackTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAttackTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAttackTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAttackTeamBaseState::mOnFlashingButtonPressed()
{}
void tAttackTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAttackTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tAttackTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tAttackTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tAttackTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tAttackTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tAttackTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tAttackTeamBaseState::mOnHostChanged()
{}
void tAttackTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if ( timerID == kTooLongID ) {
		gWC.fTooLong = kTrue;
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We have taken too long, Captain - now we need to take out their reinforcements before we can safely disengage!", kNonCommScriptMessage );
	} 

	if ( timerID == kWarningID ) {
		gWC.fTooLong = kTrue;
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Enemy reinforcements on long range scanners, Captain - we MUST leave soon!", kNonCommScriptMessage );
	} 

	if ( timerID == kStartingID )
	{
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Captain, we have a short time window to destroy the defenders, bomb", kNonCommScriptMessage );
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "the planet into submission, and leave before the enemy reinforces,", kNonCommScriptMessage );
		eRaceName  playerRace = kNoRace;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fChildActor->mGetTeam() );
		if ( playerTeam )
			playerRace = playerTeam->mGetRace();

		// return the index to the Starting message we want to use
		switch( playerRace )
		{
			case kFederation:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kFedStartAttack_msg, kNonCommScriptMessage );
			break;
			case kKlingon:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kKliStartAttack_msg, kNonCommScriptMessage );
			break;
			case kRomulan:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kRomStartAttack_msg, kNonCommScriptMessage );
			break;
			case kLyran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kLyrStartAttack_msg, kNonCommScriptMessage );
			break;
			case kGorn:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kGorStartAttack_msg, kNonCommScriptMessage );
			break;
			case kMirak:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kMirStartAttack_msg, kNonCommScriptMessage );
			break;
			case kHydran:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kHydStartAttack_msg, kNonCommScriptMessage );
			break;
			case kISC:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kISCStartAttack_msg, kNonCommScriptMessage );
			break;
			default:
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),   kStartAttack_msg, kNonCommScriptMessage );
			break;
		}
	}

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;
        if (! gScriptInterface.mIsHost()) return;

        int32 fTeamID = fChildActor->mGetTeam();
        tTeamInfo* attackTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fTeamID ) );
        if ((fTeamID != kAttackTeam1) && (fTeamID != kAttackTeam2) && (fTeamID != kAttackTeam3)) return;
        if (attackTeam->mIsTeamHumanControlled()) return;

        gST.mDebug_fprintf("--- Entering tAttackTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        gST.fTeam1DeltaFighterBPV += gST.mUpgradeFightersForTeam(attackTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam1DeltaFighterBPV);
        gST.mDebug_fprintf("--- Finished: tAttackTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }

}
void tAttackTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAttackTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

