/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_9PlanetaryAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "BaseVictory.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
   }
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}


void tOTDTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

	eRace race = mGetRace();
	int32 mapChoice = gWC.fMapPicked;

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;
   		ShipList planet_sl = gST.mFilterShipListToSingleRace(gST.narrowed_sl, race);

		std::list< eEnhancedClassTypes >  planetClassTypeList;
		planetClassTypeList.push_back(kEClassPlanet);
		fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
		gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  planetRolesList;
		switch (mapChoice) {
			case 2: // homeworld hex
				gST.mDebug_fprintf("Selecting planet homeworld role.\n");
				planetRolesList.push_back(kHomeWorld);
				fShiplistCount = planet_sl.Subset(planetRolesList, true);
				break;
			case 1: // coreworld hex
				gST.mDebug_fprintf("Selecting planet armed world role.\n");
				planetRolesList.push_back(kArmedWorld);
				fShiplistCount = planet_sl.Subset(planetRolesList, true);
				break;
			default: // colony world hex
				gST.mDebug_fprintf("Filtering out homeworlds and armed worlds: colony planet.\n");
				planetRolesList.push_back(kHomeWorld);
				planetRolesList.push_back(kArmedWorld);
				fShiplistCount = planet_sl.Subset(planetRolesList, false);
				break;
		}
		gST.mDebug_fprintf("subset for planet eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		{
		SHIP_LIST_ITER(planet_sl)
			std::string badplanet = "";
			switch (race) {
				case kKlingon: badplanet = "K-PE"; break;
				case kRomulan: badplanet = "R-PE"; break;
				case kLyran: badplanet = "L-PE"; break;
				case kGorn: badplanet = "G-PE"; break;
				case kHydran: badplanet = "H-PE"; break;
				case kMirak: badplanet = "Z-PE"; break;
				case kISC: badplanet = "I-PE"; break;
				default: break;
			}
			for (;;) {
				if (gST.mGetModelSize(iter->geometry) > 151) {
					iter = planet_sl.erase(iter);
					if (iter != planet_sl.end()) continue; else break;
				}
				if (badplanet.compare(iter->shipCode) == 0) {
					iter = planet_sl.erase(iter);
					if (iter != planet_sl.end()) continue; else break;
				}
				break;
			}
		SHIP_LIST_ITER_END
		}
		gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
		gST.listShiplistContent(planet_sl);


		Ship p1;
		if ((planet_sl.size() > 0) && (planet_sl.SelectRandomShip(p1))) {
			gST.mDebug_fprintf("planet selected! %s:  %d BPV\n", p1.shipCode.c_str(), p1.bpv);
			gST.mTrackOrions( p1 );

			mCreateShip( 
				typeid( tOTD ),
				p1.shipCode.c_str(),
				kStartPosition_Z, 
				kNoMetaShipID,
				kOTD, 0, 0, 0, 0, "",
				kCanAddToFleetUponCapture );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateShipsForTeam\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOTDTeam::mOldCreateShipsForTeam\n");

	std::string planettype;
	eRace selectedrace = mGetRace();
	std::string planet = "";
	int32 mapChoice = gWC.fMapPicked;

	switch (mGetRace()) {
		case kFederation: planet = "F-PL"; break;
		case kKlingon: planet = "K-PL"; break;
		case kRomulan: planet = "R-PL"; break;
		case kLyran: planet = "L-PL"; mapChoice--; break;
		case kGorn: planet = "G-PL"; break;
		case kHydran: planet = "H-PL"; break;
		case kMirak: planet = "Z-PL"; break;
		case kISC: planet = "I-PL"; break;
		default: planet = "O-PL"; break;
	}

	switch (mapChoice) {
		case 2: // homeworld hex
			switch (mGetRace()) {
				case kFederation: planet = "F-PE"; break;
				case kKlingon: planet = "K-PKZ"; break;
				case kRomulan: planet = "R-PRO"; break;
				case kLyran: planet = "L-PLY"; break;
				case kGorn: planet = "G-PG1"; break;
				case kHydran: planet = "H-PHY"; break;
				case kMirak: planet = "Z-PM1"; break;
				case kISC: planet = "I-PI1"; break;
				default: planet = "O-PE"; break;
			}
			break;
		case 1: // coreworld hex
			planet += "Ph";
			switch (gWC.fMissionSeed % 4) {
				case 1: planet += "1"; break;
				case 2: planet += "7"; break;
				case 3: planet += "16"; break;
				default: planet += "13"; break;
			}
			break;
		default: // colony world hex
			switch (gWC.fMissionSeed % 4) {
				case 1: planet += "8"; break;
				case 2: planet += "3"; break;
				case 3: planet += "9"; break;
				default: planet += "0"; break;
			}
			break;
	}

	// the ok planet types are PL0, 1, 3, 7, 8, 9, 13, 16
	//      and their armed variants PLPh0, 1, 3, 7, 8, 9, 13, 16
	// the others can be large enough to trash orbiting defenders
	mCreateShip( 
		typeid( tOTD ),
		planet.c_str(),
		kStartPosition_Z, 
		kNoMetaShipID,
		kOTD, 0, 0, 0, 0, "",
		kCanAddToFleetUponCapture );

	gST.mDebug_fprintf("---- Finished: tOTDTeam::mOldCreateShipsForTeam\n");
}

void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	// we've switched to using the static planet creation routine above,
	// rather than the drafted planet

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
	return;
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}

