/********************************************************************************************************\

	File Name:	OTDTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/


#include "OTDTeamBaseState.h"
#include "Met_9PlanetaryAssault.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tOTDTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kTooLongID, 25.0f );
}
void tOTDTeamBaseState::mEndState()
{}

void tOTDTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tOTDTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tOTDTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tOTDTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	static iTruth notSet = kTrue;
	if ( damagedShip->mGetCustomID() != kOTD )  return;
	if (gWC.fPlanetSubdued) {
		if (notSet) {
			fChildActor->mSetRelations(kTeam1, kTeamNeutral, kGoalIgnore, 0.0f );
			fChildActor->mSetRelations(kTeam2, kTeamNeutral, kGoalIgnore, 0.0f );
			fChildActor->mSetRelations(kTeam3, kTeamNeutral, kGoalIgnore, 0.0f );
			notSet = kFalse;
		}
	}
}
void tOTDTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tOTDTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tOTDTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tOTDTeamBaseState::mOnFlashingButtonPressed()
{}
void tOTDTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tOTDTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tOTDTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tOTDTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tOTDTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tOTDTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tOTDTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tOTDTeamBaseState::mOnHostChanged()
{}
void tOTDTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if ( timerID == kTooLongID ) {
		static iTruth reinforced = kFalse;

		if (gWC.fPvP) return;
		if (reinforced) return;

		reinforced = kTrue;

		if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
			int32 fShiplistCount;
			gST.mDebug_fprintf("--- tOTDTeamBaseState::mOnTimedInterval -- Calling reinforcements!\n");

			tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( kTeam7 );
			ShipList reinf_sl = gST.mFilterShipListToSingleRace(gST.narrowed_sl, planetTeam->mGetRace());
			reinf_sl = gST.stripOutRestricted( reinf_sl );
			reinf_sl = gST.stripOutSPECIAL( reinf_sl );

			std::list< eEnhancedClassTypes >  shipClassTypeList;
			shipClassTypeList.push_back(kEClassSmallMonster);
			shipClassTypeList.push_back(kEClassMediumMonster);
			shipClassTypeList.push_back(kEClassLargeMonster);
			for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
				shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
			fShiplistCount = reinf_sl.Subset(shipClassTypeList, true);
			gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

			reinf_sl = gST.selectWarShips(reinf_sl);

			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ) );
			int32 reinfBPV = 89;
			if (playerTeam) reinfBPV = playerTeam->mGetCombatBPV();

			if (reinfBPV < 60) reinfBPV += 60;
			mCreateSingleShip_refactored( reinf_sl, reinfBPV );

			if (reinfBPV < 120) reinfBPV += 60;
			mCreateSingleShip_refactored( reinf_sl, reinfBPV );

			if (reinfBPV < 150) reinfBPV += 50;
			mCreateSingleShip_refactored( reinf_sl, reinfBPV );

			gST.mDebug_fprintf("--- tOTDTeamBaseState::mOnTimedInterval -- Done calling reinforcements!\n");
		} else {
			mOldCreateReinforcements();
		}
		// And now we set up the new ships' fighters
		mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 1.0f + (0.2f * fChildActor->mGetTeam()) );
	}

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;
        if (! gScriptInterface.mIsHost()) return;

        int32 fTeamID = fChildActor->mGetTeam();
        tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fTeamID ) );
        if (fTeamID != kPlanetTeam) return;
        if (planetTeam->mIsTeamHumanControlled()) return;

        gST.mDebug_fprintf("--- Entering tOTDTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        gST.fTeam3DeltaFighterBPV += gST.mUpgradeFightersForTeam(planetTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", gST.fTeam3DeltaFighterBPV);
        gST.mDebug_fprintf("--- Finished: tOTDTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}

void tOTDTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tOTDTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tOTDTeamBaseState::mOldCreateReinforcements()
{
	gST.mDebug_fprintf("---- Entering: tOTDTeamBaseState::mOldCreateReinforcements\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kAttackTeam1 ) );
	int32 reinfBPV = 89;
	if (playerTeam) reinfBPV = playerTeam->mGetCombatBPV();

	if (reinfBPV < 60) reinfBPV += 60;
	fChildActor->mCreateShip( typeid( tShipInfo ),
					reinfBPV,
					kClassFrigate,
					kClassBattleship,
					0,
					kStartPosition_V, -1, 0, 0, -1, -1, NULL, 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
	
	if (reinfBPV < 120) reinfBPV += 60;
	fChildActor->mCreateShip( typeid( tShipInfo ),
					reinfBPV,
					kClassFrigate,
					kClassBattleship,
					0,
					kStartPosition_V, -1, 0, 0, -1, -1, NULL, 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );

	if (reinfBPV < 150) reinfBPV += 50;
	fChildActor->mCreateShip( typeid( tShipInfo ),
					reinfBPV,
					kClassFrigate,
					kClassBattleship,
					0,
					kStartPosition_V, -1, 0, 0, -1, -1, NULL, 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );

	gST.mDebug_fprintf("---- Finished: tOTDTeamBaseState::mOldCreateReinforcements\n");
}


void tOTDTeamBaseState::mCreateSingleShip_refactored( ShipList this_sl, int32 BPV)
{
	Ship s1;

	int32 fShiplistCount = this_sl.Subset( this_sl.kSubsetBPV, int(0.875 * BPV), int(1.125 * BPV), true);
	gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
		int(0.875 * BPV), int(1.125 * BPV), fShiplistCount);
	gST.listShiplistContent(this_sl);

	if ( (this_sl.size() > 0) && (this_sl.SelectRandomShip(s1)) ) {
		gST.mDebug_fprintf("small ship selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
		fChildActor->mCreateShip( 
			typeid( tShipInfo ),
			s1.shipCode.c_str(),
			kStartPosition_V,
			fChildActor->kNoMetaShipID,
			-1, 0, 0, -1, -1,
			NULL,
			static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
	} else {
		fChildActor->mCreateShip( 
			typeid( tShipInfo ),
			BPV,
			kClassFrigate,
			kClassBattleship,
			0,
			kStartPosition_V, -1, 0, 0, -1, -1, NULL, 
			static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
	}
}