/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}
void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}
void tOrionTeam::mSetupCommState()
{
}


void tOrionTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam:::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;

	
   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;

		// Planet
   		ShipList planet_sl = gST.narrowed_sl;
		eRace race = mGetRace();
		std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = planet_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. planets: %d\n", race, fShiplistCount);

        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  planetRolesList;
        gST.mDebug_fprintf("Selecting planet armed world role.\n");
        planetRolesList.push_back(kArmedWorld);
        fShiplistCount = planet_sl.Subset(planetRolesList, true);
        gST.mDebug_fprintf("subset for ArmedWorld eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        {
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);


		// Monster
		ShipList monster_sl = gST.narrowed_sl;
        std::list< eRace >  monsterList;
        monsterList.push_back(kMonster);
        fShiplistCount = monster_sl.Subset(monsterList);
        gST.mDebug_fprintf("subset for kMonster race filter completed. ships: %d\n", fShiplistCount);

        monster_sl = gST.stripOutSPECIAL(monster_sl);
        monster_sl = gST.stripOutRestricted(monster_sl);

        std::list< eEnhancedClassTypes >  monsterClassTypeList;
        monsterClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassLightCruiser; i <= kEClassMediumCruiser; i++) {
                monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = monster_sl.Subset(monsterClassTypeList, true);
        gST.mDebug_fprintf("subset for monster classes filter completed. ships: %d\n", fShiplistCount);

        gST.listShiplistContent(monster_sl);

		if ((planet_sl.size() > 0) && (monster_sl.size() > 0)) {
			Ship s1;

			planet_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("Planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );

			mCreateShip( 
				typeid( tShipInfo ), 
				s1.shipCode.c_str(), 
				kStartPosition_Z, 
				kNoMetaShipID, 
				-1, 0, 0, -1, -1, 
				"Colony", 
				kDefaultShipOptions );

			switch (gWC.fMissionSeed % 17) {
				case 0:
				case 1:
				case 2:
				case 3:
					monster_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("Planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
					gST.mTrackOrions( s1 );

					mCreateShip( 
						typeid( tShipInfo ), 
						s1.shipCode.c_str(), 
						kStartPosition_T, 
						kNoMetaShipID, 
						-1, 0, 0, -1, -1, 
						"Colony", 
						kDefaultShipOptions );

				default:
					break;
			}
		} else {
			mOldCreateShipsForTeam();
		}

	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tOrionTeam:::mCreateShipsForTeam\n");
}

void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam:::mCreateShipsForTeam\n");

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;
	mCreateShip( typeid( tShipInfo ), "PLPh0", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Colony", kDefaultShipOptions );
	switch (gWC.fMissionSeed % 17) {
	case 0:
		mCreateShip( typeid( tShipInfo ), "M-AMM", kStartPosition_T, 
				kNoMetaShipID, -1, 0, 0, -1, -1, "Colony", kDefaultShipOptions );
		break;
	case 1:
		mCreateShip( typeid( tShipInfo ), "M-LCM", kStartPosition_T, 
				kNoMetaShipID, -1, 0, 0, -1, -1, "Colony", kDefaultShipOptions );
		break;
	case 2:
		mCreateShip( typeid( tShipInfo ), "M-MTM", kStartPosition_T, 
				kNoMetaShipID, -1, 0, 0, -1, -1, "Colony", kDefaultShipOptions );
		break;
	case 3:
		mCreateShip( typeid( tShipInfo ), "M-SGM", kStartPosition_T, 
				kNoMetaShipID, -1, 0, 0, -1, -1, "Colony", kDefaultShipOptions );
		break;
	default:
		break;
	}

	gST.mDebug_fprintf("---- Finished: tOrionTeam:::mCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}



