/********************************************************************************************************\

	File Name:	AttackTeam3.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "AttackTeam3.h"
#include "AttackTeamInitialState.h"
#include "AttackTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "AttackTeam3InitialState.h"
#include "ShipClasses.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tAttackTeam3> gAttackTeam3Allocator;

tAttackTeam3::tAttackTeam3()
{
	gWC.mAddAttackTeam();
}

void tAttackTeam3::mSetupVictoryCondition()
{
	fVictoryCondition = new tAttackVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam3::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tAttackTeam3::mSetupState()
{
	mRegisterState( new tAttackTeam3InitialState() );

	mGotoState( typeid( tAttackTeam3InitialState ) );
	
}
void tAttackTeam3::mSetupCommState()
{
}
void tAttackTeam3::mCreateShipsForTeam()
{
	if ((gWC.fDraftedHumans == 0) && (!mIsTeamHumanControlled())) {
		mCreateShip(	typeid( tAttackTeamShip ), 
					fShipType, 
					kStartPosition_Y,
					fMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );

	}
	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftedBPV += BPV;
	}
}
void tAttackTeam3::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam3::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tAttackTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	if (!mIsTeamHumanControlled()) {
		strcpy(fShipType, shipScriptDescription.fClassName.c_str());
		fTeamID = shipScriptDescription.fTeamID;
		fMetaShipID = metaid;
	} else {
		gWC.fDraftedHumans++;
		mCreateShip( typeid( tAttackTeamShip ), 
					 shipScriptDescription.fClassName.c_str(), 
					 kStartPosition_Y,
					 metaid,
					 gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
					 shipScriptDescription.fCustomName.c_str(),
					 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
	}

	gST.mDebug_fprintf("--- Finished: tAttackTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}


void tAttackTeam3::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}