/********************************************************************************************************\

	File Name:	DefendTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "DefendTeam.h"
#include "DefendTeamInitialState.h"
#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tDefendTeam> gDefendTeamAllocator;

tDefendTeam::tDefendTeam()
{
	gWC.mAddDefendTeam();
}
void tDefendTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tDefendTeam::mSetupState()
{
	mRegisterState( new tDefendTeamInitialState() );

	mGotoState( typeid( tDefendTeamInitialState ) );
	
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tDefendTeam::mSetupCommState()
{
}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tDefendTeam::mCreateShipsForTeam()
{
	if ((gWC.fDraftedHumans == 0) && (!mIsTeamHumanControlled())) {
		mCreateShip(	typeid( tDefendTeamShip ), 
					fShipType, 
					kStartPosition_G,
					fMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );

	}
	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftingBPV += BPV;
	}

}
void tDefendTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tDefendTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	if (!mIsTeamHumanControlled()) {
		strcpy(fShipType, shipScriptDescription.fClassName.c_str());
		fTeamID = shipScriptDescription.fTeamID;
		fMetaShipID = metaid;
	} else {
		gWC.fDraftingHumans++;
		mCreateShip( typeid( tDefendTeamShip ), 
					 shipScriptDescription.fClassName.c_str(), 
					 kStartPosition_G,
					 metaid,
					 gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
					 shipScriptDescription.fCustomName.c_str(),
					 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
	}
	gST.mDebug_fprintf("--- Finished: tDefendTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tDefendTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}