//	<Name>	DefendTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Met_10Patrol.h"
#include "ShipInfo.h"
#include "DefendTeamShip.h"
#include "DefendTeamShipInitialState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

tDefendTeamShipInitialState::tDefendTeamShipInitialState( )
{
	fDeepScanON = false;
}

void tDefendTeamShipInitialState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}



void tDefendTeamShipInitialState::mBeginState()
{
	fChildActor->mSetShipCallbackEvents( kNotifyProbeLaunch );
	fChildActor->mSetShipCallbackEvents( kNotifyBeginTractor );
	fChildActor->mSetShipCallbackEvents( kNotifyDamage );
	// may have to use gGetMapPicked() for some missions
	int32 map = 1;
	int32 cust = fChildActor->mGetCustomID(); if (cust < 1) cust = 1;
	int32 seed = gWC.fMissionSeed; if (seed < 1) seed = 1;
	int32 bpv = fChildActor->mGetShipBPV(); if (bpv < 1) bpv = 1;
	int32 Team = fChildActor->mGetTeam(); if (Team < 1) Team = 1;
	int32 id = fChildActor->mGetShipID(); if (id < 1) id = 1;
	fShipRandValue = map * cust * seed * bpv * Team * id;
	if (fShipRandValue < 0) fShipRandValue *= -1;
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ));
	if (team && (team->mGetRace() == kRomulan) && (!team->mIsTeamHumanControlled())) {
		fChildActor->mCloak( kFalse );
		fChildActor->mChangeAIOfficerState( kCloakOfficer, kFalse );
	}
}


void tDefendTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
}
void tDefendTeamShipInitialState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	// we only care about damage we inflict on the monster
	fChildActor->mSetShipCallbackEvents( kNotifyDamage );
	if (!damagedShip || !hostileShip) return;
	if (damagedShip->mGetCustomID() != 37) return;
	if (hostileShip->mGetShipID() != fChildActor->mGetShipID()) return;
	if (!gWC.fDefenderScannedMonster) return;

	// once damaged, we can no longer communicate with the monster and must destroy it
	if (gWC.fDeathTechnique == 4) gWC.fDeathTechnique = 0;

	bool destroyMonster = false;
	switch (damageType) {
		case kDamageByProbe:
			if (gWC.fDeathTechnique == 3) destroyMonster = true;
			break;
		case kDamageByMine:
			if (gWC.fDeathTechnique == 2) destroyMonster = true;
			break;
		default: return;
	}
	if (destroyMonster) {
		if (damagedShip) damagedShip->mDestroyShip();
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "That should do it Sir!", kNonCommScriptMessage );
	}
}

void tDefendTeamShipInitialState::mOnBeginTractor( tShipInfo* TractorTarget, tShipInfo* TractoringShip )
{
	// we only care about us tractoring the monster
	if (!TractorTarget || !TractoringShip) return;
	if (TractoringShip->mGetShipID() != fChildActor->mGetShipID()) return;
	if (gWC.fDeathTechnique != 1) return;
	if (!gWC.fDefenderScannedMonster) return;
	if (TractorTarget->mGetCustomID() == 37) 
	{
		TractorTarget->mDestroyShip();
		// TractorTarget->mDestroyShipSilently();
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "That should do it Sir!", kNonCommScriptMessage );
	}
	fChildActor->mSetShipCallbackEvents( kNotifyBeginTractor );
}

void tDefendTeamShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if (!deepScanOn) return;
	tTeamInfo* monsterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kOrionTeam) );
	tShipIterator monsterShip = monsterTeam->mGetFirstShip();
	if (monsterShip->mGetCustomID() != 37) return;

	if (ship && (ship->mGetShipID() == fChildActor->mGetShipID())) {
			if (ship->mGetTacticalDistance(*monsterShip) < 2)
				gWC.fDefenderScannedMonster = true;
			else
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We need to get closer to the monster before we attempt a scan", kNonCommScriptMessage );
	}
	if (!gWC.fDefenderScannedMonster) return;
	switch (gWC.fDeathTechnique) {
		case 1: // a tractor lock will kill the monster
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir, I believe a tractor lock will kill the monster.", kNonCommScriptMessage );
			break;
		case 2: // mine damage will kill the monster
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir, I believe a mine can be used to kill the monster.", kNonCommScriptMessage );
			break;
		case 3: // probe damage will kill the monster
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir, I believe we can modify a probe so that, fired as a weapon, it will kill the monster.", kNonCommScriptMessage );
			break;
		case 4: // we can communicate with the monster
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir, I believe the monster is not inherently hostile, it may be possible to establish communication.", kNonCommScriptMessage );
			break;
		case 7: // Andro!
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir! This is not a monster, but a strange hostile ship! We may have to attempt destroying it.", kNonCommScriptMessage );
			break;
		default: // we must kill the monster
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Sir, I believe the only way to kill the monster is combat.", kNonCommScriptMessage );
			break;
	}
}

void tDefendTeamShipInitialState::mOnProbeLaunch( tShipInfo* Ship )
{
	// we only care about us tractoring the monster
	if (!Ship) return;
	if (Ship->mGetShipID() != fChildActor->mGetShipID()) return;
	if (gWC.fDeathTechnique != 3) return;
	if (!gWC.fDefenderScannedMonster) return;
	{
		tTeamInfo* monsterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kOrionTeam) );
		tShipIterator monsterShip = monsterTeam->mGetFirstShip();
		(*monsterShip)->mDestroyShip();
		// TractorTarget->mDestroyShipSilently();
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "That should do it Sir!", kNonCommScriptMessage );
	}
	fChildActor->mSetShipCallbackEvents( kNotifyProbeLaunch );
}

void tDefendTeamShipInitialState::mOnDeath( tShipInfo* ship )
{
}
void tDefendTeamShipInitialState::mOnCapture( tShipInfo* ship )
{
}
void tDefendTeamShipInitialState::mOnDisengage( tShipInfo* ship )
{
}
void tDefendTeamShipInitialState::mEndState()
{
}
