/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}
void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}
void tOrionTeam::mSetupCommState()
{
}

void tOrionTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

	int32 selection = gWC.fMissionSeed % 6;
	switch (gWC.fMissionSeed % 19) {
		case 0: gWC.fDeathTechnique = 4; break;
		case 1:
		case 2:
		case 3: gWC.fDeathTechnique = 3; break;
		case 4:
		case 5:
		case 6: gWC.fDeathTechnique = 2; break;
		case 7:
		case 8:
		case 9: gWC.fDeathTechnique = 1; break;
		default: gWC.fDeathTechnique = 0; break;
	}
	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;

		// Monster
        gST.mDebug_fprintf("---- Setting up monster shiplist subset\n");
        ShipList monster_sl = gST.narrowed_sl;
        std::list< eRace >  monsterList;
        monsterList.push_back(kMonster);
        fShiplistCount = monster_sl.Subset(monsterList);
        gST.mDebug_fprintf("subset for kMonster race filter completed. ships: %d\n", fShiplistCount);

        monster_sl = gST.stripOutSPECIAL(monster_sl);
        monster_sl = gST.stripOutRestricted(monster_sl);

		std::list< eEnhancedClassTypes >  monsterClassTypeList;
		switch (selection) {
			case 0: 
			case 1: 
			case 2:
				gST.mDebug_fprintf("We're having a small monster for dinner. selection: %d\n", selection);
				monsterClassTypeList.push_back(kEClassSmallMonster);
				{ // no, I'm not redefining i. Local only please.
					for (int32 i = kEClassFrigate; i <= kEClassWarDestroyer; i++) {
							monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
					}
				}
				break;
			case 3:
			case 4: 
				gST.mDebug_fprintf("We're having a medium monster for dinner. selection: %d\n", selection);
				monsterClassTypeList.push_back(kEClassMediumMonster);
				{ // no, I'm not redefining i. Local only please.
					for (int32 i = kEClassLightCruiser; i <= kEClassNewHeavyCruiser; i++) {
							monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
					}
				}
				break;
			default: 
				gST.mDebug_fprintf("We're having a large monster for dinner. selection: %d\n", selection);
				monsterClassTypeList.push_back(kEClassLargeMonster);
				{ // no, I'm not redefining i. Local only please.
					for (int32 i = kEClassHeavyBattlecruiser; i <= kEClassBattleship; i++) {
							monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
					}
				}
				break;
		}
		fShiplistCount = monster_sl.Subset(monsterClassTypeList, true);
        gST.mDebug_fprintf("subset for monster classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(monster_sl);

		Ship s1;
		if ((monster_sl.size() > 0) && (monster_sl.SelectRandomShip(s1))) {
			gST.mDebug_fprintf("monster selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );

			// Make sure we apply the alternate kill methods ONLY to actual monsters
			if (
				(s1.enhancedShipClass != kEClassSmallMonster) &&
				(s1.enhancedShipClass != kEClassMediumMonster) &&
				(s1.enhancedShipClass != kEClassLargeMonster)
			) { 
				gWC.fDeathTechnique = 7;  // 7 is outside of the random selectable range of 0-5
				gST.mDebug_fprintf("Not an actual monster. Disabling special kill method\n");
			}

			mCreateShip( 
				typeid( tShipInfo ), 
				s1.shipCode.c_str(), 
				kStartPosition_Z, 
				kNoMetaShipID, 
				37, 0, 0, -1, -1, 
				"Nemesis", 
				kDeathUponCapture );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}

void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam::mOldCreateShipsForTeam\n");

	int32 selection = gWC.fMissionSeed % 31;
	switch (gWC.fMissionSeed % 19) {
		case 0: gWC.fDeathTechnique = 4; break;
		case 1:
		case 2:
		case 3: gWC.fDeathTechnique = 3; break;
		case 4:
		case 5:
		case 6: gWC.fDeathTechnique = 2; break;
		case 7:
		case 8:
		case 9: gWC.fDeathTechnique = 1; break;
		default: gWC.fDeathTechnique = 0; break;
	}

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;
	switch (selection) {
	case 0: 
	case 1: 
	case 2: mCreateShip( typeid( tShipInfo ), "M-AMS", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 3:
	case 4: mCreateShip( typeid( tShipInfo ), "M-AMM", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 5: mCreateShip( typeid( tShipInfo ), "M-AML", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 6:
	case 7:
	case 8:
		mCreateShip( typeid( tShipInfo ), "M-LCS", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 9:
	case 10:
		mCreateShip( typeid( tShipInfo ), "M-LCM", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 11:
		mCreateShip( typeid( tShipInfo ), "M-LCL", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 12:
	case 13:
	case 14:
		mCreateShip( typeid( tShipInfo ), "M-MTS", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 15:
	case 16:
		mCreateShip( typeid( tShipInfo ), "M-MTM", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 17:
		mCreateShip( typeid( tShipInfo ), "M-MTL", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 18:
	case 19:
	case 20:
		mCreateShip( typeid( tShipInfo ), "M-SGS", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	case 21:
	case 22:
		mCreateShip( typeid( tShipInfo ), "M-SGM", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	case 23:
		// generate a monster
		mCreateShip( typeid( tShipInfo ), "M-SGL", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	case 24:
	case 25:
	case 26:
		mCreateShip( typeid( tShipInfo ), "M-SSS", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	case 27:
	case 28:
		mCreateShip( typeid( tShipInfo ), "M-SSM", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	default:
		mCreateShip( typeid( tShipInfo ), "M-SSL", kStartPosition_Z, 
			kNoMetaShipID, 37, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	}
	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// census only: not actually drafting

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOrionTeam::mCreateOrions()
{
}

