/********************************************************************************************************\

	File Name:	AttackTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "AttackTeam2.h"
#include "AttackTeamInitialState.h"
#include "AttackTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "AttackTeam2InitialState.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tAttackTeam2> gAttackTeam2Allocator;

tAttackTeam2::tAttackTeam2()
{
	gWC.mAddAttackTeam();
	fNumShips = 0;
}

void tAttackTeam2::mSetupVictoryCondition()
{
	fVictoryCondition = new tAttackVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam2::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tAttackTeam2::mSetupState()
{
	mRegisterState( new tAttackTeam2InitialState() );

	mGotoState( typeid( tAttackTeam2InitialState ) );
	
}
void tAttackTeam2::mSetupCommState()
{
}
void tAttackTeam2::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam2::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
	gST.mCompleteNarrowedList();

	// these ships are actually drafted in mCreateNewShip. See the fShipType in there?
	if ((gWC.fDraftedHumans == 0) && (!mIsTeamHumanControlled())) {
		mCreateShip(	typeid( tAttackTeamShip ), 
					fShipType, 
					kStartPosition_X,
					fMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
		if (fNumShips > 1) {
			mCreateShip(	typeid( tAttackTeamShip ), 
					fShipType2, 
					kStartPosition_X,
					fMetaShipID2,
					gWC.fNWCustID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
		}
	}

	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftedBPV += BPV;
	}

	gST.mDebug_fprintf("--- Finishing: tAttackTeam2::mCreateShipsForTeam\n");
}
void tAttackTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tAttackTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	fNumShips++;

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	if (!mIsTeamHumanControlled()) {
		fTeamID = shipScriptDescription.fTeamID;
		if (fNumShips < 2) {
			strcpy(fShipType, shipScriptDescription.fClassName.c_str());
			fMetaShipID = metaid;
		} else if (fNumShips < 3) {
			strcpy(fShipType2, shipScriptDescription.fClassName.c_str());
			fMetaShipID2 = metaid;
		}
	} else {
		gWC.fDraftedHumans++;
		mCreateShip(
			typeid( tAttackTeamShip ), 
			shipScriptDescription.fClassName.c_str(), 
			kStartPosition_X,
			metaid,
			gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
			shipScriptDescription.fCustomName.c_str(),
			static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
	}

	gST.mDebug_fprintf("--- Finished: tAttackTeam2::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam2::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}