//	<Name>	AttackVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "AttackVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_10Patrol.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "AttackTeam.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"

#include "WorldConditions.h"
#include "OrionTeam.h"

tAttackVictoryState::tAttackVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tAttackVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );
	attackTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam2 ) );
	attackTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam3 ) );
			
	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	defendTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam2 ) );
	defendTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam3 ) );
	
	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 1;
	fTeamDisengaged = false;
	fThisPlayerDead = false;


	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled()))) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}

void tAttackVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );

	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}
	if( attackTeam2 )
	{
		attackBPV += attackTeam2->mGetCombatBPV();
	}
	if( attackTeam3 )
	{
		attackBPV += attackTeam3->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( defendTeam2 )
	{
		defendBPV += defendTeam2->mGetCombatBPV();
	}
	if( defendTeam3 )
	{
		defendBPV += defendTeam3->mGetCombatBPV();
	}

	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
}

void tAttackVictoryState::mEndState()
{
}

eVictoryLevels tAttackVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kOrionTeam ) );
	tTeamInfo* thisTeam = fMissionInfo->mGetTeamHandle( ID );

	if (!thisTeam) return kDraw;
	else if (thisTeam->mShipsCurrentlyControlled() > 0) {
		if (gWC.fPirateUsed) return kAstoundingVictory;
		else return kVictory;
	} else {
		if (gWC.fPirateUsed && orionTeam && (orionTeam->mShipsCurrentlyControlled() < 1)) return kDefeat;
		else return kDevastatingDefeat;
	}


/**********START ORIGINAL VERSION ****************
	int32 attackDamagePoints = 0;
	float attackDamageRatio = 0.0;
	int32 defendDamagePoints = 0;
	float defendDamageRatio = 0.0;
	float damageRatio = 0.0;
	int i;

	for( i = 1; i <= gWC.mGetNumAttackShips(); ++i )
	{
		attackDamagePoints +=	gWC.mGetAttackDamageCond( i ); 
	}
	for( i = 1; i <= gWC.mGetNumDefendShips(); ++i )
	{
		defendDamagePoints +=	gWC.mGetDefendDamageCond( i );
	}
	attackDamageRatio = (float)attackDamagePoints / gWC.mGetNumAttackShips();
	defendDamageRatio = (float)defendDamagePoints / gWC.mGetNumDefendShips();

	if( attackDamageRatio )
	{
		damageRatio = defendDamageRatio / attackDamageRatio;
	}
	else if( defendDamageRatio )
	{
		damageRatio = 3;
	}
	else
	{
		damageRatio = 0.5;
	}

	if( attackBPV > defendBPV && fTeamDisengaged && ( gWC.mGetRemainingDefendShips() > 0 ) )
	{
//		mAddToBonusPrestige( -40 );
	}

	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if ( fTeamDisengaged )			// 20020531 JEK fix disengage wins you a sector bug
	{
		//if (attackBPV < defendBPV)
			//return kDraw;
		//else
			return kDefeat;
	}
	else if( damageRatio > 2 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 50 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		
		return	kAstoundingVictory;
	}
	else if( damageRatio > 1 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 25 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 62 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kVictory;
	}
	else if( damageRatio > 0.5 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 12 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 31 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kDraw;
	}
	else if( damageRatio > 0.25 )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}
 **********END ORIGINAL VERSION ****************/
}
int32 tAttackVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();
	if ((teamID->mGetTeam() == kOrionTeam) && (!gWC.fPirateUsed)) return 0;

	return teamShipsInPlay;
}
void tAttackVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	if (!ship) return;
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kOrionTeam ) );

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			gWC.mRemoveAttackShip();
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	iTruth checkend = kFalse;
	if( fTeam && fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() ) 
	{
			fThisPlayerDead = true;
			gWC.mRemoveAttackTeam();
			fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
			checkend = kTrue;
	}

	if( orionTeam && ship->mGetOriginalTeam() == orionTeam->mGetTeam() )
	{
		if (orionTeam->mShipsCurrentlyControlled() < 1)
			mAddToBonusPrestige( orionTeam->mGetCombatBPV() / 2 );	
		checkend = kTrue;
	}

	if (checkend) gWC.mCheckForGameEnd();
}
void tAttackVictoryState::mOnDeath( tShipInfo* ship )
{

	int32 thisShipID;

	if (!ship) return;
	if (ship->mGetClassType() == kClassPseudoFighter) return;

	if (!ship) return;
	eTeamID teamID = ship->mGetTeam();
	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(600);
				mAddToBonusPrestige(60);
			}
		}
	}
	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
		case kDefendTeam3:
		default:
		//Empty
		break;
	}

	mCheckAllTeams( ship );
}
void tAttackVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;
	if (!ship) return;
	if (ship->mGetClassType() == kClassPseudoFighter) return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 30 );
	}

	if (!ship) return;
	eTeamID teamID = ship->mGetTeam();
	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(900);
				mAddToBonusPrestige(90);
			}
		}
	}
	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kAttackTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		case kDefendTeam2:
		case kDefendTeam3:
			//Break;
		default:
		break;
	}

	mCheckAllTeams( ship );
}

void tAttackVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if (!ship) return;
	gWC.mRestoreSpares(ship);
	if (ship->mGetClassType() == kClassPseudoFighter) return;
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
			tTeamInfo* myteam = fChildActor->mGetTeam();
			if (myteam) {
				eTeamID myteamID = myteam->mGetTeam();
				if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
					if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(300);
					mAddToBonusPrestige(30);
				}
			}
		}
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveAttackTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				//gWC.mCheckForGameEnd();
				mSetTimedInterval(kTimerSecondsSet, kEndingID, 5.0f);
			}
		}
	}
}
void tAttackVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tAttackVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	} 
}
void tAttackVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveAttackTeam();
		fPlayerLeftAlready = true;
		mSetTimedInterval(kTimerSecondsSet, kEndingID, 5.0f);
	}
}
