/********************************************************************************************************\

	Mission:	Patrol
	Script:		"Met_10Patrol" //Enemy space version of patrol
	Descript:	<DESCRIPTION>

	Created by:		David F		06/19/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "AttackTeam3.h"
#include "OrionTeam.h"
#include "WorldConditions.h"
#include "DefendTeamShip.h"
#include "AttackTeamShip.h"
#include "Met_10Patrol.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_10Patrol );

tMet_10Patrol::tMet_10Patrol()
{
	srand(time(NULL));
}
void tMet_10Patrol::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fMinBPV = kMinBPV;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement;
	float AIchance = 1.0f;

	std::string friendlymsg("Fleet Patrol");
	std::string hostilemsg("Fleet Patrol");

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tDefendTeam ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryOpponentTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kSameAs | kDefendTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ),
														kMaxChance, kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,  kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam3 ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam3 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Orion Team", static_cast< eTeamID>( kOrionTeam ), kMaxChance, kTrue,
														kNPCTeam,
														kMinChance,
														kSpecOrion,
														kAnyRace,
														typeid( tOrionTeam ) );



	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}
int32 tMet_10Patrol::mInitializeStart( tDynaverseScriptInfo& info )
{
	gWC.mSetMapPicked(info.fMapPicked);
	return 1;
}
void tMet_10Patrol::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_10Patrol::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMet_10Patrol::mNewInternalShip( tShipInfo* Ship )
{
}


void tMet_10Patrol::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Patrol (10enemy)");
	gWC.mCheckCRC();

	const int kRelationsCount = 7;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
		kOrionTeam,		//Orn
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Def3	Atk2	Atk3	Orn
		/*Def1*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",	"WD",
		/*Atk1*/	"WD",	"--",	"WD",	"WD",	"AI",	"AI",	"WD",
		/*Def2*/	"AI",	"WD",	"--",	"AI",	"WD",	"WD",	"WD",
		/*Def3*/	"AI",	"WD",	"AI",	"--",	"WD",	"WD",	"WD",
		/*Atk2*/	"WD",	"AI",	"WD",	"WD",	"--",	"AI",	"WD",	
		/*Atk3*/	"WD",	"AI",	"WD",	"WD",	"AI",	"--",	"WD",
		/*Orn*/		"WD",	"WD",	"WD",	"WD",	"WD",	"WD",	"--",
	};
	std::string caprel[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Def3	Atk2	Atk3	Orn
		/*Def1*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",	"WD",
		/*Atk1*/	"WX",	"--",	"WX",	"WD",	"AI",	"AI",	"WD",
		/*Def2*/	"AI",	"WD",	"--",	"AI",	"WD",	"WD",	"WD",
		/*Def3*/	"AI",	"WD",	"AI",	"--",	"WD",	"WD",	"WD",
		/*Atk2*/	"WX",	"AI",	"WX",	"WD",	"--",	"AI",	"WD",	
		/*Atk3*/	"WX",	"AI",	"WX",	"WD",	"AI",	"--",	"WD",
		/*Orn*/		"WD",	"WD",	"WD",	"WD",	"WD",	"WD",	"--",
	};

	if ((gWC.fDraftedHumans < 1) && (gWC.fDraftingHumans < 2) && ((rand() % 3) < 1)) fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );
	else fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_10Patrol::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace1|kTerrainSpace3|kTerrainSpace5, kPlanetNone, kBaseNone );
	case 1:		return tMapTerrain( kTerrainSpace2|kTerrainSpace4|kTerrainSpace6, kPlanetNone, kBaseNone );
	case 2:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseNone );
	case 3:		return tMapTerrain( kTerrainAsteroids1|kTerrainAsteroids3|kTerrainAsteroids5, kPlanetNone, kBaseNone );
	case 4:		return tMapTerrain( kTerrainAsteroids2|kTerrainAsteroids4|kTerrainAsteroids6, kPlanetNone, kBaseNone );
	case 5:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseNone );
	case 6:		return tMapTerrain( kTerrainNebula1|kTerrainNebula3|kTerrainNebula5, kPlanetNone, kBaseNone );
	case 7:		return tMapTerrain( kTerrainNebula2|kTerrainNebula4|kTerrainNebula6, kPlanetNone, kBaseNone );
	case 8:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseNone );
	case 9:		return tMapTerrain( kTerrainBlackHole1|kTerrainBlackHole3|kTerrainBlackHole5, kPlanetNone, kBaseNone );
	case 10:		return tMapTerrain( kTerrainBlackHole2|kTerrainBlackHole4|kTerrainBlackHole6, kPlanetNone, kBaseNone );
	case 11:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseNone );
	case 12:		return tMapTerrain( kTerrainDustclouds, kPlanetNone, kBaseNone );
	case 13:		return tMapTerrain( kTerrainShippingLane, kPlanetNone, kBaseNone );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_10Patrol::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;

}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_10Patrol::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.56f, 0.99f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.6f, 1.02f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.64f, 1.04f );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.62f, 1.14f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.12f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.58f, 1.10f );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.585f, 1.08f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.56f, 1.06f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.535f, 1.04f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.05f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.63f, 1.07f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.66f, 1.09f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.65f, 1.11f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.68f, 1.14f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.71f, 1.17f );
}
void tMet_10Patrol::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kDefendTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefing_msg;
		}
	}
}