/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}
void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}
void tOrionTeam::mSetupCommState()
{
}

void tOrionTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	int32 orionBPV = 60;

	if (playerTeam) orionBPV = playerTeam->mGetCombatBPV() - (gWC.fMissionSeed % 23);

	if ( 
		(orionBPV < 75) || (gWC.fDraftedHumans > 0)
	) {
		gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam -- not enough BPV (%d) or drafted humans present (%d)\n",
			orionBPV, gWC.fDraftedHumans);
		return;
	}

	int32 selection = gWC.fMissionSeed % 13;

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		gST.mDebug_fprintf("--- Choosing pirates with enhanced shiplist!\n");

		int32 fShiplistCount;
		ShipList pirate_sl = gST.narrowed_sl;
		std::list< eRace > raceList;
		raceList.push_back(mGetRace());
		fShiplistCount = pirate_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for race completed. ships: %d\n", fShiplistCount);

		pirate_sl = gST.stripOutRestricted(pirate_sl);

		// pirate cargo freighters
		ShipList pirate_freighter_sl = pirate_sl;
		std::list< eEnhancedRoles >  pirate_freighter_enh_roles;
		pirate_freighter_enh_roles.push_back(kCargo);
		fShiplistCount = pirate_freighter_sl.Subset(pirate_freighter_enh_roles, true);
		gST.mDebug_fprintf("subset for eEnhancedRoles for kCargo completed. cargo ships: %d\n", fShiplistCount);

		std::list< eEnhancedClassTypes >  pirate_freighter_enh_class;
		pirate_freighter_enh_class.push_back(kEClassCivilianFreighter);
		pirate_freighter_enh_class.push_back(kEClassPirateFreighter);
		pirate_freighter_enh_class.push_back(kEClassPriorityTransport);
		pirate_freighter_enh_class.push_back(kEClassSmallFreighter);
		pirate_freighter_enh_class.push_back(kEClassLargeFreighter);
		pirate_freighter_enh_class.push_back(kEClassHeavyFreighter);
		fShiplistCount = pirate_freighter_sl.Subset(pirate_freighter_enh_class, true);
		gST.mDebug_fprintf("subset for eEnhancedClassTypes for freighters completed. cargo freighters: %d\n", fShiplistCount);

		Ship aShip;

		switch (selection) {
		case 0:	
			{
				ShipList pirate_bases_sl = pirate_sl;
				std::list< eEnhancedClassTypes >  pirate_bases_enh_class;
				pirate_bases_enh_class.push_back(kEClassMobileBase);
				pirate_bases_enh_class.push_back(kEClassBaseStation);
				pirate_bases_enh_class.push_back(kEClassBattleStation);
				fShiplistCount = pirate_bases_sl.Subset(pirate_bases_enh_class, true);
				gST.mDebug_fprintf("subset for eEnhancedClassTypes for bases completed. bases: %d\n", fShiplistCount);

				if ((pirate_bases_sl.size() > 0) && (pirate_bases_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Eden II", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) 
					);
				} else {
					// fall back to old code
					mCreateShip( typeid( tShipInfo ),
						orionBPV,
						kClassBaseStation,
						kClassStarBase,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Eden II", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
				}
			}
			
			break;
		case 4: // generate a pirate convoy
			{
				ShipList temp_sl = pirate_freighter_sl;
				temp_sl.Subset( temp_sl.kSubsetBPV, 25, 35, true );
				if ((temp_sl.size() > 0) && (temp_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Convoy 5", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );		
				} else {
					mCreateShip( typeid( tShipInfo ),
							30,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 5", 
							static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
				}
			}
		case 5:
			{
				ShipList temp_sl = pirate_freighter_sl;
				temp_sl.Subset( temp_sl.kSubsetBPV, 30, 40, true );
				if ((temp_sl.size() > 0) && (temp_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Convoy 4", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );		
				} else {
					mCreateShip( typeid( tShipInfo ),
							35,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 4", 
							static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
				}
			}
		case 6:
			{
				ShipList temp_sl = pirate_freighter_sl;
				temp_sl.Subset( temp_sl.kSubsetBPV, 35, 45, true );
				if ((temp_sl.size() > 0) && (temp_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Convoy 3", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );		
				} else {
					mCreateShip( typeid( tShipInfo ),
							40,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 3", 
							static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
				}
			}
		case 7:
			{
				ShipList temp_sl = pirate_freighter_sl;
				temp_sl.Subset( temp_sl.kSubsetBPV, 40, 50, true );
				if ((temp_sl.size() > 0) && (temp_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Convoy 2", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );		
				} else {
					mCreateShip( typeid( tShipInfo ),
							40,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 2", 
							static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
				}
			}
		case 8:
			{
				ShipList temp_sl = pirate_freighter_sl;
				temp_sl.Subset( temp_sl.kSubsetBPV, 45, 55, true );
				if ((temp_sl.size() > 0) && (temp_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Convoy 1", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );		
				} else {
					mCreateShip( typeid( tShipInfo ),
							40,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 1", 
							static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
				}
			}
			break;
		default:
			{
				// pirate ship OR monster ship (Andros!)
				ShipList pirate_ships_sl = gST.narrowed_sl;

				std::list< eRace > raceList;
				raceList.push_back(mGetRace());
				raceList.push_back(kMonster);
				fShiplistCount = pirate_ships_sl.Subset(raceList);
				gST.mDebug_fprintf("subset for race completed. ships: %d\n", fShiplistCount);

				// Strip out ships with SPECIAL or 'R'estricted
				pirate_ships_sl = gST.stripOutRestricted( pirate_ships_sl );
				pirate_ships_sl = gST.stripOutSPECIAL( pirate_ships_sl );

				std::list< eEnhancedClassTypes >  pirate_ships_enh_class;
				// Auxiliary units
				pirate_ships_enh_class.push_back(kEClassSmallAuxiliary);
				pirate_ships_enh_class.push_back(kEClassLargeAuxiliary);
				pirate_ships_enh_class.push_back(kEClassHeavyAuxiliary);
				// Tugs
				pirate_ships_enh_class.push_back(kEClassLightTug);
				pirate_ships_enh_class.push_back(kEClassTug);
				pirate_ships_enh_class.push_back(kEClassNewTug);
				// Ships
				pirate_ships_enh_class.push_back(kEClassFrigate);
				pirate_ships_enh_class.push_back(kEClassDestroyer);
				pirate_ships_enh_class.push_back(kEClassWarDestroyer);
				pirate_ships_enh_class.push_back(kEClassLightCruiser);
				pirate_ships_enh_class.push_back(kEClassWarCruiser);
				pirate_ships_enh_class.push_back(kEClassHeavyWarDestroyer);
				pirate_ships_enh_class.push_back(kEClassMediumCruiser);
				pirate_ships_enh_class.push_back(kEClassHeavyCruiser);
				pirate_ships_enh_class.push_back(kEClassNewHeavyCruiser);
				pirate_ships_enh_class.push_back(kEClassHeavyBattlecruiser);
				pirate_ships_enh_class.push_back(kEClassMonitor);
				pirate_ships_enh_class.push_back(kEClassDreadnought);
				pirate_ships_enh_class.push_back(kEClassHeavyDreadnought);
				pirate_ships_enh_class.push_back(kEClassBattleship);
				fShiplistCount = pirate_ships_sl.Subset(pirate_ships_enh_class, true);
				gST.mDebug_fprintf("subset for eEnhancedClassTypes for ships completed. ships: %d\n", fShiplistCount);

				std::list< eEnhancedRoles >  pirate_ships_enh_roles;
				pirate_ships_enh_roles.push_back(kLineShip);
				pirate_ships_enh_roles.push_back(kWarShip);
				pirate_ships_enh_roles.push_back(kLeader);
				pirate_ships_enh_roles.push_back(kDroneBombardmentShip);
				pirate_ships_enh_roles.push_back(kEscort);
				pirate_ships_enh_roles.push_back(kCarrier);
				pirate_ships_enh_roles.push_back(kPFTender);
				pirate_ships_enh_roles.push_back(kFastCruiser);
				fShiplistCount = pirate_ships_sl.Subset(pirate_ships_enh_roles, true);
				gST.mDebug_fprintf("subset for eEnhancedRoles for combat ships completed. ships: %d\n", fShiplistCount);

				fShiplistCount = pirate_ships_sl.Subset( pirate_ships_sl.kSubsetBPV, int(0.8 * orionBPV), int(1.2 * orionBPV), true);
				gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n", 
					int(0.8 * orionBPV), int(1.2 * orionBPV), fShiplistCount);


				if ((pirate_ships_sl.size() > 0) && (pirate_ships_sl.SelectRandomShip(aShip))) {
					gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n", 
						aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
					gST.mTrackOrions( aShip );

					mCreateShip( 
						typeid( tShipInfo ),
						aShip.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, "Nemesis", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );		
				} else {
					// fallback to original code
					mCreateShip( typeid( tShipInfo ),
								orionBPV,
								kClassFrigate,
								kClassBattleship,
								0,
								kStartPosition_Z, -1, 0, 0, -1, -1, "Nemesis", 
								static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
				}
				break;
			}
		}

	} else {
		gST.mDebug_fprintf("--- Fallback to original creation code\n");
		switch (selection) {
		case 0:
			// generate a pirate base
			mCreateShip( typeid( tShipInfo ),
						orionBPV,
						kClassBaseStation,
						kClassStarBase,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Eden II", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
			break;
		case 4: // generate a pirate convoy
			mCreateShip( typeid( tShipInfo ),
						25,
						kClassFreighter,
						kClassFreighter,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 5", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		case 5:
			mCreateShip( typeid( tShipInfo ),
						30,
						kClassFreighter,
						kClassFreighter,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 4", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		case 6:
			mCreateShip( typeid( tShipInfo ),
						35,
						kClassFreighter,
						kClassFreighter,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 3", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		case 7:
			mCreateShip( typeid( tShipInfo ),
						40,
						kClassFreighter,
						kClassFreighter,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 2", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		case 8:
			mCreateShip( typeid( tShipInfo ),
						45,
						kClassFreighter,
						kClassFreighter,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 1", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
			break;
		default:
			// generate a pirate ship
			mCreateShip( typeid( tShipInfo ),
						orionBPV,
						kClassFrigate,
						kClassBattleship,
						0,
						kStartPosition_Z, -1, 0, 0, -1, -1, "Nemesis", 
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
			break;
		}
	}
	gST.mDebug_fprintf("---Finished: tOrionTeam::mCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}


