/********************************************************************************************************\

	Mission:	Patrol
	Script:		"Met_NW17Patrol"
	Descript:	This is the player-territory version of Patrol, no pirates.

	Created by:		David F		06/19/00
	Last Updated:	DMW	01/18/03
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "AttackTeam3.h"
#include "WorldConditions.h"
#include "Met_17Patrol.h"
#include "DefendTeamShip.h"
#include "AttackTeamShip.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_17Patrol );

tMet_17Patrol::tMet_17Patrol()
{
	srand(time(NULL));
}
void tMet_17Patrol::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kPlayerRaceHex, kAnyTerrainHex );
	Specs->fWait	= 2;
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fMinBPV = kMinBPV;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags;

	std::string friendlymsg("Patrol");
	std::string hostilemsg("Patrol");

	/*
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kEnemyRaceHex, kAsteroidHex | kNebulaHex | kBlackHoleHex | kShippingLaneHex | kDustCloudsHex | kEmptySpaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;
	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tDefendTeam ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryOpponentTeam,
														kMaxChance,
														kSpecPlayableRace|kSpecOrion,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ),
														kMaxChance, kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,  kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam3 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace|kSpecOrion,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam3 ),
														kNoTeamTag,
														hostilemsg );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}
int32 tMet_17Patrol::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_17Patrol::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_17Patrol::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMet_17Patrol::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_17Patrol::mStartMission()
{
	tScript::mStartMission();

	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Patrol (17)");
	gWC.mCheckCRC();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Def3	Atk2	Atk3
		/*Def1*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",
		/*Atk1*/	"WD",	"--",	"WD",	"WD",	"AI",	"AI",
		/*Def2*/	"AI",	"WD",	"--",	"AI",	"WD",	"WD",
		/*Def3*/	"AI",	"WD",	"AI",	"--",	"WD",	"WD",
		/*Atk2*/	"WD",	"AI",	"WD",	"WD",	"--",	"AI",
		/*Atk3*/	"WD",	"AI",	"WD",	"WD",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_17Patrol::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace1|kTerrainSpace3|kTerrainSpace5, kPlanetNone, kBaseNone );
	case 1:		return tMapTerrain( kTerrainSpace2|kTerrainSpace4|kTerrainSpace6, kPlanetNone, kBaseNone );
	case 2:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseNone );
	case 3:		return tMapTerrain( kTerrainAsteroids1|kTerrainAsteroids3|kTerrainAsteroids5, kPlanetNone, kBaseNone );
	case 4:		return tMapTerrain( kTerrainAsteroids2|kTerrainAsteroids4|kTerrainAsteroids6, kPlanetNone, kBaseNone );
	case 5:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseNone );
	case 6:		return tMapTerrain( kTerrainNebula1|kTerrainNebula3|kTerrainNebula5, kPlanetNone, kBaseNone );
	case 7:		return tMapTerrain( kTerrainNebula2|kTerrainNebula4|kTerrainNebula6, kPlanetNone, kBaseNone );
	case 8:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseNone );
	case 9:		return tMapTerrain( kTerrainBlackHole1|kTerrainBlackHole3|kTerrainBlackHole5, kPlanetNone, kBaseNone );
	case 10:		return tMapTerrain( kTerrainBlackHole2|kTerrainBlackHole4|kTerrainBlackHole6, kPlanetNone, kBaseNone );
	case 11:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseNone );
	case 12:		return tMapTerrain( kTerrainDustclouds, kPlanetNone, kBaseNone );
	case 13:		return tMapTerrain( kTerrainShippingLane, kPlanetNone, kBaseNone );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_17Patrol::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_17Patrol::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.55f, 1.155f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.575f, 1.175f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.6f, 1.2f );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tMet_17Patrol::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )

	{
		for ( int32 i = kDefendTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefing_msg;
		}
	}
}