/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerTeam.h"
#include "CommonVictoryState.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "CommonEnemyTeam.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState(	int32 astoundingVictory,
											int32 victory,
											int32 draw,
											int32 defeat,
											int32 devastatingDefeat ) : tChildActorVictoryState< tCommonVictory >(	astoundingVictory,
																													victory,
																													draw,
																													defeat,
																													devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled()))) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}
void tCommonVictoryState::mEndState()
{}

eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* thisTeam = fMissionInfo->mGetTeamHandle( ID );

	if (!thisTeam) return kDraw;
	if (gWC.fDraftedHumans > 0) {
		if (thisTeam->mShipsCurrentlyControlled() > 0) return kAstoundingVictory;
		else return kDefeat;
	} else {
		if (thisTeam->mShipsCurrentlyControlled() > 0) return kVictory;
		else return kDevastatingDefeat;
	}

/**********START ORIGINAL VERSION ****************
	//
	//	Add up the hull integrity of each ship on the team.  Then get a percentage and distribute
	//	that amount to any bonus prestige points, if any.
	//
	tCommonPlayerTeam* team = dynamic_cast< tCommonPlayerTeam* >( fMissionInfo->mGetTeamHandle( ID ) );
	float teamHealth = 0.0f;
	if ( team )
	{
		float tempTotal = 0.0f;
		for ( tShipIterator ship = team->mGetFirstShip(); *ship != NULL; ship++ )
		{
			tempTotal += ship->mGetDamagePercent();
		}
		teamHealth = ( ( 100.0f - tempTotal ) / team->mNumShips() );
	}
	else 
		return kDraw;	//	default victory level


	//
	//	Now figure out which victory level this team should be.
	//
	if ( gWC.mTeamWon( ID ) )
	{
		if ( !gWC.mWasCaptured( ID ) )
			return kAstoundingVictory;
		else
			return kVictory;			
	}
	else
	{
		if ( !gWC.mWasCaptured( ID ) )
			return kDefeat;
		else
			return kDevastatingDefeat;
	}

	return kDraw;
 **********END ORIGINAL VERSION ****************/
}

//
//	Ship events
//

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;
	
	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(600);
				mAddToBonusPrestige(60);
			}
		}
	}

	if ( !team->mShipsCurrentlyControlled() )
	{
		gWC.mSetTeamWon( teamID, kFalse );
//		gScriptInterface.mSetGameStatus( teamID, kPlayerDestroyed );
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 30 );
	}

	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(900);
				mAddToBonusPrestige(90);
			}
		}
	}


	if ( !team->mShipsCurrentlyControlled() )
	{
		gWC.mSetTeamWon( teamID, kFalse );
//		gScriptInterface.mSetGameStatus( teamID, kPlayerCaptured );
	}

	//
	//	Let's subtract prestige points from this team's score and add them to the other team.
	//
	gWC.mSetWasCaptured( teamID, kTrue );

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	if ( !team )
		return;

	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(300);
				mAddToBonusPrestige(30);
			}
		}
	}


	if ( !team->mShipsCurrentlyControlled() )
	{
		gWC.mSetTeamWon( teamID, kFalse );
//		gScriptInterface.mSetGameStatus( teamID, kPlayerDisengaged );
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{
}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
