/********************************************************************************************************\

	File Name:	CommonPlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonPlayerShip.h"
#include "CommonPlayerTeam.h"
#include "CommonPlayerTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


tClassAllocator<tCommonPlayerTeam> gCommonPlayerTeamAllocator;

tCommonPlayerTeam::tCommonPlayerTeam() : fNumShips( 0 )
{}

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tCommonPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCommonPlayerTeam::mSetupState()
{
	mRegisterState( new tCommonPlayerTeamBaseState() );
	mGotoState( typeid( tCommonPlayerTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tCommonPlayerTeam::mSetupCommState()
{}


void tCommonPlayerTeam::mCreateShipsForTeam()
{
    if (! gScriptInterface.mIsHost() ) return;
    gST.mDebug_fprintf("--- Entering: tCommonPlayerTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

	if ((gWC.fDraftedHumans == 0) && (!mIsTeamHumanControlled())) {
		mCreateShip(
			typeid( tCommonPlayerShip ), 
			fShipType, 
			kStartPosition_G + fTeamID,
			fMetaShipID,
			gWC.fNWCustID++, 0, 0, -1, -1,
			"",
			kDefaultShipOptions );

		if (
			(mGetRace() == kKlingon) || 
			(mGetRace() == kFederation) || 
			(mGetRace() == kMirak) || 
			(mGetRace() == kHydran)
			) return;

		if (fNumShips > 1) {
			mCreateShip(
				typeid( tCommonPlayerShip ), 
				fShipType2, 
				kStartPosition_G + fTeamID,
				fMetaShipID2,
				gWC.fNWCustID++, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
	}
	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftingBPV += BPV;
	}

	gST.mDebug_fprintf("--- Finished: tCommonPlayerTeam::mCreateShipsForTeam\n");
}

void tCommonPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( mIsTeamHumanControlled() )
	{
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	//
	//	Set this team to 'won'.  All teams that are in the game should be set to this value.
	//
	gWC.mSetTeamWon( mGetTeam(), kTrue );
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tCommonPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tCommonPlayerTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	fNumShips++;
    int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
        metaid = kNoMetaShipID;
    }
#endif

	if (gWC.fNumPlayerShips < 9) {
		gWC.fPlayerShips[gWC.fNumPlayerShips] = shipScriptDescription.fClassName;
		gWC.fPlayerShipBPVs[gWC.fNumPlayerShips++] = gGetCombatBPV(shipScriptDescription.fClassName);
	}
	if (!mIsTeamHumanControlled()) {
		fTeamID = shipScriptDescription.fTeamID;
		if (fNumShips < 2) {
			fMetaShipID = metaid;
			strcpy(fShipType, shipScriptDescription.fClassName.c_str());
		} else if (fNumShips < 3) {
			fMetaShipID2 = metaid;
			strcpy(fShipType2, shipScriptDescription.fClassName.c_str());
		} else if (fNumShips < 4) {
			fMetaShipID3 = metaid;
			strcpy(fShipType3, shipScriptDescription.fClassName.c_str());
		}
	} else {
		mCreateShip(
			typeid( tCommonPlayerShip ), 
			shipScriptDescription.fClassName.c_str(), 
			kStartPosition_G + shipScriptDescription.fTeamID,
			metaid,
			gWC.fNWCustID++, 0, 0, -1, -1,
			shipScriptDescription.fCustomName.c_str(),
			kDefaultShipOptions );

		gWC.fDraftingHumans++;
		fNumShips++;
	}

	gST.mDebug_fprintf("--- Finished: tCommonPlayerTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}





void tCommonPlayerTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonPlayerTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonPlayerTeam::mNumShips()
{
	return fNumShips;
}

void tCommonPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}