//	<Name>	DefendVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "DefendVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_10Patrol.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "DefendTeam.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "WorldConditions.h"

tDefendVictoryState::tDefendVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tDefendVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );
	attackTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam2 ) );
	attackTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam3 ) );
			
	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	defendTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam2 ) );
	defendTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam3 ) );
	
	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 1;
	fTeamDisengaged = false;
	fPlayerLeftAlready = false;
	fThisPlayerDead = false;

	if (gWC.fBadCRC) {
		gWC.mMyInvokeGameStatus( fTeam->mGetTeam(), kLeftEarly );
	}
	if (fTeam->mIsTeamHumanControlled()) {
		tTeamInfo *enemyteam1 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam1));
		tTeamInfo *enemyteam2 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam2));
		tTeamInfo *enemyteam3 = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kAttackTeam3));
		if (((enemyteam1) && (enemyteam1->mIsTeamHumanControlled())) ||
			((enemyteam2) && (enemyteam2->mIsTeamHumanControlled())) ||
			((enemyteam3) && (enemyteam3->mIsTeamHumanControlled()))) {
			mAddToPrestige( kAstoundingVictory, gWC.fAstoundingVictoryPvPBonus );
			mAddToPrestige( kVictory, gWC.fVictoryPvPBonus );
			mAddToPrestige( kDraw, gWC.fDrawPvPBonus );
			mAddToPrestige( kDefeat, gWC.fDefeatPvPBonus );
			mAddToPrestige( kDevastatingDefeat, gWC.fDevastatingDefeatPvPBonus );
		}
	}
}

void tDefendVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );

	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}
	if( attackTeam2 )
	{
		attackBPV += attackTeam2->mGetCombatBPV();
	}
	if( attackTeam3 )
	{
		attackBPV += attackTeam3->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( defendTeam2 )
	{
		defendBPV += defendTeam2->mGetCombatBPV();
	}
	if( defendTeam3 )
	{
		defendBPV += defendTeam3->mGetCombatBPV();
	}
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(25);
	mAddToPrestige(25);
}

void tDefendVictoryState::mEndState()
{
}

eVictoryLevels tDefendVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kOrionTeam ) );
	tTeamInfo* thisTeam = fMissionInfo->mGetTeamHandle( ID );

	if (!thisTeam) return kDraw;
	else if (thisTeam->mShipsCurrentlyControlled() > 0) {
		if (gWC.fPirateUsed) return kAstoundingVictory;
		else return kVictory;
	} else {
		if (gWC.fPirateUsed && orionTeam && (orionTeam->mShipsCurrentlyControlled() < 1)) return kDefeat;
		else return kDevastatingDefeat;
	}

}
int32 tDefendVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	if ((teamID->mGetTeam() == kOrionTeam) && (!gWC.fPirateUsed)) return 0;

	return teamShipsInPlay;
}
void tDefendVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	if (!ship) return;
	eTeamID		teamID = ship->mGetOriginalTeam();
	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kOrionTeam ) );

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	iTruth checkend = kFalse;
	if( orionTeam && ship->mGetOriginalTeam() == orionTeam->mGetTeam() )
	{
		checkend = kTrue;
		if (orionTeam->mShipsCurrentlyControlled() < 1)
			mAddToBonusPrestige( orionTeam->mGetCombatBPV() / 2 );	
	}

	if( fTeam && fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
			fThisPlayerDead = true;
			gWC.mRemoveDefendTeam();
			fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
			checkend = kTrue;
	}
	if (checkend) gWC.mCheckForGameEnd();
}
void tDefendVictoryState::mOnDeath( tShipInfo* ship )
{
	int32 thisShipID;

	if (!ship) return;
	eTeamID teamID = ship->mGetTeam();
	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(600);
				mAddToBonusPrestige(60);
			}
		}
	}
	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
		case kAttackTeam3:
		//Empty
		break;
		case kDefendTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		case kDefendTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		case kDefendTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		default:
		break;
	}

	mCheckAllTeams( ship );
}
void tDefendVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	if (!ship) return;
	eTeamID teamID = ship->mGetTeam();
	if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
		tTeamInfo* myteam = fChildActor->mGetTeam();
		if (myteam) {
			eTeamID myteamID = myteam->mGetTeam();
			if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
				if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(900);
				mAddToBonusPrestige(90);
			}
		}
	}
	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
		case kAttackTeam3:
			//Empty
		break;
		case kDefendTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		case kDefendTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		case kDefendTeam3:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		default:
		break;
	}

	mCheckAllTeams( ship );
}

void tDefendVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if (!ship) return;
	gWC.mRestoreSpares(ship);
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if (ship->mGetTeam() == ship->mGetOriginalTeam()) {
			tTeamInfo* myteam = fChildActor->mGetTeam();
			if (myteam) {
				eTeamID myteamID = myteam->mGetTeam();
				if (gWC.mShipIsMyEnemy(teamID, myteamID)) {
					if (gWC.fDraftedHumans > 0) mAddToBonusPrestige(300);
					mAddToBonusPrestige(30);
				}
			}
		}
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveDefendTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				//gWC.mCheckForGameEnd();
				mSetTimedInterval(kTimerSecondsSet, kEndingID, 5.0f);
			}
		}
	}
		
}
void tDefendVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tDefendVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tDefendVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveDefendTeam();
		fPlayerLeftAlready = true;
		mSetTimedInterval(kTimerSecondsSet, kEndingID, 5.0f);
	}
}
