/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}
void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}
void tOrionTeam::mSetupCommState()
{
}

void tOrionTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	int32 orionBPV = 60;
	if (playerTeam) orionBPV = playerTeam->mGetCombatBPV() - (gWC.fMissionSeed % 23);

	if ( (orionBPV < 75) || (gWC.fDraftedHumans > 0) ) {
		gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
		return;
	}

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;
	int32 selection = gWC.fMissionSeed % 37;


   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;
   		ShipList orion_sl = gST.narrowed_sl;

		if ((selection >= 0) && (selection <= 16)) {
			// 1 Monster. Medium or small, or andro up to BCH
			eRace race = kMonster;

			std::list< eRace >  raceList;
			raceList.push_back(race);
			fShiplistCount = orion_sl.Subset(raceList);
			gST.mDebug_fprintf("subset for kMonster race filter completed. ships: %d\n", fShiplistCount);

			orion_sl = gST.stripOutSPECIAL(orion_sl);
			orion_sl = gST.stripOutRestricted(orion_sl);

			std::list< eEnhancedClassTypes >  monsterClassTypeList;
			monsterClassTypeList.push_back(kEClassSmallMonster);
			monsterClassTypeList.push_back(kEClassMediumMonster);
			monsterClassTypeList.push_back(kEClassLargeMonster);
			for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
	            	monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
			fShiplistCount = orion_sl.Subset(monsterClassTypeList, true);
			gST.mDebug_fprintf("subset for monster classes filter completed. ships: %d\n", fShiplistCount);

			fShiplistCount = orion_sl.Subset( orion_sl.kSubsetBPV, int(0.8 * orionBPV), int(1.2 * orionBPV), true);
			gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(0.8 * orionBPV), int(1.2 * orionBPV), fShiplistCount);

			gST.listShiplistContent(orion_sl);

			Ship s1;
			if ((orion_sl.size() > 0) && (orion_sl.SelectRandomShip(s1))) {
				gST.mDebug_fprintf("Monster selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );
				mCreateShip(
					typeid( tShipInfo ),
					s1.shipCode.c_str(),
					kStartPosition_Z,
					kNoMetaShipID,
					-1, 0, 0, -1, -1,
					"Nemesis",
					static_cast<eShipOptions>(kDeathUponCapture|kStartPositionCanBeOffset) );
			} else {
				mOldCreateShipsForTeam();
			}

		} else if ((selection >= 19) && (selection <= 23)) {
			// Orion Base
			eRace race = kOrion;

			std::list< eRace >  raceList;
			raceList.push_back(race);
			fShiplistCount = orion_sl.Subset(raceList);
			gST.mDebug_fprintf("subset for kOrion race filter completed. ships: %d\n", fShiplistCount);

			orion_sl = gST.stripOutRestricted(orion_sl);

            std::list< eEnhancedClassTypes >  pirate_bases_enh_class;
            pirate_bases_enh_class.push_back(kEClassMobileBase);
            pirate_bases_enh_class.push_back(kEClassBaseStation);
            pirate_bases_enh_class.push_back(kEClassBattleStation);
            fShiplistCount = orion_sl.Subset(pirate_bases_enh_class, true);
            gST.mDebug_fprintf("subset for eEnhancedClassTypes for bases completed. bases: %d\n", fShiplistCount);

			gST.listShiplistContent(orion_sl);

			Ship s1;
			if ((orion_sl.size() > 0) && (orion_sl.SelectRandomShip(s1))) {
				gST.mDebug_fprintf("Base selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );

				mCreateShip(
					typeid( tShipInfo ),
					s1.shipCode.c_str(),
					kStartPosition_Z,
					kNoMetaShipID,
					-1, 0, 0, -1, -1,
					"Eden II",
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
			} else {
				mOldCreateShipsForTeam();
			}

		} else if ((selection >= 24) && (selection <= 28)) {
			// pirate convoy, freighters
			eRace race = kOrion;

			std::list< eRace >  raceList;
			raceList.push_back(race);
			fShiplistCount = orion_sl.Subset(raceList);
			gST.mDebug_fprintf("subset for kOrion race filter completed. ships: %d\n", fShiplistCount);

			orion_sl = gST.stripOutSPECIAL(orion_sl);
			orion_sl = gST.stripOutRestricted(orion_sl);

            // keep only freighters.
            std::list< eEnhancedClassTypes >  enhancedClassTypeListF;
            enhancedClassTypeListF.push_back(kEClassSmallFreighter);
            enhancedClassTypeListF.push_back(kEClassPriorityTransport);
            enhancedClassTypeListF.push_back(kEClassCivilianFreighter);
            enhancedClassTypeListF.push_back(kEClassLargeFreighter);
            enhancedClassTypeListF.push_back(kEClassHeavyFreighter);
            fShiplistCount = orion_sl.Subset(enhancedClassTypeListF);
            gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

            // no Qship role for this ship
            std::list< eEnhancedRoles >  qshipList;
            qshipList.push_back(kQship);
            fShiplistCount = orion_sl.Subset(qshipList, false);
            gST.mDebug_fprintf("subset for NO qship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

			// CARGO role
			std::list< eEnhancedRoles >  CargoList;
			CargoList.push_back(kCargo);
			fShiplistCount = orion_sl.Subset(CargoList);
			gST.mDebug_fprintf("subset for Cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
            gST.listShiplistContent(orion_sl);
	
			if (orion_sl.size() > 0) {
				Ship s1;
				for (int i = 1; i <= 5; i++) {
					orion_sl.SelectRandomShip(s1);

					char digit[1];
					itoa(i, digit, 10);
					std::string shipname = "Convoy ";
					shipname += digit;

					gST.mDebug_fprintf("Convoy ship '%s' selected! %s:  %d BPV\n", shipname.c_str(), s1.shipCode.c_str(), s1.bpv);
					gST.mTrackOrions( s1 );

					mCreateShip(
						typeid( tShipInfo ),
						s1.shipCode.c_str(),
						kStartPosition_Z,
						kNoMetaShipID,
						-1, 0, 0, -1, -1,
						shipname.c_str(),
						static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );

				}
			} else {
				mOldCreateShipsForTeam();
			}

		} else {
			// pirate OR Monster ship named Nemesis  (Andro!)  (OP+ enhancement)
			std::list< eRace >  raceList;
			raceList.push_back(kOrion);
			raceList.push_back(kMonster);
			fShiplistCount = orion_sl.Subset(raceList);
			gST.mDebug_fprintf("subset for kOrion/kMonster race filter completed. ships: %d\n", fShiplistCount);

			orion_sl = gST.stripOutSPECIAL(orion_sl);
			orion_sl = gST.stripOutRestricted(orion_sl);

			std::list< eEnhancedClassTypes >  orionClassTypeList;
			for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
	            	orionClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
			fShiplistCount = orion_sl.Subset(orionClassTypeList, true);
			gST.mDebug_fprintf("subset for ships completed. ships: %d\n", fShiplistCount);

            std::list< eEnhancedRoles >  warRolesList;
            warRolesList.push_back(kLineShip);
            warRolesList.push_back(kWarShip);
            warRolesList.push_back(kLeader);
            warRolesList.push_back(kDroneBombardmentShip);
            warRolesList.push_back(kEscort);
            warRolesList.push_back(kCarrier);
            warRolesList.push_back(kPFTender);
            warRolesList.push_back(kFastCruiser);
            warRolesList.push_back(kMauler);
            warRolesList.push_back(kPolice);
            fShiplistCount = orion_sl.Subset(warRolesList, true);
            gST.mDebug_fprintf("subset for warship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

			fShiplistCount = orion_sl.Subset( orion_sl.kSubsetBPV, int(0.8 * orionBPV), int(1.2 * orionBPV), true);
			gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(0.8 * orionBPV), int(1.2 * orionBPV), fShiplistCount);

            gST.listShiplistContent(orion_sl);


			Ship s1;
			if ((orion_sl.size() > 0) && (orion_sl.SelectRandomShip(s1))) {
				gST.mDebug_fprintf("Convoy ship %d selected! %s:  %d BPV\n", i, s1.shipCode.c_str(), s1.bpv);
				gST.mTrackOrions( s1 );

				mCreateShip( 
					typeid( tShipInfo ),
					s1.shipCode.c_str(),
					kStartPosition_Z,
					kNoMetaShipID,
					-1, 0, 0, -1, -1, 
					"Nemesis", 
					static_cast<eShipOptions>( kDeathUponCapture ) );
			} else {
				mOldCreateShipsForTeam();
			}
		}
		
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}

void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	int32 orionBPV = 60;
	if (playerTeam) orionBPV = playerTeam->mGetCombatBPV() - (gWC.fMissionSeed % 23);
	if (orionBPV < 75) return;
	if (gWC.fDraftedHumans > 0) return;
	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;
	int32 selection = gWC.fMissionSeed % 37;

	switch (selection) {
	case 0: mCreateShip( typeid( tShipInfo ), "M-AMS", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 1: mCreateShip( typeid( tShipInfo ), "M-AMM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 3:
		mCreateShip( typeid( tShipInfo ), "M-LCS", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 4:
		mCreateShip( typeid( tShipInfo ), "M-LCM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 6:
		mCreateShip( typeid( tShipInfo ), "M-MTS", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 7:
		mCreateShip( typeid( tShipInfo ), "M-MTM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
		break;
	case 9:
		mCreateShip( typeid( tShipInfo ), "M-SGS", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
			break;
	case 10:
		mCreateShip( typeid( tShipInfo ), "M-SGM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
			break;
	case 15:
		mCreateShip( typeid( tShipInfo ), "M-SSS", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
			break;
	case 16:
		mCreateShip( typeid( tShipInfo ), "M-SSM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture ); // The only one used
			break;
	
	case 19:
	case 20:
	case 21:
	case 22:
	case 23:
		// generate a pirate base
		mCreateShip( typeid( tShipInfo ),
					orionBPV,
					kClassBaseStation,
					kClassStarBase,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Eden II", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
		break;
	case 24:
	case 25:
	case 26:
	case 27:
	case 28: // generate a pirate convoy
		mCreateShip( typeid( tShipInfo ),
					25,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 1", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		mCreateShip( typeid( tShipInfo ),
					30,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 2", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		mCreateShip( typeid( tShipInfo ),
					35,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 3", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		mCreateShip( typeid( tShipInfo ),
					40,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 4", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		mCreateShip( typeid( tShipInfo ),
					45,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 5", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		break;
	default:
		// generate a pirate ship
		mCreateShip( typeid( tShipInfo ),
					orionBPV,
					kClassFrigate,
					kClassBattleship,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Nemesis", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
		break;
	}

	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}
void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Not actually drafting. Just gathering data.

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}
