/********************************************************************************************************\

	File Name:	AttackShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackShipBaseState.h"
#include "Met_7BaseAssault.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include "CommonDefs.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAttackShipBaseState::mBeginState()
{
	fChildActor->mSetShipCallbackEvents(kNotifyDeepScan);
}

void tAttackShipBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}


void tAttackShipBaseState::mEndState()
{
}

void tAttackShipBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAttackShipDestroyed_msg, kNonCommScriptMessage );
		}
	}
}
void tAttackShipBaseState::mOnCapture( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAttackShipCaptured_msg, kNonCommScriptMessage );
		}
	}
}
void tAttackShipBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( fChildActor->mGetShipID() == ship->mGetShipID() )
	{
		for ( int32 i = kAttackTeam1; i <= kDefendTeam3; i++ )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kAttackShipDisengaged_msg, kNonCommScriptMessage );
		}
	}
}
void tAttackShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tAttackShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tAttackShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tAttackShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if (!ship) return;
	if (ship->mGetShipID() == fChildActor->mGetShipID()) {
		// this ship just turned on or off it's scanners
		if (deepScanOn) {
			// see if we're close enough to a target to count it as scanned
			if (ship->mGetShipTargetID() == gWC.fTargetIDs[0]) {
				// we know which installation we've targetted
				if (gWC.fTargetScanned[0]) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(), "The target has already been scanned sir", kNonCommScriptMessage );
				} else if (fChildActor->mGetTacticalDistance(gWC.fTargets[0]) < 5) {
					gWC.fTargetScanned[0] = true;
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(), "Target scanned and data recorded sir", kNonCommScriptMessage );
				} else {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(), "We are still too far away sir, I suggest we move closer and try again", kNonCommScriptMessage );
				}
			}

		}

		// remember to let us know the next time the scanners go on or off
		ship->mSetShipCallbackEvents(kNotifyDeepScan);
	}
}
void tAttackShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAttackShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAttackShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAttackShipBaseState::mOnFlashingButtonPressed()
{}
void tAttackShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAttackShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tAttackShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tAttackShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tAttackShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tAttackShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tAttackShipBaseState::mOnHostChanged()
{}
void tAttackShipBaseState::mOnTimedInterval( int32 timerID )
{}
void tAttackShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAttackShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

