/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

/*

	HELPER NOTE:	Add or delete from this list.  It is not necessary to have these exact maps in each
					multiplayer mission; however, each map should have valid starting positions for each
					of the 6 teams.

*/
int32 TotalMaps = 8;
const char* MapTypes[] = { "Open Space", "Asteroids", "Sun",
							"Blackhole", "Blackhole planet", "Nebula", "Nebula planet", "Moon" };
const char* MapLines[]=
{
	// Asteroids: a,A,*
	// Black Holes: b,B
	// Nebula: c,C
	// Dust clouds: d,D
	// Start positions: G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z
	// End positions:   g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z
	// Planets: 1-6
	// Suns: 7-9
	//
   "Met_7BaseAssault",
   "// Open Space",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |................Ss.................|",
   " |...................................|",
   "1|...................................|",
   " |.........Y..............y..........|",
   " |..............Tt...................|",
   " |...................................|",
   " |........W..........................|",
   "2|.....X..................w..........|",
   " |..........Z............xz..........|",
   " |...................................|",
   " |.......Uu..........................|",
   " |..................h................|",
   "3|.Vv...........g.i..................|",
   " |...................................|",
   " |...................................|",
   " |................IGH................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Asteroids",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |........TT.........................|",
   "1|...Ss.........Uu...................|",
   " |.........Y.........................|",
   " |...................Vv..............|",
   " |................[..................|",
   " |........W...Z......................|",
   "2|.....X.............................|",
   " |............[..[..*................|",
   " |...................................|",
   " |...........]...*...................|",
   " |..................h................|",
   "3|...........*..g....................|",
   " |...................................|",
   " |........*..]...iGH.................|",
   " |...............I...................|",
   " |........]..........................|",
   "4|.........[.........................|",
   " |...................................|",
   " |.......]...........................|",
   " |...................................|",
   " |...................................|",
   "5|.wxyz..............................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Sun",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "1|.......................Ss..........|",
   " |.........Y......[tT................|",
   " |...................[..[............|",
   " |...................................|",
   " |........W.....Uu........w..........|",
   "2|.....X..................x..........|",
   " |...Z....................y..........|",
   " |.................Vv.....z..........|",
   " |........]..[.]......]..............|",
   " |.................].................|",
   "3|...........[.igh...................|",
   " |....]....].........................|",
   " |................GHI................|",
   " |...................................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "5|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Blackhole",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "1|....Z..............................|",
   " |.........Y.........................|",
   " |...................................|",
   " |...................................|",
   " |........W..........................|",
   "2|.....X.............................|",
   " |...................................|",
   " |..................Ss...............|",
   " |...................................|",
   " |........,...............x..........|",
   "3|.................Tt................|",
   " |........................z..........|",
   " |...................................|",
   " |........................y..........|",
   " |...ghi...........Uu................|",
   "4|...GHI.............................|",
   " |...................................|",
   " |...................................|",
   " |....................Vv.............|",
   " |...................................|",
   "5|............w......................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Blackhole planet",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |.............,.....................|",
   " |...................................|",
   " |...................................|",
   "1|....................GHI............|",
   " |.........Y.........ghi.............|",
   " |...................................|",
   " |........Z..........................|",
   " |........W..........................|",
   "2|.....X.............................|",
   " |...................................|",
   " |.....................Ss............|",
   " |...................................|",
   " |...................................|",
   "3|...................................|",
   " |.....................Tt............|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "4|..................Uu...............|",
   " |...................................|",
   " |...........Vv......................|",
   " |...................................|",
   " |...................................|",
   "5|...............wxyz................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Nebula",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |.?.................................|",
   " |...................................|",
   " |...................................|",
   "1|...................................|",
   " |.........Y.........................|",
   " |........................w..........|",
   " |.......W...........................|",
   " |...................................|",
   "2|.....X.............................|",
   " |..........Z.............y..........|",
   " |...................................|",
   " |...................................|",
   " |..................h................|",
   "3|...Ss.........gi..GHI..............|",
   " |...................................|",
   " |...................................|",
   " |......Tt...........................|",
   " |...................................|",
   "4|...................................|",
   " |..........Uu.......................|",
   " |...................................|",
   " |...............Vv..................|",
   " |...................................|",
   "5|...x......z........................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Nebula planet",
   " +____1____2____3____4____5____6____7+",
   " |..................z................|",
   " |.&.................................|",
   " |...................................|",
   " |...................................|",
   "1|...................................|",
   " |.........Y..............y..........|",
   " |...................................|",
   " |.......W...........................|",
   " |...................................|",
   "2|.....X.............................|",
   " |..........Z........................|",
   " |........................w..........|",
   " |...................................|",
   " |..................h................|",
   "3|....Uu........gi..GHI..............|",
   " |...................................|",
   " |...................................|",
   " |......Ss...Tt......................|",
   " |...................................|",
   "4|...................................|",
   " |...................................|",
   " |.........Vv........................|",
   " |...................................|",
   " |...................................|",
   "5|..........x........................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "// Moon",
   " +____1____2____3____4____5____6____7+",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "1|...................................|",
   " |..Ss.....Y..............w..........|",
   " |...................................|",
   " |.......W...........................|",
   " |...................................|",
   "2|.....X..........Vv.................|",
   " |..........Z........................|",
   " |...................................|",
   " |...................................|",
   " |.Tt...............h................|",
   "3|..............gi..GHI..............|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |....Uu.............................|",
   "4|y..................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "5|...z.......x.......................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "6|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   "7|...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " |...................................|",
   " +-----------------------------------+",
   "      1    2    3    4    5    6    7",
   "///",
   0
};
