/********************************************************************************************************\

	File Name:	OTDTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Met_7BaseAssault.h"
#include "OTD.h"
#include "OTDTeam.h"
#include "OTDTeamBaseState.h"
#include "BaseVictory.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tOTDTeam> gOTDTeamAllocator;

tOTDTeam::tOTDTeam()
{
	fNumShips = 0;
}

void tOTDTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tOTDTeam::mSetupState()
{
	mRegisterState( new tOTDTeamBaseState() );
	mGotoState( typeid( tOTDTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tOTDTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/

void tOTDTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam()\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;
   		ShipList planet_sl = gST.narrowed_sl;
        eRace race = mGetRace();
        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = planet_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race (%d) filter completed. planets: %d\n", race, fShiplistCount);

        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  planetRolesList;
        gST.mDebug_fprintf("Selecting unarmed planet role.\n");
        planetRolesList.push_back(kArmedWorld);
        planetRolesList.push_back(kHomeWorld);
        fShiplistCount = planet_sl.Subset(planetRolesList, false);
        gST.mDebug_fprintf("subset for unarmed planets completed. ships: %d\n", fShiplistCount);

		{
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);

        
        if (planet_sl.size() > 0) {
			Ship s1, s2, s3, s4;

			planet_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("Planet 1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 );
			mCreateShip( 
				typeid( tOTD ),
				s1.shipCode.c_str(), 
				kStartPosition_S,
				kNoMetaShipID,
				kOTD, 0, 0, -1, -1,
				"Target S",
				kDeathUponCapture );

			planet_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("Planet 2 selected! %s:  %d BPV\n", s2.shipCode.c_str(), s2.bpv);
			gST.mTrackOrions( s2 );
			mCreateShip( 
				typeid( tOTD ),
				s2.shipCode.c_str(), 
				kStartPosition_T,
				kNoMetaShipID,
				kOTD, 0, 0, -1, -1,
				"Target T",
				kDeathUponCapture );

			planet_sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("Planet 3 selected! %s:  %d BPV\n", s3.shipCode.c_str(), s3.bpv);
			gST.mTrackOrions( s3 );
			mCreateShip( 
				typeid( tOTD ),
				s3.shipCode.c_str(), 
				kStartPosition_U,
				kNoMetaShipID,
				kOTD, 0, 0, -1, -1,
				"Target U",
				kDeathUponCapture );

			planet_sl.SelectRandomShip(s4);
            gST.mDebug_fprintf("Planet 4 selected! %s:  %d BPV\n", s4.shipCode.c_str(), s4.bpv);
			gST.mTrackOrions( s4 );
			mCreateShip( typeid( tOTD ),
				s4.shipCode.c_str(), 
				kStartPosition_V,
				kNoMetaShipID,
				kOTD, 0, 0, -1, -1,
				"Target V",
				kDeathUponCapture );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateShipsForTeam()\n");
}

void tOTDTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tOTDTeam::mOldCreateShipsForTeam()\n");

	mCreateShip( typeid( tOTD ),
		gWC.fTargetTypes.c_str(), 
		kStartPosition_S,
		kNoMetaShipID,
		kOTD, 0, 0, -1, -1,
		"Target S",
		kDeathUponCapture );
	mCreateShip( typeid( tOTD ),
			gWC.fTargetTypes.c_str(), 
			kStartPosition_T,
			kNoMetaShipID,
			kOTD, 0, 0, -1, -1,
			"Target T",
			kDeathUponCapture );
	mCreateShip( typeid( tOTD ),
			gWC.fTargetTypes.c_str(), 
			kStartPosition_U,
			kNoMetaShipID,
			kOTD, 0, 0, -1, -1,
			"Target U",
			kDeathUponCapture );
	mCreateShip( typeid( tOTD ),
			gWC.fTargetTypes.c_str(), 
			kStartPosition_V,
			kNoMetaShipID,
			kOTD, 0, 0, -1, -1,
			"Target V",
			kDeathUponCapture );

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mOldCreateShipsForTeam()\n");
}

void tOTDTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tOTDTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// not actually drafting, but the info gathered is useful.

	gST.mDebug_fprintf("--- Finished: tOTDTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

eClassTypes tOTDTeam::mTranslateHexIntoType( tMetaLocationSpec hex )
{
	eClassTypes returnClass = kClassStarBase;

	if ( hex.fLocationObjectFlags & kBaseStationHex )
		returnClass = kClassBaseStation;
	else if ( hex.fLocationObjectFlags & kBattleStationHex )
		returnClass = kClassBattleStation;
	else if ( hex.fLocationObjectFlags & kStarbaseHex )
		returnClass = kClassStarBase;

	return returnClass;
}

void tOTDTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}