/********************************************************************************************************\

	File Name:	DefendTeam3.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "DefendTeam3.h"
#include "DefendTeamInitialState.h"
#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "DefendTeam3InitialState.h"
#include "DefendVictoryCondition.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tDefendTeam3> gDefendTeam3Allocator;

tDefendTeam3::tDefendTeam3()
{
	gWC.mAddDefendTeam();
}

void tDefendTeam3::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam3::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tDefendTeam3::mSetupState()
{
	mRegisterState( new tDefendTeam3InitialState() );
	mGotoState( typeid( tDefendTeam3InitialState ) );
}
void tDefendTeam3::mSetupCommState()
{
}
void tDefendTeam3::mCreateShipsForTeam()
{
  	gST.mDebug_fprintf("--- Entering: tDefendTeam3::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gWC.fDraftedHumans == 0) && (!mIsTeamHumanControlled())) {
		mCreateShip(	typeid( tDefendTeamShip ), 
					fShipType, 
					kStartPosition_I,
					fMetaShipID,
					gWC.fNWCustID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );

	}
	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftingBPV += BPV;
	}
	gST.mDebug_fprintf("--- Finished: tDefendTeam3::mCreateShipsForTeam\n");
}
void tDefendTeam3::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam3::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tDefendTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
        metaid = kNoMetaShipID;
    }
#endif

	if (!mIsTeamHumanControlled()) {
		strcpy(fShipType, shipScriptDescription.fClassName.c_str());
		fTeamID = shipScriptDescription.fTeamID;
		fMetaShipID = metaid;
	} else {
		gWC.fDraftingHumans++;
		mCreateShip( 
			typeid( tDefendTeamShip ), 
			shipScriptDescription.fClassName.c_str(), 
			kStartPosition_I,
			metaid,
			gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
			shipScriptDescription.fCustomName.c_str(),
			static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
	}

   	gST.mDebug_fprintf("--- Finished: tDefendTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tDefendTeam3::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}