/********************************************************************************************************\

	Mission:	Patrol
	Script:		"Met_10Patrol" //Enemy space version of patrol
	Descript:	<DESCRIPTION>

	Created by:		David F		06/19/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "AttackTeam3.h"
#include "OrionTeam.h"
#include "WorldConditions.h"
#include "Met_10Patrol.h"
#include "DefendTeamShip.h"
#include "AttackTeamShip.h"
#include <time.h>
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMet_10Patrol );

tMet_10Patrol::tMet_10Patrol()
{
	srand(time(NULL));
}
void tMet_10Patrol::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kPlayerRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fMinBPV = kMinBPV;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fWait	= 1;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kPlaceBaseMission;

	std::string friendlymsg("Patrol");
	std::string hostilemsg("Base Placement");
	float AIchance = 0.0f;
	if (gWC.fDisableAIDuringPvP) AIchance = 0.0f;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tDefendTeam ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryOpponentTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ),
														kMaxChance, kFalse,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,  kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam3 ),
														kNoTeamTag,
														hostilemsg );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
														kMaxChance,
														kFalse,
														kPlayableTeam,
														kMaxChance,
														kSpecPlayableRace,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam3 ),
														kNoTeamTag,
														friendlymsg );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Orion Team", static_cast< eTeamID>( kOrionTeam ), kMaxChance, kFalse,
														kNPCTeam,
														kMaxChance,
														kSpecOrion,
														kAnyRace,
														typeid( tOrionTeam ) );



	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}
int32 tMet_10Patrol::mInitializeStart( tDynaverseScriptInfo& info )
{
	gWC.mSetMapPicked(info.fMapPicked);
	return 1;
}
void tMet_10Patrol::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_10Patrol::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMet_10Patrol::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_10Patrol::mStartMission()
{
	tScript::mStartMission();
	gWC.mGetSettingsFromConfig();
	gWC.mStartMissionLog("Base placement ");
	gWC.mCheckCRC();

	const int kRelationsCount = 7;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
		kOrionTeam,		//Orn
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Def3	Atk2	Atk3	Orn
		/*Def1*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",	"WD",
		/*Atk1*/	"WD",	"--",	"WD",	"WD",	"AI",	"AI",	"AI",
		/*Def2*/	"AI",	"WD",	"--",	"AI",	"WD",	"WD",	"WD",
		/*Def3*/	"AI",	"WD",	"AI",	"--",	"WD",	"WD",	"WD",
		/*Atk2*/	"WD",	"AI",	"WD",	"WD",	"--",	"AI",	"AI",	
		/*Atk3*/	"WD",	"AI",	"WD",	"WD",	"AI",	"--",	"AI",
		/*Orn*/		"WD",	"AI",	"WD",	"WD",	"AI",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_10Patrol::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetNone, kBaseNone );
	case 1:		return tMapTerrain( kTerrainDustclouds | kTerrainShippingLane, kPlanetNone, kBaseNone );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetNone, kBaseNone );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetNone, kBaseNone );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetNone, kBaseNone );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_10Patrol::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;

}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_10Patrol::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.5f, 0.96f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.525f, 1.00f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.55f, 1.04f );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.10f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.575f, 1.08f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.55f, 1.06f );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.675f, 1.15f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.65f, 1.125f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.625f, 1.10f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.6f, 1.06f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.625f, 1.09f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.65f, 1.12f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.55f, 1.01f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.575f, 1.04f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.6f, 1.07f );
}
void tMet_10Patrol::mDefineBriefings( void )
{
	eRace lastteam = kMirak;


	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		fBriefingIndex[kDefendTeam1][j] = kBriefing_msg;
		fBriefingIndex[kDefendTeam2][j] = kBriefing_msg;
		fBriefingIndex[kDefendTeam3][j] = kBriefing_msg;
		fBriefingIndex[kAttackTeam1][j] = kShortBriefing_msg;
		fBriefingIndex[kAttackTeam2][j] = kShortBriefing_msg;
		fBriefingIndex[kAttackTeam3][j] = kShortBriefing_msg;
	}
} 