/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}

void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}

void tOrionTeam::mSetupCommState()
{
}

void tOrionTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;

		int32 orionBPV = 0;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam2 ) );
		int32 playerBPV = 75;
		if (playerTeam) playerBPV = 2 * playerTeam->mGetCombatBPV() / 3;
		if (playerBPV < 60) return;

		orionBPV = playerBPV;
		gWC.mSetOrionTeamInGame( true );
		gWC.fPirateUsed = kTrue;

   		ShipList pirate_sl = gST.narrowed_sl;

		// Orion OR Monster (Andros!)
        std::list< eRace >  raceList;
        raceList.push_back(mGetRace());
		raceList.push_back(kMonster);
        fShiplistCount = pirate_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for race completed. ships: %d\n", fShiplistCount);

        pirate_sl = gST.stripOutSPECIAL(pirate_sl);
        pirate_sl = gST.stripOutRestricted(pirate_sl);

        std::list< eEnhancedClassTypes >  pirateClassTypeList;
        { // no, I'm not redefining i. Local only please.
            for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
				pirateClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
            }
        }
        fShiplistCount = pirate_sl.Subset(pirateClassTypeList, true);
        gST.mDebug_fprintf("subset for pirate classes filter completed. ships: %d\n", fShiplistCount);

		pirate_sl = gST.selectWarShips( pirate_sl );

        int32 MinPirateBPV = int(orionBPV * 0.875);
        int32 MaxPirateBPV = int(orionBPV * 1.125);

		fShiplistCount = pirate_sl.Subset(ShipList::kSubsetBPV, MinPirateBPV, MaxPirateBPV);
		gST.mDebug_fprintf("BPV range subset: %d  (%d to %d)\n", fShiplistCount, MinPirateBPV, MaxPirateBPV);

		gST.listShiplistContent(pirate_sl);

		if (pirate_sl.size() > 0) {
			Ship s1;
			pirate_sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("Ship1 selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
			gST.mTrackOrions( s1 ); // the game strips the digit at the end! yuck!

			mCreateShip( 
				typeid( tShipInfo ),
				s1.shipCode.c_str(),
				kStartPosition_Z,
				kNoMetaShipID,
				-1, 0, 0, -1, -1, 
				NULL, 
				static_cast<eShipOptions>( kCanBeCarrier | kCanBeRestricted | kDeathUponCapture ) );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

   	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}

void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam::mOldCreateShipsForTeam\n");

	int32 orionBPV = 0;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam2 ) );
	int32 playerBPV = 75;
	if (playerTeam) playerBPV = 2 * playerTeam->mGetCombatBPV() / 3;
	if (playerBPV < 60) return;

		orionBPV = playerBPV;
		// orionBPV = orionBPV * gWC.fAIBPVModifier;
		gWC.mSetOrionTeamInGame( true );
		gWC.fPirateUsed = kTrue;

		mCreateShip( typeid( tShipInfo ),
							orionBPV,
							kClassFrigate,
							kClassBattleship,
							0,
							kStartPosition_Z, -1, 0, 0, -1, -1, NULL, 
							static_cast<eShipOptions>( kCanBeCarrier | kCanBeRestricted | kDeathUponCapture ) );	
		
	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );
	
	// no draft actually done.

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOrionTeam::mCreateOrions()
{

	int32 nStartPos = kStartPosition_Z;

	int32 bpv = 100;

	mCreateShip( typeid( tShipInfo ), bpv, kClassLightCruiser, 0, nStartPos, -1, 0,0,0,0, NULL, kDefaultShipOptions );

}

