/********************************************************************************************************\

	File Name:	AttackTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "AttackTeam.h"
#include "AttackTeamInitialState.h"
#include "AttackTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tAttackTeam> gAttackTeamAllocator;

tAttackTeam::tAttackTeam()
{
	gWC.mAddAttackTeam();
	fNumShips = 0;
}

void tAttackTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tAttackVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tAttackTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kAttackDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tAttackTeam::mSetupState()
{
	mRegisterState( new tAttackTeamInitialState() );
	mGotoState( typeid( tAttackTeamInitialState ) );
}
void tAttackTeam::mSetupCommState()
{
}

void tAttackTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {

		ShipList group_sl = gST.narrowed_sl;
		eRace race = mGetRace();
		int32 fShiplistCount = 0;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = group_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for %d race filter completed. ships: %d\n", race, fShiplistCount);

		group_sl = gST.stripOutSPECIAL(group_sl);
		group_sl = gST.stripOutRestricted(group_sl);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = group_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		group_sl = gST.selectWarShips( group_sl );

		if (group_sl.size() > 0) {
			int32 BPV = 150 - (10 * fNumShips);
			while (fNumShips < 4) {
				fNumShips++;
				BPV -= 10;

				gST.mDebug_fprintf("---- tAttackTeam::mCreateShipsForTeam generating ship #: %d\n", fNumShips);

				ShipList this_pass_sl = group_sl;
				fShiplistCount = this_pass_sl.Subset( this_pass_sl.kSubsetBPV, int(0.875 * BPV), int(1.125 * BPV), true);
				gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
					int(0.875 * BPV), int(1.125 * BPV), fShiplistCount);
				gST.listShiplistContent(this_pass_sl);

				Ship s1;
				this_pass_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("Attack ship %d selected! %s:  %d BPV\n", fNumShips, s1.shipCode.c_str(), s1.bpv);

				mCreateShip( typeid( tShipInfo ),
					s1.shipCode.c_str(),
					kStartPosition_W, 
					kNoMetaShipID,
					gWC.fNWCustID++, 
					0, 0, -1, -1, "", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
			}

			if (mShipsCurrentlyControlled() > 0) {
				int32 BPV = gGetCombatBPV(fShipType);
				if (BPV < 1) BPV = mGetCombatBPV();
				if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
				gWC.fDraftedBPV += BPV;
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

   	gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateShipsForTeam\n");
}

void tAttackTeam::mOldCreateShipsForTeam()
{
   	gST.mDebug_fprintf("---- Entering: tAttackTeam::mOldCreateShipsForTeam\n");

	int32 BPV = 150 - (10 * fNumShips);
	while (fNumShips < 4) {
		fNumShips++;
		BPV -= 10;
		mCreateShip( typeid( tShipInfo ),
					BPV,
					kClassFreighter,
					kClassBattleship,
					30,
					kStartPosition_W, gWC.fNWCustID++, 0, 0, -1, -1, "", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
	}

	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftedBPV += BPV;
	}

   	gST.mDebug_fprintf("---- finished: tAttackTeam::mOldCreateShipsForTeam\n");
}

void tAttackTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tAttackTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	fNumShips++;

	if (mIsTeamHumanControlled()) {
		gWC.fDraftedHumans++;
	}

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif
	
	mCreateShip( typeid( tAttackTeamShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_W,
				 metaid,
				 gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("--- Finished: tAttackTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tAttackTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}