/********************************************************************************************************\

	File Name:	DefendTeam3.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "DefendTeam3.h"
#include "DefendTeamInitialState.h"
#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "DefendTeam3InitialState.h"
#include "DefendVictoryCondition.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"

tClassAllocator<tDefendTeam3> gDefendTeam3Allocator;

tDefendTeam3::tDefendTeam3()
{
	gWC.mAddDefendTeam();
	fNumShips = 0;
}

void tDefendTeam3::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam3::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tDefendTeam3::mSetupState()
{
	mRegisterState( new tDefendTeam3InitialState() );

	mGotoState( typeid( tDefendTeam3InitialState ) );
	
}
void tDefendTeam3::mSetupCommState()
{
}

void tDefendTeam3::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tDefendTeam3::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {

		ShipList group_sl = gST.narrowed_sl;
		eRace race = mGetRace();
		int32 fShiplistCount = 0;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = group_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for %d race filter completed. ships: %d\n", race, fShiplistCount);

		group_sl = gST.stripOutSPECIAL(group_sl);
		group_sl = gST.stripOutRestricted(group_sl);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = group_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		group_sl = gST.selectWarShips( group_sl );

		if (group_sl.size() > 0) {
			int32 BPV = 150 - (10 * fNumShips);
			while (fNumShips < 4) {
				fNumShips++;
				BPV -= 10;

				gST.mDebug_fprintf("---- tDefendTeam3::mCreateShipsForTeam generating ship #: %d\n", fNumShips);

				ShipList this_pass_sl = group_sl;
				fShiplistCount = this_pass_sl.Subset( this_pass_sl.kSubsetBPV, int(0.9 * BPV), int(1.1 * BPV), true);
				gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
					int(0.9 * BPV), int(1.1 * BPV), fShiplistCount);
				gST.listShiplistContent(this_pass_sl);

				Ship s1;
				this_pass_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("Attack ship %d selected! %s:  %d BPV\n", fNumShips, s1.shipCode.c_str(), s1.bpv);

				mCreateShip( typeid( tShipInfo ),
					s1.shipCode.c_str(),
					kStartPosition_I, 
					kNoMetaShipID,
					gWC.fNWCustID++, 
					0, 0, -1, -1, "", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
			}

			if (mShipsCurrentlyControlled() > 0) {
				int32 BPV = gGetCombatBPV(fShipType);
				if (BPV < 1) BPV = mGetCombatBPV();
				if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
				gWC.fDraftedBPV += BPV;
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

   	gST.mDebug_fprintf("--- Finished: tDefendTeam3::mCreateShipsForTeam\n");
}

void tDefendTeam3::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tDefendTeam3::mOldCreateShipsForTeam\n");

	int32 BPV = 150 - (10 * fNumShips);
	while (fNumShips < 4) {
		fNumShips++;
		BPV -= 10;
		mCreateShip( typeid( tShipInfo ),
					BPV,
					kClassFreighter,
					kClassBattleship,
					30,
					kStartPosition_I, gWC.fNWCustID++, 0, 0, -1, -1, "", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
	}

	if (mShipsCurrentlyControlled() > 0) {
		int32 BPV = gGetCombatBPV(fShipType);
		if (BPV < 1) BPV = mGetCombatBPV();
		if (mIsTeamHumanControlled()) BPV = 11 * BPV / 10;
		gWC.fDraftingBPV += BPV;
	}

	gST.mDebug_fprintf("---- Entering: tDefendTeam3::mOldCreateShipsForTeam\n");
}

void tDefendTeam3::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam3::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tDefendTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	fNumShips++;

	if (mIsTeamHumanControlled()) {
		gWC.fDraftingHumans++;
	}

	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

	mCreateShip( typeid( tDefendTeamShip ), 
		shipScriptDescription.fClassName.c_str(), 
		kStartPosition_I,
		metaid,
		gWC.fNWCustID++, 0, 0, -1, -1, //Default data,so we can set the custom name
		shipScriptDescription.fCustomName.c_str(),
		static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

   	gST.mDebug_fprintf("---Finished: tDefendTeam3::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tDefendTeam3::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}