/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_10Patrol.h"
#include "OrionTeam.h"
#include "OrionTeamInitialState.h"
//#include "OrionTeamShip.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
}
void tOrionTeam::mSetupVictoryCondition()
{
}
void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamInitialState() );
	mGotoState( typeid( tOrionTeamInitialState ) );
}
void tOrionTeam::mSetupCommState()
{
}

void tOrionTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
   	gST.mInitializeSL();  // just to be safe
   	gST.mCompleteNarrowedList();

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	int32 orionBPV = 60;
	if (playerTeam) orionBPV = playerTeam->mGetCombatBPV() - (gWC.fMissionSeed % 23);
	int32 selection = gWC.fMissionSeed % 31;

	if (
		(orionBPV < 65) || (gWC.fDraftedHumans > 0) || (selection > 30)
		) {
		gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
		return;
	}

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;
   		ShipList orion_sl = gST.narrowed_sl;

		gWC.mSetOrionTeamInGame( true );
		gWC.fPirateUsed = kTrue;


		// Monster
		gST.mDebug_fprintf("---- Setting up monster shiplist subset\n");
        ShipList monster_sl = gST.narrowed_sl;
        std::list< eRace >  monsterList;
        monsterList.push_back(kMonster);
        fShiplistCount = monster_sl.Subset(monsterList);
        gST.mDebug_fprintf("subset for kMonster race filter completed. ships: %d\n", fShiplistCount);

        monster_sl = gST.stripOutSPECIAL(monster_sl);
        monster_sl = gST.stripOutRestricted(monster_sl);

        std::list< eEnhancedClassTypes >  monsterClassTypeList;
		monsterClassTypeList.push_back(kEClassSmallMonster);
        monsterClassTypeList.push_back(kEClassMediumMonster);
		monsterClassTypeList.push_back(kEClassLargeMonster);
		{ // no, I'm not redefining i. Local only please.
			for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
					monsterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
		}
        fShiplistCount = monster_sl.Subset(monsterClassTypeList, true);
        gST.mDebug_fprintf("subset for monster classes filter completed. ships: %d\n", fShiplistCount);

		// here's the deal: there's no promise that these monsters still exist in a modded shiplist. 
		// If that's the case, we want to detect it properly instead of creating ships blindly.
		// Additionally, we want to make sure that these monsters exist before 0 until 999
		Ship s1;
		Ship s2;

		bool bAstroMiner = false;
		// Astro Miner
		if (
			monster_sl.Find("M-AMM", s1) &&
			monster_sl.Find("M-AML", s2) &&
			(s1.yearFirstAvail <= gST.fEarliestYear) && (s1.yearLastAvail >= gST.fLatestYear) &&
			(s2.yearFirstAvail <= gST.fEarliestYear) && (s2.yearLastAvail >= gST.fLatestYear)
			) {
			gST.mDebug_fprintf("FYI: Papa and Mama astrominers both exist/are available.\n");
			bAstroMiner = true;
		}

		bool bLivingCage = false;
		// Living Cage
		if (
			monster_sl.Find("M-LCM", s1) &&
			monster_sl.Find("M-LCL", s2) &&
			(s1.yearFirstAvail <= gST.fEarliestYear) && (s1.yearLastAvail >= gST.fLatestYear) &&
			(s2.yearFirstAvail <= gST.fEarliestYear) && (s2.yearLastAvail >= gST.fLatestYear)
			) {
			gST.mDebug_fprintf("FYI: Papa and Mama living cages both exist/are available.\n");
			bLivingCage = true;
		}

		bool bMEater = false;
		// M-Eaters
		if (
			monster_sl.Find("M-MTM", s1) &&
			monster_sl.Find("M-MTL", s2) &&
			(s1.yearFirstAvail <= gST.fEarliestYear) && (s1.yearLastAvail >= gST.fLatestYear) &&
			(s2.yearFirstAvail <= gST.fEarliestYear) && (s2.yearLastAvail >= gST.fLatestYear)
			) {
			gST.mDebug_fprintf("FYI: Papa and Mama M-Eaters both exist/are available.\n");
			bMEater = true;
		}

		bool bSunGlider = false;
		// Sun Gliders
		if (
			monster_sl.Find("M-SGM", s1) &&
			monster_sl.Find("M-SGL", s2) &&
			(s1.yearFirstAvail <= gST.fEarliestYear) && (s1.yearLastAvail >= gST.fLatestYear) &&
			(s2.yearFirstAvail <= gST.fEarliestYear) && (s2.yearLastAvail >= gST.fLatestYear)
			) {
			gST.mDebug_fprintf("FYI: Papa and Mama Sun Gliders both exist/are available.\n");
			bSunGlider = true;
		}

		bool bSpaceShell = false;
		// SpaceShell
		if (
			monster_sl.Find("M-SSM", s1) &&
			monster_sl.Find("M-SSL", s2) &&
			(s1.yearFirstAvail <= gST.fEarliestYear) && (s1.yearLastAvail >= gST.fLatestYear) &&
			(s2.yearFirstAvail <= gST.fEarliestYear) && (s2.yearLastAvail >= gST.fLatestYear)
			) {
			gST.mDebug_fprintf("FYI: Papa and Mama Space Shells both exist/are available.\n");
			bSpaceShell = true;
		}

		ShipList monster_ships_sl = monster_sl;
        std::list< eEnhancedClassTypes >  monsterShipClassTypeList;
		{ // no, I'm not redefining i. Local only please.
			for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
					monsterShipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
		}
        fShiplistCount = monster_ships_sl.Subset(monsterShipClassTypeList, true);
        gST.mDebug_fprintf("subset for monster ship classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(monster_ships_sl);


		// Pirate OR monster ships (Andros!)
		gST.mDebug_fprintf("---- Setting up pirate ships shiplist subset\n");
		ShipList pirate_sl = gST.narrowed_sl;

        std::list< eRace >  pirateList;
        pirateList.push_back(kOrion);
		pirateList.push_back(kMonster);
        fShiplistCount = pirate_sl.Subset(pirateList);
        gST.mDebug_fprintf("subset for pirate/monster race filter completed. ships: %d\n", fShiplistCount);

        pirate_sl = gST.stripOutSPECIAL(pirate_sl);
        pirate_sl = gST.stripOutRestricted(pirate_sl);

        std::list< eEnhancedClassTypes >  pirateClassTypeList;
		{ // no, I'm not redefining i. Local only please.
			for (int32 i = kEClassFirstShip ; i <= kEClassLastShip; i++) {
				pirateClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
		}
        fShiplistCount = pirate_sl.Subset(pirateClassTypeList, true);
        gST.mDebug_fprintf("subset for pirate classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(pirate_sl);


		// pirate bases
		gST.mDebug_fprintf("---- Setting up pirate base shiplist subset\n");
		ShipList pirate_bases_sl = gST.narrowed_sl;

		{ // no, not redefining raceList.
	    std::list< eRace >  raceList;
        raceList.push_back(kOrion);
        fShiplistCount = pirate_bases_sl.Subset(raceList);
        gST.mDebug_fprintf("subset for kOrion race filter completed. ships: %d\n", fShiplistCount);
		}

		pirate_bases_sl = gST.stripOutSPECIAL(pirate_bases_sl);
        pirate_bases_sl = gST.stripOutRestricted(pirate_bases_sl);

        std::list< eEnhancedClassTypes >  pirate_bases_enh_class;
        pirate_bases_enh_class.push_back(kEClassMobileBase);
        pirate_bases_enh_class.push_back(kEClassBaseStation);
        pirate_bases_enh_class.push_back(kEClassBattleStation);
        fShiplistCount = pirate_bases_sl.Subset(pirate_bases_enh_class);
        gST.mDebug_fprintf("subset for eEnhancedClassTypes for bases completed. bases: %d\n", fShiplistCount);
        gST.listShiplistContent(pirate_bases_sl);


		// pirate freighters
		gST.mDebug_fprintf("---- Setting up pirate freighters shiplist subset\n");
		ShipList pirate_freighters_sl = gST.narrowed_sl;

		{ // no, not redefining raceList.
		std::list< eRace >  raceList;
		raceList.push_back(kOrion);
		fShiplistCount = pirate_freighters_sl.Subset(raceList);
		gST.mDebug_fprintf("subset for kOrion race filter completed. ships: %d\n", fShiplistCount);
		}

        pirate_freighters_sl = gST.stripOutSPECIAL(pirate_freighters_sl);
        pirate_freighters_sl = gST.stripOutRestricted(pirate_freighters_sl);

        // keep only freighters.
        std::list< eEnhancedClassTypes >  enhancedClassTypeListF;
        enhancedClassTypeListF.push_back(kEClassSmallFreighter);
        enhancedClassTypeListF.push_back(kEClassPriorityTransport);
        enhancedClassTypeListF.push_back(kEClassCivilianFreighter);
        enhancedClassTypeListF.push_back(kEClassLargeFreighter);
        enhancedClassTypeListF.push_back(kEClassHeavyFreighter);
        fShiplistCount = pirate_freighters_sl.Subset(enhancedClassTypeListF);
        gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

        // no Qship role for this ship
        std::list< eEnhancedRoles >  qshipList;
        qshipList.push_back(kQship);
        fShiplistCount = pirate_freighters_sl.Subset(qshipList, false);
        gST.mDebug_fprintf("subset for NO qship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        // CARGO role
        std::list< eEnhancedRoles >  CargoList;
        CargoList.push_back(kCargo);
        fShiplistCount = pirate_freighters_sl.Subset(CargoList);
        gST.mDebug_fprintf("subset for Cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(pirate_freighters_sl);


		gST.mDebug_fprintf("selection: %d\n", selection);


		// Monsters
		if (
			(selection >= 1) && (selection <= 10)
		) {
			gST.mDebug_fprintf("creating monster(s)\n");
			switch (selection) {
			case 1: 
				if (bAstroMiner) {
					gST.mDebug_fprintf("spawning M-AMM\n");
					mCreateShip( typeid( tShipInfo ), "M-AMM", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no astro miners available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
			case 2: 
				if (bAstroMiner) {
					gST.mDebug_fprintf("spawning M-AML\n");
					mCreateShip( typeid( tShipInfo ), "M-AML", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no astro miners available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
				break;

			case 3:
				if (bLivingCage) {
					gST.mDebug_fprintf("spawning M-LCM\n");
					mCreateShip( typeid( tShipInfo ), "M-LCM", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no living cages available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
			case 4:
				if (bLivingCage) {
					gST.mDebug_fprintf("spawning M-LCL\n");
					mCreateShip( typeid( tShipInfo ), "M-LCL", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no living cages available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
				break;

			case 5:
				if (bMEater) {
					gST.mDebug_fprintf("spawning M-MTM\n");
					mCreateShip( typeid( tShipInfo ), "M-MTM", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no M-Eaters available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
			case 6:
				if (bMEater) {
					gST.mDebug_fprintf("spawning M-MTL\n");
					mCreateShip( typeid( tShipInfo ), "M-MTL", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no M-Eaters available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
				break;

			case 7:
				if (bSunGlider) {
					gST.mDebug_fprintf("spawning M-SGM\n");
					mCreateShip( typeid( tShipInfo ), "M-SGM", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no Sun Gliders available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
			case 8:
				if (bSunGlider) {
					gST.mDebug_fprintf("spawning M-SGL\n");
					mCreateShip( typeid( tShipInfo ), "M-SGL", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no Sun Gliders available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
				break;

			case 9:
				if (bSpaceShell) {
					gST.mDebug_fprintf("spawning M-SSL\n");
					mCreateShip( typeid( tShipInfo ), "M-SSM", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no Space Shells available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
			case 10:
				if (bSpaceShell) {
					gST.mDebug_fprintf("spawning M-SSL\n");
					mCreateShip( typeid( tShipInfo ), "M-SSL", kStartPosition_Z, 
						kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
				} else if (monster_ships_sl.size() > 0) {
					gST.mDebug_fprintf("no Space Shells available. skipping to monster ships\n");
					selection = 11;
					break;
				} else {
					gST.mDebug_fprintf("no monsters available. skipping to pirate ships\n");
					selection = 30;
					break;
				}
				break;
			}
		}


		// Monster Ships
		if (
			(selection >= 11) && (selection <= 15)
		) {
			if (monster_ships_sl.size() > 0) {
				int32 monster_ship_team_BPV = orionBPV;
				gST.mDebug_fprintf("creating monster ships. (I have %d BPV to use)\n", monster_ship_team_BPV);
				int32 ships_created = 0;
				int32 minimumBPV_used = monster_ships_sl.MinimumBPV();
				Ship s1;
				while (
					(ships_created < 3) && 
					(minimumBPV_used <= monster_ship_team_BPV) &&
					(monster_ships_sl.size() > 0) && 
					(monster_ships_sl.SelectRandomShip(s1))
				) {
					if (s1.bpv > monster_ship_team_BPV) {
						monster_ships_sl.Remove(s1.shipCode.c_str());
					}
					ships_created++;
					monster_ship_team_BPV -= s1.bpv;
					gST.mDebug_fprintf("monster ship %d selected! %s:  %d BPV\n", ships_created, s1.shipCode.c_str(), s1.bpv);
					mCreateShip( 
						typeid( tShipInfo ), 
						s1.shipCode.c_str(), 
						kStartPosition_Z, 
						kNoMetaShipID, 
						-1, 0, 0, -1, -1, 
						"Nemesis", 
						kDeathUponCapture );
				}
			} else {
				gST.mDebug_fprintf("no monster ships available. skipping to pirate ships\n");
				selection = 30;
			}
		}
		

		// Pirate Base
		if (
			(selection >= 19) && (selection <= 23)
		) {
			Ship s1;
			if ((pirate_bases_sl.size() > 0) && (pirate_bases_sl.SelectRandomShip(s1))) {
	
				gST.mDebug_fprintf("pirate base selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
				mCreateShip( 
					typeid( tShipInfo ), 
					s1.shipCode.c_str(), 
					kStartPosition_Z, 
					kNoMetaShipID, 
					-1, 0, 0, -1, -1, 
					"Eden II", 
					kDeathUponCapture );

			} else {
				gST.mDebug_fprintf("no pirate base available. skipping to pirate ships\n");
				selection = 30;
			}
		}


		// pirate convoy
		if (
			(selection >= 24) && (selection <= 28)
		) {
			int32 fNumConvoyToCreate = 2;
			switch (selection) {
				case 24: 
					fNumConvoyToCreate = 5;
					break;
				case 25: 
					fNumConvoyToCreate = 4;
					break;
				case 26: 
					fNumConvoyToCreate = 3;
					break;
				default: 
					fNumConvoyToCreate = 2;
					break;
			}

			gST.mDebug_fprintf("Pirate Convoy: selection: %d. Getting %d ships\n", selection, fNumConvoyToCreate);

			Ship s1;
            if (pirate_freighters_sl.size() > 0) {
                Ship s1;
                for (int i = 1; i <= fNumConvoyToCreate; i++) {
                    pirate_freighters_sl.SelectRandomShip(s1);

                    gST.mDebug_fprintf("Pirate Convoy ship '%d' selected! %s:  %d BPV\n", i, s1.shipCode.c_str(), s1.bpv);

                    mCreateShip(
                        typeid( tShipInfo ),
                        s1.shipCode.c_str(),
                        kStartPosition_Z,
                        kNoMetaShipID,
                        -1, 0, 0, -1, -1,
                        "", 
                        static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );

                }
            } else {
				gST.mDebug_fprintf("no pirate freighters available. skipping to pirate ships\n");
				selection = 30;
            }
		}


		// Pirate Ships
		if (selection >= 29) {
			if (pirate_sl.size() > 0) {
				int32 pirate_ship_team_BPV = orionBPV;
				gST.mDebug_fprintf("creating monster ships. (I have %d BPV to use)\n", pirate_ship_team_BPV);
				int32 ships_created = 0;
				int32 minimumBPV_used = pirate_sl.MinimumBPV();
				Ship s1;
				while (
					(ships_created < 3) && 
					(minimumBPV_used <= pirate_ship_team_BPV) &&
					(pirate_sl.size() > 0) && 
					(pirate_sl.SelectRandomShip(s1))
				) {
					if (s1.bpv > pirate_ship_team_BPV) {
						pirate_sl.Remove(s1.shipCode.c_str());
					}
					ships_created++;
					pirate_ship_team_BPV -= s1.bpv;
					gST.mDebug_fprintf("monster ship %d selected! %s:  %d BPV\n", ships_created, s1.shipCode.c_str(), s1.bpv);
					mCreateShip( 
						typeid( tShipInfo ), 
						s1.shipCode.c_str(), 
						kStartPosition_Z, 
						kNoMetaShipID, 
						-1, 0, 0, -1, -1, 
						"", 
						kDeathUponCapture );
				}
			} else {
				gST.mDebug_fprintf("pirate shiplist empty. going to old method: mOldCreateShipsForTeam();\n");
				mOldCreateShipsForTeam();
			}
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}


void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kDefendTeam1 ) );
	int32 orionBPV = 60;
	if (playerTeam) orionBPV = playerTeam->mGetCombatBPV() - (gWC.fMissionSeed % 23);
	int32 selection = gWC.fMissionSeed % 31;

	if (orionBPV < 65) return;
	if (gWC.fDraftedHumans > 0) return;
	if (selection > 30) return;

	gWC.mSetOrionTeamInGame( true );
	gWC.fPirateUsed = kTrue;

	switch (selection) {
	case 1: mCreateShip( typeid( tShipInfo ), "M-AMM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
	case 2: mCreateShip( typeid( tShipInfo ), "M-AML", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 4:
		mCreateShip( typeid( tShipInfo ), "M-LCM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
	case 5:
		mCreateShip( typeid( tShipInfo ), "M-LCL", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 7:
		mCreateShip( typeid( tShipInfo ), "M-MTM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
	case 8:
		mCreateShip( typeid( tShipInfo ), "M-MTL", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
		break;
	case 10:
		mCreateShip( typeid( tShipInfo ), "M-SGM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
	case 11:
		// generate a monster
		mCreateShip( typeid( tShipInfo ), "M-SGL", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	case 16:
		mCreateShip( typeid( tShipInfo ), "M-SSM", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
	case 17:
		mCreateShip( typeid( tShipInfo ), "M-SSL", kStartPosition_Z, 
			kNoMetaShipID, -1, 0, 0, -1, -1, "Nemesis", kDeathUponCapture );
			break;
	
	case 19:
	case 20:
	case 21:
	case 22:
	case 23:
		// generate a pirate base
		mCreateShip( typeid( tShipInfo ),
					orionBPV,
					kClassBaseStation,
					kClassStarBase,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Eden II", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
		break;
	case 24: // generate a pirate convoy
		mCreateShip( typeid( tShipInfo ),
					25,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 5", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
	case 25:
		mCreateShip( typeid( tShipInfo ),
					30,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 4", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
	case 26:
		mCreateShip( typeid( tShipInfo ),
					35,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 3", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
	case 27:
	case 28:
		mCreateShip( typeid( tShipInfo ),
					40,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 2", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		mCreateShip( typeid( tShipInfo ),
					45,
					kClassFreighter,
					kClassFreighter,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Convoy 1", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture | kStartPositionCanBeOffset ) );
		break;
	default:
		// generate a pirate ship
		mCreateFleet( typeid( tShipInfo ),
					3,
					orionBPV,
					kClassFrigate,
					kClassBattleship,
					0,
					kStartPosition_Z, -1, 0, 0, -1, -1, "Nemesis", 
					static_cast<eShipOptions>( kCanBeRestricted | kDeathUponCapture ) );
		break;
	}

	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// census only: not actually drafting

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}

void tOrionTeam::mCreateOrions()
{

	int32 nStartPos = kStartPosition_Z;
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.4f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 0.55f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 0.7f;
	}

	nVarPercent += 0.30f;

	int32 bpv = 130 * nVarPercent;

	mCreateShip( typeid( tShipInfo ), bpv, kClassLightCruiser, 0, nStartPos, -1, 0,0,0,0, NULL, kDefaultShipOptions );

}

