
#include "ShipList_API.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "Tokenizer.h"
#include <sys/stat.h>

shiplist_tools gST;

ShipList global_sl;
ShipList narrowed_sl;
ShipList selected_drafted_sl;
#ifdef FIGHTERS
ShipList fighter_sl;
ShipList narrowed_fighter_sl;
#endif


shiplist_tools::setup()
{
	fEarliestYear = -999;
	fLatestYear = 999;
	fDraftedShipsSetUp = 0;
	fNarrowedListFullySetUp = 0;

	fTeam1DeltaFighterBPV = 0;
	fTeam2DeltaFighterBPV = 0;
	fTeam3DeltaFighterBPV = 0;
}


// Add me to really something early, like the Main mission script's mScriptSpecs()
void shiplist_tools::mOpenDebugFile() 
{
#ifdef DEBUGOUT
	std::string fileName = "C:\\"+Messages[kMissionTitle_msg]+".txt";
	FILE *fp = fopen(fileName.c_str(), "w");
	fprintf (fp, "Starting debug logfile\n" );
	fflush(fp);
	fclose(fp);
	ZeroTimeCount = GetTickCount();
#endif
}


void shiplist_tools::mDebug_fprintf ( const char* format, ... )
{
#ifdef DEBUGOUT
        va_list argptr ;
        va_start ( argptr, format ) ;
        int32 CurrentTimeCount = GetTickCount();

        // Insert timer counter here.
        char buf[1024];
        sprintf(buf, "%d: %s\0", CurrentTimeCount - ZeroTimeCount, format);

        // actual printf here.
		std::string fileName = "C:\\"+Messages[kMissionTitle_msg]+".txt";
		FILE *fp = fopen(fileName.c_str(), "a");
        vfprintf(fp, buf, argptr);
        va_end ( argptr ) ;
        fflush(fp);
		fclose(fp);
#endif
}


/*
	// Add this to tPlayerTeam::mCreateShipsForTeam !
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();
	gST.fHumanShipsSetUp = 1;
	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateShipsForTeam\n");
 */


// Add me just before the players' ships are created. 
// ie: at the start of PlayerTeam::mCreateShip
void shiplist_tools::mInitializeSL( void )
{
	if (global_sl.size() > 0) {
		return;
	}

	mDebug_fprintf("--- Entering: shiplist_tools::mInitializeSL\n");
	int32 fShiplistCount = 0;

	bool is_meta = true;
	// Load ShipList
	if (global_sl.size() == 0) {
		mDebug_fprintf("loading shiplist\n");
#ifdef OP
		global_sl.CreateMaster(ShipList::kOrionPirates, is_meta);
#else
		global_sl.CreateMaster(ShipList::kEmpiresAtWar, is_meta);
#endif
		mDebug_fprintf("shiplist loaded. ships: %d\n", global_sl.size());
		narrowed_sl = global_sl;
	}
	

#ifdef FIGHTERS
	if (fighter_sl.size() == 0) {
		mDebug_fprintf("loading fighter list\n");
#ifdef OP
		fighter_sl.CreateFighterList(ShipList::kOrionPirates, is_meta);
#else
		fighter_sl.CreateFighterList(ShipList::kEmpiresAtWar, is_meta);
#endif
		mDebug_fprintf("fighter list loaded. ships: %d\n", fighter_sl.size());
		narrowed_fighter_sl = fighter_sl;
	}
#endif

	mDebug_fprintf("--- Finishing: shiplist_tools::mInitializeSL\n");
}

void shiplist_tools::listShiplistContent( ShipList this_sl ) 
{
#ifdef DEBUGOUT
		{
		SHIP_LIST_ITER(this_sl)
		  for (;;)
		  {
			mDebug_fprintf("   ship %s:  %d BPV.  YFA: %d   YLA: %d\n", 
				iter->shipCode.c_str(), iter->bpv, iter->yearFirstAvail, iter->yearLastAvail);
			break;
		  }
		SHIP_LIST_ITER_END
		}
#endif
}


void shiplist_tools::mDraftedShipCensus( const char* shipCode )
{
    mDebug_fprintf("--- Entering: shiplist_tools::mDraftedShipCensus - %s\n", shipCode);

	fDraftedShipsSetUp++;

	Ship aShip;
    global_sl.Find( shipCode, aShip);
    selected_drafted_sl.push_back(aShip);

    mDebug_fprintf("Human/Drafted Ship: %s. YFA: %d  YLA: %d\n",
             shipCode, aShip.yearFirstAvail, aShip.yearLastAvail);
    mDebug_fprintf("--- Finished: shiplist_tools::mDraftedShipCensus\n");
}


ShipList shiplist_tools::stripOutSPECIAL( ShipList this_sl )
{
	// Strip out ships with SPECIAL
	std::list< eClassTypes >  classTypeList;
	classTypeList.push_back(kClassSpecial);
	int32 fShiplistCount = this_sl.Subset(classTypeList, false);
	mDebug_fprintf("subset for eClassTypes (no SPECIAL) completed. ships: %d\n", fShiplistCount);
	return this_sl;
}


ShipList shiplist_tools::stripOutRestricted( ShipList this_sl )
{
	// Strip out Qships with 'R' restricted.
	{
	SHIP_LIST_ITER(this_sl)
	  for (;;)
	  {
		if (iter->specialRole == kSpecialRoleR) {
		  iter = this_sl.erase(iter);
		  if (iter != this_sl.end()) continue; else break;
		}
		break;
	  }
	SHIP_LIST_ITER_END
	}
	mDebug_fprintf("subset for eSpecialRole (no R) completed. ships: %d\n", this_sl.size());
	return this_sl;
}


ShipList shiplist_tools::mFilterShipListToSingleRace( ShipList this_sl, eRace race ) 
{
	std::list< eRace >  raceList;
	raceList.push_back(race);
	int32 fShiplistCount = this_sl.Subset(raceList);
	mDebug_fprintf("subset for race (%d) filter completed. entries: %d\n", race, fShiplistCount);

	return this_sl;
}


ShipList shiplist_tools::selectWarShips( ShipList this_sl )
{
    if (this_sl.mIsEnhancedShiplist()) {
        int32 fShiplistCount = 0;

        std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kDroneBombardmentShip);
        warRolesList.push_back(kEscort);
        warRolesList.push_back(kCarrier);
        warRolesList.push_back(kPFTender);
        warRolesList.push_back(kFastCruiser);
        warRolesList.push_back(kMauler);
        warRolesList.push_back(kPolice);
        // and a monster
        warRolesList.push_back(kRoleMonster);
        fShiplistCount = this_sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for warship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  doNotWantRolesList;
        doNotWantRolesList.push_back(kLocalDefense);
        fShiplistCount = this_sl.Subset(doNotWantRolesList, false);
        gST.mDebug_fprintf("filtered out roles we do not want for warship encounters. ships: %d\n", fShiplistCount);
    }
    return this_sl;
}


// Add me to the first AI team to be created. Or all. Doesn't matter
// ie: tNPC1Team::mCreateShipsForTeam
void shiplist_tools::mCompleteNarrowedList( void )
{
	if (! fNarrowedListFullySetUp) {
		mDebug_fprintf("--- Entering shiplist_tools::mCompleteNarrowedList\n");

		// We need a delay until the human ships are loaded
		int32 waitForHumans=0;
		while (
			(fDraftedShipsSetUp == 0) && (waitForHumans < 200)
		) {
			waitForHumans++;
			Sleep(50); //MilliSeconds
		}
		mDebug_fprintf("Waited for humans to be set up: %d milliseconds\n", waitForHumans * 50);

		if ((narrowed_sl.size() != 0) && (selected_drafted_sl.size() != 0)) {
			bool XShip_allowed = 0;
			fEarliestYear = -999;
			fLatestYear = 999;
				
			SHIP_LIST_ITER(selected_drafted_sl)
			mDebug_fprintf("parsing human/drafted ship(%s): YFA: %d  YLA %d\n", iter->shipCode.c_str(), iter->yearFirstAvail, iter->yearLastAvail);
			if (iter->yearFirstAvail > fEarliestYear) {
				mDebug_fprintf("replacing fEarliestYear: %d to %d\n", fEarliestYear, iter->yearFirstAvail);
				fEarliestYear = iter->yearFirstAvail;
			}
			if (iter->yearLastAvail < fLatestYear) {
				mDebug_fprintf("replacing fLatestYear: %d to %d\n", fLatestYear, iter->yearLastAvail);
				fLatestYear = iter->yearLastAvail;
			}
			if (selected_drafted_sl.mIsEnhancedShiplist()) {
				if (iter->enhancedXtech != kNoXtech) {
					XShip_allowed = 1;
					mDebug_fprintf("Xship (%d) on player team: %s\n", iter->enhancedXtech, iter->shipCode.c_str());
				}
			}
			SHIP_LIST_ITER_END

			// no player has an x-ship, so AI isn't allowed one.
			// This fixes the fact that xships can exist in late era and appear too early.
			if (XShip_allowed == 0) {
				if (fLatestYear > kX1Begin) {
					fLatestYear = kX1Begin - 1;
				}
				mDebug_fprintf("There are no Xships on player team. No AI X-ships allowed.\n");
			}

			// we alread have a subset function for cleaning up the shiplist based on years of availability
			mDebug_fprintf("Estimated years of play: %d to %d\n", fEarliestYear, fLatestYear);
			int32 fShiplistCount = narrowed_sl.Subset(ShipList::kSubsetYearRangeExists, fEarliestYear, fLatestYear);
			mDebug_fprintf("subset for year range completed. ships: %d\n", fShiplistCount);

#ifdef FIGHTERS
			fShiplistCount = narrowed_fighter_sl.Subset(ShipList::kSubsetYearRangeExists, fEarliestYear, fLatestYear);
			mDebug_fprintf("fighter subset for year range completed. fighters: %d\n", fShiplistCount);
#endif
		}
		fNarrowedListFullySetUp = true;

		mDebug_fprintf("--- Finished shiplist_tools::mCompleteNarrowedList\n");
	}
}


int32 shiplist_tools::mGetModelSize( std::string geometry )
{
	FILE* modelSizFP;
	std::string modelSizFileName;
	bool registry_location_success;

#ifdef OP
	registry_location_success = FindModelsDirectory(kOrionPirates, modelSizFileName);
#else
	registry_location_success = FindModelsDirectory(kEmpiresAtWar, modelSizFileName);
#endif
	modelSizFileName += "model.siz";


	if ( ! ( registry_location_success && FileExists(modelSizFileName) ) ) {
		mDebug_fprintf("File not Found! %s\n", modelSizFileName.c_str());
		return -1;
	}

	modelSizFP = fopen(modelSizFileName.c_str(), "r");
    if (!modelSizFP)
        return -1;

    for (;;)
    {
        char inputBuf[1024];
        if (!fgets(inputBuf, 1024, modelSizFP))
            break;      // assume EOF

		inputBuf[strlen(inputBuf) - 1] = 0x00;      /* trip EOL-CR */
		Tokenizer   tok(inputBuf, "\t");

		std::string model = tok.getNextTokenC();
		float modelSize = tok.getNextTokenF();

		std::transform(model.begin(), model.end(), model.begin(), (int(*)(int)) toupper);
		std::replace( model.begin(), model.end(), '\\', '/' );
		std::transform(geometry.begin(), geometry.end(), geometry.begin(), (int(*)(int)) toupper);
		std::replace( geometry.begin(), geometry.end(), '\\', '/' );
		if (model.compare(geometry) == 0) {
			return modelSize;
			break;
		}
	}

	return -1;
}


bool shiplist_tools::FindModelsDirectory(eGame game, std::string &location)
{
    HKEY    hKey;
    LONG    lval;
    DWORD   type;
    char    path[256];
    DWORD   siz = 256;

    if (game == ShipList::kOrionPirates)
    {
        lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                            "Software\\Taldren\\Starfleet Command Orion Pirates",
                            0,
                            KEY_QUERY_VALUE,
                            &hKey);

        if (lval != ERROR_SUCCESS)
            return false;

        lval = RegQueryValueEx( hKey,
                                "Directory",
                                0,
                                &type,
                                (unsigned char *)path,
                                &siz );

        RegCloseKey(hKey);

        if (lval != ERROR_SUCCESS)
            return false;
    }
    else if (game == ShipList::kEmpiresAtWar)
    {
        lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                            "Software\\Taldren\\Starfleet Command II",
                            0,
                            KEY_QUERY_VALUE,
                            &hKey);

        if (lval != ERROR_SUCCESS)
            return false;

        lval = RegQueryValueEx( hKey,
                                "Path",
                                0,
                                &type,
                                (unsigned char *)path,
                                &siz );

        RegCloseKey(hKey);

        if (lval != ERROR_SUCCESS)
            return false;
    }
    else
    {
        DebugBreak();
    }

    path[siz] = 0;
    if (path[siz - 1] != '\\')
        strcat(path, "\\");

    strcat(path, "Assets\\Models\\");

    location = path;
    return true;
}


bool shiplist_tools::FileExists( std::string strFilename ) {
  struct stat stFileInfo;
  int intStat;

  // Attempt to get the file attributes
  intStat = stat(strFilename.c_str(),&stFileInfo);
  if(intStat == 0) {
    return true;
  } else {
    return false;
  }
  
  return false;
}


int32 shiplist_tools::mUpgradeFighters( tShipInfo *ship )
{
#ifdef FIGHTERS
	if (! ship) return 0;

	mCompleteNarrowedList(); // just in case;
    Ship s1;
	mDebug_fprintf("---- Checking for fighters on: %s.\n", ship->mGetClassName());
	narrowed_sl.Find(ship->mGetClassName(), s1);

	// check for fighters.
	// up to 4 fighter bays
	int num_bays = 0;
	int32 count = 0;
	int32 total_num_fighters = 0;
	Ship selected_fighter;
	BaseShip fighter;
	bool found_fighter = false;

	for (int k = 0; k < 4; k++)
	{
		// assume all fighters are the same in the ship
		// load only the first one.
		if (! found_fighter) {
			fighter = s1.fighters[k].first;
			found_fighter = true;
		}
		count = s1.fighters[k].second;
		if (count)
		{
			num_bays++;
			total_num_fighters += count;
		}
	}
	mDebug_fprintf("Total number of Fighters: %d\n", total_num_fighters);

	if (total_num_fighters)
	{
		// We have fighters!
		// print out: fighter.shipCode
		mDebug_fprintf("We have fighters! Bays: %d Count: %d  Type: %s\n",
			num_bays, total_num_fighters, fighter.shipCode.c_str());

		ShipList tmpFighter_sl;

		mDebug_fprintf("Removing fighter selection which are less than %d BPV.\n", fighter.bpv);

		{
		SHIP_LIST_ITER(narrowed_fighter_sl)
			if (
				(iter->race == s1.race)
				&& (iter->bpv >= fighter.bpv)
			) {
				tmpFighter_sl.push_back(*iter);
			}
		SHIP_LIST_ITER_END
		}

		if (tmpFighter_sl.size() == 0) {
			mDebug_fprintf("weird! empty fighter shiplist. Aborting!\n");
			return 0;
		}
		listShiplistContent(tmpFighter_sl);

		tmpFighter_sl.SelectRandomShip(selected_fighter);

		int32 avg_fighters = total_num_fighters / num_bays;
		if (total_num_fighters % num_bays)
			avg_fighters++;
		// print out: fighter.shipCode

		mDebug_fprintf("New Fighter type: Bays: %d   #/bay: %d  Type: %s\n",
			num_bays, avg_fighters, selected_fighter.shipCode.c_str());
			
 		ship->mSetFighters(num_bays, avg_fighters, selected_fighter.shipCode);
		mDebug_fprintf("---- Done: fighters on: %s.\n", ship->mGetClassName());

		return (num_bays * avg_fighters * selected_fighter.bpv) - (num_bays * avg_fighters * fighter.bpv);
	}
#endif // FIGHTERS
	return 0;
}


int32 shiplist_tools::mUpgradeFightersForTeam( tTeamInfo *ti )
{
	int32 fTeam_BPV_Increase = 0;
#ifdef FIGHTERS
	if (ti) {
		mDebug_fprintf("--- Entering shiplist_tools::mUpgradeFightersForTeam: Team: %d.\n", ti->mGetTeam());
		tShipIterator tsi = ti->mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;

			fTeam_BPV_Increase += mUpgradeFighters( si );

			tsi++;
		}
		mDebug_fprintf("--- Finished shiplist_tools::mUpgradeFightersForTeam: Team: %d.\n", ti->mGetTeam());
	}
#endif
	return fTeam_BPV_Increase;
}