/********************************************************************************************************\

	File Name:	CommonEnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonEnemyShip.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ShipList_API.h"

 
tClassAllocator<tCommonEnemyTeam> gCommonEnemyTeamAllocator;

tCommonEnemyTeam::tCommonEnemyTeam() : fNumShips( 0 )
{
	fHumanControlled = mIsTeamHumanControlled();
	fTeamID = -1;
	fMyShipID = -1;
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
}

void tCommonEnemyTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonEnemyTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonEnemyTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonEnemyTeamBaseState() );
	mGotoState( typeid( tCommonEnemyTeamBaseState ) );
   }
   
}

void tCommonEnemyTeam::mSetupCommState()
{}

void tCommonEnemyTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tCommonEnemyTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;

		// based on the original code..
		if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
		gWC.mSetDrafting();

		if (mIsTeamHumanControlled()) {
			int32 humanBPV = 0;
			for (int s = 0 ; s < fNumShips; s++) {
				Ship draftedShip;
				gST.narrowed_sl.Find(fShipList[s], draftedShip);
				humanBPV += draftedShip.bpv;
			}
			gWC.fDraftedBPV += (10 * humanBPV / 9);
		}

		switch (gWC.fMapPicked) {
			case 4: 
			case 5:
				if (gWC.fAddAlly < 1) gWC.fAddAlly = 1;
				break;
			case 6: 
			case 8:
			case 9:
				if (gWC.fAddAlly < 2) gWC.fAddAlly = 2;
				break;
			default:
				break;
		}
		if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) gWC.fAddAlly = 0;

		eRace baseRace;
		switch (fMyRace) {
			case kFederation: 
			case kKlingon:
			case kRomulan:
			case kLyran:
			case kGorn:
			case kHydran:
			case kMirak:
			case kISC:
				baseRace = fMyRace;
				break;
			default:
				baseRace = kNeutralRace;
				break;
		}


		if ((fTeamID == kEnemyTeam2) && (gWC.fMapPicked > 3)) {
			std::string extraship;
			iMapPosition i;

			// for all installations, planets, platforms..
			ShipList base_sl = gST.narrowed_sl;
			std::list< eRace >  baseRaceList;
			baseRaceList.push_back(baseRace);
			fShiplistCount = base_sl.Subset(baseRaceList);
			gST.mDebug_fprintf("enemy base: subset for race %d filter completed. ships: %d\n", baseRace, fShiplistCount);

			base_sl = gST.stripOutRestricted(base_sl);

			// for FRDs
			ShipList FRD_sl = base_sl;
			std::list< eEnhancedClassTypes >  FRDClassTypeList;
			FRDClassTypeList.push_back(kEClassFleetRepairDock);
			fShiplistCount = FRD_sl.Subset(FRDClassTypeList, true);
			gST.mDebug_fprintf("subset for FRD class filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(FRD_sl);

			// Defense platforms
			ShipList DEF_sl = base_sl;
			std::list< eEnhancedClassTypes >  DEFClassTypeList;
			DEFClassTypeList.push_back(kEClassPlatform);
			fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
			gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

			std::list< eEnhancedRoles >  DEFRolesList;
			DEFRolesList.push_back(kLocalDefense);
			fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
			gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(DEF_sl);

			// planets
			ShipList planet_sl;
			if (gWC.fMapPicked >= 7) {
				planet_sl = base_sl;
				std::list< eEnhancedClassTypes >  planetClassTypeList;
				planetClassTypeList.push_back(kEClassPlanet);
				fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
				gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

				{
				SHIP_LIST_ITER(planet_sl)
					std::string badplanet = "";
					switch (baseRace) {
						case kKlingon: badplanet = "K-PE"; break;
						case kRomulan: badplanet = "R-PE"; break;
						case kLyran: badplanet = "L-PE"; break;
						case kGorn: badplanet = "G-PE"; break;
						case kHydran: badplanet = "H-PE"; break;
						case kMirak: badplanet = "Z-PE"; break;
						case kISC: badplanet = "I-PE"; break;
						default: break;
					}
					for (;;) {
						if (gST.mGetModelSize(iter->geometry) > 151) {
							iter = planet_sl.erase(iter);
							if (iter != planet_sl.end()) continue; else break;
						}
						if (badplanet.compare(iter->shipCode) == 0) {
							iter = planet_sl.erase(iter);
							if (iter != planet_sl.end()) continue; else break;
						}
						break;
					}
				SHIP_LIST_ITER_END
				}
				gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
			}

			switch (gWC.fMapPicked) {
				case 4: 
					{
						ShipList bs_sl = base_sl;
						std::list< eEnhancedClassTypes >  bsClassTypeList;
						bsClassTypeList.push_back(kEClassBaseStation);
						fShiplistCount = bs_sl.Subset(bsClassTypeList);
						gST.mDebug_fprintf("subset for BS class filter completed. ships: %d\n", fShiplistCount);

						std::list< eEnhancedRoles >  bsRolesList;
						gST.mDebug_fprintf("Selecting base station 'BASE' role.\n");
						bsRolesList.push_back(kBase);
						fShiplistCount = bs_sl.Subset(bsRolesList);
						gST.mDebug_fprintf("subset for kBase eEnhancedRoles completed. ships: %d\n", fShiplistCount);

						Ship s1;
						bs_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("BS selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();
						gWC.fDraftedBPV += s1.bpv;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
							if ((rand() % 11) < 1) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 11) < 1) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}

					break;
				case 5: 
					{
						ShipList bats_sl = base_sl;
						std::list< eEnhancedClassTypes >  batsClassTypeList;
						batsClassTypeList.push_back(kEClassBattleStation);
						fShiplistCount = bats_sl.Subset(batsClassTypeList);
						gST.mDebug_fprintf("subatset for bats class filter completed. ships: %d\n", fShiplistCount);

						std::list< eEnhancedRoles >  batsRolesList;
						gST.mDebug_fprintf("Selecting base station 'BASE' role.\n");
						batsRolesList.push_back(kBase);
						fShiplistCount = bats_sl.Subset(batsRolesList);
						gST.mDebug_fprintf("subatset for kBase eEnhancedRoles completed. ships: %d\n", fShiplistCount);

						Ship s1;
						bats_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("bats selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();
						gWC.fDraftedBPV += s1.bpv;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
							if ((rand() % 13) < 2) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 13) < 2) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
					break;
				case 6: 
					{
						ShipList sb_sl = base_sl;
						std::list< eEnhancedClassTypes >  sbClassTypeList;
						sbClassTypeList.push_back(kEClassBattleStation);
						fShiplistCount = sb_sl.Subset(sbClassTypeList);
						gST.mDebug_fprintf("susbet for sb class filter completed. ships: %d\n", fShiplistCount);

						std::list< eEnhancedRoles >  sbRolesList;
						gST.mDebug_fprintf("Selecting starbase 'BASE' role.\n");
						sbRolesList.push_back(kBase);
						fShiplistCount = sb_sl.Subset(sbRolesList);
						gST.mDebug_fprintf("susbet for kBase eEnhancedRoles completed. ships: %d\n", fShiplistCount);

						Ship s1;
						sb_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("sb selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();
						gWC.fDraftedBPV += s1.bpv;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {

							if ((rand() % 17) < 3) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 17) < 3) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
					break;
				case 7: // colony world
					{
						std::list< eEnhancedRoles >  planetRolesList;
						gST.mDebug_fprintf("Selecting colony world role.\n");
						planetRolesList.push_back(kArmedWorld);
						planetRolesList.push_back(kHomeWorld);
						fShiplistCount = planet_sl.Subset(planetRolesList, false);
						gST.mDebug_fprintf("Filtered out armed and homeworlds. ships: %d\n", fShiplistCount);
						gST.listShiplistContent(planet_sl);

						Ship s1;
						planet_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();
						gWC.fDraftedBPV += s1.bpv;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
							if ((rand() % 11) < 3) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								gWC.fDraftedBPV += DEFShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 11) < 3) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								gWC.fDraftedBPV += FRDShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
					break;
				case 8: // core world
					{
						std::list< eEnhancedRoles >  planetRolesList;
						gST.mDebug_fprintf("Selecting armed world role.\n");
						planetRolesList.push_back(kArmedWorld);
						fShiplistCount = planet_sl.Subset(planetRolesList);
						gST.mDebug_fprintf("narrowed down to armed worlds. ships: %d\n", fShiplistCount);
						gST.listShiplistContent(planet_sl);

						Ship s1;
						planet_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();
						gWC.fDraftedBPV += s1.bpv;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
							if ((rand() % 13) < 2) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								gWC.fDraftedBPV += DEFShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 13) < 2) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								gWC.fDraftedBPV += FRDShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
					break;
				case 9: // home world 
					{
						std::list< eEnhancedRoles >  planetRolesList;
						gST.mDebug_fprintf("Selecting armed world role.\n");
						planetRolesList.push_back(kArmedWorld);
						fShiplistCount = planet_sl.Subset(planetRolesList);
						gST.mDebug_fprintf("narrowed down to armed worlds. ships: %d\n", fShiplistCount);
						gST.listShiplistContent(planet_sl);

						Ship s1;
						planet_sl.SelectRandomShip(s1);
						gST.mDebug_fprintf("planet selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
						extraship = s1.shipCode.c_str();

						gWC.fDraftedBPV += 450;

						if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
							if ((rand() % 17) < 3) {
								Ship DEFShip;
								DEF_sl.SelectRandomShip(DEFShip);
								gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

								gWC.fDraftedBPV += DEFShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									DEFShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
							if ((rand() % 17) < 3) {
								Ship FRDShip;
								FRD_sl.SelectRandomShip(FRDShip);
								gST.mDebug_fprintf("FRD selected! %s:  %d BPV\n", FRDShip.shipCode.c_str(), FRDShip.bpv);

								gWC.fDraftedBPV += FRDShip.bpv;
								mCreateShip(
									typeid( tCommonEnemyShip ), 
									FRDShip.shipCode.c_str(), 
									i,
									-1,
									-1, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
					break;
				default: extraship = ""; 
					break;
			}
			if (extraship != "") {
				mCreateShip(
					typeid( tCommonEnemyShip ), 
					extraship.c_str(), 
					kStartPosition_Z,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			} else {
				gST.mDebug_fprintf("Could not generate the base/planet for race %d\n", fMyRace);
			}
		}

		if (fHumanControlled || (mIsTeamHumanControlled()) || (gWC.fDraftedHumans > 0)) {
			return;
		}
		std::string shipname = "";
		std::string shiptype = gWC.fShipTypeList[mGetTeam() * 6];

		int32 bpv, i, minbpv, bpvsofar;
		bpv = 0;
		minbpv = 60;

		if (fTeamID == kEnemyTeam3) {
			gST.mDebug_fprintf("Creating ships for kEnemyTeam3\n");
			if (gWC.fUseMinefield) {
				gST.mDebug_fprintf("Creating minefield for kEnemyTeam3\n");

				// for all installations, planets, platforms..
				ShipList base_sl = gST.narrowed_sl;
				std::list< eRace >  baseRaceList;
				baseRaceList.push_back(baseRace);
				fShiplistCount = base_sl.Subset(baseRaceList);
				gST.mDebug_fprintf("enemy base: subset for race %d filter completed. ships: %d\n", baseRace, fShiplistCount);

				base_sl = gST.stripOutRestricted(base_sl);

				// Defense platforms
				ShipList DEF_sl = base_sl;
				std::list< eEnhancedClassTypes >  DEFClassTypeList;
				DEFClassTypeList.push_back(kEClassPlatform);
				fShiplistCount = DEF_sl.Subset(DEFClassTypeList);
				gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

				std::list< eEnhancedRoles >  DEFRolesList;
				DEFRolesList.push_back(kLocalDefense);
				fShiplistCount = DEF_sl.Subset(DEFRolesList);
				gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
				gST.listShiplistContent(DEF_sl);

				// Listening Post
				ShipList LP_sl = base_sl;
				std::list< eEnhancedClassTypes >  LPClassTypeList;
				LPClassTypeList.push_back(kEClassListeningPost);
				fShiplistCount = LP_sl.Subset(LPClassTypeList);
				gST.mDebug_fprintf("subset for LP class filter completed. ships: %d\n", fShiplistCount);

				std::list< eEnhancedRoles >  LPRolesList;
				LPRolesList.push_back(kListeningPost);
				fShiplistCount = LP_sl.Subset(LPRolesList);
				gST.mDebug_fprintf("subset for LP eEnhancedRoles completed. ships: %d\n", fShiplistCount);
				gST.listShiplistContent(LP_sl);


				for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
					Ship DEFShip;
					DEF_sl.SelectRandomShip(DEFShip);
					gST.mDebug_fprintf("DEF selected! %s:  %d BPV\n", DEFShip.shipCode.c_str(), DEFShip.bpv);

					mCreateShip(	
						typeid( tCommonEnemyShip ), 
						DEFShip.shipCode.c_str(), 
						i,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"lg mine",
						kDefaultShipOptions );
				}
				for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
					Ship LPShip;
					LP_sl.SelectRandomShip(LPShip);
					gST.mDebug_fprintf("LP selected! %s:  %d BPV\n", LPShip.shipCode.c_str(), LPShip.bpv);

					mCreateShip(	typeid( tCommonEnemyShip ), 
						LPShip.shipCode.c_str(), 
						i,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"sm mine",
						kDefaultShipOptions );
				}
			} else {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv += minbpv;
				bpv = (((rand() % 30) + 95) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				if (bpv < 65) bpv = 50 + (rand() % 35);
				bpvsofar = bpv;
				if (bpv > 185) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;

				Ship s1;
				gST.narrowed_sl.Find(fShipType.c_str(), s1);
				int32 draftedbpv = s1.bpv;
				gST.mDebug_fprintf("using %s to find drafted BPV: %d\n", fShipType, draftedbpv);

				if (draftedbpv < 1) draftedbpv = 100;
				if (draftedbpv < bpv) {
					id = fShipMetaID;
					shiptype = fShipType;
					bpvsofar = draftedbpv;
				} else {
					ShipList empty_sl;
					gST.mPickShip(bpvsofar, mGetRace(), s1, empty_sl);
					bpvsofar = s1.bpv;
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = s1.shipCode.c_str();
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gST.narrowed_sl.Find(shiptype.c_str(), s1);
					gWC.fDraftedBPV += s1.bpv;
					gST.mDebug_fprintf("creating ship: %s\n", shiptype.c_str());
					mCreateShip(	
						typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;

				if ((bpv - bpvsofar) >= 45) {
					int32 fleetbpv = bpv - bpvsofar;
					if (fleetbpv < 0) fleetbpv = 0;
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					gST.mDebug_fprintf("Need to pick and generate an AI fleet\n");

					switch (gWC.fMapPicked) {
						case 4:
							fleetbpv = fleetbpv - (rand() % 50);
							if (fleetbpv < 50) fleetbpv = 50 + (rand() % 23);
							break;
						case 5:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 75) fleetbpv = 70 + (rand() % 31);
							break;
						case 6:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 43);
							break;
						case 7:
							if (fleetbpv < 50) fleetbpv = 60 + (rand() % 37);
							break;
						case 8:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 37);
							break;
						case 9:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 150) fleetbpv = 120 + (rand() % 41);
							break;
						default:
							break;
					}

					if (gWC.fMapPicked > 3) switch (gWC.fDrafterRace) {
						case kLyran: fleetbpv = (4 * fleetbpv) / 5; break;
						case kGorn: fleetbpv = (5 * fleetbpv) / 6; break;
						case kRomulan: fleetbpv = (6 * fleetbpv) / 7; break;
						case kHydran: break;
						case kKlingon: fleetbpv = (6 * fleetbpv) / 5; break;
						case kMirak: 
						case kFederation: fleetbpv = (5 * fleetbpv) / 4; break;
						default: break;
					}

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// calculate the number of AI ships we can handle
						int32 fleetsize = gWC.mGetFleetSize(fleetbpv);
						// put in a call to pick the fleet
						std::string shipspicked[6];
						// create the ships
						fleetbpv = gST.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);
						for (int i = 0; i < fleetsize; i++) {
							if (shipspicked[i].length() > 2) {
								Ship s1;
								gST.narrowed_sl.Find(shipspicked[i].c_str(), s1);
								gWC.fDraftedBPV += s1.bpv;
								gST.mDebug_fprintf("ship fleetpick: %s\n", shipspicked[i].c_str());

								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_J + (rand() % 3),
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}

				if (mShipsCurrentlyControlled() < 1) {
					gST.mDebug_fprintf("failed to generate a ship/fleet. Let's add a pirate base instead.\n");
					int32 fShiplistCount = 0;

					// for all installations, planets, platforms..
					ShipList base_sl = gST.narrowed_sl;
					std::list< eRace >  baseRaceList;
					baseRaceList.push_back(kOrion);
					fShiplistCount = base_sl.Subset(baseRaceList);
					gST.mDebug_fprintf("enemy base: subset for race kOrion filter completed. ships: %d\n", fShiplistCount);

					base_sl = gST.stripOutRestricted(base_sl);

					std::list< eEnhancedClassTypes >  bsClassTypeList;
					bsClassTypeList.push_back(kEClassBaseStation);
					fShiplistCount = base_sl.Subset(bsClassTypeList);
					gST.mDebug_fprintf("subset for BS class filter completed. ships: %d\n", fShiplistCount);

					std::list< eEnhancedRoles >  bsRolesList;
					gST.mDebug_fprintf("Selecting base station 'BASE' role.\n");
					bsRolesList.push_back(kBase);
					fShiplistCount = base_sl.Subset(bsRolesList);
					gST.mDebug_fprintf("subset for kBase eEnhancedRoles completed. ships: %d\n", fShiplistCount);

					Ship s1;
					base_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("BS selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);
					gWC.fDraftedBPV += s1.bpv;

					mCreateShip(	typeid( tCommonEnemyShip ), 
						s1.shipCode.c_str(), 
						kStartPosition_L,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
			}
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tCommonEnemyTeam::mCreateShipsForTeam\n");
}

void tCommonEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tCommonEnemyTeam::mOldCreateShipsForTeam\n");

	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();
	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftedBPV += (10 * humanBPV / 9);
	}
	switch (gWC.fMapPicked) {
		case 4: 
		case 5:
			if (gWC.fAddAlly < 1) gWC.fAddAlly = 1;
			break;
		case 6: 
		case 8:
		case 9:
			if (gWC.fAddAlly < 2) gWC.fAddAlly = 2;
			break;
		default:
			break;
	}
	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) gWC.fAddAlly = 0;
	
	if ((fTeamID == kEnemyTeam2) && (gWC.fMapPicked > 3)) {
		std::string extraship, frdship, defship;
		iMapPosition i;
		switch (fMyRace) {
			case kFederation: frdship = "F-FRD"; defship = "F-DEF";
				extraship = "F-PL"; break;
			case kKlingon:  frdship = "K-FRD"; defship = "K-DEF";
				extraship = "K-PL"; break;
			case kRomulan:  frdship = "R-FRD"; defship = "R-DEF";
				extraship = "R-PL"; break;
			case kLyran:  frdship = "L-FRD"; defship = "L-DEF";
				extraship = "L-PL"; break;
			case kGorn:  frdship = "G-FRD"; defship = "G-DEF";
				extraship = "G-PL"; break;
			case kHydran:  frdship = "H-FRD"; defship = "H-DEF";
				extraship = "H-PL"; break;
			case kMirak:  frdship = "Z-FRD"; defship = "Z-DEF";
				extraship = "Z-PL"; break;
			case kISC:  frdship = "I-FRD"; defship = "I-DEF";
				extraship = "I-PL"; break;
			default:  frdship = "O-FRD"; defship = "O-DEF";
				extraship = "PL"; break;
		}
		switch (gWC.fMapPicked) {
			case 4: extraship = gWC.fBSchoice[fMyRace]; 
				gWC.fDraftedBPV += 120;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 60;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 60;
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
					if ((rand() % 11) < 1) mCreateShip(	typeid( tCommonEnemyShip ), 
						defship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
					if ((rand() % 11) < 1) mCreateShip(	typeid( tCommonEnemyShip ), 
						frdship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			case 5: extraship = gWC.fBATSchoice[fMyRace]; 
				gWC.fDraftedBPV += 240;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 60;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 60;
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
					if ((rand() % 13) < 2) mCreateShip(	typeid( tCommonEnemyShip ), 
						defship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
					if ((rand() % 13) < 2) mCreateShip(	typeid( tCommonEnemyShip ), 
						frdship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			case 6: extraship = gWC.fSBchoice[fMyRace]; 
				gWC.fDraftedBPV += 400;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 100;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 100;
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
					if ((rand() % 17) < 3) mCreateShip(	typeid( tCommonEnemyShip ), 
						defship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
					if ((rand() % 17) < 3) mCreateShip(	typeid( tCommonEnemyShip ), 
						frdship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			case 7: // colony world
				gWC.fDraftedBPV += 100;
				switch (rand() % 5) {
					case 0: extraship += "0"; break;
					case 1: extraship += "8"; break;
					case 2: extraship += "3"; break;
					case 3: extraship += "9"; break;
					default: extraship = "N-PLPh0"; break;
				}
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
					if ((rand() % 11) < 3) {
						gWC.fDraftedBPV += 40;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							defship.c_str(), 
							i,
							-1,
							-1, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
					if ((rand() % 11) < 3) {
						gWC.fDraftedBPV += 40;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							frdship.c_str(), 
							i,
							-1,
							-1, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
				}
				break;
			case 8: // core world
				gWC.fDraftedBPV += 300;
				extraship += "Ph";
				switch (rand() % 5) {
					case 0: extraship += "13"; break;
					case 1: extraship += "1"; break;
					case 2: extraship += "7"; break;
					case 3: extraship += "16"; break;
					default: extraship += "0"; break;
				}
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
					if ((rand() % 13) < 2) {
						gWC.fDraftedBPV += 40;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							defship.c_str(), 
							i,
							-1,
							-1, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
					if ((rand() % 13) < 2) {
						gWC.fDraftedBPV += 40;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							frdship.c_str(), 
							i,
							-1,
							-1, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
				}
				break;
			case 9: // home world 
				gWC.fDraftedBPV += 450;
				switch (fMyRace) {
					case kFederation: extraship = "F-PE"; break;
					case kKlingon: extraship = "K-PKZ"; break;
					case kRomulan: extraship = "R-PRO"; break;
					case kLyran: extraship = "L-PLY"; break;
					case kGorn: extraship = "G-PG1"; break;
					case kHydran: extraship = "H-PHY"; break;
					case kMirak: extraship = "Z-PM1"; break;
					case kISC: extraship = "I-PI1"; break;
					default: extraship = "O-PE"; break;
				}
				if (!mIsTeamHumanControlled()) for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
					if ((rand() % 17) < 3) mCreateShip(	typeid( tCommonEnemyShip ), 
						defship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
					if ((rand() % 17) < 3) mCreateShip(	typeid( tCommonEnemyShip ), 
						frdship.c_str(), 
						i,
						-1,
						-1, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			default: extraship = ""; 
				break;
		}
		if (extraship != "") {
			mCreateShip(	typeid( tCommonEnemyShip ), 
				extraship.c_str(), 
				kStartPosition_Z,
				-1,
				fMyShipID++, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		} else gWC.mLogCall("Could not generate the base/planet for race ", fMyRace);
	}
	if (fHumanControlled || (mIsTeamHumanControlled()) || (gWC.fDraftedHumans > 0)) {
		return;
	}
	std::string shipname = fShipName;
	std::string shiptype = fShipType;

	int32 bpv, i, minbpv, bpvsofar;
	bpv = 0;
	minbpv = 60;

	if (fTeamID == kEnemyTeam3) {
		if (gWC.fUseMinefield) {
			for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-DEF", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"lg mine",
					kDefaultShipOptions );
			}
			for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-LP", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"sm mine",
					kDefaultShipOptions );
			}
		} else {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv += minbpv;
				bpv = (((rand() % 30) + 95) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				if (bpv < 65) bpv = 50 + (rand() % 35);
				bpvsofar = bpv;
				if (bpv > 185) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < 1) draftedbpv = 100;
				if (draftedbpv < bpv) {
					id = fShipMetaID;
					shiptype = fShipType;
					bpvsofar = draftedbpv;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;
				if ((bpv - bpvsofar) >= 45) {
					int32 fleetbpv = bpv - bpvsofar;
					if (fleetbpv < 0) fleetbpv = 0;
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV


					switch (gWC.fMapPicked) {
						case 4:
							fleetbpv = fleetbpv - (rand() % 50);
							if (fleetbpv < 50) fleetbpv = 50 + (rand() % 23);
							break;
						case 5:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 75) fleetbpv = 70 + (rand() % 31);
							break;
						case 6:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 43);
							break;
						case 7:
							if (fleetbpv < 50) fleetbpv = 60 + (rand() % 37);
							break;
						case 8:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 37);
							break;
						case 9:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 150) fleetbpv = 120 + (rand() % 41);
							break;
						default:
							break;
					}

					if (gWC.fMapPicked > 3) switch (gWC.fDrafterRace) {
						case kLyran: fleetbpv = (4 * fleetbpv) / 5; break;
						case kGorn: fleetbpv = (5 * fleetbpv) / 6; break;
						case kRomulan: fleetbpv = (6 * fleetbpv) / 7; break;
						case kHydran: break;
						case kKlingon: fleetbpv = (6 * fleetbpv) / 5; break;
						case kMirak: 
						case kFederation: fleetbpv = (5 * fleetbpv) / 4; break;
						default: break;
					}

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// calculate the number of AI ships we can handle
						int32 fleetsize = gWC.mGetFleetSize(fleetbpv);
						// put in a call to pick the fleet
						std::string shipspicked[6];
						// create the ships
						fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);
						for (int i = 0; i < fleetsize; i++) {
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_J + (rand() % 3),
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}

				if (mShipsCurrentlyControlled() < 1) {
					gWC.fDraftedBPV += 100;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						"O-BS", 
						kStartPosition_L,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
		}
	}

	gST.mDebug_fprintf("---- Finished: tCommonEnemyTeam::mOldCreateShipsForTeam\n");
}

void tCommonEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( mIsTeamHumanControlled() )
	{
		if (gWC.fDebug && (fTeamID != kTeam1)) return;
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

}

void tCommonEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tCommonEnemyTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fMyShipID < 0) switch (fTeamID) {
		case kEnemyTeam1: fMyShipID = kET1Ship1; break;
		case kEnemyTeam2: fMyShipID = kET2Ship1; break;
		case kEnemyTeam3: fMyShipID = kET3Ship1; break;
		case kEnemyTeam4: fMyShipID = kET4Ship1; break;
		case kEnemyTeam5: fMyShipID = kET5Ship1; break;
		case kEnemyTeam6: fMyShipID = kET6Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	fShipMetaID = shipScriptDescription.fMetaDatabaseID;
	fHumanControlled = mIsTeamHumanControlled();
	if (!fHumanControlled) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		gWC.mSetDrafting();
		fHumanControlled = kFalse;
	} else {
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		fHumanControlled = kTrue;
		// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		std::string msg = "Drafted human enemy ship: ";
		msg += fShipType.c_str();
		msg += ", hullsize: ";
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, gWC.fDraftedBPV);
		mCreateShip(	typeid( tCommonEnemyShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftedHumans++;
	}
	fNumShips++;

	gST.mDebug_fprintf("--- Finished: tCommonEnemyTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}





void tCommonEnemyTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonEnemyTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonEnemyTeam::mNumShips()
{
	return fNumShips;
}

void tCommonEnemyTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}



