/********************************************************************************************************\

	File Name:	CommonPlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerShipBaseState.h"
#include "WorldConditions.h"
#include "MissionText.h"

void tCommonPlayerShipBaseState::mBeginState()
{
	fCurrAlert = kAlertRed;
	fDataptr = NULL;
	fTransTurns = 3 + (rand() % 5);
	fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
	tTeamInfo* team = gWC.fTeamList[ fChildActor->mGetTeam() ];
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	fChildActor->mSetShipCallbackEvents( kNotifyDamage);
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
	fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem);
	fChildActor->mSetShipCallbackEvents( kNotifyHostChanged);
	
	fDetectionRange = 80.0f;
	fCurrentlyScanning  = false;
	fTractorOn = false;
}

void tCommonPlayerShipBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}

void tCommonPlayerShipBaseState::mEndState()
{
}

void tCommonPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
}
void tCommonPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	float analysistime = 200.0f / fDetectionRange;
	// if we take damage we have a chance of breakdown
	if (damagedShip && (damagedShip->mGetTeam() <= kTeam3) && hostileShip && (hostileShip->mGetTeam() >= kTeam4)) {
		gWC.fEnemyFired = true;
	}
}
void tCommonPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	// make sure the jamming has been set for both sides
	if (!gWC.fJammingDetermined) {
		gWC.fJammingDetermined = true;
		int32 total = gWC.fPlayerJammingPotential + gWC.fEnemyJammingPotential;
		if (total > 0) {
			if ((rand() % total) < gWC.fPlayerJammingPotential) {
				gWC.mLogCall("Player able to jam enemy communications", 1);
				gWC.fPlayerJamming = true;
			}
			if ((rand() % total) < gWC.fEnemyJammingPotential) {
				gWC.mLogCall("Enemy able to jam player communications", 1);
				gWC.fEnemyJamming = true;
			}
		}
	}

	// make sure a scanning ship really exists
	if (!ship) return;

	// only process this scan with a live player's ship
	if (fChildActor->mGetTeam() > kTeam3) {
		return;
	}

	// keep track of whether or not our ship is currently scanning
	if (ship == fChildActor) {
		fCurrentlyScanning = deepScanOn;
	}

	if (gWC.fPlayerScanCompleted) return;

	mChkFinalAnalysis();
	
	// we might have completed the mission
	if (gWC.fPlayerGoalCompleted) 
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	
	// set notification up for the next scan
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
}
void tCommonPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{
}
void tCommonPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
}
void tCommonPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonPlayerShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonPlayerShipBaseState::mOnHostChanged()
{
	gWC.fHostLeft = true;
}

void tCommonPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo *thisteam = gWC.fTeamList[ fChildActor->mGetTeam() ];
	tTeamInfo *drafterteam= NULL;
	tShipInfo *ship = NULL;
	bool shipsdetected = false;
	float range = fDetectionRange;
	float mutinyTime = 7 + (rand() % 9);
	switch (timerID) {
		case kStartingID:
			if (!fDataptr) {
				fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
			}
			if (fDataptr) {
				fDetectionRange = 80.0f + fDataptr->jamming;
				if (fDetectionRange < 80.0f) fDetectionRange = 80.0f;
				if (fDataptr->isscout) fDetectionRange *= 2;
				gWC.fPlayerJammingPotential = fDataptr->jamming;
			} else fDetectionRange = 80.0f;
			if (gWC.fUsableAsteroid) {	
				if (fChildActor->mGetCustomID() == kMaxShips) fChildActor->mSetShipMaxSpeed(0);
				else if (fChildActor->mGetTeam() == kTeam1) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"There is an we may be able to tractor and use as cover, Sir", 
						kMissionScriptMessage );
				}
				if (fChildActor->mGetTeam() == kTeam3) {
					// this is the asteroid, make sure it starts out damaged
					fChildActor->mSetSystemHealthFromOrig( kHullGroupHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kForwardHullHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kAftHullHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kCenterHullHardPoint, 0.25f, kFalse );
				}
			}

				if (gWC.fMapPicked > 6) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nWe need to take out this planet and any defenders, Sir!", 
						kMissionScriptMessage );
				} else if (gWC.fMapPicked > 3) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nWe need to take out this base and any defenders, Sir!", 
						kMissionScriptMessage );
				} else {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a hostile warship, Sir!", 
						kMissionScriptMessage );
					mSetShootOnSightRelations();
				}
				if (gWC.fDoomsDay) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"That thing is massive!  We\'ve got to deal with that!", 
						kMissionScriptMessage );
				} else if (gWC.fMutiny) {
					mSetTimedInterval( kTimerTurnsSet, kMutiny, mutinyTime );
				}
			break;
		case kMutiny: mSetMutinyRelations();
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Sir - our ally is turning to attack!!!!", 
					kMissionScriptMessage );
				break;
		case kCheckDistance:
			// see if we've reached the minimum distance to an enemy ship
			shipsdetected = mChkPrelimAnalysis();
			if (!gWC.fAlreadyDetected) {
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
			} 
			// see if we've reached the minimum distance to an enemy ship
			break;
		case kCheckGoalIndicator:
			if (!gWC.fPlayerScanCompleted) {
				mChkFinalAnalysis();
				if (!gWC.fPlayerScanCompleted && ((rand() % 7) > 3)) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We must get closer before we can verify the target, Sir", kNonCommScriptMessage );
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "I am updating the plotted path towards them now", kNonCommScriptMessage );
				}
			}
			fDetectionRange *= 1.25f;
			fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
			mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
			break;
		case kCheckTransportCompletion:
			gWC.fPlayerGoalCompleted = true;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Transfer complete, Sir!", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kAwayTeamDone:
			if ((rand() % 3) == 0) {
				gWC.fPlayerGoalCompleted = true;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Something\'s gone wrong on the derelict, Sir!", kMissionScriptMessage );
				gWC.fEnemyTargetShip->mDestroyShip();
				mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			} else {
				gWC.fPlayerGoalCompleted = true;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The team is done, Sir, we can proceed with demolishing the derelict", kMissionScriptMessage );
				gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransBlackBox, 1);
			}
			break;
		case kAllComplete:
			gWC.fPlayerGoalCompleted = true;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				"All complete, Sir - everything checks out", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kCheckGameEnd:
			gWC.mCheckForGameEnd();
			break;
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tCommonPlayerShipBaseState::mSetFriendlyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm2*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm3*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetHostileRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetEscapeRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetMutinyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetShootOnSightRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}


bool tCommonPlayerShipBaseState::mChkPrelimAnalysis()
{
	float range = fDetectionRange;
	if (gWC.fEnemyTargetShip) {
		if (!gWC.fEnemyTargetable) {
			fDetectionRange *= 1.25f;
			if (fCurrentlyScanning) range = 2.0f * range;
			if (gWC.fMapPicked == 2) range = range * 0.75f;
			else if (gWC.fMapPicked == 3) range = range * 0.90f;
			if (gWC.fEnemyTargetShip->mCloakOn()) range = range * 0.75f;
			if (gWC.fEnemyTargetShip->mStealthy()) range = range * 0.75f;
			if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) < fDetectionRange) {
				gWC.fEnemyTargetable = true;
				fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
				gWC.fAlreadyDetected = true;
				switch (gWC.fMissionType) {
				case tWorldConditions::kShootOnSight:
					if ((rand() % 3) == 0) {
						if (gWC.fEnemyTargetShip) {
							gWC.fEnemyTargetShip->mFollowEntity(fChildActor, kAbsolutePriority);
						}
					}
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a naval warship, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re demanding we leave immediately, sir!", 
						kMissionScriptMessage );
					break;
				default:
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nUnknown disturbance detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				}
				if (gWC.fUseMinefield) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\nI\'m also picking up what appears to be a minefield, Sir", 
						kMissionScriptMessage );
				}
			}
		}
	}
	return(gWC.fAlreadyDetected);
}

void tCommonPlayerShipBaseState::mChkFinalAnalysis()
{
	if (!gWC.fAlreadyDetected) return;
	
	// get the target for the scanning ship, if any
	tShipInfo *target = NULL;
	int32 targetid = -1;
	int32 x = 0, y = 0;
	if (fChildActor) targetid = fChildActor->mGetShipTargetID();
	if ((targetid > -1)  && fMissionInfo) {
		target = fMissionInfo->mFindShipID(targetid);
	}

	if (!target) return;

	// set the modifier for final detection range,
	//     larger numbers mean the player has to get closer
	float RangeModifier = 1.0f;
	// =========================================
	switch (gWC.fMissionType) {
		// special case
		case tWorldConditions::kShootOnSight:
			RangeModifier = 15.0f;
			break;
		default:
			RangeModifier = 25.0f;
			break;
	}
	//=======================================
	if (fCurrentlyScanning) RangeModifier *= 0.5f;

	// check and see if the player has been detected in an eavesdropping mission

	if (fChildActor->mGetTeam() == kTeam1) { 
		if (target && (target->mGetTeam() > kTeam3) && (fChildActor->mGetTacticalDistance(target) < (fDetectionRange / RangeModifier))){
			gWC.fPlayerScanCompleted = kTrue;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Scans are inconclusive sir", kMissionScriptMessage );

		}
	} 
}
