/********************************************************************************************************\

	File Name:	CommonPlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonPlayerShip.h"
#include "CommonPlayerTeam.h"
#include "CommonPlayerTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"
#include "ShipList_API.h"


tClassAllocator<tCommonPlayerTeam> gCommonPlayerTeamAllocator;
 
tCommonPlayerTeam::tCommonPlayerTeam()
{
	fTeamID = -1; 
	fMyShipID = -1; 
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
	fHumanControlled = mIsTeamHumanControlled();
}

void tCommonPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonPlayerTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonPlayerTeamBaseState() );
	mGotoState( typeid( tCommonPlayerTeamBaseState ) );
   }
}

void tCommonPlayerTeam::mSetupCommState()
{}

void tCommonPlayerTeam::mCreateShipsForTeam()
{
   	gST.mDebug_fprintf("--- Entering: tCommonPlayerTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

   	if ((gST.narrowed_sl.size() != 0) && gST.narrowed_sl.mIsEnhancedShiplist()) {
   		int32 fShiplistCount = 0;
   		ShipList player_sl = gST.narrowed_sl;

		mOldCreateShipsForTeam();
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tCommonPlayerTeam::mCreateShipsForTeam\n");
}

void tCommonPlayerTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tCommonPlayerTeam::mOldCreateShipsForTeam\n");

	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();


	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftingBPV += (10 * humanBPV / 9);
		return;
	}

	switch (gWC.fMapPicked) {
		case 4: 
		case 5:
			if (gWC.fAddAlly < 1) gWC.fAddAlly = 1;
			break;
		case 6: 
		case 8:
		case 9:
			if (gWC.fAddAlly < 2) gWC.fAddAlly = 2;
			break;
		default:
			break;
	}
	if ((fTeamID == kPlayerTeam3) && (gWC.fAddAlly > 0) && (gWC.fDraftedHumans < 1)) {
		int32 allyBPV = 85 + gWC.fYearLowerBound + (rand() % 43);
		if (gWC.fMapPicked > 3) {
			switch (gWC.fMapPicked) {
				case 4: 
					allyBPV += (rand() % 25);
					break;
				case 5:
					allyBPV += (25 + (rand() % 25));
					break;
				case 6: 
					allyBPV += (50 + (rand() % 25));
					break;
				case 8:
					allyBPV += (50 + (rand() % 35));
					break;
				case 9:
					allyBPV += (65 + (rand() % 50));
				default:
					break;
			}
			switch (mGetRace()) {
				case kLyran: allyBPV = (4 * allyBPV) / 3; break;
				case kGorn: allyBPV = (5 * allyBPV) / 4; break;
				case kRomulan: allyBPV = (6 * allyBPV) / 5; break;
				case kHydran: break;
				case kKlingon: allyBPV = (4 * allyBPV) / 5; break;
				case kMirak: 
				case kFederation: allyBPV = (3 * allyBPV) / 4; break;
				default: break;
			}
		}
		std::string allyshiptype;
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if (allyshiptype != "") {
			gWC.fDraftingBPV += (5 * allyBPV / 6);
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_H,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
		if (gWC.fAddAlly > 1) {
			allyBPV = 105 + gWC.fYearLowerBound + (rand() % 61);
			if (gWC.fMapPicked > 3) {
				switch (gWC.fMapPicked) {
				case 4: 
					allyBPV += (rand() % 25);
					break;
				case 5:
					allyBPV += (25 + (rand() % 25));
					break;
				case 6: 
					allyBPV += (50 + (rand() % 25));
					break;
				case 8:
					allyBPV += (50 + (rand() % 35));
					break;
				case 9:
					allyBPV += (65 + (rand() % 50));
				default:
					break;
				}
				switch (mGetRace()) {
					case kLyran: allyBPV = (4 * allyBPV) / 3; break;
					case kGorn: allyBPV = (5 * allyBPV) / 4; break;
					case kRomulan: allyBPV = (6 * allyBPV) / 5; break;
					case kHydran: break;
					case kKlingon: allyBPV = (4 * allyBPV) / 5; break;
					case kMirak: 
					case kFederation: allyBPV = (3 * allyBPV) / 4; break;
					default: break;
				}
			}
			allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
			if (allyshiptype != "") {
				gWC.fDraftingBPV += (5 * allyBPV / 6);
				mCreateShip(	typeid( tCommonPlayerShip ), 
					allyshiptype.c_str(), 
					kStartPosition_I,
					-1,
					-1, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			}
		}
	}
	// have a chance for for fleet battles, in which each non-human team gets three ships
	else if (((rand() % 29) < 3) && (gWC.fDraftedHumans < 1) && (gWC.fMapPicked == 0)) {
		int32 allyBPV = 69 + gWC.fYearLowerBound + (rand() % 59);
		std::string allyshiptype;
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if (allyshiptype != "") {
			gWC.fDraftingBPV += 19 * allyBPV / 20;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_H,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
		allyBPV = 84 + gWC.fYearLowerBound + (rand() % 59);
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if ((allyshiptype != "") && ((rand() % 7) > 4)) {
			gWC.fDraftingBPV += 9 * allyBPV / 10;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_I,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
		allyBPV = 99 + gWC.fYearLowerBound + (rand() % 59);
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if ((allyshiptype != "") && ((rand() % 7) > 4)) {
			gWC.fDraftingBPV += 11 * allyBPV / 12;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_G,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
	}

	gST.mDebug_fprintf("---- Finished: tCommonPlayerTeam::mOldCreateShipsForTeam\n");
}

void tCommonPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( fHumanControlled )
	{
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
			if (!gWC.fPlayerShip) gWC.fPlayerShip = (tCommonPlayerShip*)(*ship);
		}
	}
	
}


void tCommonPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
   	gST.mDebug_fprintf("--- Entering: tCommonPlayerTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
   	gST.mInitializeSL();
   	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fTeamID == kTeam1) gWC.fDrafterRace = fMyRace;
	if (fMyShipID < 0) switch (fTeamID) {
		case kPlayerTeam1: fMyShipID = kPT1Ship1; break;
		case kPlayerTeam2: fMyShipID = kPT2Ship1; break;
		case kPlayerTeam3: fMyShipID = kPT3Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (fTeamID == kTeam1) gWC.mPickPlacement();
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	if (!mIsTeamHumanControlled()) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		fHumanControlled = kFalse;
		if (fTeamID < kTeam4) gWC.fMutineerShip = fShipType;
	} else {
		if (fTeamID == kTeam1) gWC.fMutineerShip = fShipType;
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		fHumanControlled = kTrue;
		gWC.fDraftingHulls += gWC.mGetHullsize(fShipType);
		std::string msg;
		if (fTeamID == kTeam1) msg = "Drafter ship: ";
		else msg = "Human wing: ";
		msg += fShipType.c_str();
		msg += ", api hullsize id: ";
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, gWC.fDraftingBPV);
		// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		mCreateShip(	typeid( tCommonPlayerShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftingHumans++;
	}
	fNumShips++;

   	gST.mDebug_fprintf("--- Finished: tCommonPlayerTeam::mCreateNewShip: %s\n", shipScriptDescription.fClassName.c_str());
}





void tCommonPlayerTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonPlayerTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonPlayerTeam::mNumShips()
{
	return fNumShips;
}

void tCommonPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}

