/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerTeam.h"
#include "CommonVictoryState.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerShip.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState(	int32 astoundingVictory,
											int32 victory,
											int32 draw,
											int32 defeat,
											int32 devastatingDefeat ) : tChildActorVictoryState< tCommonVictory >(	astoundingVictory,
																													victory,
																													draw,
																													defeat,
																													devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{
	int32 dronerace = kMirak | kFederation | kKlingon;
	int32 weakrace = kLyran | kRomulan;
	if (fTeam->mGetRace() & weakrace) mAddToPrestige(70);
}
void tCommonVictoryState::mEndState()
{}

eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* thisTeam = fMissionInfo->mGetTeamHandle( ID );
/*
		kPlayerTeam1,kPlayerTeam2,kPlayerTeam3, // defending fleet
		kEnemyTeam1,kEnemyTeam2,kEnemyTeam3, // invading fleet
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // a visiting enemy diplomat or invading recon vessel
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
*/
	// PvP victories are worth more than victories against the AI,
	// PvP defeats are treated as less serious that defeats against the AI
	if (!thisTeam) return kDraw;
	if ((gWC.fDraftedHumans > 0) || (gWC.fMissionType == tWorldConditions::kShootOnSight)) {
		if (thisTeam->mShipsCurrentlyControlled() > 0) return kAstoundingVictory;
		else return kDefeat;
	} else {
		// add any prestige from the sale of black market goods

		// add a bonus in ambush/rescue missions
		if (gWC.fAmbushed) {
			this->mAddToBonusPrestige(150);
		}
		if (gWC.fRescued) {
			this->mAddToBonusPrestige(150);
		}

		// figure out who's left
		iTruth	playerTeamHasShips = kFalse;
		iTruth	enemyTeamHasShips = kFalse;
		iTruth	thisTeamHasShips = kFalse;
		int32 i;
		tTeamInfo* team = gWC.fTeamList[ID];
		if (!team) {
			team = fMissionInfo->mGetTeamHandle(eTeamID(ID));
			if (team) gWC.fTeamList[ID] = team;
		}
		if (!team) return kDraw;
		if ( team->mShipsCurrentlyControlled() ) thisTeamHasShips = kTrue;

		for ( i = kPlayerTeam1; i < kMaxMissionTeams; i++ )
		{
			tTeamInfo* team = gWC.fTeamList[i];
			if (!team) {
				team = fMissionInfo->mGetTeamHandle(eTeamID(i));
				if (team) gWC.fTeamList[i] = team;
			}
			if ( team )
			{
				if ( team->mShipsCurrentlyControlled() ) {
					switch (i) {
					case kPlayerTeam1:
					case kPlayerTeam2:
					case kPlayerTeam3:
						playerTeamHasShips = kTrue;
						break;
					case kPlayerTeam4:
					case kPlayerTeam5:
					case kPlayerTeam6:
					case kEnemyTeam1:
					case kEnemyTeam2:
					case kEnemyTeam3:
					case kEnemyTeam5:
					case kEnemyTeam6:
						enemyTeamHasShips = kTrue;
						break;
					case kEnemyTeam4:
						enemyTeamHasShips = kTrue;
						break;
					}
				}
			}
		}
		// figure out and assign the results for this team's perpec
		int32 score = 0;
		if (gWC.fPlayerGoalFailed) score = -2;

		switch (gWC.fMissionType) {
			// cases where you scan (ai only) then fight
			case tWorldConditions::kShootOnSight:
				if (thisTeamHasShips && (ID < kTeam4)) score++;
				if (thisTeamHasShips && (ID > kTeam3)) score--;
				if (enemyTeamHasShips) score--;
				if (playerTeamHasShips) score++;
				if (gWC.fPlayerScanCompleted || gWC.fPlayerGoalCompleted) score++;
				if (gWC.fPlayerFiredEarly) score--;
				if (ID > kTeam3) {
					// results for AI team
					switch (score) {
						case 3: return kDevastatingDefeat;
						case 2: return kDefeat;
						case 1: 
						case 0:
						case -1: return kDraw;
						case -2: return kVictory;
						default: return kAstoundingVictory;
					}
				} else {
					// results for player team
					switch (score) {
						case 3: return kAstoundingVictory;
						case 2: return kVictory;
						case 1:
						case 0: 
						case -1: return kDraw;
						case -2: return kDefeat;
						default: return kDevastatingDefeat;
					}
				}
				break;
			default:
				if (gWC.fPlayerScanCompleted) score++;
				if (gWC.fPlayerGoalCompleted) score+=2;
				if (gWC.fPlayerFired) score -= 2;
				if (ID > kTeam3) {
					// results for AI team
					switch (score) {
						case 5: return kDevastatingDefeat;
						case 4: 
						case 3: return kDefeat;
						case 2: return kDraw;
						case 1: 
						case 0: return kVictory;
						default: return kAstoundingVictory;
					}
				} else {
					// results for player team
					switch (score) {
						case 5: return kAstoundingVictory;
						case 4: 
						case 3: return kVictory;
						case 2: return kDraw;
						case 1: 
						case 0: return kDefeat;
						default: return kDevastatingDefeat;
					}
				}
				break;
		}
	}
	return kDraw;
}

//
//	Ship events
//

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = gWC.fTeamList[ teamID ];
	Assert( team );
	if ( !team )
		return;	

	// if an enemy ship is destroyed give us a bonus of 50pp
	// if one of our own ships is destroyed give us a penalty of 50pp
	bool playership = kFalse;
	bool playerteam = kFalse;
	bool enemyship = kFalse;
	bool enemyteam = kFalse;
	switch (ship->mGetTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playership = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyship = kTrue;
			break;
		case kEnemyTeam4:
			enemyship = kTrue;
			break;
	}
	switch (fTeam->mGetTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playerteam = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyteam = kTrue;
			break;
		case kEnemyTeam4:
			enemyteam = kTrue;
			break;
	}
	switch (gWC.fMissionType) {
		case tWorldConditions::kShootOnSight:
			if (playerteam) {
				if (playership) {
					this->mAddToPrestige( -25 );
					this->mAddToBonusPrestige( -25 );
				} else if (enemyship) {
					this->mAddToPrestige( 25 );
					this->mAddToBonusPrestige( 25 );
				}
			} else if (enemyteam) {
				if (enemyship) {
					this->mAddToPrestige( -25 );
					this->mAddToBonusPrestige( -25 );
				} else if (playership) {
					this->mAddToPrestige( 25 );
					this->mAddToBonusPrestige( 25 );
				}
			}
			break;
		default:
			this->mAddToPrestige(-25);
			this->mAddToBonusPrestige(-25);
			break;
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = gWC.fTeamList[ teamID ];
	Assert( team );
	if ( !team )
		return;

	// if an enemy ship is destroyed give us a bonus of 50pp
	// if one of our own ships is destroyed give us a penalty of 50pp
	bool playership = kFalse;
	bool playerteam = kFalse;
	bool enemyship = kFalse;
	bool enemyteam = kFalse;
	switch (ship->mGetOriginalTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playership = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyship = kTrue;
			break;
		case kEnemyTeam4:
			enemyship = kTrue;
			break;
	}
	switch (fTeam->mGetTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playerteam = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyteam = kTrue;
			break;
		case kEnemyTeam4:
			enemyteam = kTrue;
			break;
	}
	switch (gWC.fMissionType) {
		case tWorldConditions::kShootOnSight:
			if (playerteam) {
				if (playership) {
					this->mAddToPrestige( -25 );
					this->mAddToBonusPrestige( -25 );
				} else if (enemyship) {
					this->mAddToPrestige( 25 );
					this->mAddToBonusPrestige( 25 );
				}
			} else if (enemyteam) {
				if (enemyship) {
					this->mAddToPrestige( -25 );
					this->mAddToBonusPrestige( -25 );
				} else if (playership) {
					this->mAddToPrestige( 25 );
					this->mAddToBonusPrestige( 25 );
				}
			}
			break;
		default:
			this->mAddToPrestige(-25);
			this->mAddToBonusPrestige(-25);
			break;
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = gWC.fTeamList[ teamID ];
	Assert( team );
	if ( !team )
		return;


	// if an enemy ship disengaged give us a bonus of 25pp
	// if one of our own ships disengaged give us a penalty of 25pp
	bool playership = kFalse;
	bool playerteam = kFalse;
	bool enemyship = kFalse;
	bool enemyteam = kFalse;
	switch (ship->mGetTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playership = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyship = kTrue;
			break;
		case kEnemyTeam4:
			enemyship = kTrue;
			break;
	}
	switch (fTeam->mGetTeam()) {
		case kPlayerTeam1:
		case kPlayerTeam2:
		case kPlayerTeam3:
			playerteam = kTrue;
			break;
		case kPlayerTeam4:
		case kPlayerTeam5:
		case kPlayerTeam6:
		case kEnemyTeam1:
		case kEnemyTeam2:
		case kEnemyTeam3:
		case kEnemyTeam5:
		case kEnemyTeam6:
			enemyteam = kTrue;
			break;
		case kEnemyTeam4:
			enemyteam = kTrue;
			break;
	}
	switch (gWC.fMissionType) {
		case tWorldConditions::kShootOnSight:
		default:
			gWC.fPlayerGoalFailed = true;
			if (playerteam) {
					this->mAddToPrestige( -25 );
					this->mAddToBonusPrestige( -25 );
			} 
			break;
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}

void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{
}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
