/********************************************************************************************************\

	File Name:	CommonEnemyShip.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonEnemyShip.h"
#include "CommonEnemyShipBaseState.h"
#include "WorldConditions.h"
#include "ScriptInterface.h"
#include "MissionInfo.h"
#include "MissionText.h"

tClassAllocator<tCommonEnemyShip> gCommonEnemyShipAllocator;

void tCommonEnemyShip::mSetupState()
{
   tTeamInfo* ti = gWC.fTeamList[ mGetTeam() ];
   if ((ti) && (! ( (!ti->mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) ))
      {	
	mRegisterState( new tCommonEnemyShipBaseState() );
	mGotoState( typeid( tCommonEnemyShipBaseState ) );
   }
}

 
void tCommonEnemyShip::mSetupCommState()
{
}

int32 tCommonEnemyShip::mPickValue(int32 max)
{
	int32 smaller, middle, larger;
	if (max <= 12) {
		smaller = max;
		middle = 0;
		larger = 0;
	} else if (max <= 24) {
		smaller = 12;
		middle = max - 12;
		larger = 0;
	} else {
		smaller = 12;
		middle = 12;
		larger = max - 24;
	}
	smaller = ((67 + (rand() % 34)) * smaller) / 100;
	middle = ((50 + (rand() % 26)) * middle) / 100;
	larger = ((34 + (rand() % 34)) * larger) / 100;
	return smaller + middle + larger;
}


void tCommonEnemyShip::mSetupShip()
{

   tTeamInfo* ti = gWC.fTeamList[ mGetTeam() ];
   if (ti && !ti->mIsTeamHumanControlled()) {
	   gWC.mLogCall(mGetClassName(), gWC.mGetBPV(mGetClassName()));
   }
	if (gWC.fMissionType == tWorldConditions::kShootOnSight) return;
	if (gWC.fMissionType == tWorldConditions::kFleetBattle) return;
	if (gWC.fDraftedHumans) return;

	int32 speed = 31;
	switch (gWC.fMissionType) {
	case tWorldConditions::kEnemyCombat:
		speed = 16 + (rand() % 13);
		break;
	case tWorldConditions::kPirateInnocent:
	case tWorldConditions::kPirateWanted:
	case tWorldConditions::kPirateHostile:
	case tWorldConditions::kPirateSpy:
	case tWorldConditions::kPirateSmuggler:
		speed = 15 + (rand() % 17);
		break;
	case tWorldConditions::kConvoy:
	case tWorldConditions::kSwarm:
		speed = 4 + (rand() % 5);
		break;
	case tWorldConditions::kMonsterInnocent:
	case tWorldConditions::kMonsterHostile:
	case tWorldConditions::kPirateBase: 
		speed = 12 + (rand() % 13);
		break;
	case tWorldConditions::kCleanup:
	case tWorldConditions::kDerelict: 
	case tWorldConditions::kOutpostRaid:
		speed = rand() % 5;
		break;
	case tWorldConditions::kEvacuateTerrain:
	case tWorldConditions::kSalvageTerrain:
	default:
		speed = (rand() % 3);
		break;
	}
	switch (mGetClassType()) {
		case kClassListeningPost:
				case kClassBaseStation:
				case kClassBattleStation:
				case kClassStarBase:
				case kClassPlanets:
					speed = 0;
					break;
				default:
					break;
	}
	char stype[10];
	strcpy(stype, mGetClassName());
	for (int i = 0; i < 8; i++) stype[i] = stype[i+1];
	if (strcmpi(stype, "-DEF") == 0) speed = 0;
	if (strcmpi(stype, "-FRD") == 0) speed = 0;
	if (strcmpi(stype, "-LP") == 0) speed = 0;
	
	mSetFinalDestination();
	mSetShipMaxSpeed(speed);
}

tCommonEnemyShip::tCommonEnemyShip()
{
	fDataptr = NULL;
}

bool tCommonEnemyShip::mCloakOn(  )
{
	return (fDataptr && fDataptr->cloak);
}

bool tCommonEnemyShip::mStealthy(  )
{
	return (fDataptr && fDataptr->stealthy);
}

void tCommonEnemyShip::mSetTargetable(bool State)
{
	tShipSystemsInfo info;
	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipTargetable;
	info.fInt32[1] = State;
	info.fShipEntityID = mGetShipID();
	gScriptInterface.mEntityMgr( info );
}
