/********************************************************************************************************\

	File Name:	CommonEnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonEnemyShip.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"

 
tClassAllocator<tCommonEnemyTeam> gCommonEnemyTeamAllocator;

tCommonEnemyTeam::tCommonEnemyTeam() : fNumShips( 0 )
{
	fHumanControlled = mIsTeamHumanControlled();
	fTeamID = -1;
	fMyShipID = -1;
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
}

void tCommonEnemyTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonEnemyTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonEnemyTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonEnemyTeamBaseState() );
	mGotoState( typeid( tCommonEnemyTeamBaseState ) );
   }
   
}

void tCommonEnemyTeam::mSetupCommState()
{}


void tCommonEnemyTeam::mCreateShipsForTeam()
{
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();
	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftedBPV += (10 * humanBPV / 9);
	}

	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) gWC.fAddAlly = 0;
	if ((gWC.fDraftedHumans > 0) && (gWC.fMapPicked < 4)) {
		return;
	}

	if ((fTeamID == kEnemyTeam3) && (gWC.fMapPicked > 3)) {
		gWC.mLogCall("Creating planet or base in map type ", gWC.fMapPicked);
		std::string extraship;
		switch (fMyRace) {
			case kFederation: extraship = "F-PL"; break;
			case kKlingon: extraship = "K-PL"; break;
			case kRomulan: extraship = "R-PL"; break;
			case kLyran: extraship = "L-PL"; break;
			case kGorn: extraship = "G-PL"; break;
			case kHydran: extraship = "H-PL"; break;
			case kMirak: extraship = "Z-PL"; break;
			case kISC: extraship = "I-PL"; break;
			default: extraship = "PL"; break;
		}
		switch (gWC.fMapPicked) {
			case 4: extraship = gWC.fBSchoice[fMyRace]; 
				gWC.fDraftedBPV += 120;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 60;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 60;
				break;
			case 5: extraship = gWC.fBATSchoice[fMyRace]; 
				gWC.fDraftedBPV += 240;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 60;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 60;
				break;
			case 6: extraship = gWC.fSBchoice[fMyRace]; 
				gWC.fDraftedBPV += 400;
				if (gWC.fYearLowerBound > 7) gWC.fDraftedBPV += 100;
				if (gWC.fYearLowerBound > 19) gWC.fDraftedBPV += 100;
				break;
			case 7: // colony world
				gWC.fDraftedBPV += 100;
				switch (rand() % 5) {
					case 0: extraship += "0"; break;
					case 1: extraship += "8"; break;
					case 2: extraship += "3"; break;
					case 3: extraship += "9"; break;
					default: extraship = "N-PLPh0"; break;
				}
				break;
			case 8: // core world
				extraship += "Ph";
				gWC.fDraftedBPV += 300;
				switch (rand() % 5) {
					case 0: extraship += "0"; break;
					case 1: extraship += "1"; break;
					case 2: extraship += "7"; break;
					case 3: extraship += "16"; break;
					default: extraship += "13"; break;
				}
				break;
			case 9: // home world 
				gWC.fDraftedBPV += 450;
				switch (fMyRace) {
					case kFederation: extraship = "F-PE"; break;
					case kKlingon: extraship = "K-PKZ"; break;
					case kRomulan: extraship = "R-PRO"; break;
					case kLyran: extraship = "L-PLY"; break;
					case kGorn: extraship = "G-PG1"; break;
					case kHydran: extraship = "H-PHY"; break;
					case kMirak: extraship = "Z-PM1"; break;
					case kISC: extraship = "I-PI1"; break;
					default: extraship = "O-PE"; break;
				}
				break;
			default: extraship = ""; 
				break;
		}
		if (extraship == "") {
				switch (fMyRace) {
					case kFederation: extraship = "F-FRD"; break;
					case kKlingon: extraship = "K-FRD"; break;
					case kRomulan: extraship = "R-FRD"; break;
					case kLyran: extraship = "L-FRD"; break;
					case kGorn: extraship = "G-FRD"; break;
					case kHydran: extraship = "H-FRD"; break;
					case kMirak: extraship = "Z-FRD"; break;
					case kISC: extraship = "I-FRD"; break;
					default: break;
				}
		}
		if (extraship != "") {
			if (gWC.fDebug) gWC.mLogCall(extraship, fMyShipID);
			mCreateShip(	typeid( tCommonEnemyShip ), 
				extraship.c_str(), 
				kStartPosition_Z,
				-1,
				fMyShipID++, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		} else gWC.mLogCall("Could not generate a base/planet for race ", fMyRace);
	}
	if (fHumanControlled || (gWC.fDraftedHumans > 0) || (mIsTeamHumanControlled())) {
		return;
	}
	std::string shipname = fShipName;
	std::string shiptype = fShipType;

	int32 bpv, i, x, y, minbpv, bpvsofar;
	bpv = 0;
	minbpv = 60;


	if ((gWC.fMissionType == tWorldConditions::kEnemyCombat) ||
		(gWC.fMissionType == tWorldConditions::kShootOnSight)) {
		if ((fTeamID == kEnemyTeam2) && ((rand() % 17) < 6)) {
			if ((gWC.fDraftedHumans < 1) && (fShipType != "")) {
				gWC.fExtraDraftedAIBPV = gWC.mGetBPV(fShipType);
				gWC.fDraftedBPV += gWC.fExtraDraftedAIBPV;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_K,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			}
		}
	}

	if (gWC.fMissionType == tWorldConditions::kFleetBattle) {
		if ((fTeamID == kEnemyTeam2) || (fTeamID == kEnemyTeam1)) {
			if (gWC.fDraftedHumans < 1) {
				bpv = 105 + (rand() % 73);
				bpv = gWC.mPickShip(bpv, mGetRace(), fShipType);
				fShipMetaID = -1;
				if (fShipType == "") fShipType = "O-CR1";
				gWC.fExtraDraftedAIBPV += gWC.mGetBPV(fShipType);
				gWC.fDraftedBPV += gWC.mGetBPV(fShipType);
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_K,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			}
		}
	}

	if (fTeamID == kEnemyTeam3) {
		if(gWC.fUseMinefield) {
			for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-DEF", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"lg mine",
					kDefaultShipOptions );
			}
			for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-LP", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"sm mine",
					kDefaultShipOptions );
			}
		} else if (gWC.fMissionType == tWorldConditions::kEnemyCombat) {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 90) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < bpv) {
					id = fShipMetaID;
					shiptype = fShipType;
					bpvsofar = draftedbpv;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gWC.fDraftedBPV += bpvsofar;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;
				if ((bpv - bpvsofar) > 45) {
					int32 fleetbpv = bpv - bpvsofar;
					if (gWC.fMapPicked > 3) switch (gWC.fDrafterRace) {
						case kLyran: fleetbpv = (3 * fleetbpv) / 4; break;
						case kGorn: fleetbpv = (4 * fleetbpv) / 5; break;
						case kRomulan: fleetbpv = (5 * fleetbpv) / 6; break;
						case kHydran: break;
						case kKlingon: fleetbpv = (5 * fleetbpv) / 4; break;
						case kMirak: 
						case kFederation: fleetbpv = (4 * fleetbpv) / 3; break;
						default: break;
					}
					if (fleetbpv < 0) fleetbpv = 0;
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					// calculate the number of AI ships we can handle
					int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

					// put in a call to pick the fleet
					std::string shipspicked[6];
					fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// create the ships
						for (int i = 0; i < fleetsize; i++) {
							int32 id = -1;
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_L,
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}
		} else if ((gWC.fMissionType == tWorldConditions::kShootOnSight) || (gWC.fMissionType == tWorldConditions::kFleetBattle)){
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv += minbpv;
				bpv = (((rand() % 30) + 90) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < bpv) {
					id = fShipMetaID;
					shiptype = fShipType;
					bpvsofar = draftedbpv;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gWC.fDraftedBPV += bpvsofar;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;
				if ((bpv - bpvsofar) > 45) {
					int32 fleetbpv = bpv - bpvsofar;
					switch (gWC.fMapPicked) {
						case 4:
							fleetbpv = fleetbpv - (rand() % 50);
							if (fleetbpv < 50) fleetbpv = 50 + (rand() % 23);
							break;
						case 5:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 75) fleetbpv = 70 + (rand() % 31);
							break;
						case 6:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 43);
							break;
						case 7:
							if (fleetbpv < 50) fleetbpv = 60 + (rand() % 37);
							break;
						case 8:
							fleetbpv = fleetbpv - (rand() % 100);
							if (fleetbpv < 100) fleetbpv = 90 + (rand() % 37);
							break;
						case 9:
							fleetbpv = fleetbpv - (rand() % 200);
							if (fleetbpv < 150) fleetbpv = 120 + (rand() % 41);
							break;
						default:
							break;
					}
					if (gWC.fMapPicked > 3) {
						switch (gWC.fDrafterRace) {
							case kLyran: fleetbpv = (4 * fleetbpv) / 5; break;
							case kGorn: fleetbpv = (5 * fleetbpv) / 6; break;
							case kRomulan: fleetbpv = (6 * fleetbpv) / 7; break;
							case kHydran: break;
							case kKlingon: fleetbpv = (6 * fleetbpv) / 5; break;
							case kMirak: 
							case kFederation: fleetbpv = (5 * fleetbpv) / 4; break;
							default: break;
						}
					} else if (gWC.fMissionType == tWorldConditions::kFleetBattle) {
						switch (mGetRace()) {
							case kLyran: fleetbpv = (10 * fleetbpv) / 9; break;
							case kGorn:
							case kRomulan: break;
							case kHydran: fleetbpv = (9 * fleetbpv) / 10; break;
							case kKlingon: fleetbpv = (5 * fleetbpv) / 6; break;
							case kMirak: 
							case kFederation: fleetbpv = (4 * fleetbpv) / 5; break;
							default: break;
						}
					}
					if (fleetbpv < 0) fleetbpv = 0;
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					// calculate the number of AI ships we can handle
					int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

					// put in a call to pick the fleet
					std::string shipspicked[6];
					fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// create the ships
						for (int i = 0; i < fleetsize; i++) {
							int32 id = -1;
							if ((rand() % 7) < 5) id = fShipMetaID;
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_L,
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}
		} else if (gWC.fMissionType == tWorldConditions::kSwarm) {
			for (i = kStartPosition_S; i < kStartPosition_X; i++) {
				std::string swarmtype = "M-SSS";
				switch (rand() % 8) {
					case 0: swarmtype = "M-LCS"; break;
					case 1: swarmtype = "M-MTS"; break;
					case 2: swarmtype = "N-PR2"; break;
					case 3: swarmtype = "T-INT"; break;
					default: swarmtype = "M-SSS"; break;
				}
				gWC.fDraftedBPV += 30;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					swarmtype.c_str(), 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDeathUponCapture );
			}
		} else if (gWC.fMissionType == tWorldConditions::kConvoy) {
			for (i = kStartPosition_S; i < kStartPosition_X; i++) {
				std::string frtrtype = "";
				switch (mGetRace()) {
					case kFederation: frtrtype = "F"; break;
					case kKlingon: frtrtype = "K"; break;
					case kRomulan: frtrtype = "R"; break;
					case kMirak: frtrtype = "Z"; break;
					case kLyran: frtrtype = "L"; break;
					case kHydran: frtrtype = "H"; break;
					case kISC: frtrtype = "I"; break;
					case kGorn: frtrtype = "G"; break;
					default: frtrtype = "F"; break;
				}
				if ((rand() % 17) > 10) frtrtype += "-FQL";
				else frtrtype += "-FLPh";
				gWC.fDraftedBPV += 50;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					frtrtype.c_str(), 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDeathUponCapture );
			}
		}
	} else if (fTeamID == kEnemyTeam4) {
		/*
		*/
		std::string extraship;
		switch (gWC.fMissionType) {
			case tWorldConditions::kOutpostRaid:
				// set up one FRD
				switch (fMyRace) {
					case kFederation: extraship = "F-FRD"; break;
					case kKlingon: extraship = "K-FRD"; break;
					case kRomulan: extraship = "R-FRD"; break;
					case kLyran: extraship = "L-FRD"; break;
					case kGorn: extraship = "G-FRD"; break;
					case kHydran: extraship = "H-FRD"; break;
					case kMirak: extraship = "Z-FRD"; break;
					case kISC: extraship = "I-FRD"; break;
					default: extraship = "O-FRD"; break;
				}
				gWC.fDraftedBPV += 80;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					extraship.c_str(), 
					kStartPosition_Z,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDeathUponCapture );
				// set up a mix of cargo boxes
				for (i = kStartPosition_S; i <= kStartPosition_Z; i++) {
					if ((rand() % 7) < 3) 
						mCreateShip(	typeid( tCommonEnemyShip ), 
							"N-BOX", 
							i,
							-1,
							fMyShipID++, 0, 0, -1, -1,
							"",
							kDeathUponCapture );
				}
				break;
			case tWorldConditions::kConvoy:
				bpv = (rand() % 43) + 75;
				bpv = gWC.mPickShip(bpv, mGetRace(), shiptype);
				if (shiptype == "") shiptype = "N-E4";
				gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_Z,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDeathUponCapture );
				break;
			case tWorldConditions::kSwarm:
				gWC.fDraftedBPV += 65;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"N-E4", 
					kStartPosition_Z,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"Mentor",
					kDeathUponCapture );
				break;
			case tWorldConditions::kOneOfOurOwn:
				bpv = gWC.fDraftingBPV - gWC.fDraftedBPV;
				bpv = (((rand() % 30) + 90) * bpv) / 100;
				if (bpv < 80) bpv = 65 + (rand() % 43);
				bpv = gWC.mPickShip(bpv, gWC.fDrafterRace, shiptype);
				if (shiptype == "") shiptype = gWC.fMutineerShip;
				gWC.fDraftedBPV += bpv;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_N,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			case tWorldConditions::kCleanup:
				fMissionInfo->mCreateMapObject('B', gWC.xCtr, gWC.yCtr);
			case tWorldConditions::kDerelict:
				gWC.fDraftedBPV += 15;
				if (shiptype == "") shiptype = "O-CR1";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_N,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	} else if (fTeamID == kEnemyTeam5) {
		std::string extraship;
		switch (gWC.fMissionType) {
			case tWorldConditions::kPirateBase:
				shiptype = "O-BS";
				gWC.fDraftedBPV += 100;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_P,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-FRD", 
					kStartPosition_P,
					-1,
					-1, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
				// possibly generate an Orion ship as well
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 90) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = gWC.mPickShip(bpv, kOrion, shiptype);
				if (shiptype == "") {
					shiptype = "O-CR1";
					bpvsofar = 80;
				}
				if ((bpvsofar > 30) && (shiptype != ""))  {
					gWC.fDraftedBPV += bpvsofar;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_P,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			case tWorldConditions::kPirateSmuggler:
			case tWorldConditions::kPirateWanted:
			case tWorldConditions::kPirateHostile:
			case tWorldConditions::kPirateSpy:
				fRace = kOrion;
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 90) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				bpvsofar = gWC.mPickShip(bpvsofar, kOrion, shiptype);
				if (shiptype == "") {
					shiptype = "O-CR1";
					bpvsofar = 80;
				}
				gWC.fDraftedBPV += bpvsofar;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_P,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				// possibly add a second hostile orion
				if (((bpv - bpvsofar) > 45) && (gWC.fMissionType == tWorldConditions::kPirateHostile)) {
					bpvsofar = gWC.mPickShip(bpv - bpvsofar, kOrion, shiptype);
					if (shiptype == "") shiptype = "O-CR1";
					gWC.fDraftedBPV += bpvsofar;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_P,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			default:
				break;
		}
	} else if (fTeamID == kEnemyTeam6) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kMonsterInnocent:
				gWC.fDraftedBPV += 80;
			case tWorldConditions::kMonsterHostile:
				gWC.fDraftedBPV += 80;
				switch (rand() % 31) {
					case 0: shiptype = "M-AMS"; break;
					case 1:
					case 2: shiptype = "M-AMM"; gWC.fDraftedBPV += 80;break;
					case 4: shiptype = "M-AML"; gWC.fDraftedBPV += 80;break;
					case 5: shiptype = "M-SSS"; break;
					case 6:
					case 7: shiptype = "M-SSM"; break;
					case 8:
					case 9: shiptype = "M-SSL"; gWC.fDraftedBPV += 60;break;
					case 10: shiptype = "M-LCS"; break;
					case 11:
					case 12: shiptype = "M-LCM"; gWC.fDraftedBPV += 60;break;
					case 13:
					case 14: shiptype = "M-LCL"; gWC.fDraftedBPV += 60;break;
					case 15: shiptype = "M-MTS"; break;
					case 16:
					case 17: shiptype = "M-MTM"; gWC.fDraftedBPV += 60;break;
					case 18:
					case 19: shiptype = "M-MTL"; gWC.fDraftedBPV += 60;break;
					case 20: shiptype = "M-SGS"; break;
					case 22: shiptype = "M-SGM"; gWC.fDraftedBPV += 80;break;
					default: shiptype = "M-AMS";
						break;
				}
				if (shiptype == "") shiptype = "M-AMM";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_R,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			case tWorldConditions::kEvacuateTerrain:
			case tWorldConditions::kSalvageTerrain:
				shiptype = "N-AST";
				gWC.fDraftedBPV += 20;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_R,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	}
	else if (fTeamID == kPlayerTeam5) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kOursInDistress:
				if (!gWC.fUseMinefield && ((rand() % 3) == 0)) {
					// add ion storms around position 0
					fMissionInfo->mGetGridPosition('O', &x, &y);
					fMissionInfo->mCreateMapObject('{', (x-3), (y-3));
					fMissionInfo->mCreateMapObject('{', (x+3), (y-3));
					fMissionInfo->mCreateMapObject('{', (x-3), (y+3));
					fMissionInfo->mCreateMapObject('{', (x+3), (y+3));
				}
				gWC.fDraftedBPV += 1;
				bpv = 65 + (rand() % 101);
				bpv = gWC.mPickShip(bpv, gWC.fDrafterRace, shiptype);
				if (shiptype == "") shiptype = "N-FLPh";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_O,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	} else if (fTeamID == kPlayerTeam4) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kPirateInnocent:
				gWC.fDraftedBPV += 50;
				fRace = kOrion;
				bpv = 50 + (rand() % 101);
				bpv = gWC.mPickShip(bpv, kOrion, shiptype);
				if (shiptype == "") shiptype = "O-CR1";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_M,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	}
}

void tCommonEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( mIsTeamHumanControlled())
	{
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

}

void tCommonEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fMyShipID < 0) switch (fTeamID) {
		case kEnemyTeam1: fMyShipID = kET1Ship1; break;
		case kEnemyTeam2: fMyShipID = kET2Ship1; break;
		case kEnemyTeam3: fMyShipID = kET3Ship1; break;
		case kEnemyTeam4: fMyShipID = kET4Ship1; break;
		case kEnemyTeam5: fMyShipID = kET5Ship1; break;
		case kEnemyTeam6: fMyShipID = kET6Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	fShipMetaID = shipScriptDescription.fMetaDatabaseID;
	fHumanControlled = mIsTeamHumanControlled();
	if (!fHumanControlled) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		gWC.mSetDrafting();
	} else {
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		gWC.fMissionType = tWorldConditions::kShootOnSight;		
			// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		std::string msg = "Drafted human enemy ship: ";
		msg += fShipType.c_str();
		msg += ", hullsize: ";
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, shipScriptDescription.fClassType);
		mCreateShip(	typeid( tCommonEnemyShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftedHumans++;
	}
	fNumShips++;
}





void tCommonEnemyTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonEnemyTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonEnemyTeam::mNumShips()
{
	return fNumShips;
}

void tCommonEnemyTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}



