/********************************************************************************************************\

	File Name:	CommonPlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerShipBaseState.h"
#include "WorldConditions.h"
#include "MissionText.h"

void tCommonPlayerShipBaseState::mBeginState()
{
	fCurrAlert = kAlertRed;
	fDataptr = NULL;
	fTransTurns = 3 + (rand() % 5);
	fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
	tTeamInfo* team = gWC.fTeamList[ fChildActor->mGetTeam() ];
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	fChildActor->mSetShipCallbackEvents( kNotifyDamage);
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
	fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem);
	fChildActor->mSetShipCallbackEvents( kNotifyHostChanged);
	
	fDetectionRange = 56.0f;
	fCurrentlyScanning  = false;
	fTractorOn = false;
}

bool tCommonPlayerShipBaseState::mIsPlayerFirstShip()
{
	tTeamInfo* team = gWC.fTeamList[ fChildActor->mGetTeam() ];
	if (!team) return false;
	if (!team->mIsTeamHumanControlled()) return false;
	tShipIterator ship = team->mGetFirstShip();
	if (fChildActor == (*ship)) return true;
	return false;
}


void tCommonPlayerShipBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}

void tCommonPlayerShipBaseState::mEndState()
{
}

void tCommonPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
}
void tCommonPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	float analysistime = 200.0f / fDetectionRange;
	// if we take damage we have a chance of breakdown
	if (damagedShip && (damagedShip->mGetTeam() > kTeam3) && hostileShip && (hostileShip->mGetTeam() < kTeam4)) {
		gWC.fPlayerFired = true;
		switch (gWC.fMissionType) {
			case tWorldConditions::kMonsterInnocent:
			case tWorldConditions::kPirateInnocent:
			case tWorldConditions::kOursInDistress:
			case tWorldConditions::kCleanup:
					if (gWC.fPlayerFired && !gWC.fPlayerJamming && !gWC.fEnemyFired) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					break;
			default:
				break;
		}
	} else if (damagedShip && (damagedShip->mGetTeam() <= kTeam3) && hostileShip && (hostileShip->mGetTeam() >= kTeam4)) {
		gWC.fEnemyFired = true;
	}
}
void tCommonPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	// make sure the jamming has been set for both sides
	if (!gWC.fJammingDetermined) {
		gWC.fJammingDetermined = true;
		int32 total = gWC.fPlayerJammingPotential + gWC.fEnemyJammingPotential;
		if (total > 0) {
			if ((rand() % total) < gWC.fPlayerJammingPotential) {
				gWC.fPlayerJamming = true;
			}
			if ((rand() % total) < gWC.fEnemyJammingPotential) {
				gWC.fEnemyJamming = true;
			}
		}
	}

	// make sure a scanning ship really exists
	if (!ship) return;

	// only process this scan with a live player's ship
	if (fChildActor->mGetTeam() > kTeam3) {
		return;
	}

	// keep track of whether or not our ship is currently scanning
	if (ship == fChildActor) {
		fCurrentlyScanning = deepScanOn;
	}

	if (gWC.fPlayerScanCompleted) return;

	mChkFinalAnalysis();
	
	// we might have completed the mission
	if (gWC.fPlayerGoalCompleted) 
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	
	// set notification up for the next scan
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
}
void tCommonPlayerShipBaseState::mDisableTargetting(tShipInfo *ship)
{
	if ((ship == fChildActor) && fMissionInfo) {
		fChildActor->mRemoveTargets();
		tShipSystemsInfo info;
		info.fAction = kShipTargetingStatus;
		info.fInt32[0] = kMakeShipAbleToTarget;
		info.fInt32[1] = kFalse;
		info.fShipEntityID = fChildActor->mGetShipID();
		gScriptInterface.mEntityMgr( info );
		if (gWC.fDebug) {
			gWC.mLogCall("Disabling targeting on ship with id", fChildActor->mGetShipID());
			gWC.mLogCall("   and custom id", fChildActor->mGetCustomID());
		}
	}
}
void tCommonPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{
	if ((ship == fChildActor) && fMissionInfo) {
		fCurrAlert = alertStatus;
	}
}
void tCommonPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	// handle cases where our ship is beaming something onto a target ship
	if (shipTo && (shipFrom == fChildActor) && (shipTo->mGetTeam() > kTeam3)) {
		if ((item == kTransMedicalSupplies) || (item == kTransEngineers)) {
			if (gWC.fMissionType == tWorldConditions::kDerelict) {
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The engineering team has begun their investigation, Sir", kMissionScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kAwayTeamDone, 25 );
			} else {
				gWC.fPlayerGoalCompleted = true;
				mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			}
			mSetFriendlyRelations();
		} else if (item == kTransMarineDude) {
			tTeamInfo *team = NULL;
			switch (gWC.fMissionType) {
				case tWorldConditions::kPirateSpy:
					gWC.fPlayerGoalCompleted = true;
					if ((rand() % 7) > 2) {
						fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "They\'re trying to escape, Sir!", kMissionScriptMessage );
						mSetEscapeRelations();
						gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
						int32 speed = 17 + (rand() % 8);
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					} else {
						fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "They\'re attacking, Sir!", kMissionScriptMessage );
						int32 speed = 17 + (rand() % 8);
						gWC.fEnemyTargetShip->mClearCommands();
						gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
						team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
						if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
						mSetShootOnSightRelations();
					}
				default:
					break;
			}
		}
	}
	// handle cases where something is getting beamed off a target ship and onto our ship
	else if (shipFrom && (shipTo == fChildActor) && (shipFrom->mGetTeam() > kTeam3)) {
		if (gWC.fMissionType == tWorldConditions::kOursInDistress) {
			if ((item >= kTransMarineDude) && (item <= kTransEngineers)) {
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "They are safely aboard, Sir", kMissionScriptMessage );
				gWC.fRescued = true;
			}
		} else if ((item == kTransDilithiumCrystals) || (item == kTransInjuredDude) || (item == kTransRomulanAle)) {
			gWC.fBonusPrestige += 50;
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransTribbles) {
			gWC.fBonusPrestige -= 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Not TRIBBLES, Sir - they\'re going to be a nightmare to get rid of!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransHarryMudd) {
			gWC.fBonusPrestige -= 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Not Harry Mudd, Sir - he\'s going to be nothing but trouble!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransBlackBox) {
			gWC.fBonusPrestige += 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We have retrieved the derelict\'s black box, Sir", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransEngineers) {
			gWC.fBonusPrestige += 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We have retrieved the engineering team, Sir!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransCaptainDude) {
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "That\'s got their attention, Sir ... I\'m sure we can get this worked out now.", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			gWC.fPlayerGoalFailed = false;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		}
	}
}
void tCommonPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonPlayerShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonPlayerShipBaseState::mOnHostChanged()
{
	gWC.fHostLeft = true;
}
void tCommonPlayerShipBaseState::mSetTargetable(bool State)
{
	tShipSystemsInfo info;
	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipTargetable;
	info.fInt32[1] = State;
	info.fShipEntityID = fChildActor->mGetShipID();
	gScriptInterface.mEntityMgr( info );
}
void tCommonPlayerShipBaseState::mSetTargetting(bool State)
{
	tShipSystemsInfo info;
	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipAbleToTarget;
	info.fInt32[1] = State;
	info.fShipEntityID = fChildActor->mGetShipID();
	gScriptInterface.mEntityMgr( info );
}

void tCommonPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo *thisteam= NULL;
	tTeamInfo *drafterteam= NULL;
	tShipInfo *ship = NULL;
	bool shipsdetected = false;
	float range = fDetectionRange;
	float mutinyTime = 7 + (rand() % 9);
	switch (timerID) {
		case kStartingID:
			if (!fDataptr) {
				fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
			}
			if (fDataptr) {
				fDetectionRange = 56.0f + fDataptr->jamming;
				if (fDetectionRange < 56.0f) fDetectionRange = 56.0f;
				if (fDataptr->isscout) fDetectionRange *= 2;
				gWC.fPlayerJammingPotential = fDataptr->jamming;
			} else fDetectionRange = 56.0f;
			if (gWC.fUsableAsteroid) {	
				if (fChildActor->mGetCustomID() == kMaxShips) fChildActor->mSetShipMaxSpeed(0);
				else if (fChildActor->mGetTeam() == kTeam1) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"There is an we may be able to tractor and use as cover, Sir", 
						kMissionScriptMessage );
				}
				if (fChildActor->mGetTeam() == kTeam3) {
					// this is the asteroid, make sure it starts out damaged
					fChildActor->mSetSystemHealthFromOrig( kHullGroupHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kForwardHullHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kAftHullHardPoint, 0.25f, kFalse );
					fChildActor->mSetSystemHealthFromOrig( kCenterHullHardPoint, 0.25f, kFalse );
				}
			}

			if ((gWC.fDraftedHumans < 1) && (gWC.fMissionType != tWorldConditions::kShootOnSight) && (gWC.fMissionType != tWorldConditions::kFleetBattle)) {
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"\n\nWe need more data - I recommend we activate long range scanners, Sir", 
					kMissionScriptMessage );
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
				mSetTargetting(kFalse);
			} else {
				if (gWC.fMapPicked > 6) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nWe need to take out this planet and any defenders, Sir!", 
						kMissionScriptMessage );
				} else if (gWC.fMapPicked > 3) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nWe need to take out this base and any defenders, Sir!", 
						kMissionScriptMessage );
				} else {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a hostile enemy, coming towards us fast, Sir!", 
						kMissionScriptMessage );
					mSetShootOnSightRelations();
				}
				if (gWC.fDoomsDay && (gWC.fMissionType != tWorldConditions::kFleetBattle)) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"That thing is massive!  We\'ve got to deal with that!", 
						kMissionScriptMessage );
				} else if (gWC.fMutiny && (gWC.fMissionType != tWorldConditions::kFleetBattle)) {
					mSetTimedInterval( kTimerTurnsSet, kMutiny, mutinyTime );
				}
			}
			break;
		case kMutiny: mSetMutinyRelations();
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Sir - our ally is turning to attack!!!!", 
					kMissionScriptMessage );
				break;
		case kCheckDistance:
			// see if we've reached the minimum distance to an enemy ship
			shipsdetected = mChkPrelimAnalysis();
			if (!gWC.fAlreadyDetected) {
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
			} else mSetTargetting(true);
			// see if we've reached the minimum distance to an enemy ship
			break;
		case kCheckGoalIndicator:
			if (!gWC.fPlayerScanCompleted) {
				mChkFinalAnalysis();
				if (!gWC.fPlayerScanCompleted && ((rand() % 19) < 6)) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We must get closer before we can complete our analysis, Sir", kNonCommScriptMessage );
					if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) > 80.0f) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "I am updating the plotted path towards them now", kNonCommScriptMessage );
					}
				}
				if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) > 80.0f) {
					fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				} else {
					// hopefully one of these four methods turns off the indicator
					fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), NULL);
					fMissionInfo->mSetGoalIndicator( fChildActor, false, -1, -1 );
					fMissionInfo->mSetGoalIndicator( fChildActor->mGetTeam(), false, -1, -1 );
				}
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
			}
			fDetectionRange *= 1.25f;
			break;
		case kCheckTransportCompletion:
			gWC.fPlayerGoalCompleted = true;
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Transfer complete, Sir!", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kAwayTeamDone:
			if ((rand() % 3) == 0) {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Something\'s gone wrong on the derelict, Sir!", kMissionScriptMessage );
				gWC.fEnemyTargetShip->mDestroyShip();
				mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			} else {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The team is done, Sir, we can proceed with demolishing the derelict", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransBlackBox, 1);
				}
			}
			break;
		case kAllComplete:
			gWC.fPlayerGoalCompleted = true;
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				"All complete, Sir - everything checks out", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kCheckGameEnd:
			gWC.mLogCall("Timed check for game end", 0);
			gWC.mCheckForGameEnd();
			break;
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tCommonPlayerShipBaseState::mSetFriendlyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm2*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm3*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetHostileRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetEscapeRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetMutinyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetShootOnSightRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mCarryOutTrade()
{
	if (fChildActor->mGetRace() <= kMirak) return;
	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) {
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"It doesn\'t look promising, Sir.  There\'s nothing much in their inventory, and they don\'t seem eager to buy.", 
			kMissionScriptMessage );
	}
}

bool tCommonPlayerShipBaseState::mChkPrelimAnalysis()
{
	float range = fDetectionRange;
	//gWC.mLogCall("Checking for prelim analysis, detection range", fDetectionRange);
	if (gWC.fMissionType == tWorldConditions::kMineSweeper) {
		mSetTargetting(true);
		gWC.fAlreadyDetected = true;
		gWC.fEnemyTargetable = true;
		gWC.fPlayerScanCompleted = true;
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"\n\nIt looks like a pirate minefield, Sir - we have plotted a heading towards it", kMissionScriptMessage );
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"\nThat\'s going to be a menace to our troops in the area, we should eliminate it, Sir", kMissionScriptMessage );
		fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), true, gWC.fMineX, gWC.fMineY);
	} else if (gWC.fEnemyTargetShip) {
		if (!gWC.fEnemyTargetable) {
			fDetectionRange *= 1.25f;
			if (fCurrentlyScanning) range = 2.0f * range;
			if (gWC.fMapPicked == 2) range = range * 0.75f;
			else if (gWC.fMapPicked == 3) range = range * 0.90f;
			if (gWC.fEnemyTargetShip->mCloakOn()) range = range * 0.75f;
			if (gWC.fEnemyTargetShip->mStealthy()) range = range * 0.75f;
			if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) < fDetectionRange) {
				mSetTargetting(true);
				gWC.fEnemyTargetable = true;
				fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
				gWC.fAlreadyDetected = true;
				switch (gWC.fMissionType) {
				case tWorldConditions::kConvoy:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nI\'m picking up a mix of signals, Sir, looks like a convoy - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kSwarm:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nI\'m picking up an odd mix of small signals, Sir, moving slowly - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kEnemyCombat:
					if (fDetectionRange < 75.0f) fDetectionRange += 75.0f;
					else if (fDetectionRange < 100.0f) fDetectionRange += 50.0f;
					else if (fDetectionRange < 125.0f) fDetectionRange += 25.0f;
					else fDetectionRange += 15.0f;
				case tWorldConditions::kShootOnSight:
				case tWorldConditions::kFleetBattle:
					if ((rand() % 3) == 0) {
						if (gWC.fEnemyTargetShip) {
							gWC.fEnemyTargetShip->mFollowEntity(fChildActor, kAbsolutePriority);
						}
					}
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"\n\nIt looks like a naval warship, Sir - we have plotted a heading towards it", 
					kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re demanding we leave immediately, sir!", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kDerelict:
				case tWorldConditions::kCleanup:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"It looks like a naval vessel, but they\'re not responding to hails, sir", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kOneOfOurOwn:
				case tWorldConditions::kOursInDistress:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "\n\nIt appears to be an allied vessel, Sir - we have plotted a heading towards it", kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateInnocent:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nAn unidentified vessel, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateWanted:
				case tWorldConditions::kPirateHostile:
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kPirateSmuggler:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a pirate vessel, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					if ((rand() % 7) > 3) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re claiming they are authorized to be in the area, sir", 
						kMissionScriptMessage );
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re not responding to hails, sir", 
						kMissionScriptMessage );
					}
					break;
				case tWorldConditions::kMonsterInnocent:
				case tWorldConditions::kMonsterHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a life form, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kEvacuateTerrain:
				case tWorldConditions::kSalvageTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nSome sort of terrain anomaly detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kOutpostRaid: 
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nThe signals are consistent with a repair post, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateBase: 
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nThe signals are consistent with some form of base, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				default:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nUnknown disturbance detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				}
				if (gWC.fUseMinefield) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\nI\'m also picking up what appears to be a minefield, Sir", 
						kMissionScriptMessage );
				}
			}
		}
	} else {
		gWC.mLogCall("No enemy ship targetable?", 0);
	}
	return(gWC.fAlreadyDetected);
}

void tCommonPlayerShipBaseState::mChkFinalAnalysis()
{
	if (!gWC.fAlreadyDetected) return;
	
	// get the target for the scanning ship, if any
	tShipInfo *target = NULL;
	int32 targetid = -1;
	int32 x = 0, y = 0;
	if (fChildActor) targetid = fChildActor->mGetShipTargetID();
	if ((targetid > -1)  && fMissionInfo) {
		target = fMissionInfo->mFindShipID(targetid);
	}

	if (!target) return;

	// set the modifier for final detection range,
	//     larger numbers mean the player has to get closer
	float RangeModifier = 1.0f;
	// =========================================
	switch (gWC.fMissionType) {
		// special case
		// hostiles are detected farther away, just to be nice
		case tWorldConditions::kOutpostRaid:
		case tWorldConditions::kConvoy:
		case tWorldConditions::kSwarm:
		case tWorldConditions::kPirateBase: 
		case tWorldConditions::kEnemyCombat:
		case tWorldConditions::kShootOnSight:
		case tWorldConditions::kFleetBattle:
		case tWorldConditions::kPirateHostile:
		case tWorldConditions::kMonsterHostile:
			RangeModifier = 15.0f;
			break;
		// fight-or-flee cases are detected at midrange
		case tWorldConditions::kMineSweeper:
		case tWorldConditions::kOneOfOurOwn:
		case tWorldConditions::kPirateSmuggler:
		case tWorldConditions::kPirateWanted:
		case tWorldConditions::kPirateSpy:
			RangeModifier = 20.0f;
			break;
		// defenseless cases are detected at close range
		case tWorldConditions::kCleanup:
		case tWorldConditions::kPirateInnocent:
		case tWorldConditions::kMonsterInnocent:
		case tWorldConditions::kDerelict:
		case tWorldConditions::kOursInDistress:
		case tWorldConditions::kEvacuateTerrain:
		case tWorldConditions::kSalvageTerrain:
		default:
			RangeModifier = 25.0f;
			break;
	}
	//=======================================
	if (fCurrentlyScanning) RangeModifier *= 0.5f;


	int32 speed;
	if (fChildActor->mGetTeam() == kTeam1) { 
		if (target && (target->mGetTeam() > kTeam3) && (fChildActor->mGetTacticalDistance(target) < (fDetectionRange / RangeModifier))){
			switch (gWC.fMissionType) {
				case tWorldConditions::kEnemyCombat:
				case tWorldConditions::kPirateHostile:
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kPirateSmuggler:
				case tWorldConditions::kOneOfOurOwn:
				case tWorldConditions::kPirateWanted:
					if (gWC.fEnemyJamming) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"\nThey are jamming our communications\n, Sir", kMissionScriptMessage );
					}
					break;
				default:
					break;
			}
			gWC.fPlayerScanCompleted = kTrue;
			tTeamInfo *team = NULL;
			switch (gWC.fMissionType) {
				case tWorldConditions::kOutpostRaid:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir, if we can destroy the outpost and the cargo boxes in the vicinity it should put a big dent in enemy activity in this sector!", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kCleanup:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Command wants the derelict dumped in the black hole behind us, Sir.  They don\'t want any wreckage left for anyone to snoop through.", 
						kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We should be careful not to damage it before we dump it in the black hole, to guarantee we don\'t leave any debris behind.", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateBase: 
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"This base is headquarters for a very hostile faction, Sir - we need to dispose of it!", 
							kMissionScriptMessage );
					break;
				case tWorldConditions::kConvoy:
				case tWorldConditions::kSwarm:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re definitely hostile!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam3) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					speed = 7 + (rand() % 7);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kPirateHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re definitely hostile!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					speed = 17 + (rand() % 8);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kMonsterHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re definitely hostile!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam6) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					speed = 17 + (rand() % 8);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kEnemyCombat:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re attacking!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam3) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					speed = 17 + (rand() % 8);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kDerelict:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"She\'s definitely abandoned, Sir, though it looks like some systems are still on autopilot... \nI suggest we put an engineering team aboard her to investigate", 
							kMissionScriptMessage );
					fChildActor->mAddItemToStores( kTransporterItem, kTransEngineers, 1);
					break;
				case tWorldConditions::kPirateSpy:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"There is an order out to investigate this vessel, Sir... \nWe need to get a security team aboard her, forcibly if necessary", 
							kMissionScriptMessage );
					fChildActor->mAddItemToStores( kTransporterItem, kTransMarineDude, 1);
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kOursInDistress:
					if ((rand() % 7) > 3) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They appear to be suffering critical control difficulties sir, that\'s why they haven\'t responded to hails.", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"If we can put a repair team aboard, they should be able to fix them up in no time, Sir", kMissionScriptMessage );
						fChildActor->mAddItemToStores( kTransporterItem, kTransEngineers, 1);
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They appear to be suffering a medical emergency sir, that\'s why they haven\'t responded to hails.", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"The medical team is preparing a supply kit, if we can get it through to them, Sir", kMissionScriptMessage );
						fChildActor->mAddItemToStores( kTransporterItem, kTransMedicalSupplies, 1);
					}
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					// need to add the requirement to beam a rescue party aboard
					mSetFriendlyRelations();
					if (gWC.fEnemyTargetShip) {
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldOneHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldTwoHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldThreeHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldFourHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldFiveHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldSixHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f, kFalse );
					}
					break;
				case tWorldConditions::kPirateSmuggler:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We\'re detecting substantial amounts of contraband aboard her sir!", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"I\'m sending a message to stand down and be boarded", kMissionScriptMessage );
					if ((rand() % 7) > 2) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re turning to attack!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mClearCommands();
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
						gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
						team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
						if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
						mSetShootOnSightRelations();
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re trying to evade!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
						speed = 17 + (rand() % 8);
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
						mSetEscapeRelations();
					}
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kOneOfOurOwn:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"This ship was reported lost with all hands, sir...", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Apparently she has either mutineed or been taken over!", kMissionScriptMessage );
					if ((rand() % 7) > 2) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re turning to attack!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mClearCommands();
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
						gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
						team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam4) );
						if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
						mSetShootOnSightRelations();
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re trying to evade!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
						speed = 17 + (rand() % 8);
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
						mSetEscapeRelations();
					}
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kPirateWanted:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"There\'s a warrant out on this vessel sir!", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"I\'m sending a message to stand down and be boarded", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We should ready the marines...", kMissionScriptMessage );
					if ((rand() % 7) > 2) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re turning to attack!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mClearCommands();
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
						gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
						team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
						if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
						mSetShootOnSightRelations();
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - they\'re trying to evade!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
						speed = 21 + (rand() % 8);
						gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
						mSetEscapeRelations();
					}
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kPirateInnocent:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"All appears above-board sir, looks like we have to let them go", kMissionScriptMessage );
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					gWC.fPlayerGoalCompleted = true;
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					break;
				case tWorldConditions::kMonsterInnocent:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"It appears to be non-threatening sir", kMissionScriptMessage );
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					gWC.fPlayerGoalCompleted = true;
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					break;
				case tWorldConditions::kEvacuateTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - there appears to be a wreck on the surface, with life signs aboard!", kMissionScriptMessage );
					if ((gWC.fMapPicked == 1) || ((rand() % 7) > 3)) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"We need to get a beam them up immediately!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransInjuredDude, 1);
					} else {
					// need to add requirement to evacuate the survivors
						if (fDataptr && (fDataptr->trans > 0)) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"This could take some time sir, we\'ll need to stay within transporter range", kMissionScriptMessage );
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"The engineers will notify us when the transfer is complete", kMissionScriptMessage );
							if (fDataptr && (fDataptr->trans > 0)) fTransTurns = fDataptr->dorms / fDataptr->trans;
							if (fTransTurns > 5.0f) fTransTurns = 5.0f;
							if (fTransTurns < 1.0f) fTransTurns = 1.0f;
							gWC.mLogCall("Expected tranfer time:", fTransTurns);
							gWC.fBonusPrestige = 10 * fDataptr->dorms ;
							mSetTimedInterval( kTimerTurnsSet, kCheckTransportCompletion, fTransTurns );
						}
					}
					mSetFriendlyRelations();
					break;
				case tWorldConditions::kSalvageTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - there appears to be a wreck on the surface, and signs of some valuable cargo aboard", kMissionScriptMessage );
					if ((gWC.fMapPicked == 1) || ((rand() % 7) > 3)) {
						if ((rand() % 7) > 3)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransDilithiumCrystals, 1);
						else
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransRomulanAle, 1);
						if ((rand() % 5) == 0)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransTribbles, 1);
						else if ((rand() % 7) == 0)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransHarryMudd, 1);
					} else {
						// need to add requirement to transport the goods
						if (fDataptr && (fDataptr->trans > 0)) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"This could take some time sir, we\'ll need to stay within transporter range", kMissionScriptMessage );
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"The engineers will notify us when the transfer is complete", kMissionScriptMessage );
							if (fDataptr && (fDataptr->trans > 0)) fTransTurns = fDataptr->storage / fDataptr->trans;
							if (fTransTurns > 5.0f) fTransTurns = 5.0f;
							if (fTransTurns < 1.0f) fTransTurns = 1.0f;
							gWC.mLogCall("Expected tranfer time:", fTransTurns);
							mSetTimedInterval( kTimerTurnsSet, kCheckTransportCompletion, fTransTurns );
							gWC.fBonusPrestige = 10 * fDataptr->storage ;
						}
					}
					mSetFriendlyRelations();
					break;
				default:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Scans are inconclusive sir", kMissionScriptMessage );
					break;
			}

		}
	} 
}
