/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include <time.h>
#include <fstream.h>

class tWorldConditions gWC;

void tWorldConditions::mSetDrafting()
{
	// the type of mission is determined
	//    by a call to this routine from the first drafted AI team
	if (fDraftingDecided) return;
	fDraftingDecided = kTrue;

	int32 modfactor = kNumMissions - 1;
	fMissionType = eMissionType((rand() % modfactor) + 1);

	if ((fMapPicked != 0) && (fMissionType == kFleetBattle)) fMissionType = kShootOnSight;


	if (fMapPicked == 0) {
		switch (fMissionType) {
			case kEvacuateTerrain:
			case kSalvageTerrain:
			case kMineSweeper:
			case kOneOfOurOwn:
			case kDerelict:
				if ((rand() % 7) > 3) fMissionType = kConvoy;
				break;
			default: break;
		}
	} else if (fMapPicked == 1) {
		switch (fMissionType) {
			case kOursInDistress:
			case kPirateSpy:
			case kPirateSmuggler:
			case kPirateWanted:
			case kSalvageTerrain:
			case kEvacuateTerrain:
			case kDerelict:
			case kCleanup:
				fMissionType = kEnemyCombat;
				break;
			default:
				break;
		}
	}

	if (fMapPicked > 3) {
		fMissionType = kShootOnSight;
	} else if (fMapPicked > 3) {
		if (fMapPicked != 7) fMissionType = kShootOnSight;
	}

	if (fDraftedHumans > 0) {
		fMissionType = kShootOnSight;
	} else {
		switch (fMissionType) {
			case kMonsterInnocent:
				if ((rand() % 7) > 3) fMissionType = kMonsterHostile;
				break;
			case kPirateWanted:
			case kPirateSpy:
			case kPirateSmuggler:
			case kPirateInnocent:
				if ((rand() % 7) > 2) fMissionType = kPirateHostile;
				break;
			case kSwarm:
				if ((rand() % 7) > 3) fMissionType = kConvoy;
				break;
			default:
				break;
		}
	}

	
	if (fMissionType == kShootOnSight) {
		switch (gWC.fMapPicked) {
			case 4: 
			case 5:
			case 6: 
			case 8:
			case 9:
				gWC.fAddAlly = true;
				break;
			default:
				break;
		}
	}
	switch (fMissionType) {
		case kNumMissions:
			fMissionType = kShootOnSight;
		case kShootOnSight:
			mLogCall("Mission type: shoot on sight (hostile naval vessel)", fMissionType);
			break;
		case kFleetBattle:
			mLogCall("Mission type: fleet battle", fMissionType);
			break;
		case kOutpostRaid:
			mLogCall("Mission type: raiding an enemy supply/repair outpost", fMissionType);
			break;
		case kConvoy:
			mLogCall("Mission type: an enemy convoy", fMissionType);
			break;
		case kSwarm:
			mLogCall("Mission type: a swarm of tiny hostiles", fMissionType);
			break;
		case kMineSweeper:
			fUseMinefield = true;
			mLogCall("Mission type: sweeping a rogue orion minefield", fMissionType);
			break;
		case kDerelict:
			mLogCall("Mission type: dealing with derelict enemy ship", fMissionType);
			break;
		case kPirateInnocent:
			mLogCall("Mission type: dealing with \'innocent\' pirate", fMissionType);
			break;
		case kPirateSmuggler:
			mLogCall("Mission type: dealing with pirate smuggler", fMissionType);
			break;
		case kPirateWanted:
			mLogCall("Mission type: dealing with wanted pirate", fMissionType);
			break;
		case kPirateHostile:
			mLogCall("Mission type: dealing with hostile pirate", fMissionType);
			break;
		case kPirateSpy:
			mLogCall("Mission type: dealing with pirate spy", fMissionType);
			break;
		case kMonsterInnocent:
			mLogCall("Mission type: dealing with non-aggressive monster", fMissionType);
			break;
		case kMonsterHostile:
			mLogCall("Mission type: dealing with hostile monster", fMissionType);
			break;
		case kEvacuateTerrain:
			mLogCall("Mission type: dealing with evacuation of terrain", fMissionType);
			break;
		case kSalvageTerrain:
			mLogCall("Mission type: dealing with salvage from terrain", fMissionType);
			break;
		case kOneOfOurOwn:
			mLogCall("Mission type: dealing with one of our vessels in enemy hands", fMissionType);
			break;
		case kOursInDistress:
			if ((rand() % 3) == 0) fUseMinefield = true;
			mLogCall("Mission type: dealing with one of our vessels in distress", fMissionType);
			break;
		case kEnemyCombat:
			mLogCall("Mission type: dealing with enemy combat vessel", fMissionType);
			break;
		case kCleanup:
			mLogCall("Mission type: disposing of derelict vessels in a black hole", fMissionType);
			break;
		case kPirateBase: 
			mLogCall("Mission type: disposing of a pirate base", fMissionType);
			break;
 		default: 
			mLogCall("Mission type: shoot on sight (hostile naval vessel)", fMissionType);
			fMissionType = kShootOnSight;
			break;
	}
	if (fMapPicked > 3) fUseMinefield = false;
}


tWorldConditions::tWorldConditions()
{
	fDebug = false;
	fPrintShiplist = false;
	fAmbushed = false;
	fRescued = false;
	fMutiny = false;
	fAddAlly = false;
	srand((unsigned int)(time(NULL)));
	if ((rand() % 17) < 9) {
		fAddAlly = true;
	}
	fExtraDraftedAIBPV = 0;

	/**/
	fSuckerRace = kOrion;
	fHostLeft = false;
	/**/
	fDoomsDay = false;

	fUsableAsteroid = false;
	fPlayerGoalFailed = false;
	fBonusPrestige = 0;
	fDrafterRace = kOrion;
	fCaptainsGrabbed = 0;
	fUseMinefield = false;
	fBasePlacement = false;
	fGameStartedOK = false;
	fPlayerFiredEarly  = false;
	fAlreadyDetected = false;
	fEnemyFired = false;
	fMissionType = kEnemyCombat;
	fJammingDetermined = false;
	fEnemyJammed = false;
	fPlayerJammingPotential = 0;
	fPlayerFired = false;
	fEnemyJammingPotential = 0;
	fPlayerScanCompleted = false;
	fPlayerGoalCompleted = false;
	fPlayerJamming = false;
	fEnemyJamming  = false;
	fAddPlayerAIBPV = 0;
	fAddEnemyAIBPV = 0;
	fDraftingHulls = 0;
	fDraftingDecided = kFalse;
	int32 i;
	fTypeSize = 0;
	fBPVSize = 0;
	fShiplistRead = kFalse;
	fResultsIn = kFalse;
	fNebula = kFalse;
	fMissionSeed = 79;
	fYearLowerBound = 0;
    fDraftedHumans = 0;
    fDraftingHumans = 0;
    fDraftedBPV = 0;
    fDraftingBPV = 0;
	fNumShips = 0;
	for ( i = 0; i < kMaxMissionTeams; i++) {
		fTeamNumShips[kMaxMissionTeams] = 0;
	    fTeamList[i] = NULL;
		fScannedEnemy[i] = kFalse;
		fInflictedDamage[i] = kFalse;
		fCalledForHelp[i] = kFalse;
	}
    for ( i = 0; i < kMaxShips; i++) { 
	    fShipTypes[i] = "";
    }
	fEnemyTargetShip = NULL;
	fEnemyTargetable = true;
	for (i = 0; i < kNumberOfRaces; i++) {
		fRaceUsed[i] = false;
		fShipsByBPV[i] = NULL;
		fShipsByType[i] = NULL;
		fBSchoice[i] = "";
		fBATSchoice[i] = "";
		fSBchoice[i] = "";
		fFRDchoice[i] = "";
		fLPchoice[i] = "";
		fDEFchoice[i] = "";
		fFreighterChoice1[i] = "";
		fFreighterChoice2[i] = "";
		fFreighterChoice3[i] = "";
		fFreighterChoice4[i] = "";
		fFreighterChoice5[i] = "";
		fFreighterChoice6[i] = "";
	}
	fRaceUsed[kOrion] = kTrue;
	fRaceUsed[kMonster] = kTrue;
	fRaceUsed[kNeutralRace] = kTrue;

	 xDim = 0, yDim = 0, xCtr = 0, yCtr = 0;
	 fDimSet = kFalse;
}

void tWorldConditions::mPickPlacement()
{
	tMissionInfo* thisMission = NULL;
	thisMission = gScriptInterface.mMission();
	if (!thisMission) return;
	if (!fDimSet) {
		if ( thisMission ) thisMission->mGetMapDimensions(&xDim, &yDim);
		xCtr = xDim / 2;
		yCtr = yDim / 2;
		fMineX = xCtr + 4 + (rand() % 5);
		fMineY = yCtr + 4 + (rand() % 5);
		fDimSet = true;
	}	
	int32 x = xCtr, y = yCtr;
	fMineX = xCtr; fMineY = yCtr;
	for (eTeamID team = kTeam1; team <= kTeam12; team = (eTeamID)(team + 1)) {
		char marker = char ('G' + team);
		if (team < kTeam4) {
			// this makes sure we don't start ridiculously close to an enemy base/planet,
			//      it will give us a range of 70-100
			switch (rand() % 4) {
				case 0:
					x = xCtr - 7;
					y = (yCtr - 7) + (rand() % 5);
					if (((x + y) % 2) > 0) x--;
					break;
				case 1:
					x = xCtr - 7;
					y = (yCtr - 9) + (rand() % 5);
					if (((x + y) % 2) > 0) x--;
					break;
				case 2:
					x = xCtr - 9;
					y = (yCtr - 6) + (rand() % 3);
					if (((x + y) % 2) > 0) x--;
					break;
				default:
					x = xCtr - 9;
					y = (yCtr - 7) + (rand() % 3);
					if (((x + y) % 2) > 0) x--;
					break;
			}
		} else if (team < kTeam7) {
				x = (xCtr + 5) + (rand() % 5);
				y = (yCtr + 5) + (rand() % 5);
				if (((x + y) % 2) > 0) x--;
		} else if (marker == 'P') {
			x = xCtr + 10 + (rand() % 7);
			y = yCtr + 10 + (rand() % 7);
			if (((x + y) % 2) > 0) x--;
		} else if (marker == 'O') {
			x = fMineX;
			y = fMineY;
			if (((x + y) % 2) > 0) x--;
		} else {
			switch (rand() % 8) {
			case 2:
				x = (xCtr + 5) + (rand() % 7);
				y = (yCtr - 11) + (rand() % 11);
				break;
			case 4:
				x = (xCtr + 5) + (rand() % 7);
				y = (yCtr - 9) + (rand() % 19);
				break;
			case 5:
				x = (xCtr - 11) + (rand() % 11);
				y = (yCtr + 5) + (rand() % 7);
				break;
			case 6:
				x = (xCtr - 9) + (rand() % 11);
				y = (yCtr + 5) + (rand() % 7);
				break;
			case 7:
				x = (xCtr + 5) + (rand() % 7);
				y = (yCtr + 5) + (rand() % 7);
				break;
			case 0:
				x = (xCtr - 9) + (rand() % 5);
				y = (yCtr + 7) + (rand() % 5);
				break;
			case 1:
				x = (xCtr + 7) + (rand() % 5);
				y = (yCtr - 9) + (rand() % 5);
				break;
			default:
				x = (xCtr + 7) + (rand() % 5);
				y = (yCtr + 7) + (rand() % 5);
				break;
			}
			if (((x + y) % 2) > 0) x++;
		}
		if ((fMissionType != kShootOnSight) && (team > kTeam3)) {
			x+=2; y+=2;
		}
		thisMission->mCreateMapObject(marker, x, y);
		if (team == kTeam1) {
			int32 minex, miney;
			minex = fMineX;
			miney = fMineY;
			thisMission->mCreateMapObject('Z', minex, miney);
			minex = fMineX + 3;
			miney = fMineY - 2;
			thisMission->mCreateMapObject('Y', minex, miney);
			minex = fMineX - 2;
			miney = fMineY + 3;
			thisMission->mCreateMapObject('X', minex, miney);
			minex = fMineX - 3;
			miney = fMineY - 2;
			thisMission->mCreateMapObject('W', minex, miney);
			minex = fMineX + 3;
			miney = fMineY;
			thisMission->mCreateMapObject('V', minex, miney);
			minex = fMineX - 3;
			miney = fMineY;
			thisMission->mCreateMapObject('U', minex, miney);
			minex = fMineX ;
			miney = fMineY + 3;
			thisMission->mCreateMapObject('T', minex, miney);
			minex = fMineX;
			miney = fMineY - 3;
			thisMission->mCreateMapObject('S', minex, miney);
		} 
   }
}

iTruth tWorldConditions::mAppendToFile(std::string filename, std::string textline)
{
	ofstream fpout;
	fpout.open(filename.c_str(), ios::ate); // might need app
	if (fpout.fail()) {
		return kFalse;
	}
	fpout << textline.c_str() << endl;
	fpout.close();
	return kTrue;
}


iTruth tWorldConditions::mLogTeamResults(eTeamID team, eRace race, int32 prestige, eGameStatus result, iTruth human)
{
	std::string teamname;
	switch (team) {
			case kTeam1: teamname = "Player (drafter)"; break;
			case kTeam2: 
			case kTeam3: teamname = "Player ally"; break;
			case kTeam7: teamname = "Harmless pirate"; break;
			case kTeam9: teamname = "Harmless civillian"; break;
			case kTeam11: teamname = "Problem civillian"; break;
			case kTeam4:
			case kTeam5:
			case kTeam6: teamname = "Enemy"; break;
			case kTeam8: teamname = "Enemy diplomat"; break;
			case kTeam10: teamname = "Problem pirate"; break;
			case kTeam12: teamname = "Monster or terrain issue"; break;
			default: teamname = "AI"; break;
	}

	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "ERROR: unable to open BattleLog.txt during results logging" << endl;
			fpout.close();
		 }
		return kFalse;
	}
	if (!fResultsIn) {
		fResultsIn = kTrue;
		time_t timer1, timer2;
		timer2 = time(&timer1);
		struct tm *datetime1 = localtime(&timer1);
		int32 month = 0;
		int32 day = 0;
		int32 year = 0;
		int32 hour = 0;
		int32 min = 0;
		if (datetime1) {
			hour = datetime1->tm_hour;
			if (hour == 0) hour = 12;
			min = datetime1->tm_min;
		}
		fpout << "mission completed at: " << hour << ":";
		if (min < 10) fpout << "0";
		fpout << min <<  endl;
	}
	fpout << "   " << teamname.c_str() << " (";
	switch (race) {
		case kFederation: fpout << "Fed"; break;
		case kKlingon: fpout << "Kli"; break;
		case kRomulan: fpout << "Rom"; break;
		case kLyran: fpout << "Lyr"; break;
		case kMirak: fpout << "Mir"; break;
		case kHydran: fpout << "Hyd"; break;
		case kGorn: fpout << "Gor"; break;
		case kISC: fpout << "ISC"; break;
		case kNeutralRace: fpout << "Neu"; break;
		case kMonster: fpout << "Mons"; break;
		default: fpout << "Ori"; break;
	}
	if (human) fpout << ", human";
	else fpout << ", ai";
	fpout << ") " << prestige << "pp,";
	if (result < 0x00000100) fpout << " drafting player won\n";
	else if (result < kLeftEarly) fpout << " drafting player lost\n";
	else fpout << " mission was a draw or disconnect\n";
	fpout << "--------------------------------------\n";
	fpout.close();
	return kTrue;
}

std::string tWorldConditions::mNumToStr(float data)
{
	std::string results = "   0.0 ";
	int32 digits = ((int32)(data)) % 10000;
	results[3] = (digits % 10) + '0';
	digits = digits / 10;
	results[2] = (digits % 10) + '0';
	digits = digits / 10;
	results[1] = (digits % 10) + '0';
	digits = digits / 10;
	results[0] = (digits % 10) + '0';
	int32 c = 0; 
	while (results[c] == '0') results[c++] = ' ';
	results[5] = (((int32)(10*data)) % 10) + '0';
	results[6] = (((int32)(100*data)) % 10) + '0';
	if (results[6] == '0') results[6] = ' ';
	return results;
}

void tWorldConditions::mLogCall(std::string source, float data)
{
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) return;
	fpout << source.c_str() << " " <<  data << endl;
	fpout.close();

	tMissionInfo* fMissionInfo = NULL;
	fMissionInfo = gScriptInterface.mMission();
	if (fDebug) {
		std::string numstr = mNumToStr(data);
		std::string output = source + numstr.c_str();
		fMissionInfo->mDisplayMessage( kTeam1,  output.c_str(), kMissionScriptMessage );
	}
}

iTruth tWorldConditions::mStartMissionLog(std::string missionname)
{
	time_t timer1, timer2;
	timer2 = time(&timer1);
	struct tm *datetime1 = localtime(&timer1);
	int32 month = 0;
	int32 day = 0;
	int32 year = 0;
	int32 hour = 0;
	int32 min = 0;
	if (datetime1) {
		month = datetime1->tm_mon + 1;
	    day = datetime1->tm_mday;
		year = datetime1->tm_year - 100;
		hour = datetime1->tm_hour;
		if (hour == 0) hour = 12;
		min = datetime1->tm_min;
	}
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (fpout.fail()) {
		 fpout.open("NWMissionDebug.txt", ios::ate); // might need app
		 if (!fpout.fail()) {
			fpout << "ERROR: unable to open BattleLog.txt during results logging" << endl;
			fpout.close();
		 }
		return kFalse;
	}
	fpout << "Date: " << month << "/" << day << "/0" << year << " at " << hour << ":";
	if (min < 10) fpout << "0";
	fpout << min << "," << missionname.c_str() << ": " ;
	if (gScriptInterface.mIsHost()) fpout << "(Hosting)\n";
	else fpout << "(you are client)\n";
	fpout.close();
	return kTrue;
}

void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	This is the place that the teams that are out of the game, are going to be officially
	//	counted out.  Also, if there is only one side left, then the game will end for everyone
	//	and everyone will receive the proper game status changes.
	//
	iTruth humansleft = kFalse;
	iTruth	playerTeamHasShips = kFalse;
	iTruth	enemyTeamHasShips = kFalse;
	iTruth enemyhumans = kFalse;
	iTruth playerhumans = kFalse;
	int32 i;
	for ( i = kPlayerTeam1; i < kMaxMissionTeams; i++ )
	{
		tTeamInfo* team = fTeamList[i];
		if (!team) {
			team = thisMission->mGetTeamHandle(eTeamID(i));
			if (team) fTeamList[i] = team;
		}
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() ) {
				switch (i) {
				case kPlayerTeam1:
				case kPlayerTeam2:
				case kPlayerTeam3:
					playerTeamHasShips = kTrue;
					if (team->mIsTeamHumanControlled()) playerhumans = true;
					break;
				case kEnemyTeam1:
				case kEnemyTeam2:
				case kEnemyTeam3:
				case kPlayerTeam4:
				case kPlayerTeam5:
				case kPlayerTeam6:
				case kEnemyTeam5:
				case kEnemyTeam6:
				case kEnemyTeam4:
					if (team->mIsTeamHumanControlled()) enemyhumans = true;
					enemyTeamHasShips = kTrue;
					break;
				}
				if ( team->mIsTeamHumanControlled() )
					humansleft = kTrue;
			}
		}
	}

	iTruth	thereIsAWinner = fPlayerGoalCompleted || !( playerTeamHasShips && enemyTeamHasShips && humansleft );
	if ((fDraftedHumans > 0) && (!enemyhumans || !playerhumans)) thereIsAWinner = true;
	if (!thereIsAWinner && fPlayerGoalFailed) {
		thereIsAWinner = true;
	}
	if (!thereIsAWinner) fGameStartedOK = true;

	if ( thereIsAWinner )
	{
		if (!fGameStartedOK) {
			fDebug  = true;
			mLogCall("EDEnemyPatrol was unable to generate the desired opposition and is shutting down with a draw,\n   please pass your Battlelog.txt file on to ED for debugging (thanks!):", fMissionType);
			if (!humansleft) mLogCall("There appear to be no humans left in the mission", 0);
			if (!playerTeamHasShips) mLogCall("The drafting team appears to have no ships remaining", 0);
			if (!enemyTeamHasShips) mLogCall("The drafted team appears to have no ships remaining", 0);
			for ( i = 0; i < kMaxMissionTeams; i++ ) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kLeftEarly );
			return;
		}
		for ( i = kTeam1; i <= kTeam12; i++ )
		{
			int32 score = 0;

			switch (fMissionType) {
				// cases where you scan (ai only) then fight
				case kShootOnSight:
				case kFleetBattle:
					if (playerTeamHasShips && humansleft )
					{
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
					}
					else  if (!humansleft)
					{
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					}
					else if (!playerhumans) {
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					} else if ((fDraftedHumans > 0) && !enemyhumans) {
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
					}
					break;
				case kConvoy:
				case kSwarm:
				case kCleanup:
				case kPirateBase: 
				case kEnemyCombat:
				case kMineSweeper:
				case kPirateHostile:
				case kMonsterHostile:
				case kOneOfOurOwn:
				case kPirateSmuggler:
				case kPirateWanted:
				case kOutpostRaid:
					if (playerTeamHasShips && humansleft )
					{
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
					}
					else  if (!humansleft)
					{
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					}
					else if (!playerhumans) {
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					} else if ((fDraftedHumans > 0) && !enemyhumans) {
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
					}
					break;
				// cases where you just scan
				case kPirateInnocent:
				case kMonsterInnocent:
					if (fPlayerGoalCompleted || fPlayerScanCompleted) {
						if (fPlayerFired) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kLeftEarly );
						else {
							if (fPlayerGoalFailed) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
							else mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
						}
					} else {
						mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					}
					break;
				// cases where you scan and meet a goal and fight
				case kPirateSpy:
				case kDerelict:
					if (fPlayerGoalFailed) score = score - 2;
					if (enemyTeamHasShips) score--;
					if (playerTeamHasShips) score++;
					if (fPlayerScanCompleted) score++;
					if (fPlayerGoalCompleted) score++;
					if (fPlayerFiredEarly) score--;
					if (score > 0) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins  );
					else if (score >= 0) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kLeftEarly );
					else mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					break;
				// cases where you scan and meet a goal
				case kOursInDistress:
				case kEvacuateTerrain:
				case kSalvageTerrain:
				default:
					if (fPlayerGoalFailed) score = score - 2;
					if (fPlayerScanCompleted) score++;
					if (fPlayerGoalCompleted) score++;
					if (fPlayerFired) score--;
					if (score > 0) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerWins  );
					else if (score == 0) mMyInvokeGameStatus( static_cast< eTeamID >( i ), kLeftEarly );
					else mMyInvokeGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
					break;
			}
		}
	} 
}




void tWorldConditions::mMyInvokeGameStatus(eTeamID team, eGameStatus stat)
{
	tMissionInfo* fMissionInfo = gScriptInterface.mMission();
	if (!fMissionInfo) return;
	tTeamInfo *fTeam =fMissionInfo->mGetTeamHandle( team );
	if (!fTeam) return;

	fTeam->fStatus = stat;
	if (fMissionInfo) fMissionInfo->fStatusArray[fTeam->mGetTeam()] = stat;

	if (fTeam->fVictoryCondition) {
		// make sure the status is properly set for every team
		fTeam->fVictoryLevel = fTeam->fVictoryCondition->mCalcVictoryStatus(fTeam->mGetTeam());
		fTeam->mScheduleNextMission(fTeam->fVictoryCondition);
		// calculate prestige and debriefing messages for human teams
		if (!fTeam->fHasEndedHost) {
			if (fTeam->mIsTeamHumanControlled()) {
				if (gScriptInterface.mIsHost()) mLogCall("Mission report code:", stat); 
				int32 result = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				result += fTeam->fVictoryCondition->mGetBonusPrestige();
				gWC.mLogTeamResults(team, fTeam->mGetRace(), result, stat, fTeam->mIsTeamHumanControlled());
				tShipSystemsInfo info;
				info.fTeam = fTeam->mGetTeam();
				info.fAction = kPlayerEvent;
				info.fInt32[0] = kPlayerStatusChange;
				info.fInt32[1] = fTeam->fStatus;
				info.fInt32[2] = fTeam->fVictoryLevel;
				info.fInt32[3] = fTeam->fVictoryCondition->mGetPrestigePoints(fTeam->fVictoryLevel);
				info.fInt32[4] = fTeam->fVictoryCondition->mGetBonusPrestige();
				fMissionInfo->mResetDebriefingCount( fTeam->mGetTeam() );
				fTeam->mSetupDebriefingList( fTeam->fVictoryLevel );
				gScriptInterface.mEntityMgr(info);
				if (gScriptInterface.mIsHost()) {
					fTeam->fHasEndedHost = kTrue;
					fTeam->fHasEndedClient = kTrue;
				}
			}
			// return the mission results and scheduling for the team
			tTeamInfo::tMissionScheduler *nextMission = fTeam->mGetNextMissionInfo();
			tShipSystemsInfo info;
			info.fTeam = fTeam->mGetTeam();
			info.fAction = kOther;
			info.fInt32[0] = kMissionResults;
			info.fInt32[1] = fTeam->fVictoryLevel;
			if (nextMission) info.fInt32[2] = nextMission->fPrestige;
			else info.fInt32[2] = 0;
			info.fInt32[3] = fTeam->fVictoryCondition->mGetBonusPrestige();
			if (nextMission) info.fInt32[4] = nextMission->fNextMissionScore;
			else info.fInt32[4] = 0;
			if (nextMission) info.fFloat[0] = static_cast<float>(nextMission->fMedal);
			else info.fFloat[0] = 0.0f;
			if (nextMission) info.fFloat[1] = static_cast<float>(nextMission->fCampaignEvent);
			else info.fFloat[1] = 0.0f;
			info.fFloat[2] = static_cast<float>(kFalse);
			info.fFloat[3] = static_cast<float>(gScriptInterface.mIsHost());
			if (!nextMission->fNextMissionTitle.empty())
				info.fTextPtr = nextMission->fNextMissionTitle.c_str();
			else info.fTextPtr = NULL;
			gScriptInterface.mEntityMgr(info);
		}
		if (fMissionInfo) fMissionInfo->mCheckForMissionEnd();
	}
	if (gScriptInterface.mIsHost()) fTeam->fHasEndedHost = kTrue;
	fTeam->fHasEndedClient = kTrue;
}

void tWorldConditions::mReadYFAs()
{
	// read the shiplist once, 
	//    checking against the entries in fShipTypes to
	//    see if a ship in the list matches one drafted
	//    and (if so) adjusting the known YFA bounds
	//    and noting the BPV adjustment for the drafted
	//    ship

	std::ifstream fpin;
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	fpin.open("./Assets/specs/shiplist.txt");
	if (fpin.fail()) {
		if (!fpout.fail()) {
			fpout << "Failed to open list for YFAs\n";
			fpout.close();
		}
		return;
	}

   // get rid of the two junk lines at the start of the file
   const MAXSTRLEN = 2048;
   char info[MAXSTRLEN];
   std::string nextfield, shiprole, shipclass, shiphull;
   fpin.getline(info, MAXSTRLEN-1);
   fpin.getline(info, MAXSTRLEN-1);
   std::string shiprace, shiptype;
   int32 yfa = 0, yla = 1000;
   fYearLowerBound = 0;
   bool keep = false;
   do {

	/* THEN READ NEXT SHIP'S INFO FROM SHIPLIST */

	// 1. Ship race, e.g. "Federation"
	   getnextfield(shiprace, fpin);
	   if (fpin.eof()) break;
	   int32 racenum = mGetRaceNum(shiprace);
	   if (shiprace == "Monster") keep = true;
	   else if (shiprace == "Neutral") keep = true;
	   else if (!mRaceUsed(shiprace)) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 2. Ship hull type, e.g "FF"
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   if ((shiphull == "AB") || (shiphull == "Asteroid") || (shiphull == "BOX") ||
		   (shiphull == "UNKNOWN") || (shiphull == "Planet") || (shiphull == "FTR")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 3. Ship type, e.g. "F-FF"
	   getnextfield(shiptype, fpin);
	   if (fpin.eof()) break;
	// If the ship race and the first letter of the type don't match, e.g. Federation and R-BH
	//    then assume it's a donor ship in the shiplist and skip it
	   if ((mGetRaceChar(shiprace) != shiptype[0]) && (shiprace != "Neutral")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 4. Class Type, e.g. "Frigate" etc
	   getnextfield(shipclass, fpin);
	   if (fpin.eof()) break;
	   if (!keep && (shipclass == "SPECIAL")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 5. BPV
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 6. Special Role
	   getnextfield(shiprole, fpin);
	   if (fpin.eof()) break;
	   if (!keep && ((shiprole == "T") || (shiprole == "NT") || (shiprole == "R"))) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 7. Year First Available
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   yfa = atoi(nextfield.c_str());

	// 8. Year Last Available
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   yla = atoi(nextfield.c_str());

	// THEN LOTS OF RUBBISH TO END IT
	   fpin.getline(info, MAXSTRLEN-1);
	   if (fpin.eof()) break;

	// check if the ship is one that has been drafted, and,
	//    if so, check the lower bound on the current year
	// std::string fShipTypes[kMaxShips];
	   bool found = false;
	   for (int32 i = 0; i < fNumShips; i++) {
		   if (fShipTypes[i] == shiptype) {
			   found = true;
			   break;
		   }
	   }
	   if (found) {
		   if (yfa > gWC.fYearLowerBound) gWC.fYearLowerBound = yfa;
	   }

   } while (!fpin.eof());

	if (!fpout.fail()) {
		fpout << "The current year is at least: " << ((int32)(fYearLowerBound + 2263)) << "\n";
		fpout << "Map picked: " << fMapPicked << "\n";
		fpout.close();
	}
	fpin.close();
}


char tWorldConditions::mGetRaceChar(std::string race)
{
	if (race == "Federation") return 'F';
	if (race == "Klingon") return 'K';
	if (race == "Romulan") return 'R';
	if (race == "Lyran") return 'L';
	if (race == "Hydran") return 'H';
	if (race == "Gorn") return 'G';
	if (race == "ISC") return 'I';
	if (race == "Mirak") return 'Z';
	if (race == "Orion") return 'O';
	if (race == "Monster") return 'M';
	if (race == "Neutral") return 'N';
	if (race == "Andro") return 'A';
	if (race == "Tholian") return '?';
	if (race == "Andromedan") return 'A';
	if (race == "LDR") return '?';
	if (race == "Wyn") return '?';
	if (race == "Jindarian") return 'J';
	if (race == "Mirror") return '?';
	return '?';
}

bool tWorldConditions::mRaceUsed(std::string race)
{
	if (race == "Federation") return fRaceUsed[kFederation];
	if (race == "Klingon") return fRaceUsed[kKlingon];
	if (race == "Romulan") return fRaceUsed[kRomulan];
	if (race == "Lyran") return fRaceUsed[kLyran];
	if (race == "Hydran") return fRaceUsed[kHydran];
	if (race == "Gorn") return fRaceUsed[kGorn];
	if (race == "ISC") return fRaceUsed[kISC];
	if (race == "Mirak") return fRaceUsed[kMirak];
	if (race == "Orion") return fRaceUsed[kOrion];
	if (race == "Monster") return fRaceUsed[kMonster];
	if (race == "Neutral") return fRaceUsed[kNeutralRace];
	if (race == "Andro") return fRaceUsed[kAndro];
	if (race == "Tholian") return fRaceUsed[kTholian];
	if (race == "Andromedan") return fRaceUsed[kAndro];
	if (race == "LDR") return fRaceUsed[kLDR];
	if (race == "Wyn") return fRaceUsed[kWYN];
	if (race == "Jindarian") return fRaceUsed[kJindarian];
	if (race == "Mirror") return fRaceUsed[kMirror];
	return false;
}

int32 tWorldConditions::mGetRaceNum(std::string race)
{
	if (race == "Federation") return kFederation;
	if (race == "Klingon") return kKlingon;
	if (race == "Romulan") return kRomulan;
	if (race == "Lyran") return kLyran;
	if (race == "Hydran") return kHydran;
	if (race == "Gorn") return kGorn;
	if (race == "Mirak") return kMirak;
	if (race == "ISC") return kISC;
	if (race == "Orion") return kOrion;
	if (race == "Monster") return kMonster;
	if (race == "Neutral") return kNeutralRace;
	if (race == "Andro") return kAndro;
	if (race == "Tholian") return kTholian;
	if (race == "Andromedan") return kAndro;
	if (race == "LDR") return kLDR;
	if (race == "Wyn") return kWYN;
	if (race == "Jindarian") return kJindarian;
	if (race == "Mirror") return kMirror;
	return -1;
}

eRace tWorldConditions::mGetRaceFromChar(char race)
{
	if (race == 'F') return kFederation;
	if (race == 'K') return kKlingon;
	if (race == 'R') return kRomulan;
	if (race == 'L') return kLyran;
	if (race == 'H') return kHydran;
	if (race == 'G') return kGorn;
	if (race == 'I') return kISC;
	if (race == 'Z') return kMirak;
	if (race == 'O') return kOrion;
	if (race == 'M') return kMonster;
	if (race == 'N') return kNeutralRace;
	if (race == 'A') return kAndro;
	if (race == 'J') return kJindarian;
	return kNeutralRace;
}

void tWorldConditions::mReadShiplist()
{
	fShiplistRead = kTrue;

	std::ifstream fpin;
	fpin.open("./Assets/specs/shiplist.txt");
	ofstream fpout;
	fpout.open("BattleLog.txt", ios::ate); // might need app
	if (!fpout.fail()) {
		if (fpin.fail()) {
			fpout << "Could not open shiplist\n";
			fpout.close();
			return;
		}
	} else if (fpin.fail()) return;

	// read the shiplist once, 
	//    checking against the entries in fShipTypes to
	//    see if a ship in the list matches one drafted
	//    and (if so) adjusting the known YFA bounds
	//    and noting the BPV adjustment for the drafted
	//    ship

	// read the shiplist again,
	//    creating a BPV-sorted binary tree of the ships
	//    available to be generated as part of an AI fleet

   // get rid of the two junk lines at the start of the file
   const MAXSTRLEN = 2048;
   char info[MAXSTRLEN];
   std::string nextfield;
   fpin.getline(info, MAXSTRLEN-1);
   fpin.getline(info, MAXSTRLEN-1);

   std::string shiprace; // Federation
   std::string shiphull; // FF
   std::string shiptype; // F-FF
   std::string shipclass; // Frigate
   std::string shiprole; // V
   std::string ftrtype; // Z-AS
   int32 i, bpv, adjbpv, nebbpv, yfa, yla, cloak, numweapons, numdrones, NumFtrs, numshuttles;
   int32 marines, mines, parts;
   int32 warp, impulse, apr, numtrans, numesg, numph4, numphg;
   int32 insert_attempts = 0, failedsearches = 0;
   float movecost, topspeed, sparepower;
   bool keep = false; // does the current ship need to be in the picklist
   bool isscout = false;
   bool stealthy = false;
   int32 jamming;
   bool cloakable = false;
   int32 storage = 0;
   int32 dorms = 0;
   bool specialty = false;
   bool cheapftrs = false;

   do {

	/* THEN READ NEXT SHIP'S INFO FROM SHIPLIST */
	   keep = false;
	   isscout = false;
	   stealthy = false;
	   cloakable = false;
	   specialty = false;


	// 1. Ship race, e.g. "Federation"
	   getnextfield(shiprace, fpin);
	   if (fpin.eof()) break;
	   int32 racenum = mGetRaceNum(shiprace);
	   if (shiprace == "Monster") keep = true;
	   else if (shiprace == "Neutral") keep = true;
	   else if (!mRaceUsed(shiprace)) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 2. Ship hull type, e.g "FF"
	   getnextfield(shiphull, fpin);
	   if (fpin.eof()) break;
	   if ((shiphull == "AB") || (shiphull == "Asteroid") || (shiphull == "BOX") ||
		   (shiphull == "UNKNOWN") || (shiphull == "Planet") || (shiphull == "FTR")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 3. Ship type, e.g. "F-FF"
	   getnextfield(shiptype, fpin);
	   if (fpin.eof()) break;

	// If the ship race and the first letter of the type don't match, e.g. Federation and R-BH
	//    then assume it's a donor ship in the shiplist and skip it
	   if ((mGetRaceChar(shiprace) != shiptype[0]) && (shiprace != "Neutral")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 4. Class Type, e.g. "Frigate" etc
	   getnextfield(shipclass, fpin);
	   if (fpin.eof()) break;
	   if (!keep && (shipclass == "SPECIAL")) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }
	   if ((shipclass == "CARRIER") || (shipclass == "Carrier")) specialty = true;
	   if ((shipclass == "Freighter") || (shipclass == "FREIGHTER")) specialty = true;
	   if ((shipclass == "BATTLESHIP") || (shipclass == "Battleship")) specialty = true;

	// 5. BPV
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   bpv = atoi(nextfield.c_str());
	   adjbpv = bpv;
	   nebbpv = bpv;

	// 6. Special Role
	   getnextfield(shiprole, fpin);
	   if (fpin.eof()) break;
	   if (!keep && ((shiprole == "T") || (shiprole == "NT") || (shiprole == "R"))) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }
	   char rolechar = toupper(shiprole[0]);
	   if ((rolechar == 'D') || (rolechar == 'V')) specialty = true;
	   if ((rolechar == 'E') || (rolechar == 'A')) specialty = true;
	   if ((rolechar == 'S') || (rolechar == 'G')) specialty = true;
	   if ((rolechar == 'M') || (rolechar == 'H')) specialty = true;
	   if ((rolechar == 'R') || (rolechar == 'T')) specialty = true;
	   if ((rolechar == 'P') || (rolechar == 'Q')) specialty = true;
	   if (rolechar == 'S') isscout = true;

	// 7. Year First Available
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   yfa = atoi(nextfield.c_str());
	   if ((yfa > gWC.fYearLowerBound) && !keep) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// 8. Year Last Available
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   yla = atoi(nextfield.c_str());
	   if ((yla < gWC.fYearLowerBound)  && !keep) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	// For non-mobile vessels (sb, bats, bs, frd, lp, def)
	//     store one current-year representative for each
	//     race, but don't go on to put it in the tree
	   if ((shiphull == "KDP") || (shiphull == "KSD") || (shiphull == "KLP") ||
		   (shiphull == "BT") ||(shiphull == "BS") || (shiphull == "SB")) {
		   // make a note of it
		   if (shiphull == "KDP") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a weapons platform to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fFRDchoice[racenum] = shiptype;
		   }
		   if (shiphull == "KLP") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a listening post to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fLPchoice[racenum] = shiptype;
		   }
		   if (shiphull == "KSD") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a shipyard to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fDEFchoice[racenum] = shiptype;
		   }
		   if (shiphull == "BS") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a base to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fBSchoice[racenum] = shiptype;
		   }
		   if (shiphull == "BT") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a battle station to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fBATSchoice[racenum] = shiptype;
		   }
		   if (shiphull == "SB") {
			   if (fPrintShiplist) {
				   mLogCall("Adding a starbase to the race list", racenum);
				   mLogCall(shiptype, racenum);
			   }
			   fSBchoice[racenum] = shiptype;
		   }

		   // then skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   } else if ((shipclass == "Freighter") || (shiphull == "F")) {
		   // make a note of a representative freighter class for each race,
		   //      but it may also wind up inserted in the tree later
		   if (fPrintShiplist) {
			   mLogCall("Adding a freighter to race list ", racenum);
			   mLogCall(shiptype, racenum);
		   }
		   if (fFreighterChoice1[racenum] == "") fFreighterChoice1[racenum] = shiptype;
		   else if (fFreighterChoice2[racenum] == "") fFreighterChoice2[racenum] = shiptype;
		   else if (fFreighterChoice3[racenum] == "") fFreighterChoice3[racenum] = shiptype;
		   else if (fFreighterChoice4[racenum] == "") fFreighterChoice4[racenum] = shiptype;
		   else if (fFreighterChoice5[racenum] == "") fFreighterChoice5[racenum] = shiptype;
		   else if (fFreighterChoice6[racenum] == "") fFreighterChoice6[racenum] = shiptype;
		   else if ((rand() % 3) == 0) {
			   switch (rand() % 6) {
			   case 0: fFreighterChoice1[racenum] = shiptype; break;
			   case 1: fFreighterChoice2[racenum] = shiptype; break;
			   case 2: fFreighterChoice3[racenum] = shiptype; break;
			   case 3: fFreighterChoice4[racenum] = shiptype; break;
			   case 4: fFreighterChoice5[racenum] = shiptype; break;
			   default: fFreighterChoice6[racenum] = shiptype; break;
			   }
		   }
	   }

	// 9. Size Class
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 10. Turn mode
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 11. Move Cost
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   movecost = atof(nextfield.c_str());

	// 12. HET +2 Turns & Nimble
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 13. HET Breakdown
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 14. Stealth (+ECM) 
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   if (atoi(nextfield.c_str()) > 0) stealthy = true;

	// 15. Regular Crew
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 16. Boarding Parties Base
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 17. Boarding Parties Max
	   getnextfield(nextfield, fpin);
	   marines = atoi(nextfield.c_str());
	   if (fpin.eof()) break;

	// 18. Deck Crews
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 19. Total Crew
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 20. Min Crew
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 21. Shield 1
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 22. Shield 2 _ 6
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 23. Shield 3 _ 5
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 24. Shield 4
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 25. Shield Total
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;


	// 26. Cloak
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   cloak = atoi(nextfield.c_str());
	   if (cloak > 0) {
		   adjbpv = (9 * adjbpv) / 10;
		   nebbpv = (5 * nebbpv) / 6;
		   cloakable = true;
	   }

	   // prep the drone rack information
	   numweapons = 0;
	   numdrones = 0;
	   numesg = 0;
	   numphg = 0;
	   numph4 = 0;

	   for (i = 0; i < 25; i++) {

		  std::string wtype;
		  getnextfield(nextfield, fpin);
		  if (fpin.eof()) break;
		  long count = atoi(nextfield.c_str());
		  if (strnicmp(wtype.c_str(), "PH3", 3)) count = (count+1) / 2;
		  if (strnicmp(wtype.c_str(), "ADD", 3) && (strnicmp(wtype.c_str(), "PLAD", 4))) 
			  numweapons += count;

		  // Type
		  getnextfield(wtype, fpin);
		   if (fpin.eof()) break;

		  // if it's a drone rack we have to update the appropriate information
		  if (strnicmp(wtype.c_str(), "Dro", 3) == 0) numdrones += count;
		  if (strnicmp(wtype.c_str(), "ESG", 3) == 0) numesg += count;
		  if (strnicmp(wtype.c_str(), "PHG", 3) == 0) numphg += count;
		  if (strnicmp(wtype.c_str(), "PH4", 3) == 0) numph4 += count;

		  //  get weapon arc
		  getnextfield(nextfield, fpin);
		  if (fpin.eof()) break;

	   }
	   // adjust bpv for weaponry cheese
	   int32 cheesetotal = ((2 * numdrones) / 3) + numesg + numphg + numph4;
	   int32 aveyear = 2;
	   if (yfa > 2) aveyear = yfa;
	   if (yla > 30) aveyear += 30;
	   else aveyear += yla;
	   aveyear = aveyear / 2;
	   if (aveyear > 4) cheesetotal += (3 * numdrones) / 4;
	   if (aveyear > 17) cheesetotal += numdrones;
	   cheesetotal = cheesetotal - (bpv / 60);
	   if (cheesetotal > 0) {
		  // positively cheesey
		  if (cheesetotal >= 8) adjbpv = adjbpv + 8 * cheesetotal;
		  else adjbpv = adjbpv + (cheesetotal*cheesetotal);
	   }
	   int32 nebcheese = cheesetotal - numesg;
	   if (nebcheese > 0) {
	      if (nebcheese >= 8) nebbpv = nebbpv + (8 * nebcheese);
		  else nebbpv = nebbpv + (nebcheese * nebcheese);
	   }

	// 102. Probes
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming = atoi(nextfield.c_str());

	// 103. T Bombs Base
	   getnextfield(nextfield, fpin);
	   mines = atoi(nextfield.c_str());
	   if (fpin.eof()) break;

	// 104. T Bombs Max
	   getnextfield(nextfield, fpin);

	// 105. Nuclear Mine Base
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 106. Nuclear Mine Max
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 107. Drone Control
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 108. ADD_6
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 109. ADD_12
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 110. Shuttles Size
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 111. Launch Rate
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 112. General Base
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 113. General Max
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   numshuttles = atoi(nextfield.c_str());
	   if (numshuttles > 2) {
		   adjbpv = adjbpv + (numshuttles - 2) * (numshuttles - 2);
		   nebbpv = nebbpv - numshuttles;
	   }

	// Then for 114 through 121 get num fighters and their type
	   NumFtrs = 0;
	   int32 ftrbays = 0;
	   for (int j = 0; j < 4; j++) {
		  // 120. Fighter Bay 4
		  getnextfield(nextfield, fpin);
		  if (fpin.eof()) break;
		  NumFtrs += atoi(nextfield.c_str());
		  if (atoi(nextfield.c_str()) > 0) ftrbays++;
   
		  // 121. Fighter Type 4
		  getnextfield(nextfield, fpin);
		  if (j == 0) ftrtype = nextfield;
	      if (fpin.eof()) break;
	   }
	   if ((NumFtrs > 0) && (mIsCheapFtr(ftrtype))) cheapftrs = true;
	   else cheapftrs = false;

	   // adjust bpv for fighters/pfs
	   if ((NumFtrs > 0) && (!cheapftrs)) {
		   nebbpv = nebbpv - 2 * NumFtrs;
		   if (NumFtrs > 12) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 9 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 9 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 8 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 8 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 6 * NumFtrs;
				} else {
				   adjbpv += 5 * NumFtrs;
				}
		   } else if (NumFtrs > 8) {
				if (yfa > 19) {
			  if (shiprace == "Romulan") adjbpv += 8 * NumFtrs;
			  else if (shiprace == "Gorn") adjbpv += 8 * NumFtrs;
			  else if (shiprace == "Klingon") adjbpv += 5 * NumFtrs;
			  else if (shiprace == "Mirak") adjbpv += 5 * NumFtrs;
			  else if (shiprace == "Federation") adjbpv += 6 * NumFtrs;
			  else if (shiprace == "ISC") adjbpv += 6 * NumFtrs;
			  else if (shiprace == "Lyran") adjbpv += 7 * NumFtrs;
			  else if (shiprace == "Hydran") adjbpv += 7 * NumFtrs;
			  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 5 * NumFtrs;
				} else {
				   adjbpv += 4 * NumFtrs;
				}
		   } else if (NumFtrs > 4) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 6 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 4 * NumFtrs;
				} else {
				   adjbpv += 3 * NumFtrs;
				}
		   } else if (NumFtrs > 0) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 5 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 3 * NumFtrs;
				} else {
				   adjbpv += 2 * NumFtrs;
				}
		   }
	   }

	// 122. Armor
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 123. Forward Hull
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   storage = atoi(nextfield.c_str());

	// 124. Center Hull
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   storage += atoi(nextfield.c_str());

	// 125. Aft Hull
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   storage += atoi(nextfield.c_str());
	   dorms = storage;

	// 126. Cargo
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   storage += 10 * atoi(nextfield.c_str());
	   dorms += atoi(nextfield.c_str());

	// 127. Barracks
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   storage += atoi(nextfield.c_str());
	   dorms += 10 * atoi(nextfield.c_str());

	// 128. Repair
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 129. R_L_Warp
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   warp = 2 * atoi(nextfield.c_str()) & 255ul;

	// 130. C_Warp
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   warp += atoi(nextfield.c_str()) & 255ul;

	// 131. Impulse
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   impulse = atoi(nextfield.c_str()) & 255ul;

	// 132. Apr
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   apr = atoi(nextfield.c_str()) & 255ul;

	   //  Compute the ship's power curve
	   float movepower = warp;
	   float disengagespeed, disengagepower;
	   if (warp > 30) disengagepower = 15;
	   else disengagepower = warp / 2.0;
	   float otherpower = apr;
	   if (movecost > impulse) {
		  movepower += impulse;
		  disengagepower += impulse;
	   } else {
		  movepower += movecost;
		  disengagepower += movecost;
		  otherpower += (impulse - movecost);
	   }
	   float lifesupport = 3.0;
	   if (movecost > 0.5) lifesupport += 1.5;
	   if (movecost > 1.25) lifesupport += 2.0;
	   if (otherpower > lifesupport) {
		  otherpower -= lifesupport;
	   } else {
		  otherpower = 0;
		  movepower = movepower + otherpower - lifesupport;
		  disengagepower = disengagepower + otherpower - lifesupport;
	   }
	   if (movecost < 0.125) {
		  topspeed =  0; 
		  disengagespeed = 31.0;
		  sparepower = movepower + otherpower;
	   } else if (movepower >= (31 * movecost)) {
		  topspeed = 31;
		  disengagespeed = disengagepower / movecost;
		  sparepower = otherpower + movepower - (31 * movecost);
	   } else {
		  disengagespeed = disengagepower / movecost;
		  topspeed = movepower / movecost;
		  sparepower = otherpower + movepower - (topspeed * movecost);
	   }

	   // don't bother including painfully slow ships in the AI fleets
	   if ((topspeed < 16.0) && !keep) {
			// skip this ship
		   fpin.getline(info, MAXSTRLEN-1);
		   if (fpin.eof()) break;
		   continue;
	   }

	   // adjust bpv for painfully slow ships
	   if (racenum <= kMirak) {
		   if ((adjbpv > 40) && (topspeed < 10)) {
			  adjbpv = (3 * adjbpv) / 5;
			  nebbpv = (3 * nebbpv) / 5;
		   } else if ((adjbpv > 33) && (topspeed < 16)) {
			  adjbpv = (7 * adjbpv) / 10;
			  nebbpv = (7 * nebbpv) / 10;
		   } else if ((adjbpv > 28) && (topspeed < 24)) {
			  adjbpv = 4 * adjbpv / 5;
			  nebbpv = 4 * nebbpv / 5;
		   } else if ((adjbpv > 25) && (topspeed < 28)) {
			  adjbpv = 9 * adjbpv / 10; 
			  nebbpv = 9 * nebbpv / 10; 
		   }
	   }

	   // increase bpv for fast, high-powered ships
	   else if (sparepower > (adjbpv / 10.0)) {
		  adjbpv = 26 * adjbpv / 20;
		  nebbpv = 26 * nebbpv / 20;
	   } else if (sparepower > (adjbpv / 15.0)) {
		  adjbpv = 25 * adjbpv / 20;
		  nebbpv = 25 * nebbpv / 20;
	   } else if (sparepower > (adjbpv / 20.0)) {
		  adjbpv = 24 * adjbpv / 20;
		  nebbpv = 24 * nebbpv / 20;
	   } else if (sparepower > (adjbpv / 30.0)) {
		  adjbpv = 23 * adjbpv / 20;
		  nebbpv = 23 * nebbpv / 20;
	   } else if (sparepower > (adjbpv / 40.0)) {
		  adjbpv = 22 * adjbpv / 20;
		  nebbpv = 22 * nebbpv / 20;
	   } else if (sparepower > (adjbpv / 50.0)) {
		  adjbpv = 21 * adjbpv / 20;
		  nebbpv = 21 * nebbpv / 20;
	   }

	// 133. Battery
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 134. Bridge
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming += 4 * atoi(nextfield.c_str());

	// 135. Security
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming += 2 * atoi(nextfield.c_str());

	// 136. Lab
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming += 6 * atoi(nextfield.c_str());

	// 137. Transporters
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   numtrans = atoi(nextfield.c_str());
	   // adjust bpv for transporters
	   if (numtrans > 5) {
		  adjbpv = 12 * adjbpv / 10;
	   } else if (numtrans > 3) {
		  adjbpv = 11 * adjbpv / 10;
	   } else if (numtrans < 2) {
		  adjbpv = 9 * adjbpv / 10;
	   }

	// 138. Tractors
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 139. Mech Tractors
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 140. Special Sensors
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   if (atoi(nextfield.c_str()) > 0) isscout = true;
	   jamming += (10 * atoi(nextfield.c_str()));

	// 141. Sensors
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming += 2 * atoi(nextfield.c_str());

	// 142. Scanners
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;
	   jamming += 2 * atoi(nextfield.c_str());

	// 143. Explosion Strength
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 144. Acceleration
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// 145. Damage Control
	   getnextfield(nextfield, fpin);
	   parts = atoi(nextfield.c_str());
	   parts *= 4;
	   if (fpin.eof()) break;

	// 146. Extra Damage
	   getnextfield(nextfield, fpin);
	   if (fpin.eof()) break;

	// THEN LOTS OF RUBBISH TO END IT
	   fpin.getline(info, MAXSTRLEN-1);
	   if (fpin.eof()) break;

	   // NOW CREATE THE NEW SHIPDATA ELEMENT AND INSERT IT IN THE TREE
	   if (adjbpv < (bpv / 2)) adjbpv = bpv / 2;
	   if (nebbpv < (bpv / 2)) nebbpv = bpv / 2;
	   sShipData *sptr = new sShipData;
	   if (sptr) {
		   insert_attempts++;
		   sptr->ftrbays = ftrbays;
		   sptr->specialty = specialty;
		   sptr->cheapftrs = cheapftrs;
		   sptr->adjbpv = adjbpv;
		   sptr->lbpv = NULL;
		   sptr->rbpv = NULL;
		   sptr->listbpv = bpv;
		   sptr->ltype = NULL;
		   sptr->nebbpv = nebbpv;
		   sptr->storage = storage;
		   sptr->rtype = NULL;
		   sptr->shiptype = shiptype;
		   sptr->topspeed = topspeed;
		   sptr->shiprace = shiprace;
		   sptr->oktodraft = true;
		   sptr->isscout = isscout;
		   sptr->stealthy = stealthy;
		   sptr->dorms = dorms;
		   sptr->trans = numtrans;
		   sptr->marines = marines;
		   sptr->mines = mines;
		   sptr->parts = parts;
		   sptr->shuttles = numshuttles;
		   sptr->fighters = NumFtrs;
		   sptr->ftrtype = ftrtype;
		   if (isscout) sptr->jamming = 3 * jamming / 2;
		   else sptr->jamming = jamming;
		   sptr->cloak = cloakable;
		   if ((shipclass == "Freighter") || (shipclass == "FREIGHTER")) sptr->oktodraft = false;
		   if (fPrintShiplist) mLogCall(shiptype, adjbpv);
		   mInsert(sptr);
	   }

   } while (!fpin.eof());
	fpin.close();

	// now can we go through and try to compute each team's real bpv,
	//	and store it in fTeamBPV[], fTeamNebBPV[],
	// using the adjusted bpv or nebula bpv values if we can find
	//  the team's ships using our mSearch routine,
	// or just falling back on the Taldren team->mGetCombatBPV()
	//  if any of the ships aren't in our search tree

	for (int32 t = 0; t < kMaxMissionTeams; t++) {
		tTeamInfo *team = fTeamList[t];
		if (!team) continue;
		int32 baseBPV = team->mGetCombatBPV();
		fTeamBPV[t] = 0;
		fTeamNebBPV[t] = 0;
		for (int32 j = 0; j < fTeamNumShips[t]; j++) {
			bool teamok = true;
			std::string ship = fShipTypeList[6*t+j];
			sShipData *s = mSearch(ship, team->mGetRace());
			if (!s) {
				failedsearches++;
				fTeamBPV[t] = baseBPV;
				switch (team->mGetRace()) {
				case kLyran:
				case kHydran:
					fTeamNebBPV[t] = (4 * fTeamBPV[t]) / 5;
					break;
				case kRomulan:
				case kGorn:
				case kISC:
					fTeamNebBPV[t] = fTeamBPV[t];
					break;
				default:
					fTeamNebBPV[t] = (19 * fTeamBPV[t]) / 20;
					break;
				}
				teamok = false;
			} else if (teamok) {
				fTeamNebBPV[t] += s->nebbpv;
				fTeamBPV[t] += s->adjbpv;
			}
		}
	}

}

int32 tWorldConditions::mGetFtrBays(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 bays = 0;
	if (!shipdata) return bays;
	else return shipdata->ftrbays;
}



void tWorldConditions::getnextfield(std::string &s, std::ifstream &fpin)
{
   s = "";
   char c;
   do {
      fpin.get(c);
      if (fpin.eof()) return;
      if (c != '\t') s += c;
   } while (c != '\t');
}

bool tWorldConditions::mInsert(sShipData *ship)
// inserts the ship in the bstree rooted at fShipsByBPV
//    and the bstree rooted at fShipsByType
// returns true if successful, false otherwise
{
	if (!ship) return false;
	bool result1 = false, result2 = false;
	if (ship->shiprace == "Federation") {
		result1 = mInsertType(fShipsByType[kFederation], ship);
		result2 = mInsertBPV(fShipsByBPV[kFederation], ship);
	}
	else if (ship->shiprace == "Klingon") {
		result1 = mInsertType(fShipsByType[kKlingon], ship);
		result2 = mInsertBPV(fShipsByBPV[kKlingon], ship);
	}
	else if (ship->shiprace == "Romulan") {
		result1 = mInsertType(fShipsByType[kRomulan], ship);
		result2 = mInsertBPV(fShipsByBPV[kRomulan], ship);
	}
	else if (ship->shiprace == "Lyran") {
		result1 = mInsertType(fShipsByType[kLyran], ship);
		result2 = mInsertBPV(fShipsByBPV[kLyran], ship);
	}
	else if (ship->shiprace == "Hydran") {
		result1 = mInsertType(fShipsByType[kHydran], ship);
		result2 = mInsertBPV(fShipsByBPV[kHydran], ship);
	}
	else if (ship->shiprace == "Gorn") {
		result1 = mInsertType(fShipsByType[kGorn], ship);
		result2 = mInsertBPV(fShipsByBPV[kGorn], ship);
	}
	else if (ship->shiprace == "Mirak") {
		result1 = mInsertType(fShipsByType[kMirak], ship);
		result2 = mInsertBPV(fShipsByBPV[kMirak], ship);
	}
	else if (ship->shiprace == "Orion") {
		result1 = mInsertType(fShipsByType[kOrion], ship);
		result2 = mInsertBPV(fShipsByBPV[kOrion], ship);
	}
	else if (ship->shiprace == "Monster") {
		result1 = mInsertType(fShipsByType[kMonster], ship);
		result2 = mInsertBPV(fShipsByBPV[kMonster], ship);
	}
	else if (ship->shiprace == "Neutral") {
		result1 = mInsertType(fShipsByType[kNeutralRace], ship);
		result2 = mInsertBPV(fShipsByBPV[kNeutralRace], ship);
	}
	else if (ship->shiprace == "Andro") {
		result1 = mInsertType(fShipsByType[kAndro], ship);
		result2 = mInsertBPV(fShipsByBPV[kAndro], ship);
	}
	else if (ship->shiprace == "Tholian") {
		result1 = mInsertType(fShipsByType[kTholian], ship);
		result2 = mInsertBPV(fShipsByBPV[kTholian], ship);
	}
	else if (ship->shiprace == "Andromedan") {
		result1 = mInsertType(fShipsByType[kAndro], ship);
		result2 = mInsertBPV(fShipsByBPV[kAndro], ship);
	}
	else if (ship->shiprace == "LDR") {
		result1 = mInsertType(fShipsByType[kLDR], ship);
		result2 = mInsertBPV(fShipsByBPV[kLDR], ship);
	}
	else if (ship->shiprace == "Wyn") {
		result1 = mInsertType(fShipsByType[kWYN], ship);
		result2 = mInsertBPV(fShipsByBPV[kWYN], ship);
	}
	else if (ship->shiprace == "Jindarian") {
		result1 = mInsertType(fShipsByType[kJindarian], ship);
		result2 = mInsertBPV(fShipsByBPV[kJindarian], ship);
	}
	else if (ship->shiprace == "Mirror") {
		result1 = mInsertType(fShipsByType[kMirror], ship);
		result2 = mInsertBPV(fShipsByBPV[kMirror], ship);
	}
	if (result1) {
		fTypeSize++;
	} 
	if (result2) {
		fBPVSize++;
	} 
	return result1 && result2;
}

bool tWorldConditions::mInsertBPV(sShipData* &tree, sShipData* ship)
{
	if (!ship) return false;
	if (!tree) {
		tree = ship;
		return true;
	}
	else if (fNebula) {
		if (tree->nebbpv > ship->nebbpv) {
			return mInsertBPV(tree->lbpv, ship);
		} else {
			return mInsertBPV(tree->rbpv, ship);
		}
	} else {
		if (tree->adjbpv > ship->adjbpv) {
			return mInsertBPV(tree->lbpv, ship);
		} else {
			return mInsertBPV(tree->rbpv, ship);
		}
	}
}

bool tWorldConditions::mInsertType(sShipData* &tree, sShipData* ship)
{
	if (!ship) return false;
	if (!tree) {
		tree = ship;
		return true;
	}
	else if (tree->shiptype == ship->shiptype) return true;
	else if (strcmp(tree->shiptype.c_str(), ship->shiptype.c_str()) > 0) {
		return mInsertType(tree->ltype, ship);
	} else {
		return mInsertType(tree->rtype, ship);
	}
	return false;
}

sShipData* tWorldConditions::mSearch(std::string ship, eRace race)
{
	// return the sShipData ptr that matches the ship type, e.g. matching "F-FF"
	if ((race < 0) || (race >= kNumberOfRaces)) return NULL;
	if (ship.length() < 2) return NULL;
	sShipData *result = fShipsByType[race];
	while (result) {
		if (result->shiptype == ship) return result;
		if (strcmp(result->shiptype.c_str(), ship.c_str()) > 0) result = result->ltype;
		else result = result->rtype;
	}
	return result;
}

void tWorldConditions::mPrintTree(sShipData *s)
{
	if (!s) return;
	if (!fDebug) return;
	mPrintTree(s->lbpv);
	if (fNebula) mLogCall(s->shiptype, s->nebbpv);
	else mLogCall(s->shiptype, s->adjbpv);
	mPrintTree(s->rbpv);
}

sShipData* tWorldConditions::mSearch(int32 bpv, eRace race)
{
	// return the sShipData ptr for the ship whose bpv is closest without going over
	if ((race < 0) || (race > kNumberOfRaces)) return NULL;
	if (bpv < 1) return NULL;
	sShipData *candidate = mSearchStock(bpv, race);
	if (((rand() % 37) < 23) && candidate) {
		return candidate;
	}
	sShipData *result = fShipsByBPV[race];
	while (result) {
		if (fNebula) {
			if ((result->nebbpv == bpv) && result->oktodraft) return result;
			if (result->nebbpv < bpv) {
				if (result->oktodraft) {
					if (!candidate) candidate = result;
					else if ((rand() % 97) < 37) candidate = result;
					else {
						// have a probability of replacing the old candidate with the new
						int32 keepchance = bpv - result->nebbpv;
						if (keepchance < 1) keepchance = 1;
						int32 skipchance = bpv - candidate->nebbpv;
						if (skipchance < 1) skipchance = 1;
						int32 total = keepchance + skipchance;
						if ((rand() % total) >= keepchance)
							candidate = result;
					}
				}	
				if (result->rbpv) result = result->rbpv;
				else return candidate;
			} else {
				if (result->lbpv) result = result->lbpv;
				else return candidate;
			}
		} else {
			if ((result->adjbpv == bpv) && result->oktodraft) return result;
			if (result->adjbpv < bpv) {
				if (result->oktodraft) {
					if (!candidate) candidate = result;
					else if ((rand() % 97) < 37) candidate = result;
					else {
						// have a probability of replacing the old candidate with the new
						int32 keepchance = bpv - result->adjbpv;
						if (keepchance < 1) keepchance = 1;
						int32 skipchance = bpv - candidate->adjbpv;
						if (skipchance < 1) skipchance = 1;
						int32 total = keepchance + skipchance;
						if ((rand() % total) >= keepchance)
							candidate = result;
					}
				}
				if (result->rbpv) result = result->rbpv;
				else return candidate;
			} else {
				if (result->lbpv) result = result->lbpv;
				else return candidate;
			}
		}
	}
	return NULL;
}

sShipData* tWorldConditions::mSearchStock(int32 bpv, eRace race)
{
	// return the sShipData ptr for the ship whose bpv is closest without going over
	if ((race < 0) || (race > kNumberOfRaces)) return NULL;
	if (bpv < 1) return NULL;
	sShipData *result = fShipsByBPV[race];
	sShipData *candidate = NULL;
	while (result) {
		if (fNebula) {
			if ((result->nebbpv == bpv) && result->oktodraft && !result->specialty) return result;
			if (result->nebbpv < bpv) {
				if (result->oktodraft && !result->specialty) {
					if (!candidate) candidate = result;
					else if ((rand() % 97) < 49) candidate = result;
					else {
						// have a probability of replacing the old candidate with the new
						int32 keepchance = bpv - result->nebbpv;
						if (keepchance < 1) keepchance = 1;
						int32 skipchance = bpv - candidate->nebbpv;
						if (skipchance < 1) skipchance = 1;
						int32 total = keepchance + skipchance;
						if ((rand() % total) >= keepchance)
							candidate = result;
					}
				}
				if (result->rbpv) result = result->rbpv;
				else return candidate;
			} else {
				if (result->lbpv) result = result->lbpv;
				else return candidate;
			}
		} else {
			if ((result->adjbpv == bpv) && result->oktodraft && !result->specialty) return result;
			if (result->adjbpv < bpv) {
				if (result->oktodraft && !result->specialty) {
					if (!candidate) candidate = result;
					else if ((rand() % 97) < 49) candidate = result;
					else {
						// have a probability of replacing the old candidate with the new
						int32 keepchance = bpv - result->adjbpv;
						if (keepchance < 1) keepchance = 1;
						int32 skipchance = bpv - candidate->adjbpv;
						if (skipchance < 1) skipchance = 1;
						int32 total = keepchance + skipchance;
						if ((rand() % total) >= keepchance)
							candidate = result;
					}
				}
				if (result->rbpv) result = result->rbpv;
				else return candidate;
			} else {
				if (result->lbpv) result = result->lbpv;
				else return candidate;
			}
		}
	}
	return NULL;
}

int32 tWorldConditions::mGetCombatBPV(eTeamID team)
{
	if (fNebula) {
		if (fTeamNebBPV[team] < 1) {
			tTeamInfo *t = fTeamList[team];
			if (t) return t->mGetCombatBPV();
			else return 80;
		}
		return fTeamNebBPV[team];
	} else {
		if (fTeamBPV[team] < 1) {
			tTeamInfo *t = fTeamList[team];
			if (t) return t->mGetCombatBPV();
			else return 80;
		}
		return fTeamBPV[team];
	}
}

int32 tWorldConditions::mGetSmallestBPV(eRace race)
{
	if ((race < 0) || (race > kNumberOfRaces)) return NULL;
	int32 bpv = 0;
	sShipData *ship = fShipsByBPV[race];
	if (!ship) return 0;
	while (ship->lbpv) ship = ship->lbpv;
	if (fNebula) return ship->nebbpv;
	else return ship->adjbpv;
}

int32 tWorldConditions::mGetLargestBPV(eRace race)
{
	if ((race < 0) || (race > kNumberOfRaces)) return NULL;
	int32 bpv = 0;
	sShipData *ship = fShipsByBPV[race];
	if (!ship) return 0;
	while (ship->rbpv) ship = ship->rbpv;
	if (fNebula) return ship->nebbpv;
	else return ship->adjbpv;
}

int32 tWorldConditions::mPickFleet(int32 bpv, int32 numships, eRace race, std::string chosenShips[])
{
	// bail out if they give us impossible options
	if ((race < 0) || (race > kNeutralRace)) return 0;
	if ((numships < 1) || (bpv < 1) || (numships > 6)) return 0;

	// find the range of available ship bpvs
	int32 smallval = mGetSmallestBPV(race);
	int32 largeval = mGetLargestBPV(race);
	if (smallval > bpv) return 0;
	if (smallval < 1) smallval = 1;
	if (largeval < smallval) largeval = smallval;

	// using the tree created by mReadShiplist,
	// create a fleet of ships with the target race, bpv, and number of ships
	//    and return the actual bpv of the resulting fleet
	// the ship types are stored in the chosenShips array,
	//    so it is up to the caller to then call the mCreateShip methods

	// initialize the ship names, 
	//   just to make sure they didn't pass us something
	//   that had values in it already
	int32 i;
	int32 bpvremaining = bpv;
	int32 actualbpv = 0;
	int32 shipsremaining = numships;
	for (i = 0; i < 6; i++) chosenShips[i] = "";

	// otherwise look for a reasonable breakdown of bpvs into the number of ships
	int32 currship = 0;
	while ((shipsremaining > 0) && (bpvremaining > smallval)) {
		// pick an upper bound for the bpv of the next ship to add
		int32 thisbpv = bpvremaining;
		switch (shipsremaining) {
		   case 6: // if this is the sixth to last ship, it gets 15-25% of the remaining bpv
			   thisbpv = (thisbpv * (15 + (rand() % 10))) / 100; 
			   break;
		   case 5: // if this is the fifth to last ship, it gets 20-30% of the remaining bpv
			   thisbpv = (thisbpv * (20 + (rand() % 10))) / 100; 
			   break;
		   case 4: // if this is the fourth to last ship, it gets 25-35% of the remaining bpv
			   thisbpv = (thisbpv * (25 + (rand() % 10))) / 100; 
			   break;
		   case 3: // if this is the third to last ship, it gets 35-50% of the remaining bpv
			   thisbpv = (thisbpv * (35 + (rand() % 15))) / 100; 
			   break;
		   case 2: // if this is the second to last ship, it gets 50-70% of the remaining bpv
			   thisbpv = (thisbpv * (50 + (rand() % 20))) / 100; 
			   break;
		   default: // in general this ship gets all the remaining bpv
			   break;
		}
		thisbpv = mPickShip(thisbpv, race, chosenShips[currship]);
		actualbpv += thisbpv;
		bpvremaining = bpvremaining - thisbpv;
		shipsremaining--;
		if (actualbpv > 0) currship++;
	}

	return actualbpv;
}

int32 tWorldConditions::mPickShip(int32 bpv, eRace race, std::string &ship)
{
	sShipData *sptr = NULL;
	sptr = mSearch(bpv, race);
	if (sptr) {
		ship = sptr->shiptype;
		if (fNebula) return sptr->nebbpv;
		else return sptr->adjbpv;
	} else {
		ship = "";
		return 0;
	}
}


int32 tWorldConditions::mGetMaxParts(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 parts = 4;
	if (!shipdata) return parts;
	else return shipdata->parts;
}

int32 tWorldConditions::mGetMaxShuttles(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 shuttles = 2;
	if (!shipdata) return shuttles;
	else return shipdata->shuttles;
}

int32 tWorldConditions::mGetMaxMarines(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 marines = 8;
	if (!shipdata) return marines;
	else return shipdata->marines;
}

int32 tWorldConditions::mGetMaxMines(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 mines = 4;
	if (!shipdata) return mines;
	else return shipdata->mines;
}

int32 tWorldConditions::mGetMaxFtrs(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 ftrs = 0;
	if (!shipdata) return ftrs;
	else return shipdata->fighters;
}

int32 tWorldConditions::mGetHullsize(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 size = 4;
	if (!shipdata) return size;
	else return 1 + (shipdata->adjbpv / 30);
}

std::string tWorldConditions::mGetFtrType(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	std::string ftrtype = "N-SH";
	if (!shipdata) {
		if (ship.length() > 2) ftrtype[0] = ship[0];
		return ftrtype;
	}
	else return shipdata->ftrtype;
}

sShipData* tWorldConditions::mSearch(std::string ship) 
{
	if (ship.length() < 3) return NULL;
	eRace race = mGetRaceFromChar(ship[0]);
	return mSearch(ship, race);
}

int32 tWorldConditions::mGetBPV(std::string ship)
{
	sShipData *shipdata = mSearch(ship);
	int32 bpv = 80;
	if (!shipdata) return bpv;
	else if (gWC.fMapPicked == 1) return shipdata->nebbpv;
	else return shipdata->adjbpv;
}

bool tWorldConditions::mIsCheapFtr(std::string ftr)
{
	if (ftr.length() == 4) {
		if ((ftr[2] != 'S') && (ftr[2] != 's')) return false;
		if ((ftr[3] != 'H') && (ftr[3] != 'h')) return false;
		return true;
	} else if (ftr.length() == 5) {
		if ((ftr[2] != 'I') && (ftr[2] != 'i')) return false;
		if ((ftr[3] != 'N') && (ftr[3] != 'n')) return false;
		if ((ftr[4] != 'T') && (ftr[4] != 't')) return false;
		return true;
	}
	return false;
}

int32 tWorldConditions::mGetFleetSize(int32 bpv)
{
	if (fYearLowerBound < 5) {
		if (bpv < 180) return 1;
		if (bpv < 360) return 2;
		if (bpv < 540) return 3;
		if (bpv < 720) return 4;
	} else if (fYearLowerBound < 12) {
		if (bpv < 200) return 1;
		if (bpv < 400) return 2;
		if (bpv < 600) return 3;
		if (bpv < 800) return 4;
	} else if (fYearLowerBound < 19) {
		if (bpv < 240) return 1;
		if (bpv < 480) return 2;
		if (bpv < 720) return 3;
		if (bpv < 960) return 4;
	} else {
		if (bpv < 280) return 1;
		if (bpv < 560) return 2;
		if (bpv < 840) return 3;
		if (bpv < 1120) return 4;
	}
	return 5;
}

int32 tWorldConditions::mGetFtrAdj(std::string ship)
{
	if (ship.length() < 3) return 0;
	int32 NumFtrs = mGetMaxFtrs(ship);
	int32 yfa = fYearLowerBound;
	int32 adjbpv = 0;
	std::string shiprace = "Orion";
	switch (ship[0]) {
		case 'F': shiprace = "Federation"; break;
		case 'K': shiprace = "Klingon"; break;
		case 'R': shiprace = "Romulan"; break;
		case 'L': shiprace = "Lyran"; break;
		case 'H': shiprace = "Hydran"; break;
		case 'G': shiprace = "Gorn"; break;
		case 'I': shiprace = "ISC"; break;
		case 'Z': shiprace = "Mirak"; break;
		default: break;
	}
	if (gWC.fMapPicked != 1) {
	   if (NumFtrs > 0) {
		   if (NumFtrs > 12) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 9 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 9 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 8 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 4 * NumFtrs;
				} else {
				   adjbpv += 3 * NumFtrs;
				}
		   } else if (NumFtrs > 8) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 8 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 8 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 7 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 4 * NumFtrs;
				} else {
				   adjbpv += 2 * NumFtrs;
				}
		   } else if (NumFtrs > 4) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 7 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 5 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 6 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 3 * NumFtrs;
				} else {
				   adjbpv += 2 * NumFtrs;
				}
		   } else if (NumFtrs > 0) {
				if (yfa > 19) {
				  if (shiprace == "Romulan") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Gorn") adjbpv += 6 * NumFtrs;
				  else if (shiprace == "Klingon") adjbpv += 2 * NumFtrs;
				  else if (shiprace == "Mirak") adjbpv += 2 * NumFtrs;
				  else if (shiprace == "Federation") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "ISC") adjbpv += 3 * NumFtrs;
				  else if (shiprace == "Lyran") adjbpv += 4 * NumFtrs;
				  else if (shiprace == "Hydran") adjbpv += 5 * NumFtrs;
				  else adjbpv += 4 * NumFtrs ;
				} else if (yfa > 9) {
				   adjbpv += 2 * NumFtrs;
				} else {
				   adjbpv += 1 * NumFtrs;
				}
		   }
	   }
	}
	return adjbpv;
}

