/********************************************************************************************************\

	File Name:	CommonEnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonEnemyShip.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"

 
tClassAllocator<tCommonEnemyTeam> gCommonEnemyTeamAllocator;

tCommonEnemyTeam::tCommonEnemyTeam() : fNumShips( 0 )
{
	fHumanControlled = mIsTeamHumanControlled();
	fTeamID = -1;
	fMyShipID = -1;
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
}

void tCommonEnemyTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonEnemyTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonEnemyTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonEnemyTeamBaseState() );
	mGotoState( typeid( tCommonEnemyTeamBaseState ) );
   }
   
}

void tCommonEnemyTeam::mSetupCommState()
{}


void tCommonEnemyTeam::mCreateShipsForTeam()
{
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();
	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) gWC.fAddAlly = 0;
	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftedBPV += (5 * humanBPV / 4);
	}

	if (fHumanControlled || (mIsTeamHumanControlled()) || (gWC.fDraftedHumans > 0)) {
		return;
	}
	std::string shipname = fShipName;
	std::string shiptype = fShipType;

	int32 bpv, i, minbpv, bpvsofar;
	bpv = 0;
	minbpv = 60;


	if (fTeamID == kEnemyTeam3) {
		if(gWC.fUseMinefield) {
			for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-DEF", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"lg mine",
					kDefaultShipOptions );
			}
			for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-LP", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"sm mine",
					kDefaultShipOptions );
			}
		} else {
			if ((gWC.fMapPicked > 3) && (gWC.fMapPicked <= 9) && gWC.fIncludeSystem) {
				if ((rand() % 17) < 1) {
					shiptype = "M-SGL";
					gWC.fDoomsDay = true;
					gWC.fDraftedBPV += 250;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else {
					if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
					bpv += minbpv;
					bpv = (((rand() % 40) + 90) * bpv) / 100;
					bpv = bpv - gWC.fDraftedBPV;
					if (fShipType != "") {
						shiptype = fShipType;
						bpvsofar = gWC.mGetBPV(shiptype);
						// create the flagship
						gWC.fDraftedBPV += bpvsofar;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							shiptype.c_str(), 
							kStartPosition_L,
							fShipMetaID,
							fMyShipID++, 0, 0, -1, -1,
							shipname.c_str(),
							kDefaultShipOptions );
					} else bpvsofar = 0;

					int32 fleetbpv = bpv - bpvsofar;
					if (gWC.fMapPicked > 3) {
						switch (mGetRace()) {
							case kLyran: fleetbpv = (4 * fleetbpv) / 3; break;
							case kGorn: fleetbpv = (5 * fleetbpv) / 4; break;
							case kRomulan: fleetbpv = (6 * fleetbpv) / 5; break;
							case kHydran: fleetbpv = (5 * fleetbpv) / 6; break;
							case kKlingon: fleetbpv = (4 * fleetbpv) / 5; break;
							case kMirak: 
							case kFederation: fleetbpv = (3 * fleetbpv) / 4; break;
							default: break;
						}
						fleetbpv = ((60 + (rand() % 41)) * fleetbpv) / 100;
					}
					if (fleetbpv > 45) {


						// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV
						// calculate the number of AI ships we can handle
						int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

						// put in a call to pick the fleet
						std::string shipspicked[6];
						fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

						// quit if no small enough ship was found
						if (fleetbpv > 0) {
							// create the ships
							for (int i = 0; i < fleetsize; i++) {
								if (shipspicked[i].length() > 2) {
									gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
									mCreateShip(	typeid( tCommonEnemyShip ), 
										shipspicked[i].c_str(), 
										kStartPosition_J + (rand() % 3),
										-1,
										fMyShipID++, 0, 0, -1, -1,
										"",
										kDefaultShipOptions );
								}
							}
						}
					}
				}
			} else {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv += minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 185) {
					bpvsofar = ((60 + (rand() % 39)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < 1) draftedbpv = 100;
				if ((draftedbpv < bpv) && (fShipType != "")) {
					bpvsofar = draftedbpv;
					shiptype = fShipType;
					id = fShipMetaID;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype == "") shiptype = "O-CR1";
				gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_L,
					id,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				if ((bpv - bpvsofar) > 45) {
					int32 fleetbpv = bpv - bpvsofar;
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					if (gWC.fMapPicked > 3) {
						switch (mGetRace()) {
							case kLyran: fleetbpv = (4 * fleetbpv) / 3; break;
							case kGorn: fleetbpv = (5 * fleetbpv) / 4; break;
							case kRomulan: fleetbpv = (6 * fleetbpv) / 5; break;
							case kHydran: fleetbpv = (5 * fleetbpv) / 6; break;
							case kKlingon: fleetbpv = (4 * fleetbpv) / 5; break;
							case kMirak: 
							case kFederation: fleetbpv = (3 * fleetbpv) / 4; break;
							default: break;
						}
						fleetbpv = ((60 + (rand() % 41)) * fleetbpv) / 100;
					}
					// calculate the number of AI ships we can handle
					int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

					// put in a call to pick the fleet
					std::string shipspicked[6];
					fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

					// quit if no small enough ship was found
					if (fleetbpv > 30) {
						// create the ships
						for (int i = 0; i < fleetsize; i++) {
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
								shipspicked[i].c_str(), 
								kStartPosition_J + (rand() % 3),
								-1,
								fMyShipID++, 0, 0, -1, -1,
								"",
								kDefaultShipOptions );
							}
						}
					}
				}
			}
			if (mShipsCurrentlyControlled() < 1) {
					gWC.fDraftedBPV += 100;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						"O-BS", 
						kStartPosition_L,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
			}

		}
	}
}

void tCommonEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( fHumanControlled )
	{
		if (gWC.fDebug && (fTeamID != kTeam1)) return;
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

}

void tCommonEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fMyShipID < 0) switch (fTeamID) {
		case kEnemyTeam1: fMyShipID = kET1Ship1; break;
		case kEnemyTeam2: fMyShipID = kET2Ship1; break;
		case kEnemyTeam3: fMyShipID = kET3Ship1; break;
		case kEnemyTeam4: fMyShipID = kET4Ship1; break;
		case kEnemyTeam5: fMyShipID = kET5Ship1; break;
		case kEnemyTeam6: fMyShipID = kET6Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	fShipMetaID = shipScriptDescription.fMetaDatabaseID;
	if (!mIsTeamHumanControlled()) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		gWC.mSetDrafting();
		fHumanControlled = kFalse;
	} else {
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		fHumanControlled = kTrue;
		// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		std::string msg = "Drafted human enemy ship: ";
		msg += fShipType.c_str();
		msg += ", hullsize: ";
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, gWC.fDraftedBPV);
		mCreateShip(	typeid( tCommonEnemyShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftedHumans++;
	}
	fNumShips++;
}





void tCommonEnemyTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonEnemyTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonEnemyTeam::mNumShips()
{
	return fNumShips;
}

void tCommonEnemyTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}



