/********************************************************************************************************\

	File Name:	CommonPlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerShipBaseState.h"
#include "WorldConditions.h"
#include "MissionText.h"

void tCommonPlayerShipBaseState::mBeginState()
{
	fCurrAlert = kAlertRed;
	fDataptr = NULL;
	fTransTurns = 3 + (rand() % 5);
	fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	fChildActor->mSetShipCallbackEvents( kNotifyDamage);
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
	fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem);
	fChildActor->mSetShipCallbackEvents( kNotifyHostChanged);
	
	fDetectionRange = 80.0f;
	fCurrentlyScanning  = false;
	fTractorOn = false;
}

void tCommonPlayerShipBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}

void tCommonPlayerShipBaseState::mEndState()
{
}

void tCommonPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
}
void tCommonPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	float analysistime = 200.0f / fDetectionRange;
	// if we take damage we have a chance of breakdown
	if (damagedShip && (damagedShip->mGetTeam() > kTeam3) && hostileShip && (hostileShip->mGetTeam() < kTeam4)) {
		gWC.fPlayerFired = true;
	} else if (damagedShip && (damagedShip->mGetTeam() <= kTeam3) && hostileShip && (hostileShip->mGetTeam() >= kTeam4)) {
		gWC.fEnemyFired = true;
	}
}
void tCommonPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	// make sure the jamming has been set for both sides
	if (!gWC.fJammingDetermined) {
		gWC.fJammingDetermined = true;
		int32 total = gWC.fPlayerJammingPotential + gWC.fEnemyJammingPotential;
		if (total > 0) {
			if ((rand() % total) < gWC.fPlayerJammingPotential) {
				gWC.mLogCall("Player able to jam enemy communications", 1);
				gWC.fPlayerJamming = true;
			}
			if ((rand() % total) < gWC.fEnemyJammingPotential) {
				gWC.mLogCall("Enemy able to jam player communications", 1);
				gWC.fEnemyJamming = true;
			}
		}
	}

	// make sure a scanning ship really exists
	if (!ship) return;

	// only process this scan with a live player's ship
	if (fChildActor->mGetTeam() > kTeam3) {
		return;
	}

	// keep track of whether or not our ship is currently scanning
	if (ship == fChildActor) {
		fCurrentlyScanning = deepScanOn;
	}

	if (gWC.fPlayerScanCompleted) return;

	mChkFinalAnalysis();
	
	// we might have completed the mission
	if (gWC.fPlayerGoalCompleted) 
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	
	// set notification up for the next scan
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
}
void tCommonPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{
}
void tCommonPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
}
void tCommonPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
}
void tCommonPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonPlayerShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonPlayerShipBaseState::mOnHostChanged()
{
	gWC.fHostLeft = true;
}
void tCommonPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo* thisteam = gWC.fTeamList[ fChildActor->mGetTeam() ];
	tTeamInfo *drafterteam= NULL;
	tShipInfo *ship = NULL;
	bool shipsdetected = false;
	float range = fDetectionRange;
	int32 x = 0, y = 0;
	float mutinyTime = 5 + (rand() % 7);
	static int32 rangechecks = 0;
	switch (timerID) {
		case kStartingID:
			if (!fDataptr) {
				fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
			}
			if (fDataptr) {
				fDetectionRange = 80.0f + fDataptr->jamming;
				if (fDataptr->isscout) fDetectionRange *= 2;
				gWC.fPlayerJammingPotential = fDataptr->jamming;
			} else fDetectionRange = 80.0f;
			if ((gWC.fDraftedHumans < 1) && (gWC.fMissionType != tWorldConditions::kShootOnSight)) {
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"\n\nWe need more data - I recommend we activate long range scanners, Sir", 
					kMissionScriptMessage );
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
			} else {
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a hostile warship, Sir!", 
					kMissionScriptMessage );
				mSetShootOnSightRelations();
				if (gWC.fDoomsDay) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We\'ve got to protect our installations!", 
						kMissionScriptMessage );
				} else if (gWC.fMutiny) {
					mSetTimedInterval( kTimerTurnsSet, kMutiny, mutinyTime );
				}
			}
			break;
		case kMutiny: mSetMutinyRelations();
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Sir - our ally is turning to attack!!!!", 
					kMissionScriptMessage );
				break;
		case kCheckDistance:
			// see if we've reached the minimum distance to an enemy ship
			shipsdetected = mChkPrelimAnalysis();
			if (!gWC.fAlreadyDetected) {
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
			} 
			break;
		case kAssessSituation:
			mAssessSituation();
			break;
		case kCheckGameEnd:
			gWC.mCheckForGameEnd();
			break;
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tCommonPlayerShipBaseState::mSetFriendlyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm2*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm3*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetHostileRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetMutinyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetShootOnSightRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetEscapeRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

bool tCommonPlayerShipBaseState::mChkPrelimAnalysis()
{
	float range = fDetectionRange;
	if (gWC.fEnemyTargetShip) {
		if (!gWC.fEnemyTargetable) {
			fDetectionRange *= 1.15f;
		if (fCurrentlyScanning) range = 2.0f * range;
			if (gWC.fMapPicked == 2) range = range * 0.75f;
			else if (gWC.fMapPicked == 3) range = range * 0.90f;
			if (gWC.fEnemyTargetShip->mCloakOn()) range = range * 0.75f;
			if (gWC.fEnemyTargetShip->mStealthy()) range = range * 0.75f;
			if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) < fDetectionRange) {
				gWC.fEnemyTargetable = true;
				fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
				gWC.fAlreadyDetected = true;
				switch (gWC.fMissionType) {
				case tWorldConditions::kShootOnSight:
					if ((rand() % 3) == 0) {
						if (gWC.fEnemyTargetShip) {
							gWC.fEnemyTargetShip->mFollowEntity(fChildActor, kAbsolutePriority);
						}
					}
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a naval warship, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				default:
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nUnknown disturbance detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				}
				if (gWC.fUseMinefield) {
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\nI\'m also picking up what appears to be a minefield, Sir", 
						kMissionScriptMessage );
				}
			}
		}
	}
	return(gWC.fAlreadyDetected);
}

void tCommonPlayerShipBaseState::mChkFinalAnalysis()
{
	if (!gWC.fAlreadyDetected) return;
	
	// get the target for the scanning ship, if any
	tShipInfo *target = NULL;
	int32 targetid = -1;
	int32 x = 0;
	int32 y = 0;
	if (fChildActor) targetid = fChildActor->mGetShipTargetID();
	if ((targetid > -1)  && fMissionInfo) {
		target = fMissionInfo->mFindShipID(targetid);
	}

	if (!target) return;

	// set the modifier for final detection range,
	//     larger numbers mean the player has to get closer
	float RangeModifier = 1.0f;
	// =========================================
	switch (gWC.fMissionType) {
		// hostiles are detected farther away, just to be nice
		case tWorldConditions::kShootOnSight:
			RangeModifier = 15.0f;
			break;
		default:
			RangeModifier = 25.0f;
			break;
	}
	//=======================================
	if (fCurrentlyScanning) RangeModifier *= 0.5;

	// check if this player team is scanning
	if (fChildActor->mGetTeam() == kTeam1) { 
		if (target && (target->mGetTeam() > kTeam3) && (fChildActor->mGetTacticalDistance(target) < (fDetectionRange / RangeModifier))){
			gWC.fPlayerScanCompleted = kTrue;
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Scans are inconclusive sir", kMissionScriptMessage );

		}
	} 
}

void tCommonPlayerShipBaseState::mAssessSituation()
{
	if (!gWC.fEnemyTargetShip || !fChildActor) return;
	float playerBPV = gWC.fDraftingBPV * fChildActor->mGetDamagePercent() / 100.0f;
	float enemyBPV = gWC.fDraftingBPV;
	if (gWC.fEnemyTargetShip) enemyBPV = enemyBPV * gWC.fEnemyTargetShip->mGetDamagePercent() / 100.0f;
	if ((enemyBPV < 20) || ((playerBPV / enemyBPV) > 1.20f)) {
		// enemy decides to cut and run
		if (!fEvading) {
			gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
			int32 speed = 17 + (rand() % 8);
			gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
			mSetEscapeRelations();
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				 "Sir - I think they\'re trying to evade!", kMissionScriptMessage );
			fEvading = true;
		}
	} else {
		// enemy decides to fight
		if (fEvading) {
			gWC.fEnemyTargetShip->mClearCommands();
			gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
			gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
			mSetHostileRelations();
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				 "Sir - I think they\'re turning to attack!", kMissionScriptMessage );
			fEvading = false;
		}
	}
	// set a timer to check again in 2 turns
	mSetTimedInterval( kTimerTurnsSet, kAssessSituation, 2.0f );
}
