/********************************************************************************************************\

	File Name:	CommonPlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonPlayerShip.h"
#include "CommonPlayerTeam.h"
#include "CommonPlayerTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"


tClassAllocator<tCommonPlayerTeam> gCommonPlayerTeamAllocator;
 
tCommonPlayerTeam::tCommonPlayerTeam()
{
	fTeamID = -1; 
	fMyShipID = -1; 
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
	fHumanControlled = mIsTeamHumanControlled();
}

void tCommonPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonPlayerTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonPlayerTeamBaseState() );
	mGotoState( typeid( tCommonPlayerTeamBaseState ) );
   }
}

void tCommonPlayerTeam::mSetupCommState()
{}

void tCommonPlayerTeam::mCreateShipsForTeam()
{
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();

	std::string shipname = fShipName;

	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftingBPV += (5 * humanBPV / 4);
	}

	if ((gWC.fMapPicked > 3) && (fTeamID == kTeam3) && (gWC.fIncludeSystem || (gWC.fDraftedHumans > 0))) {
			std::string extraship;
			switch (gWC.fDrafterRace) {
				case kFederation: extraship = "F-PL"; break;
				case kKlingon: extraship = "K-PL"; break;
				case kRomulan: extraship = "R-PL"; break;
				case kLyran: extraship = "L-PL"; break;
				case kGorn: extraship = "G-PL"; break;
				case kHydran: extraship = "H-PL"; break;
				case kMirak: extraship = "Z-PL"; break;
				case kISC: extraship = "I-PL"; break;
				default: extraship = "PL"; break;
			}
			switch (gWC.fMapPicked) {
				case 4: extraship = gWC.fBSchoice[gWC.fDrafterRace]; 
					gWC.fDraftingBPV += 100;
					break;
				case 5: extraship = gWC.fBATSchoice[gWC.fDrafterRace]; 
					gWC.fDraftingBPV += 200;
					break;
				case 6: extraship = gWC.fSBchoice[gWC.fDrafterRace]; 
					gWC.fDraftingBPV += 300;
					break;
				case 7: // colony world
					switch (rand() % 5) {
						case 0: extraship = "N-PLPh0"; break;
						case 1: extraship += "8"; break;
						case 2: extraship += "3"; break;
						case 3: extraship += "9"; break;
						default: extraship += "0"; break;
					}
					gWC.fDraftingBPV += 120;
					break;
				case 8: // core world
					extraship += "Ph";
					switch (rand() % 5) {
						case 0: extraship += "0"; 
							gWC.fDraftingBPV -= 100;
							break;
						case 1: extraship += "1"; break;
						case 2: extraship += "7"; break;
						case 3: extraship += "16"; break;
						default: extraship += "13"; break;
					}
					gWC.fDraftingBPV += 240;
					break;
				case 9: // home world 
					switch (gWC.fDrafterRace) {
						case kFederation: extraship = "F-PE"; break;
						case kKlingon: extraship = "K-PKZ"; break;
						case kRomulan: extraship = "R-PRO"; break;
						case kLyran: extraship = "L-PLY"; break;
						case kGorn: extraship = "G-PG1"; break;
						case kHydran: extraship = "H-PHY"; break;
						case kMirak: extraship = "Z-PM1"; break;
						case kISC: extraship = "I-PI1"; break;
						default: extraship = "O-PE"; break;
					}
					gWC.fDraftingBPV += 360;
					break;
				default: extraship = ""; 
					break;
			}
			if (extraship == "") {
					switch (gWC.fDrafterRace) {
						case kFederation: extraship = "F-FRD"; break;
						case kKlingon: extraship = "K-FRD"; break;
						case kRomulan: extraship = "R-FRD"; break;
						case kLyran: extraship = "L-FRD"; break;
						case kGorn: extraship = "G-FRD"; break;
						case kHydran: extraship = "H-FRD"; break;
						case kMirak: extraship = "Z-FRD"; break;
						case kISC: extraship = "I-FRD"; break;
						default: break;
					}
			}
			if (extraship != "") {
				mCreateShip(	typeid( tCommonPlayerShip ), 
					extraship.c_str(), 
					kStartPosition_Z,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			} else gWC.mLogCall("Could not generate a base/planet for race ", gWC.fDrafterRace);
	}
	if (mIsTeamHumanControlled()) {
		return;
	}
	// add a chance of including an AI ally
	if ((fTeamID == kPlayerTeam3) && (gWC.fAddAlly > 0) && (gWC.fDraftedHumans < 1)) {
		int32 allyBPV = 103 + gWC.fYearLowerBound + (rand() % 53);
		std::string allyshiptype;
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		gWC.fDraftingBPV += (5 * allyBPV / 6);
		mCreateShip(	typeid( tCommonPlayerShip ), 
			allyshiptype.c_str(), 
			kStartPosition_H,
			-1,
			-1, 0, 0, -1, -1,
			"",
			kDefaultShipOptions );
		if (gWC.fAddAlly > 1) {
			allyBPV = 77 + gWC.fYearLowerBound + (rand() % 51);
			allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
			gWC.fDraftingBPV += (5 * allyBPV / 6);
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_I,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
	}
	// have a chance for for fleet battles, in which each non-human team gets three ships
	else if (((rand() % 19) < 1) && (gWC.fDraftedHumans < 1) && (gWC.fMapPicked == 0)) {
		int32 allyBPV = 69 + gWC.fYearLowerBound + (rand() % 59);
		std::string allyshiptype;
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if (allyshiptype != "") {
			gWC.fDraftingBPV += 19 * allyBPV / 20;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_H,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
		allyBPV = 84 + gWC.fYearLowerBound + (rand() % 59);
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if ((allyshiptype != "") && ((rand() % 7) > 4)) {
			gWC.fDraftingBPV += 9 * allyBPV / 10;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_I,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
		allyBPV = 99 + gWC.fYearLowerBound + (rand() % 59);
		allyBPV = gWC.mPickShip(allyBPV, gWC.fDrafterRace, allyshiptype);
		if ((allyshiptype != "") && ((rand() % 7) > 4)) {
			gWC.fDraftingBPV += 11 * allyBPV / 12;
			mCreateShip(	typeid( tCommonPlayerShip ), 
				allyshiptype.c_str(), 
				kStartPosition_G,
				-1,
				-1, 0, 0, -1, -1,
				"",
				kDefaultShipOptions );
		}
	}
}

void tCommonPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( fHumanControlled )
	{
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
			if (!gWC.fPlayerShip) gWC.fPlayerShip = (tCommonPlayerShip*)(*ship);
		}
	}

}

void tCommonPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fTeamID == kTeam1) gWC.fDrafterRace = fMyRace;
	if (fMyShipID < 0) switch (fTeamID) {
		case kPlayerTeam1: fMyShipID = kPT1Ship1; break;
		case kPlayerTeam2: fMyShipID = kPT2Ship1; break;
		case kPlayerTeam3: fMyShipID = kPT3Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (fTeamID == kTeam1) gWC.mPickPlacement();
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	fShipMetaID = shipScriptDescription.fMetaDatabaseID;
	if (!mIsTeamHumanControlled()) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		fHumanControlled = kFalse;
		if (fTeamID < kTeam4) gWC.fMutineerShip = fShipType;
	} else {
		if (fTeamID == kTeam1) gWC.fMutineerShip = fShipType;
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		fHumanControlled = kTrue;
		gWC.fDraftingHulls += gWC.mGetHullsize(fShipType);
		std::string msg;
		if (fTeamID == kTeam1) msg = "Drafter ship: ";
		else msg = "Human wing: ";
		msg += fShipType.c_str();
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, gWC.fDraftingBPV);
		
		if (shipScriptDescription.fClassType == kClassBaseStation) gWC.fBasePlacement = true;
		if (shipScriptDescription.fClassType == kClassBattleStation) gWC.fBasePlacement = true;
		if (shipScriptDescription.fClassType == kClassStarBase) gWC.fBasePlacement = true;
		// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		mCreateShip(	typeid( tCommonPlayerShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftingHumans++;
	}
	fNumShips++;
}





void tCommonPlayerTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonPlayerTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonPlayerTeam::mNumShips()
{
	return fNumShips;
}

void tCommonPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}

