/********************************************************************************************************\

	Mission:	Met_5FleetAction
	Script:		"Met_5FleetAction"
	Descript:	Fleets meet for battle.

	Created by:		Daniel J. Suleski		07/04/00
	Last Updated:	DJS						07/05/00
					Everything
					DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta

\********************************************************************************************************/

#include "Met_5FleetAction.h"
#include "CommonDefs.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "CommonEnemyShip.h"
#include "CommonPlayerShip.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include "time.h"
 
DEFINE_SCRIPT_CLASS( tMet_5FleetAction );

tMet_5FleetAction::tMet_5FleetAction()
{
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMet_5FleetAction::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kPlayerRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fMinBPV = kMinBPV;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fWait	= 2;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kPlayerTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Player Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyAllyOf | kPlayerTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Enemy Team 1",
												static_cast< eTeamID >( kEnemyTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast< eTeamID >( kEnemyTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Enemy Team 3",
												static_cast< eTeamID >( kEnemyTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Player Team 4",
												static_cast< eTeamID >( kPlayerTeam4 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyAllyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team8 = mCreateTeamSpec(	"Enemy Team 4",
												static_cast< eTeamID >( kEnemyTeam4 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kWorstEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team9 = mCreateTeamSpec(	"Player Team 5",
												static_cast< eTeamID >( kPlayerTeam5 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kBestAllyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team10 = mCreateTeamSpec(	"Enemy Team 5",
												static_cast< eTeamID >( kEnemyTeam5 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team11 = mCreateTeamSpec(	"Player Team 6",
												static_cast< eTeamID >( kPlayerTeam6 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team12 = mCreateTeamSpec(	"Enemy Team 6",
												static_cast< eTeamID >( kEnemyTeam6 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMonster,
												kAnyRace,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
	Specs->mSetupTeam( team9 );
	Specs->mSetupTeam( team10 );
	Specs->mSetupTeam( team11 );
	Specs->mSetupTeam( team12 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMet_5FleetAction::mInitializeStart( tDynaverseScriptInfo& info )
{
	gWC.fMapPicked = info.fMapPicked;
	if (gWC.fMapPicked == 1)
		gWC.fNebula = kTrue;
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMet_5FleetAction::mCreateExternalShips( int32 Mode )
{

	// additionally, we'll use this to  add extra terrain
	if ((gWC.fDraftedHumans < 1) && (gWC.fDraftingHumans < 2)) {
		srand((unsigned int)(time(NULL)));
	} else {
		srand(gWC.fMissionSeed);
	}
	int32 x, y, i;

	if (gWC.fMapPicked == 1) { // nebula
		// add two ion storms
		if (gWC.xDim < 21) x = gWC.xCtr;
		else x = (rand() % (gWC.xDim - 20)) + 10;
		if (gWC.yDim < 21) y = gWC.yCtr;
		else y = (rand() % (gWC.yDim - 20)) + 10;
		if (((x + y) % 2) == 0) x--;
		fMissionInfo->mCreateMapObject('{', x, y);
		if (gWC.xDim < 21) x = gWC.xCtr;
		else x = (rand() % (gWC.xDim - 20)) + 10;
		if (gWC.yDim < 21) y = gWC.yCtr;
		else y = (rand() % (gWC.yDim - 20)) + 10;
		if (((x + y) % 2) == 0) x--;
		fMissionInfo->mCreateMapObject('{', x, y);
	} else if (gWC.fMapPicked == 2) { // asteroids
		int32 basex = gWC.xCtr + 7 - (rand() % 14);
		int32 basey = gWC.yCtr + 7 - (rand() % 14);
		// add 10 small dust clouds
		for (i = 0; i < 10; i++) {
			x = (rand() % 17) + basex - 9;
			y = (rand() % 17) + basey - 9;
			if (((x + y) % 2) == 0) x--;
			fMissionInfo->mCreateMapObject('<', x, y);
		}

		// add 5 large dust clouds
		for (i = 0; i < 5; i++) {
			x = (rand() % 15) + basex - 7;
			y = (rand() % 15) + basey - 7;
			if (((x + y) % 2) == 0) x--;
			fMissionInfo->mCreateMapObject('>', x, y);
		}

		// add 10 small asteroids
		for (i = 0; i < 10; i++) {
			x = (rand() % 19) + basex - 9;
			y = (rand() % 19) + basey - 9;
			if (((x + y) % 2) == 0) x--;
			fMissionInfo->mCreateMapObject('[', x, y);
		}

		// add 5 large asteroids
		for (i = 0; i < 5; i++) {
			x = (rand() % 15) + basex - 7;
			y = (rand() % 15) + basey - 7;
			if (((x + y) % 2) == 0) x--;
			fMissionInfo->mCreateMapObject(']', x, y);
		}

		// add 5 ice asteroids
		for (i = 0; i < 5; i++) {
			x = (rand() % 13) + basex - 6;
			y = (rand() % 13) + basey - 6;
			if (((x + y) % 2) == 0) x--;
			fMissionInfo->mCreateMapObject('*', x, y);
		}
	} else if (gWC.fMapPicked == 3) { // black hole
		x = gWC.xCtr - 7 + (rand() % 14); 
		y = gWC.yCtr - 7 + (rand() % 14); 
		if (((x + y) % 2) == 0) x--;
		fMissionInfo->mCreateMapObject('B', x, y);
	} else if (gWC.fMapPicked > 6) { // add a sun or moon
		if ((rand() % 13) == 0) {
			x = 1 + (rand() % 7);
			y = 1 + (rand() % 7);
			char sun = '7' + (rand() % 3);
			fMissionInfo->mCreateMapObject(sun, x, y);
		} else if ((rand() % 3) == 0) {
			x = 5 + (rand() % 11);
			y = 5 + (rand() % 11);
			char planet = '1' + (rand() % 6);
			fMissionInfo->mCreateMapObject(planet, x, y);
		}
	}

    gWC.mLogCall("Final drafting team rank", gWC.fDraftingBPV);
    gWC.mLogCall("Final drafted team rank", gWC.fDraftedBPV);
}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMet_5FleetAction::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMet_5FleetAction::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMet_5FleetAction::mStartMission()
{
	tScript::mStartMission();
	gWC.mStartMissionLog("ED Player Combat");

	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	switch (gWC.fMissionType) {
		case tWorldConditions::kEnemyCombat:
		default: 
			fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
			break;
	}


	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_5FleetAction::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
		case 0:		return tMapTerrain( kTerrainSpace|kTerrainShippingLane, kPlanetNone, kBaseNone );
		case 1:		return tMapTerrain( kTerrainNebula, kPlanetNone, kBaseNone );
		case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetNone, kBaseNone );
		case 3:		return tMapTerrain( kTerrainBlackholes, kPlanetNone, kBaseNone );
		case 4:		return tMapTerrain( kTerrainAnyTerrain, kPlanetNone, kBaseBaseStation );
		case 5:		return tMapTerrain( kTerrainAnyTerrain, kPlanetNone, kBaseBattleStation );
		case 6:		return tMapTerrain( kTerrainAnyTerrain, kPlanetNone, kBaseStarbase );
		case 7:		return tMapTerrain( kTerrainAnyTerrain, kPlanetColony, kBaseNone );
		case 8:		return tMapTerrain( kTerrainAnyTerrain, kPlanetCoreWorld, kBaseNone );
		case 9:		return tMapTerrain( kTerrainAnyTerrain, kPlanetHomeWorld, kBaseNone );
		default:	return tMapTerrain();
	}
	
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_5FleetAction::mASCIIOverrides( tMapOverrides* Map )
{
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_5FleetAction::mDefineBriefings( void )
{
	eRace lastteam = kMirak;

// update the briefing list if this is being compiled for OP
#ifdef OP
	lastteam = kOrionCamboro;
#endif

	for ( int32 j = kFederation; j <= lastteam; j++ )
	{
		for ( int32 i = kPlayerTeam1; i <= kEnemyTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefing_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_5FleetAction::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kPlayerTeam2, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam2, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam2, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);

	mSetTeamShipStrength( kPlayerTeam3, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam3, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam3, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV);

	mSetTeamShipStrength( kEnemyTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFreighter, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassDreadnought, 75l, 240l );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassDreadnought, 75l, 240l );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassDreadnought, 75l, 240l );

	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassDreadnought, 85l, 300l );
	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassDreadnought, 85l, 300l );
	mSetTeamShipStrength( kEnemyTeam3, kClassFrigate, kClassDreadnought, 85l, 300l );

	mSetTeamShipStrength( kPlayerTeam4, kClassFreighter, kClassHeavyCruiser, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam4, kClassFreighter, kClassHeavyCruiser, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam4, kClassFreighter, kClassHeavyCruiser, kMinBPV, kMaxBPV);

	mSetTeamShipStrength( kEnemyTeam4, kClassFrigate, kClassCarrier, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam4, kClassFrigate, kClassCarrier, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam4, kClassFrigate, kClassCarrier, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kPlayerTeam5, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam5, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam5, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);

	mSetTeamShipStrength( kEnemyTeam5, kClassFreighter, kClassCarrier, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam5, kClassFreighter, kClassCarrier, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam5, kClassFreighter, kClassCarrier, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kPlayerTeam6, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam6, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);
	mSetTeamShipStrength( kPlayerTeam6, kClassFreighter, kClassFreighter, kMinBPV, kMaxBPV);

	mSetTeamShipStrength( kEnemyTeam6, kClassMonster, kClassMonster, 100l, 300l );
	mSetTeamShipStrength( kEnemyTeam6, kClassMonster, kClassMonster, 100l, 400l );
	mSetTeamShipStrength( kEnemyTeam6, kClassMonster, kClassMonster, 100l, 500l );

}