/********************************************************************************************************\

	File Name:	worldconditions.h
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#if !defined( WorldConditions_h )
#define WorldConditions_h

#include "CommonDefs.h"
#include "CommonEnemyShip.h"
#include "CommonPlayerShip.h"
#include <iostream>
#include <fstream>

class tWorldConditions
{
public:

	enum eMissionType { 
		kShootOnSight, kEnemyCombat, kEnemyRecon, kEnemyDiplomat, kOursInDistress,
		kCivillianInDistress, kCivillianInnocent, kTowAwayZone, 
		kCivillianSmuggler, kCivillianWanted, kCivillianHostile, kCivillianSpy,
		kPirateInnocent, kPirateSmuggler, kPirateWanted, kPirateHostile, kPirateSpy,
		kMonsterInnocent, kMonsterHostile, kEvacuateTerrain, kSalvageTerrain,
		kPrisonBreak, kShorePatrol, kArbitrator,
		kEscortDuty, kCourierDuty, kPirateBase, kBlackMarket,
		kRepairRendezvous,
		kDealGoneBad, kPirateDoubleX, kCourierSetup,
		// add some mission variants:
		//    kPirateInnocent: could have kPirateDoubleX (you have to cap pirate to get your money or goods)
		//    kBlackMarket: could have kDealGoneBad (you have to cap pirate to get your money or goods)
		//    kCourierDuty: could have kCourierSetup (makes base mad at you, you have to go back and cap courier)
		//    kDerelict: could have a chance of a returning ally
		//    kPirateBase: could add chance of a returning pirate
		kNumMissions, 
	} fMissionType;
	tWorldConditions();
	int32 fMissionSeed;
	std::string fMutineerShip;

	bool fDebug;
	bool fPrintShiplist;
	std::string mNumToStr(float data);

	eRace fDrafterRace;
	bool fPlayerFired, fPlayerFiredEarly, fEnemyFired;
	bool fPlayerScanCompleted;
	bool fPlayerGoalCompleted;
	bool fPlayerGoalFailed;
	bool fPlayerJamming, fEnemyJamming, fEnemyJammed;
	int32 fPlayerJammingPotential, fEnemyJammingPotential;
	bool fJammingDetermined;
	bool fAlreadyDetected;
	int32 fBonusPrestige;
	bool fGameStartedOK;
	bool fUseMinefield;
	int32 fMineX, fMineY;
	bool fBasePlacement;
	int32 fCaptainsGrabbed;
	int32 fSunX, fSunY;

	bool fRescued;
	bool fMutiny;
	int32 fAddAlly;
	bool fIncludeSystem;

	bool fDoomsDay;
	bool fHostLeft;

	// routines for controlling the game end and server reporting
	void mCheckForGameEnd( void );
	void mMyInvokeGameStatus(eTeamID team, eGameStatus stat);

	// routines for recording results in the local BattleLog.txt file
	void mLogCall(std::string source, float data);
	iTruth mAppendToFile(std::string filename, std::string textline);
	iTruth mStartMissionLog(std::string missionname);
	iTruth mLogTeamResults(eTeamID team, eRace race, int32 prestige, eGameStatus result, iTruth human);
	iTruth fResultsIn;

	// variables for tracking vc data
	iTruth fScannedEnemy[kMaxMissionTeams];
	iTruth fInflictedDamage[kMaxMissionTeams];

	// record if any variants are to be applied in ai generation
	void mSetDrafting();
	bool fDraftingDecided;

	// record the size of AI fleets to be generated
	int32 fAddPlayerAIBPV, fAddEnemyAIBPV;
	int32 fExtraDraftedAIBPV;

	// keeping track of number of humans on each side and their bpv
	int32 fDraftingHumans, fDraftedHumans; // the number of humans on each side
	int32 fDraftedBPV, fDraftingBPV; // the total bpv for a side, including ai
	int32 fTeamBPV[kMaxMissionTeams];	// stores the adjusted bpv for each team non-ai ships
	int32 fTeamNebBPV[kMaxMissionTeams]; // stores the nebula-adjusted bpv for each team's non-ai ships
	int32 mGetCombatBPV(eTeamID team); // based on the map type (neb or not), looks up the team's stored bpv
	int32 fDraftingHulls; // combined hull sizes of humans on the drafting side

	// hold pointers to each of the teams
	tTeamInfo *fTeamList[kMaxMissionTeams];

	// hold pointers to the key AI target ships
	tCommonEnemyShip *fEnemyTargetShip;
	tCommonPlayerShip *fPlayerShip;
	bool fEnemyTargetable;

	// record the current map information
	int32 fMapPicked; // tracks which map number was selected
	bool  fNebula; // true iff the mission is being run in a nebula
	int32 xDim, yDim, xCtr, yCtr; // store the size and center point of the map
	bool fDimSet; // true once the dimensions of the map have been read
	void	mPickPlacement(); // set the starting positions G through Z

	// record specialty ships available for each race
	std::string fBSchoice[kNumberOfRaces];
	std::string fBATSchoice[kNumberOfRaces];
	std::string fSBchoice[kNumberOfRaces];
	std::string fFRDchoice[kNumberOfRaces];
	std::string fLPchoice[kNumberOfRaces];
	std::string fDEFchoice[kNumberOfRaces];
	std::string fFreighterChoice1[kNumberOfRaces];
	std::string fFreighterChoice2[kNumberOfRaces];
	std::string fFreighterChoice3[kNumberOfRaces];
	std::string fFreighterChoice4[kNumberOfRaces];
	std::string fFreighterChoice5[kNumberOfRaces];
	std::string fFreighterChoice6[kNumberOfRaces];

	// translate a race string into a number matching the eRace values
	int32 mGetRaceNum(std::string race);
	// lookup if a particular race was drafted for the mission
	bool fRaceUsed[kNumberOfRaces];
	bool mRaceUsed(std::string);
	char mGetRaceChar(std::string race);
	eRace mGetRaceFromChar(char race);

	// store ship data in binary trees sorted by type and bpv
	struct sShipData *fShipsByBPV[kNumberOfRaces], *fShipsByType[kNumberOfRaces];
	int32 fBPVSize, fTypeSize;
	bool mInsert(sShipData *ship);
	bool mInsertBPV(sShipData* &tree, sShipData* ship);
	bool mInsertType(sShipData* &tree, sShipData* ship);
	void mPrintTree(sShipData *s);

	// access the stored ship information for fleet/ship data
	int32 mGetSmallestBPV(eRace race);
	int32 mGetLargestBPV(eRace race);
	sShipData *mSearch(int32 bpv, eRace race);
	sShipData *mSearch(std::string ship, eRace race);
	sShipData* mSearchStock(int32 bpv, eRace race);
	int32 mPickFleet(int32 bpv, int32 numships, eRace race, std::string chosenShips[]);
	int32 mPickShip(int32 bpv, eRace race, std::string &ship);
	int32 fbiggestHuman;
	sShipData* mSearch(std::string ship); 
	int32 mGetMaxParts(std::string ship);
	int32 mGetMaxShuttles(std::string ship);
	int32 mGetMaxMarines(std::string ship);
	int32 mGetMaxMines(std::string ship);
	int32 mGetMaxFtrs(std::string ship);
	int32 mGetHullsize(std::string ship);
	int32 mGetBPV(std::string ship);
	std::string mGetFtrType(std::string ship);
	bool mIsCheapFtr(std::string ftr);
	int32 mGetFtrBays(std::string ship);
	int32 mGetFtrAdj(std::string ship);

	// storing info on the ships drafted 
	std::string fShipTypeList[kMaxShips]; // list of ships belonging to humans
	int32 fTeamNumShips[kMaxMissionTeams]; // number of ships belonging to humans
	std::string fShipTypes[kMaxShips]; // list of all drafted ships for YFA purposes
	int32 fNumShips; // total number of drafted ships

	// routines for processing the shiplist 
	bool fShiplistRead;
	void mReadShiplist();
	void getnextfield(std::string &s, std::ifstream &fpin);

	// keep track of the current year
	int32 fYearLowerBound;
	void mReadYFAs();
	int32 mGetFleetSize(int32 bpv);


};

extern tWorldConditions gWC;

#endif
