/********************************************************************************************************\

	File Name:	CommonEnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonVictory.h"
#include "CommonEnemyShip.h"
#include "CommonEnemyTeam.h"
#include "CommonEnemyTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"

 
tClassAllocator<tCommonEnemyTeam> gCommonEnemyTeamAllocator;

tCommonEnemyTeam::tCommonEnemyTeam() : fNumShips( 0 )
{
	fHumanControlled = mIsTeamHumanControlled();
	fTeamID = -1;
	fMyShipID = -1;
	fMyRace = kNeutralRace;
	fShipType = "";
	fNumShips = 0;
	for (int i = 0; i < 12; i++) fShipList[i] = "";
}

void tCommonEnemyTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonEnemyTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;
	}
}

void tCommonEnemyTeam::mSetupState()
{
   if (! ( (!mIsTeamHumanControlled()) && (!gScriptInterface.mIsHost()) ) )
      {	
	mRegisterState( new tCommonEnemyTeamBaseState() );
	mGotoState( typeid( tCommonEnemyTeamBaseState ) );
   }
   
}

void tCommonEnemyTeam::mSetupCommState()
{}


void tCommonEnemyTeam::mCreateShipsForTeam()
{
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (!gWC.fShiplistRead) {
		gWC.mReadYFAs();
		gWC.mReadShiplist();
	}
	gWC.mSetDrafting();
	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) gWC.fAddAlly = 0;
	std::string shipname = fShipName;
	std::string shiptype = fShipType;

	if (mIsTeamHumanControlled()) {
		int32 humanBPV = 0;
		for (int s = 0 ; s < fNumShips; s++) {
			std::string shiptype = fShipList[s];
			gWC.mLogCall(shiptype, gWC.mGetBPV(shiptype));
			humanBPV += gWC.mGetBPV(shiptype);
			humanBPV += gWC.mGetFtrAdj(shiptype);
		}
		gWC.fDraftedBPV += (5 * humanBPV / 4);
		return;
	}

	if (gWC.fDraftedHumans > 0) {
		return;
	}

	int32 bpv, i, x, y, minbpv, bpvsofar;
	bpv = 0;
	minbpv = 60;


	if ((gWC.fMissionType == tWorldConditions::kEnemyCombat) ||
		(gWC.fMissionType == tWorldConditions::kShootOnSight)) {
		if ((fTeamID == kTeam5) && ((gWC.fMapPicked > 3) || (gWC.fDraftingHumans > 1))) {
			if ((gWC.fDraftedHumans < 1) && (fShipType != "")) {
				gWC.fExtraDraftedAIBPV = gWC.mGetBPV(fShipType);
				gWC.fDraftedBPV += gWC.fExtraDraftedAIBPV;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_K,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
			}
		}
	}

	if (fTeamID == kEnemyTeam3) {
		if(gWC.fUseMinefield) {
			for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-DEF", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"lg mine",
					kDefaultShipOptions );
			}
			for (i = kStartPosition_W; i <= kStartPosition_Z; i++) {
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-LP", 
					i,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"sm mine",
					kDefaultShipOptions );
			}
		} else if ((gWC.fMissionType == tWorldConditions::kEnemyCombat) ||
					(gWC.fMissionType == tWorldConditions::kRepairRendezvous) ) {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < bpv) {
					bpvsofar = draftedbpv;
					shiptype = fShipType;
					id = fShipMetaID;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gWC.fDraftedBPV += bpvsofar;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;
				if ((bpv - bpvsofar) > 45) {
					int32 fleetbpv = bpv - bpvsofar;
					if (gWC.fMapPicked > 3) {
						fleetbpv = ((65 + (rand() % 35)) * fleetbpv) / 100;
					}
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					// calculate the number of AI ships we can handle
					int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

					// put in a call to pick the fleet
					std::string shipspicked[6];
					fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// create the ships
						for (int i = 0; i < fleetsize; i++) {
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_L,
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}
		} else if (gWC.fMissionType == tWorldConditions::kShootOnSight) {
			if ((gWC.fMapPicked > 3) && (gWC.fMapPicked <= 9)) {
				if ((rand() % 23) < 1) {
					gWC.mLogCall("Adding doomsday monster to attack on player installation", fTeamID);
					shiptype = "M-SGL";
					gWC.fDoomsDay = true;
					gWC.fDraftedBPV += 250;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						-1,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else if ((rand() % 23) < 1) {
					gWC.mLogCall("Adding a swarm of tiny attackers against the base/planet", fTeamID);
					for (i = kStartPosition_S; i <= kStartPosition_V; i++) {
						bpv = 50 + (rand() % 35);
						bpv = gWC.mPickShip(bpv, mGetRace(), shiptype);
						if (shiptype == "") {
							shiptype = "M-SSS";
							bpv = 55;
						}
						gWC.fDraftedBPV += bpv;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							shiptype.c_str(), 
							i,
							-1,
							fMyShipID++, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
					for (i = kStartPosition_W; i < kStartPosition_Z; i++) {
						bpv = 50 + (rand() % 35);
						bpv = gWC.mPickShip(bpv, mGetRace(), shiptype);
						if (shiptype == "") {
							shiptype = "M-AMS";
							bpv = 55;
						}
						gWC.fDraftedBPV += bpv;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							shiptype.c_str(), 
							i,
							-1,
							fMyShipID++, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
				} else {
					if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
					bpv += minbpv;
					bpv = (((rand() % 30) + 85) * bpv) / 100;
					bpv = bpv - gWC.fDraftedBPV;
					bpvsofar = gWC.mGetBPV(fShipType);
					shiptype = fShipType;
					// create the flagship
					if (shiptype != "") {
						gWC.fDraftedBPV += bpvsofar;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							shiptype.c_str(), 
							kStartPosition_L,
							fShipMetaID,
							fMyShipID++, 0, 0, -1, -1,
							shipname.c_str(),
							kDefaultShipOptions );
					} else bpvsofar = 0;
					int32 fleetbpv = bpv - bpvsofar;
					if (gWC.fMapPicked > 3) {
						switch (mGetRace()) {
							case kLyran: fleetbpv = (4 * fleetbpv) / 3; break;
							case kGorn: fleetbpv = (5 * fleetbpv) / 4; break;
							case kRomulan: fleetbpv = (6 * fleetbpv) / 5; break;
							case kHydran: fleetbpv = (5 * fleetbpv) / 6; break;
							case kKlingon: fleetbpv = (4 * fleetbpv) / 5; break;
							case kMirak: 
							case kFederation: fleetbpv = (3 * fleetbpv) / 4; break;
							default: break;
						}
						fleetbpv = ((50 + (rand() % 50)) * fleetbpv) / 100;
					}
					if (fleetbpv > 45) {


						// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

						// calculate the number of AI ships we can handle
						int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

						// put in a call to pick the fleet
						std::string shipspicked[6];
						fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

						// quit if no small enough ship was found
						if (fleetbpv > 0) {
							// create the ships
							for (int i = 0; i < fleetsize; i++) {
								if (shipspicked[i].length() > 2) {
									gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
									mCreateShip(	typeid( tCommonEnemyShip ), 
										shipspicked[i].c_str(), 
										kStartPosition_L,
										-1,
										fMyShipID++, 0, 0, -1, -1,
										"",
										kDefaultShipOptions );
								}
							}
						}
					}
				}
			} else {
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv += minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				// pick the main (big) ship, 70-100% of the desired bpv
				int32 id = -1;
				int32 draftedbpv = gWC.mGetBPV(fShipType);
				if (draftedbpv < bpv) {
					bpvsofar = draftedbpv;
					shiptype = fShipType;
					id = fShipMetaID;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 30) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						id = fShipMetaID;
						bpvsofar = draftedbpv;
					}
				}
				if (shiptype != "") {
					gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_L,
						id,
						fMyShipID++, 0, 0, -1, -1,
						shipname.c_str(),
						kDefaultShipOptions );
				} else bpvsofar = 0;
				if ((bpv - bpvsofar) > 45) {
					int32 fleetbpv = bpv - bpvsofar;
					if (gWC.fMapPicked > 3) {
						fleetbpv = ((50 + (rand() % 50)) * fleetbpv) / 100;
					}
					// NEED TO PICK AND GENERATE AN AI FLEET WITH BPV gWC.fAddEnemyAIBPV

					// calculate the number of AI ships we can handle
					int32 fleetsize = gWC.mGetFleetSize(fleetbpv);

					// put in a call to pick the fleet
					std::string shipspicked[6];
					fleetbpv = gWC.mPickFleet(fleetbpv, fleetsize, mGetRace(), shipspicked);

					// quit if no small enough ship was found
					if (fleetbpv > 0) {
						// create the ships
						for (int i = 0; i < fleetsize; i++) {
							if (shipspicked[i].length() > 2) {
								gWC.fDraftedBPV += gWC.mGetBPV(shipspicked[i]);
								mCreateShip(	typeid( tCommonEnemyShip ), 
									shipspicked[i].c_str(), 
									kStartPosition_L,
									-1,
									fMyShipID++, 0, 0, -1, -1,
									"",
									kDefaultShipOptions );
							}
						}
					}
				}
			}
		}
	} else if (fTeamID == kEnemyTeam4) {
		/*
		*/
		int32 bpvsofar, draftedbpv, id;
		switch (gWC.fMissionType) {
			case tWorldConditions::kEnemyRecon:
			case tWorldConditions::kEnemyDiplomat:
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				draftedbpv = gWC.mGetBPV(fShipType);
				id = -1;
				if (draftedbpv < bpv) {
					shiptype = fShipType;
					bpvsofar = draftedbpv;
					id = fShipMetaID;
				} else {
					bpvsofar = gWC.mPickShip(bpvsofar, mGetRace(), shiptype);
					if ((bpvsofar < 20) || (shiptype.length() < 3)) {
						shiptype = fShipType;
						bpvsofar = draftedbpv;
						id = fShipMetaID;
					}
				}
				if (shiptype == "") {
					shiptype = "O-CR1";
					bpvsofar = 80;
				}
				gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_N,
					id,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	} else if (fTeamID == kEnemyTeam5) {
		int32 bpvsofar, id;
		fRace = kOrion;
		switch (gWC.fMissionType) {
			case tWorldConditions::kPirateBase:
				shiptype = "O-BS";
				gWC.fDraftedBPV += 100;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_P,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				mCreateShip(	typeid( tCommonEnemyShip ), 
					"O-FRD", 
					kStartPosition_P,
					-1,
					-1, 0, 0, -1, -1,
					"",
					kDefaultShipOptions );
				// possibly generate an Orion ship as well
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpv = gWC.mPickShip(bpv, kOrion, shiptype);
				id = -1;
				if ((bpv < 20) || (shiptype.length() < 3)) {
					shiptype = fShipType;
					bpv = gWC.mGetBPV(fShipType);
					id = fShipMetaID;
				}
				if ((bpv > 30) && (shiptype != "")) {
					gWC.fDraftedBPV += bpv;
					mCreateShip(	typeid( tCommonEnemyShip ), 
						shiptype.c_str(), 
						kStartPosition_P,
						id,
						fMyShipID++, 0, 0, -1, -1,
						"",
						kDefaultShipOptions );
				}
				break;
			case tWorldConditions::kPirateSmuggler:
			case tWorldConditions::kPirateWanted:
			case tWorldConditions::kPirateHostile:
			case tWorldConditions::kPirateSpy:
				if (gWC.fDraftingBPV > minbpv) minbpv = gWC.fDraftingBPV;
				bpv = minbpv;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpvsofar = bpv;
				if (bpv > 160) {
					bpvsofar = ((60 + (rand() % 49)) * bpv) / 100;
					if (((bpv - bpvsofar) < 50) && ((bpv - bpvsofar) > 15)) bpv += 35;
				}
				bpvsofar = gWC.mPickShip(bpvsofar, kOrion, shiptype);
				id = -1;
				if ((bpvsofar < 20) || (shiptype.length() < 3)) {
					shiptype = fShipType;
					bpvsofar = gWC.mGetBPV(fShipType);
					id = fShipMetaID;
				}
				if (shiptype == "") {
					bpvsofar = 80;
					shiptype = "O-CR1";
				}
				gWC.fDraftedBPV += bpvsofar;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_P,
					id,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				if ((bpv - bpvsofar) > 45) {
					// generate a second pirate ship
					bpv = gWC.mPickShip(bpv - bpvsofar, kOrion, shiptype);
					if (shiptype != "") {
						gWC.fDraftedBPV += bpv;
						mCreateShip(	typeid( tCommonEnemyShip ), 
							shiptype.c_str(), 
							kStartPosition_P,
							-1,
							fMyShipID++, 0, 0, -1, -1,
							"",
							kDefaultShipOptions );
					}
				}
				break;
			default:
				break;
		}
	} else if (fTeamID == kEnemyTeam6) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kPrisonBreak:
			case tWorldConditions::kShorePatrol:
				shiptype = "N-AB";
				gWC.fDraftedBPV += 100;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_R,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			case tWorldConditions::kMonsterHostile:
				gWC.fDraftedBPV += 80;
			case tWorldConditions::kMonsterInnocent:
				gWC.fDraftedBPV += 80;
				switch (rand() % 31) {
					case 0: shiptype = "M-AMS"; break;
					case 1:
					case 2: shiptype = "M-AMM"; gWC.fDraftedBPV += 80;break;
					case 5: shiptype = "M-SSS"; break;
					case 6:
					case 7: shiptype = "M-SSM"; break;
					case 8:
					case 9: shiptype = "M-SSL"; gWC.fDraftedBPV += 60;break;
					case 10: shiptype = "M-LCS"; break;
					case 11:
					case 12: shiptype = "M-LCM"; gWC.fDraftedBPV += 60;break;
					case 13:
					case 14: shiptype = "M-LCL"; gWC.fDraftedBPV += 60;break;
					case 15: shiptype = "M-MTS"; break;
					case 16:
					case 17: shiptype = "M-MTM"; gWC.fDraftedBPV += 60;break;
					case 18:
					case 19: shiptype = "M-MTL"; gWC.fDraftedBPV += 60;break;
					case 20: shiptype = "M-SGS"; break;
					default: shiptype = "M-AMS";
						break;
				}
				if (shiptype == "") shiptype = "M-AMM";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_R,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			case tWorldConditions::kEvacuateTerrain:
			case tWorldConditions::kSalvageTerrain:
				shiptype = "N-AST";
				gWC.fDraftedBPV += 10;
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_R,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	}
	else if (fTeamID == kPlayerTeam5) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kOursInDistress:
				if (!gWC.fUseMinefield && ((rand() % 3) == 0)) {
					// add ion storms around position 0
					fMissionInfo->mGetGridPosition('O', &x, &y);
					fMissionInfo->mCreateMapObject('{', (x-3), (y-3));
					fMissionInfo->mCreateMapObject('{', (x+3), (y-3));
					fMissionInfo->mCreateMapObject('{', (x-3), (y+3));
					fMissionInfo->mCreateMapObject('{', (x+3), (y+3));
				}
				bpv = gWC.fDraftingBPV;
				bpv = (((rand() % 30) + 85) * bpv) / 100;
				bpv = bpv - gWC.fDraftedBPV;
				bpv = gWC.mPickShip(bpv, gWC.fDrafterRace, shiptype);
				if (shiptype == "") shiptype == gWC.fMutineerShip;
				gWC.fDraftedBPV += gWC.mGetBPV(shiptype);
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_O,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			case tWorldConditions::kArbitrator:
				gWC.fDraftedBPV += 50;
				if (fShipType == "") fShipType = "N-FLPh";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_U,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					"agitator",
					kDefaultShipOptions );
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_S,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					"agitant",
					kDefaultShipOptions );
				break;
			case tWorldConditions::kTowAwayZone:
				gWC.fDraftedBPV += 50;
				if (!gWC.fUseMinefield) {
					// add ion storms around position 0
					fMissionInfo->mGetGridPosition('O', &x, &y);
					fMissionInfo->mCreateMapObject('{', (x+3), y);
					fMissionInfo->mCreateMapObject('{', x, (y+3));
				}
			case tWorldConditions::kCivillianInDistress:
				gWC.fDraftedBPV += 50;
				if (!gWC.fUseMinefield) {
					// add ion storms around position 0
					fMissionInfo->mGetGridPosition('O', &x, &y);
					fMissionInfo->mCreateMapObject('{', (x-3), y);
					fMissionInfo->mCreateMapObject('{', x, (y-3));
				}
			case tWorldConditions::kCourierSetup:
				gWC.fDraftedBPV += gWC.mGetBPV(fShipType);
			case tWorldConditions::kCourierDuty:
			case tWorldConditions::kEscortDuty:
			case tWorldConditions::kCivillianInnocent:
				gWC.fDraftedBPV += 50;
				if (fShipType == "") fShipType = "N-FLPh";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_O,
					fShipMetaID,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	} else if (fTeamID == kPlayerTeam4) {
		fRace = kOrion;
		std::string shiptype;
		switch (gWC.fMissionType) {
			case tWorldConditions::kPirateInnocent:
				gWC.fDraftedBPV += 50;
				bpv = 55 + (rand() % 101);
				bpv = gWC.mPickShip(bpv, kOrion, shiptype);
				if (shiptype == "") shiptype = "O-CR1";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					shiptype.c_str(), 
					kStartPosition_M,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	} else if (fTeamID == kPlayerTeam6) {
		switch (gWC.fMissionType) {
			case tWorldConditions::kCivillianSmuggler:
			case tWorldConditions::kCivillianWanted:
			case tWorldConditions::kCivillianHostile:
			case tWorldConditions::kCivillianSpy:
				gWC.fDraftedBPV += gWC.mGetBPV(fShipType);
				if (fShipType == "") fShipType = "N-FLPh";
				mCreateShip(	typeid( tCommonEnemyShip ), 
					fShipType.c_str(), 
					kStartPosition_Q,
					-1,
					fMyShipID++, 0, 0, -1, -1,
					shipname.c_str(),
					kDefaultShipOptions );
				break;
			default:
				break;
		}
	}
}

void tCommonEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	//
	//	If this is a human controlled team, then the AI needs to give up control of the ship.
	//
	if ( fHumanControlled )
	{
		if (gWC.fDebug && (fTeamID != kTeam1)) return;
		tShipIterator ship = mGetFirstShip();
		tShipInPlayEval x;
		ship.mAddEvaluator( &x );
		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

}


void tCommonEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	fShipType = shipScriptDescription.fClassName;
	gWC.fShipTypes[gWC.fNumShips++] = shipScriptDescription.fClassName;
	gWC.fMissionSeed += shipScriptDescription.fMetaDatabaseID;
	gWC.fMissionSeed += shipScriptDescription.fClassName.length();
	gWC.fMissionSeed += shipScriptDescription.fCustomName.length();
	fMyRace = mGetRace(); if (fMyRace < 0) fMyRace = kNeutralRace;
	gWC.fRaceUsed[fMyRace] = kTrue;
	fTeamID = shipScriptDescription.fTeamID;
	if ((fTeamID < 0) || (fTeamID > 11)) fTeamID = mGetTeam();
	if (fMyShipID < 0) switch (fTeamID) {
		case kEnemyTeam1: fMyShipID = kET1Ship1; break;
		case kEnemyTeam2: fMyShipID = kET2Ship1; break;
		case kEnemyTeam3: fMyShipID = kET3Ship1; break;
		case kEnemyTeam4: fMyShipID = kET4Ship1; break;
		case kEnemyTeam5: fMyShipID = kET5Ship1; break;
		case kEnemyTeam6: fMyShipID = kET6Ship1; break;
		case kPlayerTeam4: fMyShipID = kPT4Ship1; break;
		case kPlayerTeam5: fMyShipID = kPT5Ship1; break;
		case kPlayerTeam6: fMyShipID = kPT6Ship1; break;
		default: fMyShipID = fTeamID * 6; break;
	}
	srand(gWC.fMissionSeed);
	if (!gWC.fTeamList[fTeamID]) gWC.fTeamList[fTeamID] = this;
	fShipMetaID = shipScriptDescription.fMetaDatabaseID;
	if (!mIsTeamHumanControlled()) {
		fShipType = shipScriptDescription.fClassName;
		fShipName = shipScriptDescription.fCustomName;
		gWC.mSetDrafting();
		fHumanControlled = kFalse;
	} else {
		int32 shipindex = (6 * fTeamID) + gWC.fTeamNumShips[fTeamID];
		gWC.fShipTypeList[shipindex] = shipScriptDescription.fClassName;
		gWC.fTeamNumShips[fTeamID] += 1;
		fHumanControlled = kTrue;
		gWC.fMissionType = tWorldConditions::kShootOnSight;		
		// human player ships are at GHI, drafted AI allies are at MOQ
		// enemy human ships are at JKL, drafted AI enemies are at NPR
		// spots STUVWXYZ are left open for special purposes
		std::string msg = "Drafted human enemy ship: ";
		msg += fShipType.c_str();
		msg += ", hullsize: ";
		fShipList[fNumShips] = shipScriptDescription.fClassName;
		gWC.mLogCall(msg, shipScriptDescription.fClassType);
		mCreateShip(	typeid( tCommonEnemyShip ), 
					shipScriptDescription.fClassName.c_str(), 
					kStartPosition_G + fTeamID,
					shipScriptDescription.fMetaDatabaseID,
					fMyShipID++, 0, 0, -1, -1,
					shipScriptDescription.fCustomName.c_str(),
					kDefaultShipOptions );

		gWC.fDraftedHumans++;
	}
	fNumShips++;
}





void tCommonEnemyTeam::mIncrementNumShips()
{
	fNumShips++;
}
void tCommonEnemyTeam::mDecrementNumShips()
{
	fNumShips--;
	fNumShips = fNumShips < 0 ? 0 : fNumShips;
}
int32 tCommonEnemyTeam::mNumShips()
{
	return fNumShips;
}

void tCommonEnemyTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}



