/********************************************************************************************************\

	File Name:	CommonPlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonPlayerShipBaseState.h"
#include "WorldConditions.h"
#include "MissionText.h"

void tCommonPlayerShipBaseState::mBeginState()
{
	fCurrAlert = kAlertRed;
	fDataptr = NULL;
	fTransTurns = 3 + (rand() % 5);
	fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
	tTeamInfo* team = gWC.fTeamList[ fChildActor->mGetTeam() ];
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 5.0f );
	fChildActor->mSetShipCallbackEvents( kNotifyDamage);
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
	fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem);
	fChildActor->mSetShipCallbackEvents( kNotifyHostChanged);
	
	fDetectionRange = 60.0f;
	fCurrentlyScanning  = false;
	fTractorOn = false;
}

bool tCommonPlayerShipBaseState::mIsPlayerFirstShip()
{
	tTeamInfo* team = gWC.fTeamList[ fChildActor->mGetTeam() ];
	if (!team) return false;
	if (!team->mIsTeamHumanControlled()) return false;
	tShipIterator ship = team->mGetFirstShip();
	if (fChildActor == (*ship)) return true;
	return false;
}

void tCommonPlayerShipBaseState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
}

void tCommonPlayerShipBaseState::mEndState()
{
}

void tCommonPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{
	switch (gWC.fMissionType) {
		case tWorldConditions::kShorePatrol:
		case tWorldConditions::kPrisonBreak:
			if ((ship->mGetCustomID() == gWC.fEnemyTargetShip->mGetCustomID()) &&
				(ship->mGetShipID() == gWC.fEnemyTargetShip->mGetShipID())) {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Command is going to be REALLY upset that the base was destroyed, Sir", kMissionScriptMessage );
			}
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{
	switch (gWC.fMissionType) {
		case tWorldConditions::kShorePatrol:
		case tWorldConditions::kPrisonBreak:
			if ((ship->mGetCustomID() == gWC.fEnemyTargetShip->mGetCustomID()) &&
				(ship->mGetShipID() == gWC.fEnemyTargetShip->mGetShipID())) {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) {
						fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Our security forces have restored order, Sir!", kMissionScriptMessage );
				}
			}
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{
}
void tCommonPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
	if ((ship) && (gWC.fMissionType == tWorldConditions::kTowAwayZone) && (fTractorOn) && (ship == fChildActor)) {
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"That should do it, Sir!", kMissionScriptMessage );
		gWC.fPlayerGoalCompleted = true;
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	}
}
void tCommonPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{
	if ((gWC.fMissionType == tWorldConditions::kEscortDuty) || (gWC.fMissionType == tWorldConditions::kCourierDuty)) {
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"That\'s close enough, Sir - mission accomplished", 
			kMissionScriptMessage );
		gWC.fPlayerGoalCompleted = true;
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	} else if (gWC.fMissionType == tWorldConditions::kCourierSetup) {
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"Urgent message from the base, Sir - the courier was an imposter!", 
			kMissionScriptMessage );
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"They\'ve ordered us to turn around and hunt them down!", 
			kMissionScriptMessage );
		gWC.fPlayerGoalCompleted = true;
		mAssessSituation();
	}
}
void tCommonPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
}
void tCommonPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	float analysistime = 200.0f / fDetectionRange;
	// if we take damage we have a chance of breakdown
	if (damagedShip && (damagedShip->mGetTeam() > kTeam3) && hostileShip && (hostileShip->mGetTeam() < kTeam4)) {
		gWC.fPlayerFired = true;
		switch (gWC.fMissionType) {
			case tWorldConditions::kEnemyDiplomat:
			case tWorldConditions::kCivillianInDistress:
			case tWorldConditions::kCivillianInnocent:
			case tWorldConditions::kMonsterInnocent:
			case tWorldConditions::kPirateInnocent:
			case tWorldConditions::kOursInDistress:
					if (gWC.fPlayerFired && !gWC.fPlayerJamming && !gWC.fEnemyFired) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					break;
			default:
				break;
		}
	} else if (damagedShip && (damagedShip->mGetTeam() <= kTeam3) && hostileShip && (hostileShip->mGetTeam() >= kTeam4)) {
		gWC.fEnemyFired = true;
	}
}
void tCommonPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	// make sure the jamming has been set for both sides
	if (!gWC.fJammingDetermined) {
		gWC.fJammingDetermined = true;
		int32 total = gWC.fPlayerJammingPotential + gWC.fEnemyJammingPotential;
		if (total > 0) {
			if ((rand() % total) < gWC.fPlayerJammingPotential) {
				gWC.fPlayerJamming = true;
			}
			if ((rand() % total) < gWC.fEnemyJammingPotential) {
				gWC.fEnemyJamming = true;
			}
		}
	}

	// make sure a scanning ship really exists
	if (!ship) return;

	// only process this scan with a live player's ship
	if (fChildActor != ship) {
		return;
	}

	// keep track of whether or not our ship is currently scanning
	if (ship == fChildActor) {
		fCurrentlyScanning = deepScanOn;
	}

	if (gWC.fPlayerScanCompleted) return;

	mChkFinalAnalysis();
	
	// we might have completed the mission
	if (gWC.fPlayerGoalCompleted) 
		mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	
	// set notification up for the next scan
	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan);
}
void tCommonPlayerShipBaseState::mDisableTargetting(tShipInfo *ship)
{
	if ((ship == fChildActor) && fMissionInfo ) {
		fChildActor->mRemoveTargets();
		tShipSystemsInfo info;
		info.fAction = kShipTargetingStatus;
		info.fInt32[0] = kMakeShipAbleToTarget;
		info.fInt32[1] = kFalse;
		info.fShipEntityID = fChildActor->mGetShipID();
		gScriptInterface.mEntityMgr( info );
		if (gWC.fDebug) {
			gWC.mLogCall("Disabling targeting on ship with id", fChildActor->mGetShipID());
			gWC.mLogCall("   and custom id", fChildActor->mGetCustomID());
		}
	}
}
void tCommonPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{
	if ((ship == fChildActor) && fMissionInfo) {
		fCurrAlert = alertStatus;
	}
}
void tCommonPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if (tractorTarget && (tractoringShip == fChildActor) && (gWC.fMissionType == tWorldConditions::kTowAwayZone)) {
		if (tractorTarget->mGetTeam() > kTeam3) {
			fTractorOn = true;
		}
	}
}
void tCommonPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if ((tractoringShip == fChildActor) && (gWC.fMissionType == tWorldConditions::kTowAwayZone)) {
			fTractorOn = false;
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
	}
}
void tCommonPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	// handle cases where our ship is beaming something onto a target ship
	if (shipTo && (shipFrom == fChildActor) && (shipTo->mGetTeam() > kTeam3)) {
		if ((item == kTransMedicalSupplies) || (item == kTransEngineers)) {
				gWC.fPlayerGoalCompleted = true;
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Team away, Sir", kMissionScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			mSetFriendlyRelations();
		} else if (item == kTransMarineDude) {
			tTeamInfo *team = NULL;
			switch (gWC.fMissionType) {
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kEnemyRecon:
				case tWorldConditions::kCivillianSpy:
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Team away, Sir...", kMissionScriptMessage );
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The away team is under attack - they report numerous hostiles aboard, Sir!", kMissionScriptMessage );
					gWC.fPlayerGoalCompleted = true;
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam6) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
				default:
					break;
			}
		}
	}
	// handle cases where something is getting beamed off a target ship and onto our ship
	else if ((shipTo == fChildActor) && shipFrom && (shipFrom->mGetTeam() > kTeam3)) {
		if ((gWC.fMissionType == tWorldConditions::kOursInDistress) || (gWC.fMissionType == tWorldConditions::kCivillianInDistress)) {
			if ((item >= kTransMarineDude) && (item <= kTransEngineers)) {
				fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "They are safely aboard, Sir", kMissionScriptMessage );
				gWC.fRescued = true;
			}
		} else if ((item == kTransDilithiumCrystals) || (item == kTransInjuredDude) || (item == kTransRomulanAle)) {
			gWC.fBonusPrestige += 50;
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Got it, Sir", kMissionScriptMessage );
		} else if (item == kTransTribbles) {
			gWC.fBonusPrestige -= 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Not TRIBBLES, Sir - they\'re going to be a nightmare to get rid of!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransHarryMudd) {
			gWC.fBonusPrestige -= 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Not Harry Mudd, Sir - he\'s going to be nothing but trouble!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransBlackBox) {
			gWC.fBonusPrestige += 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We have retrieved the derelict\'s black box, Sir", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransEngineers) {
			gWC.fBonusPrestige += 50;
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We have retrieved the engineering team, Sir!", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		} else if (item == kTransCaptainDude) {
			fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "That\'s got their attention, Sir ... I\'m sure we can get this worked out now.", kMissionScriptMessage );
			gWC.fPlayerGoalCompleted = true;
			gWC.fPlayerGoalFailed = false;
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
		}
	}
}
void tCommonPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonPlayerShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonPlayerShipBaseState::mOnHostChanged()
{
	gWC.fHostLeft = true;
}
void tCommonPlayerShipBaseState::mSetTargetable(bool State)
{
	tShipSystemsInfo info;
	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipTargetable;
	info.fInt32[1] = State;
	info.fShipEntityID = fChildActor->mGetShipID();
	gScriptInterface.mEntityMgr( info );
}
void tCommonPlayerShipBaseState::mSetTargetting(bool State)
{
	tShipSystemsInfo info;
	info.fAction = kShipTargetingStatus;
	info.fInt32[0] = kMakeShipAbleToTarget;
	info.fInt32[1] = State;
	info.fShipEntityID = fChildActor->mGetShipID();
	gScriptInterface.mEntityMgr( info );
}

void tCommonPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo *thisteam= NULL;
	tTeamInfo *drafterteam= NULL;
	tShipInfo *ship = NULL;
	bool shipsdetected = false;
	float range = fDetectionRange;
	int32 x = 0, y = 0;
	float mutinyTime = 5 + (rand() % 7);
	static int32 rangechecks = 0;
	switch (timerID) {
		case kStartingID:
			if (!fDataptr) {
				fDataptr = gWC.mSearch(fChildActor->mGetClassName(), fChildActor->mGetRace());
			}
			if (fDataptr) {
				fDetectionRange = 60.0f + fDataptr->jamming;
				if (fDataptr->isscout) fDetectionRange *= 2;
				gWC.fPlayerJammingPotential = fDataptr->jamming;
			} else fDetectionRange = 60.0f;
			if ((gWC.fDraftedHumans < 1) && (gWC.fMissionType != tWorldConditions::kShootOnSight)) {
				if (mIsPlayerFirstShip()) 
					fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"\n\nWe need more data - I recommend we activate long range scanners, Sir", 
					kMissionScriptMessage );
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
				mSetTargetting(kFalse);
			} else {
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"\n\nIt looks like a hostile enemy, coming towards us fast, Sir!", 
					kMissionScriptMessage );
				mSetShootOnSightRelations();
				if (gWC.fDoomsDay) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We\'ve got to protect our installations!", 
						kMissionScriptMessage );
				} else if (gWC.fMutiny) {
					mSetTimedInterval( kTimerTurnsSet, kMutiny, mutinyTime );
				}
			}
			if (gWC.fMissionType == tWorldConditions::kRepairRendezvous) {
				std::string shipname = fChildActor->mGetCustomName();
				std::string targetname = "Save my bacon";
				// need to damage the cruiser here
				if ((fChildActor->mGetCustomID() == kMaxShips) || (fChildActor->mGetShipID() == kMaxShips) ||
					(shipname == targetname)) {
					fChildActor->mSetSystemHealthFromOrig( kShieldOneHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShieldTwoHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShieldThreeHardPoint, 0.5f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShieldFourHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShieldFiveHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShieldSixHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.1f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kRightWarpHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kLeftWarpHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kCenterWarpHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kAPRPowerHardPoint, 0.5f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kImpulsePowerHardPoint, 0.3f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kAllEnginesHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kAllWeaponsHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kInternalRepairHardPoint, 0.5f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kImpulsePowerHardPoint, 0.25f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kAPRPowerHardPoint, 0.5f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kTractorHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kMechTractorHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kShuttleBayHardPoint, 0.0f, kTrue );
					fChildActor->mSetSystemHealthFromOrig( kFighterBayHardPoint, 0.0f, kTrue );
				} else {
					gWC.mLogCall("Not the damaged cruiser, id:", fChildActor->mGetCustomID());
				}
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Captain\'s Log: we have come to the aid of a badly damaged cruiser - fortunately there is also a shipyard nearby", 
					kMissionScriptMessage );
			}
			break;
		case kMutiny: mSetMutinyRelations();
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
					"Sir - our ally is turning to attack!!!!", 
					kMissionScriptMessage );
				break;
		case kCheckDistance:
			// see if we've reached the minimum distance to an enemy ship
			shipsdetected = mChkPrelimAnalysis();
			if (!gWC.fAlreadyDetected) {
				mSetTimedInterval( kTimerTurnsSet, kCheckDistance, 0.5f );
			} else mSetTargetting(true);
			break;
		case kCheckRangeToGoal:
				if ((gWC.fMissionType == tWorldConditions::kEscortDuty) || (gWC.fMissionType == tWorldConditions::kCourierDuty)) {
					rangechecks++;
					if (fChildActor->mGetTacticalDistance(gWC.xCtr, gWC.yCtr) <= 12.0f) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"That\'s close enough, Sir - mission accomplished", 
							kMissionScriptMessage );
						gWC.fPlayerGoalCompleted = true;
						mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					} else if ((gWC.fMissionType == tWorldConditions::kCourierSetup)) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Urgent message from the base, Sir - the courier was an imposter!", 
							kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They\'ve ordered us to turn around and hunt them down!", 
							kMissionScriptMessage );
						gWC.fPlayerGoalCompleted = true;
						mAssessSituation();
					} else {
						if ((rangechecks % 6) == 1) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"We need to get closer, Sir", 
								kMissionScriptMessage );
						}
						fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), true, gWC.xCtr, gWC.yCtr);
						mSetTimedInterval( kTimerTurnsSet, kCheckRangeToGoal, 0.5f );
					}
				} else if (gWC.fMissionType == tWorldConditions::kTowAwayZone) {
					rangechecks++;
					fMissionInfo->mGetGridPosition('O', &x, &y);
					if (fChildActor->mGetTacticalDistance(x, y) >= 50.0f) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"That\'s far enough, Sir - mission accomplished", 
							kMissionScriptMessage );
						gWC.fPlayerGoalCompleted = true;
						mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					} else {
						if ((rangechecks % 6) == 1) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"We need to get farther away, Sir", 
								kMissionScriptMessage );
						}
						mSetTimedInterval( kTimerTurnsSet, kCheckRangeToGoal, 0.5f );
					}
				}
				break;
		case kCheckGoalIndicator:
			if (!gWC.fPlayerScanCompleted) {
				mChkFinalAnalysis();
				if (!gWC.fPlayerScanCompleted && ((rand() % 19) < 6)) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "We must get closer before we can complete our analysis, Sir", kNonCommScriptMessage );
					if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) > 80.0f) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "I am updating the plotted path towards them now", kNonCommScriptMessage );
					}
				}
				if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) > 80.0f) {
					fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				} else {
					// hopefully one of these four methods turns off the indicator
					fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), NULL);
					fMissionInfo->mSetGoalIndicator( fChildActor, false, -1, -1 );
					fMissionInfo->mSetGoalIndicator( fChildActor->mGetTeam(), false, -1, -1 );
				}
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
			}
			fDetectionRange *= 1.10f;
			break;
		case kCheckTransportCompletion:
			gWC.fPlayerGoalCompleted = true;
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Transfer complete, Sir!", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kAwayTeamDone:
			if ((rand() % 3) == 0) {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "Something\'s gone wrong on the derelict, Sir!", kMissionScriptMessage );
				gWC.fEnemyTargetShip->mDestroyShip();
				mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			} else {
				gWC.fPlayerGoalCompleted = true;
				if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "The team is done, Sir, we can proceed with demolishing the derelict", kMissionScriptMessage );
				gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransBlackBox, 1);
			}
			break;
		case kAllComplete:
			gWC.fPlayerGoalCompleted = true;
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				"All complete, Sir - everything checks out", kMissionScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
			break;
		case kAssessSituation:
			mAssessSituation();
			break;
		case kCheckGameEnd:
			gWC.mCheckForGameEnd();
			break;
		default:
			break;
	}
}
void tCommonPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tCommonPlayerShipBaseState::mSetFriendlyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm2*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*PTm3*/ "AI", "AI", "AI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetHostileRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetMutinyRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm4*/ "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "WD", "WD", "WD", 
		/*PTm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", "WD",
		/*ETm5*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD", "WD", 
		/*PTm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}
void tCommonPlayerShipBaseState::mSetShootOnSightRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm4*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm4*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm5*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm5*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
		/*PTm6*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "AI", "WD", "AI", "WD",
		/*ETm6*/ "WD", "WD", "WD", "AI", "AI", "AI", "WD", "AI", "WD", "AI", "WD", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mSetEscapeRelations()
{
	const int kRelationsCount = 12;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
		kPlayerTeam4, // a harmless pirate
		kEnemyTeam4, // an enemy diplomat or scout
		kPlayerTeam5, // a harmless civillian
		kEnemyTeam5, // a pirate smuggler, wanted pirate, or hostile pirate
		kPlayerTeam6, // a civillian smuggler, wanted criminal, or hostile
		kEnemyTeam6, // a monster
	};

	gWC.mSetDrafting();
	std::string relDefault[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3  PTm4  ETm4  PTm5  ETm5  PTm6  ETm6
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "WD", "WD", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm1*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm2*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*ETm3*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm4*/ "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI", "NI", "NI",
		/*ETm4*/ "NI", "NI", "NI", "AI", "AI", "AI", "NI", "AI", "NI", "NI", "NI", "NI", 
		/*PTm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", "NI",
		/*ETm5*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI", "NI", 
		/*PTm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", "NI",
		/*ETm6*/ "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "NI", "AI", 
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relDefault, kRelationsCount );
}

void tCommonPlayerShipBaseState::mCarryOutTrade()
{
	if (fChildActor->mGetRace() <= kMirak) return;
	if (!mIsPlayerFirstShip())  return;
	if ((gWC.fDraftedHumans > 0) || (gWC.fDraftingHumans > 1)) {
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"It doesn\'t look promising, Sir.  There\'s nothing much in their inventory, and they don\'t seem eager to buy.", 
			kMissionScriptMessage );
	}
}

bool tCommonPlayerShipBaseState::mChkPrelimAnalysis()
{
	float range = fDetectionRange;
	if (gWC.fMissionType == tWorldConditions::kRepairRendezvous) {
		mSetTargetting(true);
		gWC.fAlreadyDetected = true;
		gWC.fEnemyTargetable = true;
		gWC.fPlayerScanCompleted = true;
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"Sir, an enemy vessel is closing in!", 
			kMissionScriptMessage );
		if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
			"If we can get the ship to the shipyard, we should be able to get enough repair parts aboard to get her in fighting shape...", 
			kMissionScriptMessage );
	} else if (gWC.fEnemyTargetShip) {
		if (!gWC.fEnemyTargetable) {
			fDetectionRange *= 1.15f;
		if (fCurrentlyScanning) range = 2.0f * range;
			if (gWC.fMapPicked == 2) range = range * 0.75f;
			else if (gWC.fMapPicked == 3) range = range * 0.90f;
			if (gWC.fEnemyTargetShip->mCloakOn()) range = range * 0.75f;
			if (gWC.fEnemyTargetShip->mStealthy()) range = range * 0.75f;
			if (fChildActor->mGetTacticalDistance(gWC.fEnemyTargetShip) < fDetectionRange) {
				mSetTargetting(true);
				gWC.fEnemyTargetable = true;
				fMissionInfo->mSetGoalIndicator(fChildActor->mGetTeam(), gWC.fEnemyTargetShip);
				mSetTimedInterval( kTimerTurnsSet, kCheckGoalIndicator, 1.0f );
				gWC.fAlreadyDetected = true;
				switch (gWC.fMissionType) {
				case tWorldConditions::kShorePatrol:
				case tWorldConditions::kPrisonBreak:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nThe source seems to be an asteroid base, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kEnemyCombat:
					if (fDetectionRange < 75.0f) fDetectionRange += 75.0f;
					else if (fDetectionRange < 100.0f) fDetectionRange += 50.0f;
					else if (fDetectionRange < 125.0f) fDetectionRange += 25.0f;
					else fDetectionRange += 15.0f;
				case tWorldConditions::kEnemyRecon:
				case tWorldConditions::kEnemyDiplomat:
				case tWorldConditions::kShootOnSight:
					if ((rand() % 3) == 0) {
						if (gWC.fEnemyTargetShip) {
							gWC.fEnemyTargetShip->mFollowEntity(fChildActor, kAbsolutePriority);
						}
					}
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a naval warship, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re claiming they are authorized to be here, sir", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kCivillianHostile:
				case tWorldConditions::kCivillianInnocent:
				case tWorldConditions::kCivillianSmuggler:
				case tWorldConditions::kCivillianSpy:
				case tWorldConditions::kCivillianWanted:
				case tWorldConditions::kCivillianInDistress:
				case tWorldConditions::kCourierDuty:
				case tWorldConditions::kCourierSetup:
				case tWorldConditions::kTowAwayZone:
				case tWorldConditions::kArbitrator:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "\n\nIt looks like a civillian, Sir - we have plotted a heading towards it", kMissionScriptMessage );
					break;
				case tWorldConditions::kEscortDuty:
				case tWorldConditions::kOursInDistress:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  "\n\nIt appears to be an allied vessel, Sir - we have plotted a heading towards it", kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateInnocent:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nAn unidentified vessel, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateWanted:
				case tWorldConditions::kPirateHostile:
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kPirateSmuggler:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a pirate vessel, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					if ((rand() % 7) > 3) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They\'re claiming they have authorization to be here, sir", 
							kMissionScriptMessage );
					} else {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They\'re not responding to hails, sir", 
							kMissionScriptMessage );
					}
					break;
				case tWorldConditions::kMonsterInnocent:
				case tWorldConditions::kMonsterHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nIt looks like a life form, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kEvacuateTerrain:
				case tWorldConditions::kSalvageTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nSome sort of terrain anomaly detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				case tWorldConditions::kPirateBase: 
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nThe signals are consistent with some form of base, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				default:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\n\nUnknown disturbance detected, Sir - we have plotted a heading towards it", 
						kMissionScriptMessage );
					break;
				}
				if (gWC.fUseMinefield) {
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"\nI\'m also picking up what appears to be a minefield, Sir", 
						kMissionScriptMessage );
				}
			}
		}
	}
	return(gWC.fAlreadyDetected);
}

void tCommonPlayerShipBaseState::mChkFinalAnalysis()
{
	if (!gWC.fAlreadyDetected) return;
	
	// get the target for the scanning ship, if any
	tShipInfo *target = NULL;
	int32 targetid = -1;
	int32 x = 0;
	int32 y = 0;
	if (fChildActor) targetid = fChildActor->mGetShipTargetID();
	if ((targetid > -1)  && fMissionInfo) {
		target = fMissionInfo->mFindShipID(targetid);
	}

	if (!target) return;

	// set the modifier for final detection range,
	//     larger numbers mean the player has to get closer
	float RangeModifier = 1.0f;
	// =========================================
	switch (gWC.fMissionType) {
		// hostiles are detected farther away, just to be nice
		case tWorldConditions::kPirateBase: 
		case tWorldConditions::kEnemyCombat:
		case tWorldConditions::kShootOnSight:
		case tWorldConditions::kPirateHostile:
		case tWorldConditions::kMonsterHostile:
			RangeModifier = 15.0f;
			break;
		// fight-or-flee cases are detected at midrange
		case tWorldConditions::kPrisonBreak:
		case tWorldConditions::kCivillianHostile:
		case tWorldConditions::kCivillianSmuggler:
		case tWorldConditions::kPirateSmuggler:
		case tWorldConditions::kCivillianWanted:
		case tWorldConditions::kPirateWanted:
		case tWorldConditions::kShorePatrol:
		case tWorldConditions::kEnemyRecon:
		case tWorldConditions::kPirateSpy:
		case tWorldConditions::kCivillianSpy:
			RangeModifier = 20.0f;
			break;
		// defenseless cases are detected at close range
		case tWorldConditions::kCivillianInnocent:
		case tWorldConditions::kPirateInnocent:
		case tWorldConditions::kMonsterInnocent:
		case tWorldConditions::kEnemyDiplomat:
		case tWorldConditions::kOursInDistress:
		case tWorldConditions::kCivillianInDistress:
		case tWorldConditions::kTowAwayZone:
		case tWorldConditions::kEvacuateTerrain:
		case tWorldConditions::kSalvageTerrain:
		case tWorldConditions::kArbitrator:
		case tWorldConditions::kEscortDuty:
		case tWorldConditions::kCourierDuty:
		case tWorldConditions::kCourierSetup:
		default:
			RangeModifier = 25.0f;
			break;
	}
	//=======================================
	if (fCurrentlyScanning) RangeModifier *= 0.5;

	// check if this player team is scanning
	int32 speed;
	if (fChildActor->mGetTeam() == kTeam1) { 
		if (target && (target->mGetTeam() > kTeam3) && (fChildActor->mGetTacticalDistance(target) < (fDetectionRange / RangeModifier))){
			switch (gWC.fMissionType) {
				case tWorldConditions::kEnemyCombat:
				case tWorldConditions::kCivillianHostile:
				case tWorldConditions::kPirateHostile:
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kEnemyRecon:
				case tWorldConditions::kCivillianSpy:
				case tWorldConditions::kCivillianSmuggler:
				case tWorldConditions::kPirateSmuggler:
				case tWorldConditions::kCivillianWanted:
				case tWorldConditions::kPirateWanted:
					break;
				default:
					break;
			}
			gWC.fPlayerScanCompleted = kTrue;
			tTeamInfo *team = NULL;
			switch (gWC.fMissionType) {
				case tWorldConditions::kPrisonBreak:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"It\'s a maximum security facility Sir, and it looks like the prisoners are rioting.\nWe need to beam down troops before they can take over the station\'s weaponry!", 
							kMissionScriptMessage );
					break;
				case tWorldConditions::kShorePatrol:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"It\'s chaos down there, Sir\nThey\'re requesting we beam down some marines to help restore order", 
							kMissionScriptMessage );
					break;
				case tWorldConditions::kArbitrator:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"It appears these two yahoos are squabbling over the cargo, Sir\nPerhaps nabbing one of their captains will get their attention...", 
							kMissionScriptMessage );
					break;
				case tWorldConditions::kEscortDuty:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"There must be something important aboard her Sir, we\'ve just been ordered to stand guard until they reach the planet!", 
							kMissionScriptMessage );
					speed = 17 + (rand() % 8);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					gWC.fEnemyTargetShip->mSetShipCallbackEvents(kNotifyAtDistance);
					gWC.fEnemyTargetShip->mTrackTacticalDistance(50.0f, gWC.xCtr, gWC.yCtr, true);
					mSetTimedInterval( kTimerSecondsSet, kCheckRangeToGoal, 1.0f );
					mSetFriendlyRelations();
					break;
				case tWorldConditions::kCourierDuty:
					mSetFriendlyRelations();
				case tWorldConditions::kCourierSetup:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Sir - this ship has just sent a top security message to us, we need to return to the base with it immediately!", 
							kMissionScriptMessage );
					fChildActor->mSetShipCallbackEvents(kNotifyAtDistance);
					fChildActor->mTrackTacticalDistance(50.0f, gWC.xCtr, gWC.yCtr, true);
					mSetTimedInterval( kTimerTurnsSet, kCheckRangeToGoal, 1.0f );
					mSetFriendlyRelations();
					break;
				case tWorldConditions::kPirateBase: 
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"This base has been the source of a lot of local problems lately, Sir - we need to dispose of it!", 
							kMissionScriptMessage );
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					break;
				case tWorldConditions::kTowAwayZone:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"They\'re badly in need of a tow to get them out of the danger zone, Sir", kMissionScriptMessage );
					fMissionInfo->mGetGridPosition('O', &x, &y);
					fChildActor->mSetShipCallbackEvents(kNotifyAtDistance);
					fChildActor->mTrackTacticalDistance(50.0f, x, y, true);
					mSetTimedInterval( kTimerTurnsSet, kCheckRangeToGoal, 1.0f );
					mSetFriendlyRelations();
					break;
				case tWorldConditions::kCivillianHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re attacking!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					mSetHostileRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kPlayerTeam6) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					speed = 15 + (rand() % 5);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kPirateHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re attacking!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					mSetHostileRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					speed = 19 + (rand() % 11);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kMonsterHostile:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re attacking!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					mSetHostileRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam6) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					speed = 16 + (rand() % 7);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kEnemyCombat:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - they\'re attacking!", kMissionScriptMessage );
					gWC.fEnemyTargetShip->mClearCommands();
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
					gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
					mSetHostileRelations();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam3) );
					if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
					mSetShootOnSightRelations();
					speed = 21 + (rand() % 11);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kPirateSpy:
				case tWorldConditions::kEnemyRecon:
				case tWorldConditions::kCivillianSpy:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"There is an order out to investigate this vessel, Sir... \nWe need to get a security team aboard her, forcibly if necessary", 
							kMissionScriptMessage );
					fChildActor->mAddItemToStores( kTransporterItem, kTransMarineDude, 1);
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kEnemyDiplomat:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Technically they do have diplomatic clearance to be here and we should disengage", 
							kMissionScriptMessage );
						gWC.fPlayerGoalCompleted = kTrue;
					if (gWC.fPlayerFired && !gWC.fPlayerJamming) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					speed = 17 + (rand() % 8);
					gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
					break;
				case tWorldConditions::kCivillianInDistress:
				case tWorldConditions::kOursInDistress:
					if ((rand() % 7) > 3) {
						if (mIsPlayerFirstShip()) {
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They appear to be suffering critical control difficulties sir, that\'s why they haven\'t responded to hails.", kMissionScriptMessage );
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"If we can put a repair team aboard, they should be able to fix them up in no time, Sir", kMissionScriptMessage );
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"It sounds like they might have some folks they\'d like evacuated as well, Sir", kMissionScriptMessage );
						}
						fChildActor->mAddItemToStores( kTransporterItem, kTransEngineers, 1);
					} else {
						if (mIsPlayerFirstShip()) {
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"They appear to be suffering a medical emergency sir, that\'s why they haven\'t responded to hails.", kMissionScriptMessage );
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"The medical team is preparing a supply kit, if we can get it through to them, Sir", kMissionScriptMessage );
							fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"It sounds like they might have some folks they\'d like evacuated as well, Sir", kMissionScriptMessage );
						}
						fChildActor->mAddItemToStores( kTransporterItem, kTransMedicalSupplies, 1);
					}
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					mSetFriendlyRelations();
					if (gWC.fEnemyTargetShip) {
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldOneHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldTwoHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldThreeHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldFourHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldFiveHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kShieldSixHardPoint, 0.0f, kFalse );
						gWC.fEnemyTargetShip->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f, kFalse );
					}
					// need to add the requirement to beam a rescue party aboard
					break;
				case tWorldConditions::kCivillianInnocent:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Looks like plain old incompetence sir, they weren\'t manning their comm station", 
							kMissionScriptMessage );
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					gWC.fPlayerGoalCompleted = true;
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					break;
				case tWorldConditions::kCivillianSmuggler:
				case tWorldConditions::kPirateSmuggler:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"We\'re detecting substantial amounts of contraband aboard her sir!", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"I\'m sending a message to stand down and be boarded", kMissionScriptMessage );
					mAssessSituation();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kCivillianWanted:
				case tWorldConditions::kPirateWanted:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"There\'s a warrant out on this vessel sir!", kMissionScriptMessage );
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"I\'m sending a message to stand down and be boarded", kMissionScriptMessage );
					mAssessSituation();
					if (!gWC.fEnemyFired)
						gWC.fPlayerFiredEarly = gWC.fPlayerFired;
					if (gWC.fPlayerFiredEarly) {
						fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFiredEarly_msg);
					}
					break;
				case tWorldConditions::kPirateInnocent:
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"All appears above-board sir, I guess we have to let them go this time", kMissionScriptMessage );
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					gWC.fPlayerGoalCompleted = true;
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					break;
				case tWorldConditions::kMonsterInnocent:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"It appears to be non-threatening sir", kMissionScriptMessage );
					if (gWC.fPlayerFired) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"Command isn\'t going to be pleased with us firing on them...", kMissionScriptMessage );
						if (mIsPlayerFirstShip()) fMissionInfo->mAddToDebriefingList(fChildActor->mGetTeam(), kFriendlyFire_msg);
					}
					gWC.fPlayerGoalCompleted = true;
					mSetTimedInterval( kTimerSecondsSet, kCheckGameEnd, 10.0f );
					break;
				case tWorldConditions::kEvacuateTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - there appears to be a wreck on the surface, with life signs aboard!", kMissionScriptMessage );
					if ((gWC.fMapPicked == 1) || ((rand() % 7) > 3)) {
						if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
							"We need to get a beam them up immediately!", kMissionScriptMessage );
						gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransInjuredDude, 1);
					} else {
					// need to add requirement to evacuate the survivors
						if (fDataptr && (fDataptr->trans > 0)) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"This could take some time sir, we\'ll need to stay within transporter range", kMissionScriptMessage );
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"The engineers will notify us when the transfer is complete", kMissionScriptMessage );
							if (fDataptr && (fDataptr->trans > 0)) fTransTurns = fDataptr->dorms / fDataptr->trans;
							if (fTransTurns > 5.0f) fTransTurns = 5.0f;
							if (fTransTurns < 1.0f) fTransTurns = 1.0f;
							gWC.mLogCall("Expected tranfer time:", fTransTurns);
							gWC.fBonusPrestige = 10 * fDataptr->dorms ;
							mSetTimedInterval( kTimerTurnsSet, kCheckTransportCompletion, fTransTurns );
						}
					}
					mSetFriendlyRelations();
					break;
				case tWorldConditions::kSalvageTerrain:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Sir - there appears to be a wreck on the surface, and signs of some valuable cargo aboard", kMissionScriptMessage );
					if ((gWC.fMapPicked == 1) || ((rand() % 7) > 3)) {
						if ((rand() % 7) > 3)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransDilithiumCrystals, 1);
						else
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransRomulanAle, 1);
						if ((rand() % 5) == 0)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransTribbles, 1);
						else if ((rand() % 7) == 0)
							gWC.fEnemyTargetShip->mAddItemToStores( kTransporterItem, kTransHarryMudd, 1);
					} else {
						// need to add requirement to transport the goods
						if (fDataptr && (fDataptr->trans > 0)) {
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"This could take some time sir, we\'ll need to stay within transporter range", kMissionScriptMessage );
							if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
								"The engineers will notify us when the transfer is complete", kMissionScriptMessage );
							if (fDataptr && (fDataptr->trans > 0)) fTransTurns = fDataptr->storage / fDataptr->trans;
							if (fTransTurns > 5.0f) fTransTurns = 5.0f;
							if (fTransTurns < 1.0f) fTransTurns = 1.0f;
							gWC.mLogCall("Expected tranfer time:", fTransTurns);
							mSetTimedInterval( kTimerTurnsSet, kCheckTransportCompletion, fTransTurns );
							gWC.fBonusPrestige = 10 * fDataptr->storage ;
						}
					}
					mSetFriendlyRelations();
					break;
				default:
					if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
						"Scans are inconclusive sir", kMissionScriptMessage );
					break;
			}

		}
	} 
}

void tCommonPlayerShipBaseState::mAssessSituation()
{
	if (!gWC.fEnemyTargetShip || !fChildActor) return;
	int32 playerBPV = gWC.fDraftingBPV;
	float playerDamage = fChildActor->mGetDamagePercent();
	int32 enemyBPV = gWC.fDraftedBPV;
	float enemyDamage = gWC.fEnemyTargetShip->mGetDamagePercent();
	if ((playerBPV > enemyBPV) && (playerDamage >= enemyDamage)) {
		// enemy decides to cut and run
		if (!fEvading) {
			gWC.fEnemyTargetShip->mDisengage(kByMapEdge, kAbsolutePriority);
			int32 speed = 17 + (rand() % 8);
			gWC.fEnemyTargetShip->mSetShipMaxSpeed(speed);
			mSetEscapeRelations();
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				 "Sir - I think they\'re trying to evade!", kMissionScriptMessage );
			fEvading = true;
		}
	} else {
		// enemy decides to fight
		if (fEvading) {
			gWC.fEnemyTargetShip->mClearCommands();
			gWC.fEnemyTargetShip->mSetShipMaxSpeed(31);
			gWC.fEnemyTargetShip->mSetShipTarget(fChildActor, kAbsolutePriority);
			tTeamInfo *team = NULL;
			switch (gWC.fMissionType) {
				case tWorldConditions::kPirateInnocent:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kPlayerTeam4) );
					break;
				case tWorldConditions::kOursInDistress:
				case tWorldConditions::kArbitrator:
				case tWorldConditions::kTowAwayZone:
				case tWorldConditions::kCivillianInDistress:
				case tWorldConditions::kCourierSetup:
				case tWorldConditions::kCourierDuty:
				case tWorldConditions::kEscortDuty:
				case tWorldConditions::kCivillianInnocent:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kPlayerTeam5) );
					break;
				case tWorldConditions::kCivillianSmuggler:
				case tWorldConditions::kCivillianWanted:
				case tWorldConditions::kCivillianHostile:
				case tWorldConditions::kCivillianSpy:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kPlayerTeam6) );
					break;
				case tWorldConditions::kPirateBase:
				case tWorldConditions::kPirateSmuggler:
				case tWorldConditions::kPirateWanted:
				case tWorldConditions::kPirateHostile:
				case tWorldConditions::kPirateSpy:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam5) );
					break;
				case tWorldConditions::kPrisonBreak:
				case tWorldConditions::kShorePatrol:
				case tWorldConditions::kMonsterHostile:
				case tWorldConditions::kMonsterInnocent:
				case tWorldConditions::kEvacuateTerrain:
				case tWorldConditions::kSalvageTerrain:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam6) );
					break;
				case tWorldConditions::kEnemyRecon:
				case tWorldConditions::kEnemyDiplomat:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam4) );
					break;
				default:
					team = fMissionInfo->mGetTeamHandle( (eTeamID)(kEnemyTeam3) );
					break;
			}
			if (team && !team->mIsTeamHumanControlled()) team->mClearCommands();
			mSetShootOnSightRelations();
			if (mIsPlayerFirstShip()) fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  
				 "Sir - I think they\'re turning to attack!", kMissionScriptMessage );
			fEvading = false;
		}
	}
	// set a timer to check again in 2 turns
	mSetTimedInterval( kTimerTurnsSet, kAssessSituation, 4.0f );
}
